# MPEG-I Encoder

_The swiss army knife of development tools for MPEG-I_

The MPEG-I python package serves as a command line tool to for MPEG-I
developement. It can encode, analyse, and debug MPEG-I scenes. It also
acts as a library to be included for scripting, testing, and experimentation.

[[_TOC_]]

## Getting Started

Perform the following steps to be able to run the encoder:

1. Run `git clone URL` in the terminal of your choice, where `URL` is
   copied from the front page of the encoder git repository

2. Install Conda and setup a Conda virtual environment (see Conda Virtual
   Environment)

3. Run the encoder! (see Usage)

For an API reference, generate the documentation locally with Sphinx
(see `.gitlab-ci.yml` for instructions).


### Conda Virtual Environment

Conda is used as virtual environment manager and package manager for
this project. Just before using the encoder, make sure to activate the
virtual environment, and after usage, make sure to deactivate it again
(instructions below).

To setup the virtual environment, follow these steps:

1. Download
   [MiniConda3](https://docs.conda.io/en/latest/miniconda.html) for
   your OS.

2. Install `MiniConda3`

  - On `Windows`, follow these
    [instructions](https://docs.conda.io/projects/conda/en/latest/user-guide/install/windows.html)
    | **CHECK THE OPTION TO ADD CONDA TO PATH ENVIRONMENT VARIABLE**
  - On `Mac`, follow these
    [instructions](https://docs.conda.io/projects/conda/en/latest/user-guide/install/macos.html)
  - On `Linux`, follow these
    [instructions](https://docs.conda.io/projects/conda/en/latest/user-guide/install/linux.html)
  - Alternatively you can install these with your platforms package manager:
    Choclatey, Brew, apt-get, pacman

3. Run a terminal of your choice and run `conda --version` to make sure that
   conda is correctly installed.
   You can also try to run this and the following commands in the *Anaconda Prompt*.

4. In the main `encoder` directory,
   make a virtual environment with

   ```
   conda env create
   ```

   * On Windows, if there is an `OSError` about file access problems, you should run this:

     ```
     python -m pip config set global.user true
     ```

   * For ARM mac, create the environment with

     ```
     CONDA_SUBDIR=osx-64 conda env create
     ```

5. Activate the virtual environment:

   ```
   conda activate mpegi
   ```

6. You should now be able to execute the `mpegi` package. Test this with the command

   ```
   python -m mpegi -h
   ```

7. Once you are finished using the encoder, deactivate your environment `conda deactivate`

_Hint:
[Here](https://docs.conda.io/projects/conda/en/4.6.0/_downloads/52a95608c49671267e40c689e0bc00ca/conda-cheatsheet.pdf)
is a cheat-sheet for some of the most useful Anaconda commands_


### Docker

If you successfully created a Conda environment (as described above),
you don't need Docker and you can skip this section.

If you do want to use Docker on Windows/macOS,
you should install the *Docker Desktop* application,
which has to be running for the following commands to work.

To build the Docker image run:
```
docker build -t <image-name> .
```

To run the encoder inside a container use:
```
docker run -it --rm <docker-flags> <image-name> python -m mpegi <encoder options>
```

The `--mount` flag can be used to mount a local folder into the container. This can be useful to allow the container to access local EIF files.

A command sequence for using docker from build until encoding may be as follows:
```
# build the docker image with encoder-image as name
docker build -t encoder-image .

# test if the mpegi command works
docker run -it --rm encoder-image python -m mpegi --help

# encode scene.xml located in the /local/path folder outputing the bitstream file in that same folder
docker run -it --rm --mount type=bind,source=/local/path,target=/container/path encoder-image python -m mpegi encode /container/path/scene.xml --output /container/path/output.bit
```

When you get an error like
"The current user is not in the 'docker-users' group." while starting Docker Desktop for Windows,
you can run this in PowerShell (as admin):

    net localgroup docker-users "your-user-id" /ADD

When you get an error like
"Error response from daemon: Bad response from Docker engine" in Docker Desktop,
you can use the troubleshoot menu (little bug icon at the top right) to reset to factory defaults.

## Usage

The mpegi package is a wrapper around many commands. Call `python -m mpegi` in
the terminal of your choice. Note that the `mpegi` package does not function
without specifying a subcommand, very similar to git; one cannot call `git` by
itself, but rather must call `git push` or `git commit`.

### Encode

The encode subcommand will encode an EIF scene file into an MPEG-I bitstream.
You should have two sets of scenes, one to encode from and one to place the
bitstream file once you've finished encoding. The Encoder Input files should
have the original SN3D files where as the Encoder Output files should have the
ESD files. Additionally, the EIF file should exclusively be found in the Encoder
Input files and the encoded bitstream should be exclusively be found in the
Encoder Output files, i.e. no eif_myscene.xml in output and no eif_myscene.bit
in input. This workflow is to ensure that the correct files are used for
encoding and rendering. Violating this workflow is left to the users discretion.

Windows:

`python -m mpegi encode path\to\my\inputscenes\EIF_scenename.xml --output
path\to\my\outputscenes\EIF_scenename.bit`

### Command Line Options

To see what commands and are available call:

    python -m mpegi -h

To see what arguments are available for a given command
(e.g. `encode`) call:

    python -m mpegi encode -h


### Config File

The config file can be changed to supply default arguments whenever the script
is called. Comments with either "#" or ";" are supported.

For example:

```
[global]
--verbose

[encode]
../path/to/mySceneFile.xml
--output=bitstream.bit
```

Would look like this on the command line:

```
python -m mpegi --verbose encode mySceneFile.xml --output=bitstream.bit
```

Note that the arguments under section headings are passed into their respective
subcommands.

Different configs can be supplied with the `-c` or `--config` flag. For example:

`python -m mpegi -c myConfigFile.conf encode /path/to/mySceneFile.xml`

### Profiling

If you call mpegi with the `--profile` argument, then it will dump profiling
stats to the terminal. You can then pipe this information to a file for easier
reading.


`python -m mpegi --profile encode /path/to/scene.xml --output
/path/to/stream.bit > profile.txt`


### How to Run Reverberator

By default the encoder runs the reverberator. It can be disabled with the `--disableReverberator` flag.
See `python -m mpegi encode -h` for supported options. Setting reverberator options will override the defaults.
Otherwise defaults will be used. If run with `--storeReverbOutput`,
reverb parameters will be stored by default as `.xml` file to `src/mpegi/metadata/reverb/meta/`


## Contribution Guide

### Repository Structure

The source code to the mpegi package can be found in `src/mpegi`. Here all of
the base code that is required for the mpegi toolkit is here.

The `tests` directory is where unit tests for the encoder exist. These can be
executed with the following command:

    python -m pytest

The `external` directory is where all libraries, that are submodules, are kept.

The `docs` directory is where documentation is kept and configuration files
for Sphinx to generate HTML pages.


### Workflow

Pushing to the `main` branch is disabled - make one branch per feature and submit a
merge request. Please make atomic (small as possible and revertable) commits.
Please make descriptive commit messages. The naming convention for branch names
are lowercase words seperated by dashes (e.g `my-new-feature`).

### Style Guide

[PEP8](https://pep8.org/) is used as an overall style guide.
Please refer to this if you should have any style questions. The code formatter
[Black](https://github.com/psf/black) is used in the project. Simply run
`black .` to format all code in the current working directory, or
`black mysourcefile.py`, `black mydir/` to specify files and directories.

## Continuous Integration

Refer to the `.gitlab-ci.yml` file to see which operations are done.

## Encoder Design

Through the use of declaring/subscribing to input/output objects the encoder can
utilise caching to the disk and parallelisation. Put simply the encoder does the
following:

1. Analyses dependencies between Components
2. Sorts the Components so that they are executed in the right order
3. Retrieves Artifacts either via cache or via the construction of the Component
4. Constructs Packets, all of which take artifacts upon construction
5. Output a bitstream through the concatenation of all Packets

### Terminolgy

- Component: A small discrete "single-responsibility" unit of the encoder that
  produces Artifacts. Components subscribe to Artifacts and declare the
  Artifacts they produce.

- Artifact: A product of Components - "System Input". They are python objects
  either are serialised into packets or help other components to produce other
  Artifacts that can be serialised.

- Arguments: Configuration parameters supplied on the command line - "User
  Input"

- Inventory: A place to store and access previously made Artifacts.
  ComponentBuilder and PacketBuilder retreive their Artifacts from here. The
  inventory is also responsible for storing and retrieving the Artfacts from the
  DiskCache.

- DiskCache: Stores previously made Artifacts to disk. The name of the cache
  file coresponds to the hash of the objects of the subscribed Arguments and
  Artifacts.

- ComponentBuilder: Responsible for constructing/instantiating Components. The
  ComponentBuilder will forego construction if it can retrieve the Artifacts
  from the DiskCache via the Inventory. The ComponentBuilder will construct the
  components in the order declared by the DependencyTree.

- DependencyTree: Analyses the Artifact/Argument Subscritions and Artifact
  Manifests of all Components and constructs a graph/tree that will ensure that
  all Components which are depended on are created first. It establishes a build
  heirarchy.

- Subscription/Manifest - Components and Packets have an Artifact and Argument
  Subscription. Components have an Artifact Manifest. A Subscription tells the
  system which objects are required by the Component/Packet. A Manifest declares
  which Artifacts are produced.

- PacketBuilder: Responsible for constructing/instanciating Packets. It will
  retrieve the Packets' required Arguments and Artifacts from the Invetory

- Packet: A small component of the bitstream - compliant with MHAS Packets

- Bitstream: A binary file that can be read by the MPEG-I Renderer

### Design Diagram

```
                       +-----------------+
                       |                 |
+----------------------+   Command Args  +-------------------------+
|                      |                 |                         |
|                      +-----------------+                         |
|                                                                  |
|   +---------------------------------------------------------+    |
|   |                     encode-command                      |    |
|   |                                                         |    |
|   |   +--------------------+                                |    |
|   |   |                    |                                |    |
|   |   |   Component List   |                                |    |
|   |   |                    |                                |    |
|   |   +---------+----------+                                |    |
|   |             |                                           |    |
|   |             |                                           |    |
|   |   +---------v----------+       +------------------+     |    |
|   |   |                    |       |                  |     |    |
|   |   |  Dependency Tree   |       |     DiskCache    |     |    |
|   |   |                    |       |                  |     |    |
|   |   +---------+----------+       +--------^---------+     |    |
|   |             |                           |               |    |
|   |             |                           |               |    |
|   |   +---------v----------+       +--------+---------+     |    |
|   |   |                    +------->                  |     |    |
+---+--->  ComponentBuilder  |       |     Inventory    <-----+----+
    |   |                    <-------+                  |     |
    |   +--------------------+       +--------+---------+     |
    |                                         |               |
    |                                         |               |
    |     +-----------------+         +-------v---------+     |
    |     |                 |         |                 |     |
    |     |   Packet List   +--------->  PacketBuilder  |     |
    |     |                 |         |                 |     |
    |     +-----------------+         +-------+---------+     |
    |                                         |               |
    |                                         |               |
    +-----------------------------------------+---------------+
                                              |
                                      +-------v---------+
                                      |                 |
                                      |    Bitstream    |
                                      |                 |
                                      +-----------------+
```
