/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_CODING_UTILS
#define MPEGI_CODING_UTILS

#include <MPEGI/flex_range_coder.h>
#include <cstdio>
#include <cstdlib>

namespace MPEGI {

void encode_uint_compact(FlexRangeEncoder* fre, unsigned int n, int max_bits);
unsigned int decode_uint_compact(FlexRangeDecoder* frd, int max_bits);
float estimate_uint_compact(unsigned int n, int max_bits);
unsigned int interleave_int(int n);
int deinterleave_int(unsigned int n);
void encode_int_magnitude_sign(FlexRangeEncoder* fre, int n, int bits);
int decode_int_magnitude_sign(FlexRangeDecoder* frd, int bits);

//! Memory buffer for encoding and decoding to memory
class MemoryBuffer
{
public:
    int size = 0;
    unsigned char* data = nullptr;
    int pos = 0;
    bool has_overflown = false;
    bool has_overread = false;

    void setup(int buffer_size, unsigned char* buffer_data)
    {
        size = buffer_size;
        data = buffer_data;
        pos = 0;
        has_overflown = false;
        has_overread = false;
    }

    void write_byte(unsigned char byte_val)
    {
        if (pos < size) {
            data[pos++] = byte_val;
        } else {
            pos++; // keep counting bytes to compute the needed buffer size
            has_overflown = true;
        }
    }

    int read_byte()
    {
        int result;

        if (pos < size) {
            result = data[pos++];
        } else {
#if FRD_DUMP_EOF
            printf("read eof\n");
#endif

            result = -1; // indicate EOF using -1
            pos++; // keep counting bytes to compute the actual readed size
            has_overread = true;
        }

        return result;
    }
};

void fre_write_byte_to_memory_buffer(void* output_state, unsigned char byte_val);
int frd_read_byte_from_memory_buffer(void* input_state);

} // namespace MPEGI

#endif /* MPEGI_CODING_UTILS */
