#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include <memory>
#include <vector>

#include <MPEGI/DirectivityCoding.h>

namespace py = pybind11;

struct Cover
{
    Cover(float _f, std::vector<float> _g)
        : frequency(_f)
        , gains(_g)
    {}

    float frequency;
    std::vector<float> gains;
};

py::bytes encodeCovers(std::vector<Cover> covers, float degStepReq, float dbStepReq)
{
    auto coverList = std::make_unique<MPEGI::DirectivityCoverList>();

    for (auto& cover : covers) {
        std::size_t coverSize = cover.gains.size();
        coverList->addDirectivityCover(
            cover.frequency, degStepReq, coverSize, cover.gains.data(), dbStepReq);
    }

    int requiredBufferSize = coverList->getRequiredBufferSize();
    auto buffer = std::make_unique<unsigned char[]>(requiredBufferSize);
    int actualBufferSize = coverList->encodeToBuffer(requiredBufferSize, buffer.get());
    const char* bufferPtr = (const char*)buffer.get();
    return py::bytes(bufferPtr, actualBufferSize);
}
