"""Efficient Acoustic Material Encoding."""
from typing import List

class Coeffs:
    """Coefficients Per Frequency for Acoustic Materials"""

    f: float
    r: float
    s: float
    t: float
    c: float
    def __init__(
        self,
        *,
        f: float,
        r: float,
        s: float,
        t: float,
        c: float,
    ) -> None:
        """Construct Acoustic Material coefficients per Frequency.

        The sum of the coefficients must be less than or equal to 1.0.

        :param f: Frequency in Hertz (1.0-24000.0)
        :param r: Specular reflection coefficient (0.0-1.0)
        :param s: Diffuse Scattering coefficient (0.0-1.0)
        :param t: Transmission coefficient (0.0-1.0)
        :param c: Coupling Coefficient (0.0-1.0)
        """
        ...

class Material:
    """Acoustic Material"""

    coeffs: List[Coeffs]
    def __init__(self, coeffs: List[Coeffs]) -> None:
        """Construct an Acoustic Material

        :param coeffs: A list of coefficients
        """
        ...

def encode(materials: List[Material], *, mat_qnt_step: float) -> bytes:
    """Encode a list of materials into bytes.

    :param materials: The list of materials to encode
    :param mat_qnt_step: The quantization step size for coefficients
    :return: The encoded materials in bytes
    """
    ...
