import copy

import mcode


def test_code_symbol() -> None:
    """Encode a symbol with a given alphabet."""
    fre = mcode.flex.FlexEncoder()
    alph = mcode.flex.Alphabet("test_alph", size=26)
    symbol = 13
    fre.encode_symbol(symbol, alpha=alph)
    blob = fre.finish()
    frd = mcode.flex.FlexDecoder(blob)

    decoded_symbol = frd.decode_symbol(alph)

    assert symbol == decoded_symbol


def test_code_symbol_uniform() -> None:
    """Encode a symbol with uniform probability."""
    alph = mcode.flex.Alphabet("test_alph", size=24)
    fre = mcode.flex.FlexEncoder()
    symbol = 13
    fre.encode_symbol_uniform(symbol, alpha=alph)
    blob = fre.finish()
    frd = mcode.flex.FlexDecoder(blob)

    decoded_symbol = frd.decode_symbol_uniform(alph)

    assert symbol == decoded_symbol


def test_code_symbol_uniform_long() -> None:
    """Encode a symbol long with uniform probability."""
    alph = mcode.flex.Alphabet("test_alph", size=(2**32) - 1)
    fre = mcode.flex.FlexEncoder()
    symbol = 1024
    fre.encode_symbol_uniform(symbol, alpha=alph)
    blob = fre.finish()
    frd = mcode.flex.FlexDecoder(blob)

    decoded_symbol = frd.decode_symbol_uniform(alph)

    assert symbol == decoded_symbol


def test_code_bits() -> None:
    """Encode and the decode bits."""
    value = 255
    width = 8
    fre = mcode.flex.FlexEncoder()
    fre.encode_bits(value, width=width)
    blob = fre.finish()
    frd = mcode.flex.FlexDecoder(blob)

    read_value = frd.decode_bits(width=width)

    assert read_value == value


def test_encode_lots_of_bits() -> None:
    """Encode bits to try and overflow buffer."""
    fre = mcode.flex.FlexEncoder()
    for _ in range(267):
        fre.encode_bits(255, width=8)

    blob = fre.finish()

    assert isinstance(blob, bytes)


def test_code_bits_long() -> None:
    """Encode and the decode bits."""
    width = 32
    value = (2**width) - 1
    fre = mcode.flex.FlexEncoder()
    fre.encode_bits_long(value, width=width)
    blob = fre.finish()
    frd = mcode.flex.FlexDecoder(blob)

    read_value = frd.decode_bits_long(width=width)

    assert read_value == value


def test_encode_lots_of_bits_long() -> None:
    """Encode bits to try and overflow buffer."""
    fre = mcode.flex.FlexEncoder()
    for _ in range(267):
        fre.encode_bits_long((2**32) - 1, width=32)

    blob = fre.finish()

    assert isinstance(blob, bytes)


def test_fre_copy() -> None:
    """Testing for fre copy."""
    fre_a = mcode.flex.FlexEncoder()
    fre_b = copy.deepcopy(fre_a)
    fre_a.encode_bits(0, width=8)
    fre_b.encode_bits(16, width=9)

    blob_a = fre_a.finish()
    blob_b = fre_b.finish()

    assert blob_b != blob_a
