# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import argparse

from mpegi.cli import Command, Config
from mpegi.coding import DirectivityEncoder
from mpegi.encoder.components import DirectivityComponent


class DirectivityCommand(Command):
    @staticmethod  # Inherited from cli.Command
    def execute(config: Config) -> None:
        encoder = DirectivityEncoder()
        blob = encoder.encode(
            config["file"], config["deg"], config["db"], config["directivity_dump"]
        )

        if config["hex"]:
            blobString = blob
        else:
            blobString = f"Byte Length: {int(len(blob) / 2)}"

        print(f"Encoded directivity {config['file']}:\n{blobString}")

    @staticmethod  # Inherited from cli.Command
    def getCommandName() -> str:
        return "directivity"

    @staticmethod  # Inherited from cli.Command
    def registerArguments(argParser: argparse.ArgumentParser) -> None:
        argParser.add_argument("file", metavar="PATH", help="Path to .sofa file")
        argParser.add_argument("--hex", action="store_true", help="Dump directivity info as hex")
        DirectivityComponent.registerArguments(argParser)
