# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.

"""Bitstream Syntax constants.

All constants here are constants that are used across multiple payloads.
"""

from mpegi.framework import QuantizationRange

bitsForId = 16

# All "max" variables determine maximum number, i.e, 2^8 = 256 maximum
bitsForMaxAnchors = 16
bitsForMaxTforms = 16
bitsForMaxStreams = 16
bitsForMaxMats = 16
bitsForMaxDirecs = 16
bitsForMaxPrims = 16
bitsForMaxMeshes = 16
bitsForMaxEnvs = 16
bitsForMaxConds = 16
bitsForMaxObjSrcs = 16
bitsForMaxHoaSrcs = 16
bitsForMaxHoaGrps = 16
bitsForMaxChanSrcs = 16
bitsForMaxUpdates = 16

bitsForMaxAEPChans = 8  # AEP Input channels in an audio stream
bitsForMaxLdspkers = 8  # Loudspeakers in a channel source
bitsForMaxParams = 16  # Acoustic Parameters in an Envrionment
bitsForMaxFreqsAP = 8  # Frequencies in acoustic parameters
bitsForMaxFreqsMat = 8  # Frequencies in acoustic materials
bitsForMaxCoSrcs = 16  # Maximum number of co sources within HOA Group
bitsForMaxUpdateIdx = 16  # Max Update indexes. EIF Index and not update index
bitsForMaxMods = 16  # Max Update indexes. EIF Index and not update index
bitsForMaxVertices = 15  # Max Vertices within a mesh
bitsForMaxFaces = 16  # Max faces within a mesh

bitsForMaxEdges = 16  # Max edges within a mesh
bitsForMaxPathsPerSourceListenerPair = 16  # Max paths per key
bitsForMaxEdgesPerPath = 8  # Max edges per path
bitsForMaxGridsPerAxis = 16  # Max paths within a mesh
bitsForMaxEdgesPerVoxel = 16  # Max edges per voxel
bitsForMaxVoxelsPerSource = 16  # Max voxels per source
bitsForMaxListenerVoxels = 32  # Max listener voxels
bitsForMaxPathIds = 32  # Max paths

bitsForVoxelGridShapeX = 32
bitsForVoxelGridShapeY = 32
bitsForVoxelGridShapeZ = 32
bitsForNormalAzi = 12
bitsForNormalEle = 11
bitsForPlaneDist = 22
bitsForDiffrAngle = 24
# bitsForNumVoxelPairs = 32  # defined in _voxelDataEarlyEncoder

x_coord_qrange = QuantizationRange(min=-2000.0, max=2000.0, num_bits=24)
y_coord_qrange = QuantizationRange(min=-2000.0, max=2000.0, num_bits=24)
z_coord_qrange = QuantizationRange(min=-2000.0, max=2000.0, num_bits=24)
x_voxel_grid_pitch_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
y_voxel_grid_pitch_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
z_voxel_grid_pitch_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
x_length_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
y_length_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
z_length_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
x_primitive_size_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
y_primitive_size_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
z_primitive_size_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
ref_dist_qrange = QuantizationRange(min=0.0, max=100.0, num_bits=10)
yaw_qrange = QuantizationRange(min=-180.0, max=180.0, num_bits=10)
pitch_qrange = QuantizationRange(min=-180.0, max=180.0, num_bits=10)
roll_qrange = QuantizationRange(min=-180.0, max=180.0, num_bits=10)
edge_angle_qrange = QuantizationRange(min=0.0, max=360.0, num_bits=10)
edge_length_qrange = QuantizationRange(min=0.0, max=500.0, num_bits=19)
edge_vector_length_qrange = QuantizationRange(min=-500.0, max=500.0, num_bits=29)
edge_normalised_vector_length_qrange = QuantizationRange(min=-1.0, max=1.0, num_bits=19)
directiveness_qrange = QuantizationRange(min=0.0, max=20.0, num_bits=8)
gain_qrange = QuantizationRange(min=-127.0, max=127.0, num_bits=12)
scene_duration_qrange = QuantizationRange(min=0.0, max=180.0, num_bits=8)
update_time_qrange = QuantizationRange(min=0.0, max=180.0, num_bits=16)
update_delay_qrange = QuantizationRange(min=0.0, max=10.0, num_bits=9)
modification_duration_qrange = QuantizationRange(min=0.0, max=180.0, num_bits=17)
portal_factor_qrange = QuantizationRange(min=0.0, max=1.0, num_bits=19)
audio_diffuseness_qrange = QuantizationRange(min=0.0, max=1.0, num_bits=7)
geq_sos_quantization_range = QuantizationRange(min=-4, max=4, num_bits=31)
geq_leveldb_quantization_range = QuantizationRange(min=-127, max=127, num_bits=31)
geq_global_gain_quantization_range = QuantizationRange(min=-4, max=4, num_bits=31)
reverb_distance_gain_drop_qrange = QuantizationRange(0, 128, num_bits=16)
reverb_minimum_distance_qrange = QuantizationRange(0, 64, num_bits=12)
predelay_factor_qrange = QuantizationRange(0, 4294967295, num_bits=31)
ae_gain_qrange = QuantizationRange(0, 1, num_bits=16)
fade_in_distance_qrange = QuantizationRange(0, 128, num_bits=16)
lsdf_rdr_adjustment_qrange = QuantizationRange(min=-50.0, max=50.0, num_bits=16)
lsdf_speed_of_sound_qrange = QuantizationRange(min=150, max=700.0, num_bits=31)
is_small_scene = False
airflow_speed_range = QuantizationRange(min=0, max=100, num_bits=10)
airflow_filter_Q_range = QuantizationRange(min=0, max=10, num_bits=7)
airflow_angle_range = QuantizationRange(min=0, max=360, num_bits=9)
airflow_aziElevLimit_range = QuantizationRange(min=0, max=180, num_bits=8)
airflow_distanceAttenuationFactor_range = QuantizationRange(min=0, max=5, num_bits=8)
ccHpQ_qrange = QuantizationRange(min=0.0, max=10.0, num_bits=7)
granularDBcoordScaling_range = QuantizationRange(min=0.0, max=100.0, num_bits=10)
granularDBsmoothingTargetCoordinates_range = QuantizationRange(min=0.0, max=1.0, num_bits=10)
granularDBsearchRadius = QuantizationRange(min=0.0, max=1.0, num_bits=10)
granularDBfastGrainSwitchThreshold = QuantizationRange(min=0.0, max=2.0, num_bits=10)
granularDBprobExp = QuantizationRange(min=0.0, max=10.0, num_bits=8)
granularDBprobWeight = QuantizationRange(min=0.0, max=100.0, num_bits=16)
granularDB_qrange = QuantizationRange(min=-1.0, max=1.0, num_bits=16)  # for audio samples
granularDB_descriptor_qrange = QuantizationRange(
    min=0.0, max=1, num_bits=16
)  # for descriptor coordinates
granularDB_descriptor_trend_qrange = QuantizationRange(
    min=-5.0, max=5.0, num_bits=16
)  # for descriptor coordinate trends
granularDB_pos_within_audio_file_qrange = QuantizationRange(
    min=0, max=20, num_bits=16
)  # for position within audio file
grain_overlap_amount_qrange = QuantizationRange(
    min=0.0, max=1, num_bits=10
)  # grain overlap percentage/amount
grainProbabilityValue_qrange = QuantizationRange(min=0.0, max=1000.0, num_bits=16)
grainProbabilityGroupNo_qrange = QuantizationRange(min=0.0, max=255.0, num_bits=8)
granularDBgrainInterpDistThreshold = QuantizationRange(min=0.0, max=1.0, num_bits=10)
granularMixRule_qrange = QuantizationRange(min=0.0, max=1.0, num_bits=8)
