# This software module was originally developed by
#
# Ericsson AB
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Ericsson AB retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import argparse
import pathlib
from typing import Any, Dict, List

import numpy as np
from scipy import signal
from xml.dom.minidom import Document, parse
from mpegi.framework import Component
from mpegi.coding import GrainDBEncoder


class GranularDatabaseComponent(Component):
    """A component is a small compartmental unit of the encoder. It is responsible
    for creating artifacts (typically) from preexisting artifacts created from
    other components. This component reads a granular database from file path specified in the EIF and returns the audio grains and coordinates as output.
    """

    def __init__(self, args: Dict[str, Any], artifacts: Dict[str, Any]):

        self.grainDBCoded = {}
        encoder = GrainDBEncoder()
        for grain in artifacts["granular_sound"]:
            path = str(pathlib.Path(args["scene"]).parent.joinpath(grain.file))
            AllGrains = encoder.encode(path)
            self.grainDBCoded[grain.id] = AllGrains

    @staticmethod
    def getArgumentSubscription() -> List[str]:
        return ["scene"]

    @staticmethod
    def getArtifactSubscription() -> List[str]:
        return [
            "granular_sound"
        ]  # get granular sound as an artifact from sceneplus-where the info is read from EIF

    @staticmethod
    def getArtifactManifest() -> List[str]:
        return ["grain_coded"]

    def getArtifacts(self) -> Dict[str, Any]:
        return {"grain_coded": self.grainDBCoded}

    @staticmethod
    def getVersion() -> Any:
        return 1

    @staticmethod
    def registerArguments(argParser: argparse.ArgumentParser) -> None:
        pass
