# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from typing import Tuple, Any, Optional

from ..bitstream import Bitstream
from ._token import Token
from ._checktype import checkType
from ._generator import TokenGenerator


class Flag(Token):
    """Single bit, a boolean value"""

    def __init__(self, label: str):
        """Constructs a flag token

        :param label: The name of this flag token
        """
        super().__init__(label)
        self.label = label

    def to_bitstream(self, value: bool) -> Bitstream:
        """Takes the incoming value and puts it into a bitstream

        :param value: The value to put into a bitstream
        :return: The bitstream representation of the value
        """
        checkType(self, bool, value)
        bitstream = Bitstream()
        bitstream.addBool(value)
        return bitstream

    def from_bitstream(self, bitstream: Bitstream, start: int = 0) -> Tuple[int, Any]:
        token_as_bitstream = bitstream.copy(start, 1)
        value = token_as_bitstream.toBool()
        return (1, value)

    def get_token(self, token_generator: TokenGenerator) -> Optional[Token]:
        return self
