# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import copy
from typing import Any, List, Optional, Tuple, Union

from ..bitstream import Bitstream
from ._generator import TokenGenerator
from ._token import Token


class Repeat(Token):
    def __init__(self, iterations: Union[int, str], syntax: List[Token]) -> None:
        self.syntax = syntax
        self.iterations = iterations

        label = "ANONYMOUSTOKEN"
        if isinstance(iterations, str):
            label = iterations
        super().__init__(label)

    def to_bitstream(self, value: Any) -> Bitstream:
        """Control flow structures don't implement toBitstream"""
        return super().to_bitstream(value)

    def from_bitstream(self, bitstream: Bitstream, start: int = 0) -> Tuple[int, Any]:
        return super().from_bitstream(bitstream, start)

    def get_token(self, token_generator: TokenGenerator) -> Optional[Token]:
        iterations = self.iterations
        if isinstance(iterations, str):
            try:
                iterations = token_generator.scope[iterations]
                if isinstance(iterations, list):
                    iterations = int(iterations[-1])
            except KeyError as exc:
                raise KeyError(
                    f"Token not found in Syntax. Repeat Token expects: '{iterations}'"
                ) from exc

        tokens = copy.deepcopy(self.syntax) * iterations
        tokens.reverse()
        token_generator.token_stack.extend(tokens)
        return token_generator.get_next_token()
