# This software module was originally developed jointly by
#
# Fraunhofer IIS and Koninklijke Philips N.V.
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS and Koninklijke Philips N.V. retain full right to
# modify and use the code for their own purpose, assign or donate the code
# to a third party without consent of and without accounting to third
# parties, and to inhibit third parties from using the code for products
# that do not conform to MPEG-related ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.


from typing import Any, Optional, Tuple


from ..bitstream import Bitstream
from ._generator import TokenGenerator
from ._token import Token
from ._writeDistance import writeDistance as writeDistance


class writePosition(Token):
    """Write a distance as bitstring using LUTs"""

    def __init__(self, label: str = "", *, isSmallScene: bool = False):
        super().__init__(label)
        self.label = label
        self.isSmallScene = isSmallScene

    def to_bitstream(self, value) -> Bitstream:
        bs = ""
        if isinstance(value, list):
            position = value
        elif isinstance(value, dict):
            position = value.get("position")
            self.isSmallScene = value.get("isSmallScene")
        for coord in position:
            isNegative = coord < 0
            bs += Bitstream.to01(
                writeDistance.to_bitstream(
                    self, {"distance": abs(coord), "isSmallScene": self.isSmallScene}
                )
            )
            if isNegative:
                bs += str(1)
            else:
                bs += str(0)

        bitstream = Bitstream()
        bitstream = bitstream.from01(bs)
        return bitstream

    def from_bitstream(self, bitstream: Bitstream, start: int = 0) -> Tuple[int, Any]:
        position = []
        num_bits = 0
        for c in range(0, 3):
            num_bits_coord, coord = writeDistance.from_bitstream(
                self, bitstream, start + num_bits, isSmallScene=self.isSmallScene
            )
            num_bits += num_bits_coord
            isNegative = int(bitstream.copy(start + num_bits, 1).to01(), 2)
            num_bits += 1
            if isNegative:
                position.append(coord * -1)
            else:
                position.append(coord)
        return num_bits, position

    def get_token(self, token_generator: TokenGenerator) -> Optional[Token]:
        return self
