# This software module was originally developed by
#
# Qualcomm Technologies, Inc.
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# QUALCOMM Incorporated retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2023.

from typing import Any, Optional, Tuple

import numpy as np

from mpegi.framework import _lookup_tables as LUT

from ..bitstream import Bitstream
from ._generator import TokenGenerator
from ._token import Token
from ._util import readCodeword


class writeTolerance(Token):
    """write ID as a varlaible length bitstring"""

    def __init__(self, label: str = "", *, highResolution: bool = False):
        super().__init__(label)
        self.label = label
        self.highResolution = highResolution

    def to_bitstream(self, value: list) -> Bitstream:
        bs = ""
        # first test if the _ is in the default _ code list - convert to string first.

        # tolStr = "{:s}".format(",".join(["{:.3f}".format(x) for x in value]))
        for tolValue in value:
            if LUT.tolerance_code.keys().__contains__(tolValue):
                bs += LUT.tolerance_code[tolValue]

        bitstream = Bitstream()
        bitstream = bitstream.from01(bs)

        return bitstream

    def from_bitstream(self, bitstream: Bitstream, start: int = 0) -> Tuple[int, Any]:
        tolerance = []
        num_bits, tol_code = readCodeword(bitstream, start, LUT.tolerance_code)

        return num_bits, tol_code

    def get_token(self, token_generator: TokenGenerator) -> Optional[Token]:
        return self
