# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from enum import Enum


class Constants:
    PrecisionErrorThreshold = 1e-3

    MinEdgeAngle = 190
    MinOffsetAngle_PerEdge = 0.0  # NOTE: it can limit diffraction paths [in degree]
    MaxDistance_FromSource = 64  # 36dB Attenuation
    MaxDistance_FromSource_noAE = 64  # 36dB Attenuation
    MaxDistance_FromTheFirstEdgeToTheLastEdge_InPath = 64  # 36dB Attenuation

    fCoef_1kHz = [
        0.96236,
        0.92671,
        0.89287,
        0.8607,
        0.83008,
        0.80091,
        0.7731,
        0.74655,
        0.72118,
        0.69692,
        0.6737,
        0.65146,
        0.63014,
        0.60968,
        0.59004,
        0.57117,
        0.55331,
        0.53596,
        0.51952,
        0.50389,
        0.48899,
        0.47477,
        0.46117,
        0.44815,
        0.43592,
        0.42408,
        0.4128,
        0.40205,
        0.39179,
        0.38199,
        0.37262,
        0.36366,
        0.35507,
        0.34685,
        0.33897,
        0.33141,
        0.32416,
        0.31721,
        0.31052,
        0.3041,
        0.29793,
        0.292,
        0.28629,
        0.2808,
        0.27551,
        0.27042,
        0.26551,
        0.26078,
        0.25622,
        0.25183,
        0.24758,
        0.24348,
        0.23953,
        0.2357,
        0.23201,
        0.22844,
        0.22498,
        0.22164,
        0.21841,
        0.21528,
        0.21225,
        0.20931,
        0.20646,
        0.20371,
        0.20103,
        0.19844,
        0.19592,
        0.19348,
        0.19111,
        0.18881,
        0.18658,
        0.1844,
        0.1823,
        0.18025,
        0.17825,
        0.17632,
        0.17443,
        0.1726,
        0.17082,
        0.16908,
        0.1674,
        0.16575,
        0.16415,
        0.16259,
        0.16108,
        0.1596,
        0.15816,
        0.15675,
        0.15539,
        0.15405,
        0.15276,
        0.15149,
        0.15025,
        0.14905,
        0.14788,
        0.14673,
        0.14562,
        0.14453,
        0.14346,
        0.14243,
        0.14142,
        0.14043,
        0.13947,
        0.13853,
        0.13761,
        0.13672,
        0.13584,
        0.13499,
        0.13416,
        0.13335,
        0.13256,
        0.13179,
        0.13104,
        0.1303,
        0.12958,
        0.12888,
        0.1282,
        0.12754,
        0.12689,
        0.12626,
        0.12564,
        0.12504,
        0.12445,
        0.12388,
        0.12332,
        0.12278,
        0.12225,
        0.12173,
        0.12123,
        0.12074,
        0.12027,
        0.11981,
        0.11936,
        0.11892,
        0.11849,
        0.11808,
        0.11768,
        0.11729,
        0.11691,
        0.11654,
        0.11619,
        0.11584,
        0.11551,
        0.11518,
        0.11487,
        0.11457,
        0.11428,
        0.11399,
        0.11372,
        0.11346,
        0.11321,
        0.11296,
        0.11273,
        0.11251,
        0.11229,
        0.11209,
        0.11189,
        0.11171,
        0.11153,
        0.11136,
        0.1112,
        0.11105,
        0.11091,
        0.11078,
        0.11066,
        0.11054,
        0.11044,
        0.11034,
        0.11025,
        0.11017,
        0.1101,
        0.11004,
        0.10998,
        0.10993,
        0.1099,
        0.10987,
        0.10985,
        0.10983,
        0.10983,
        0.10983,
    ]


class DiffractionTypes(Enum):
    NotDefined = -1
    Initialized = 0
    FirstOrder = 1
    HigherOrder = 2
    AugmentedHigherOrder = 3


class ConfigurationType(Enum):
    VertexToTriangle = 0
    TriangleToTriangle = 1
    VertexToVertex = 2
    EccentricityCalculation = 3


class CoordinateSystems(Enum):
    OpenGL = 0
    XYZ = 1
