# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from .edgelistgenperscene import *
from .pathdictgenpermesh import *


class PathDictGenPerScene:
    def __init__(
        self,
        staticTrimeshName_DIFFR: AnyStr,
        edgeListGenPerScene: EdgeListGenPerScene,
        sourceVoxelDictGen: SourceVoxelDictGen,
        sceneDict: Dict,
        args: Dict,
    ):
        """
        PathDictGenPerScene provides the attributes and methods to generate a set of path dictionaries for meshes:
          - static sceneTrimeshDIFFR and included dynamic non-transparent meshes

        @param edgeListGenPerScene: instantiated edgeListGenPerScene with the given eifplusScene
        @param sourceVoxelDictGen: instantiated sourceVoxelDictGen with static edgeListGen only
        @param maximumDiffractionOrder: maximum number of edges in a single path
        """

        logging.info(f" ------------- Path Dictionary Generation Per Scene -------------")

        self.__edgeListGenPerScene = edgeListGenPerScene
        self.__sourceVoxelDictGen = sourceVoxelDictGen
        self.__maximumDiffractionOrder = args["max_order_DIFFR"]
        self.__maximumDiffractionOrderForMovingSources = args["max_order_moving_source_DIFFR"]

        # Define list of meshes to analyze
        self.__dictOfPathDictGens = {}
        for tmID, elg in edgeListGenPerScene.edgeListGenDict.items():
            isDynamicMesh = tmID != staticTrimeshName_DIFFR
            self.__dictOfPathDictGens[tmID] = PathDictGenPerMesh(
                edgeListGenerator=elg,
                sourceVoxelDictGen=self.__sourceVoxelDictGen,
                maximumDiffractionOrder=self.__maximumDiffractionOrder,
                maximumDiffractionOrderForMovingSources=self.__maximumDiffractionOrderForMovingSources,
                dynamicMesh=isDynamicMesh,
                sceneDict=sceneDict,
            )
            logging.info(
                f">> Mesh with ID[{tmID}] produces {self.__dictOfPathDictGens[tmID].nofPaths} paths"
            )

        self.__dictOfPathDicts = {}
        for tmID, pathDictGPM in self.__dictOfPathDictGens.items():
            self.__dictOfPathDicts[tmID] = pathDictGPM.pathDict

        self.__dictOfPathDictsByEdgeIds = {}
        for tmID, pathDictGPM in self.__dictOfPathDictGens.items():
            self.__dictOfPathDictsByEdgeIds[tmID] = pathDictGPM.pathDictByEdgeIds

    @property
    def dictOfPathDictGens(self):
        return self.__dictOfPathDictGens

    @property
    def dictOfPathDicts(self):
        return self.__dictOfPathDicts

    @property
    def dictOfPathDictsByEdgeIds(self):
        return self.__dictOfPathDictsByEdgeIds
