# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
#
#  Utility functions for EIF parsing
#

import numpy as np


def isID(str):
    return str is not None and len(str) > 0


def isVector3(x):
    if isinstance(x, list):
        return len(x) == 3
    if isinstance(x, np.ndarray):
        return len(x) == 3
    return False


def isSize3(x):
    if isVector3(x):
        if x[0] < 0 or x[1] < 0 or x[2] < 0:
            return False
        if x[0] == 0 and x[1] == 0 and x[2] == 0:
            return False
        return True
    else:
        return False


def isCSpace(x):
    return x.lower() in ["relative", "user"]


def numberToStr(x):
    if int(x) == x:
        return "%d" % x
    else:
        return ("%f" % x).rstrip("0")


def parsePositionFromString(str):
    t = [float(s) for s in str.strip().split(" ")]
    if len(t) != 3:
        raise SyntaxError("Invalid position")
    return np.array(t)


def positionToString(pos):
    return "<%+0.3f, %+0.3f, %+0.3f>" % (pos[0], pos[1], pos[2])


def parseOrientationFromString(str):
    t = [float(s) for s in str.strip().split(" ")]
    if len(t) != 3:
        raise SyntaxError("Invalid position")
    return np.array(t)


def orientationToString(ori):
    return "<%0.1f°, %0.1f°, %0.1f°>" % (ori[0], ori[1], ori[2])


def parseSizeFromString(str):
    t = [float(s) for s in str.strip().split(" ")]
    if len(t) != 3:
        raise SyntaxError("Invalid size")
    if t[0] == 0 and t[1] == 0 and t[2] == 0:
        raise ValueError("Invalid size")
    return np.array(t)


def parseCSpace(str):
    s = str.strip().lower()
    if not s in ["global", "relative", "listener"]:
        raise SyntaxError("Invalid coordinate space")
    return s


def parsePlaybackMode(str):
    s = str.strip().lower()
    if not s in ["continuous", "event"]:
        raise SyntaxError("Invalid playback mode")
    return s


def parseBool(str):
    s = str.strip().lower()
    if s == "true":
        return True
    if s == "false":
        return False
    raise SyntaxError("Invalid flag")


def parseInputLayout(str):
    # TODO: Implement
    s = str.strip().upper()
    # if not s in ["continuous", "event"]:
    #    raise SyntaxError("Invalid playback mode")
    return s


def parseAuthoringParameters(str):
    # TODO: Implement
    s = str.strip().lower()
    # if not s in ["continuous", "event"]:
    #    raise SyntaxError("Invalid playback mode")
    return s


def vectorToAttributeStr(x):
    """Converts a vector (position, etc.) into an XML attribute value"""
    return " ".join([numberToStr(y) for y in x])


def indent(s):
    """Indent every newline in a string with a tab"""
    return "\t" + s.replace("\n", "\n\t")
