# This software module was originally developed by
#
# Koninklijke Philips N.V.
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Koninklijke Philips N.V. retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from mpegi.metadata.eif import utils


class AcousticTuningParameters:
    def __init__(
        self,
        defaultAE=None,
        revRendererControl=None,
        voxOccCtrl=None,
        voxDiffrCtrl=None,
        voxEarlyCtrl=None,
        erGainDb=None,
        lcErOutdoorTuning=None,
    ):
        """Creates an Acoustic Tuning Parameter set"""

        self.defaultAE = defaultAE
        self.revRendererControl = revRendererControl
        self.voxOccCtrl = voxOccCtrl
        self.voxDiffrCtrl = voxDiffrCtrl
        self.voxEarlyCtrl = voxEarlyCtrl
        self.erGainDb = erGainDb
        self.lcErOutdoorTuning = lcErOutdoorTuning

    @staticmethod
    def fromxml(node):
        """Deserialises from an XML node"""

        if node.hasAttribute("defaultAE"):
            defaultAE = utils.parseBool(node.getAttribute("defaultAE").strip())
        else:
            defaultAE = False

        if node.hasAttribute("voxDirectOcclusionControl"):
            voxOccCtrl = node.getAttribute("voxDirectOcclusionControl").strip()
        else:
            voxOccCtrl = None

        if node.hasAttribute("voxDiffractionControl"):
            voxDiffrCtrl = node.getAttribute("voxDirectOcclusionControl").strip()
        else:
            voxDiffrCtrl = None

        if node.hasAttribute("earlyTuningGainDb"):
            erGainDb = node.getAttribute("earlyTuningGainDb").strip()
        else:
            erGainDb = None

        if node.hasAttribute("LCEarlyOutdoorTuning"):
            lcErOutdoorTuning = node.getAttribute("LCEarlyOutdoorTuning").strip()
        else:
            lcErOutdoorTuning = None

        return AcousticTuningParameters(
            defaultAE, voxOccCtrl, voxDiffrCtrl, erGainDb, lcErOutdoorTuning
        )

    def toxml(self, dom):
        node = dom.createElement("AcousticTuningParameters")
        if self.defaultAE != None:
            node.setAttribute("defaultAE", self.defaultAE)

        if self.voxOccCtrl != None:
            node.setAttribute("voxDirectOcclusionControl", self.voxOccCtrl)

        if self.voxDiffrCtrl != None:
            node.setAttribute("voxDiffractionControl", self.voxDiffrCtrl)

        if self.erGainDb != None:
            node.setAttribute("earlyTuningGainDb", self.erGainDb)

        if self.lcErOutdoorTuning != None:
            node.setAttribute("lcErOutdoorTuning", self.lcErOutdoorTuning)

        return node

    def __str__(self):
        s = "AcousticTuningParameters {"
        s += 'defaultAE = "%s", ' % self.defaultAE
        s += 'voxDirectOcclusionControl = "%s", ' % self.voxOccCtrl
        s += 'vocDiffractionControl = "%s", ' % self.voxDiffrCtrl
        s += 'earlyTuningGainDb = "%s" }' % self.erGainDb
        s += 'lcErOutdoorTuning = "%s" }' % self.lcErOutdoorTuning

        return s
