# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from __future__ import annotations
from typing import Optional
import warnings


class AParams:
    """The auralization of Audio Elements can be controlled using athoring
    parameters, representeing the aritstic intent
    """

    def __init__(self):
        self.noDoppler: bool = False
        self.noDistance: bool = False
        self.noCulling: bool = False
        self.noDirectionalFocus: bool = False

    @staticmethod
    def fromString(aparamsString: Optional[str]) -> AParams:
        """Instanciates an AParams object using an EIF string. Must be seperated
        by a comma with no whitespaces.
        Valid parameters:
        - "noDoppler"
        - "noDistance"
        - "noCulling"
        - "noDirectionalFocus"

        :param aparamsString: The string to instanciate aparams from
        :type aparamsString: Optional[str]
        :raises SyntaxError: Contains an invalid parameter
        :return: [description]
        """
        aparams = AParams()
        if aparamsString == None or aparamsString == "":
            return aparams

        for a in aparamsString.split(","):
            if a.lower() == "nodoppler":
                aparams.noDoppler = True
            elif a.lower() == "nodistance":
                aparams.noDistance = True
            elif a.lower() == "nodirectionalfocus":
                aparams.noDirectionalFocus = True
            elif a.lower() == "noreverb":
                warnings.warn(
                    "The 'noReverb' aparam is depricated, use the 'noReverb' attribute instead.",
                    DeprecationWarning,
                )
            elif a.lower() == "noculling":
                aparams.noCulling = True
            else:
                raise SyntaxError(f"'{a}' is an invalid authoring parameter")
        return aparams
