# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from __future__ import annotations

from enum import IntEnum


class CSpace(IntEnum):
    """CSpace describes the frame of reference in which positions and orientations
    of an eifplus.Entity are expressed.

    Relative - Relative to the parent entity. When the entity has no direct
    parent, i.e no transform parent, the position/orientation refers to the
    global coordinate system. When it is a child of a transform entity, then
    its coordinate system is relative to the transforms frame of reference

    User - Expresses the position/orientation relative to the user
    """

    Relative = 0
    User = 1

    @staticmethod
    def fromString(cspaceString: str) -> CSpace:
        """Converts a string value to a CSpace enum and returns it.
        Valid values are as follows:
        - "relative"
        - "user"

        :param cspaceString: The string to convert
        :type cspaceString: str
        :raises SyntaxError: Invalid CSpace string value
        :return: The converted CSpace enum
        """
        strippedCSpaceString = cspaceString.strip().lower()
        if strippedCSpaceString == "relative":
            return CSpace.Relative
        elif strippedCSpaceString == "user":
            return CSpace.User
        else:
            raise SyntaxError(f"'{cspaceString}' is an invalid cspace string")
