# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from typing import List, Optional

from .cspace import CSpace


class Location:
    """Location is a wrapper around the EIF attributes position, orientation,
    and CSpace. Often is the case that entities, such as sources, have all
    of these attributes in common. In addition, locations also can have a
    parent spatial transform, such as Transform or Anchor; however anchor and
    transform are mutually exclusive.
    """

    def __init__(
        self,
        position: List[float],
        orientation: List[float],
        cspace: CSpace,
        transform: Optional[str] = None,
        anchor: Optional[str] = None,
    ):
        """Creates a location.

        :param position: Spatial position in cartesian space [X, Y, Z]
        :type position: List[float]
        :param orientation: Spatial orientation in cartesian space [Yaw, Pitch, Roll]
        :type orientation: List[float]
        :param cspace: Spatial frame of reference
        :param transform: ID of the parent transform, defaults to None
        :type transform: str, optional
        :param anchor: ID of the parent anchor, defaults to None
        :type anchor: str, optional
        """
        self.position = position
        self.orientation = orientation
        self.cspace = cspace
        self.transform = transform
        self.anchor = anchor
