# This software module was originally developed by
#
# Koninklijke Philips N.V.
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Koninklijke Philips N.V. retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2023.
from typing import List

from .basetypes import Entity


class HeadphoneEQ(Entity):
    """The frequency respons edata for the headphones used by the content creator,
    to be used for compensating differences with the local playback system."""

    def __init__(
        self,
        specType: str,
        f=List[float],
        gain=List[float],
        q=List[float],
        filtType=[str],
        preset=int,
    ):
        """Creates a content creator headphone EQ
        :param specType: Defines how the EQ is specified, either 'freq-gain', 'peak-q', 'preset', 'model', or 'uri'.
        :type specType: string
        :param f: List of frequencies [Hz]
        :type f : float
        :param gain: List of gains [dB]
        :type gain: float
        :param q: List of filter q [ 0..10]
        :type q: float
        :param preset: Integer index into the table of presets
        :type preset: int
        :param manufacturer: Name of manufacturer
        :type manufacturer: string
        :param model: Model name of the headphones
        :type model: string
        :param uri: URI to rechieve the data from an external location
        :type uri: string

        ::note Length of gain and (optionally) q must match the list of frequencies.
        """
        Entity.__init__(self, "headphoneEQ")
        self.specType = specType
        self.f = f
        self.gain = gain
        self.q = q
        self.filtType = filtType
        self.preset = preset
