from typing import List, Optional

from .basetypes import Entity


class InformedSource(Entity):
    """An EIF InformedSource that is enhanced with extra metadata for the purposes of
    encoding to an MPEGI bitstream
    """

    def __init__(
        self,
        id: str,
        position: List[float],
        listenerThreshold: float,
        hoaSourceThreshold: float,
        priorityValue: int,
    ):
        """Creates an InformedSource. Typically, fromEIFInformedSource is used to instanciate
        this class instead.

        :param id: Unique identifier
        :type id: str
        :param position: Spatial position in cartesian coordinates expressed as
            a list of three floats [X, Y, Z]
        :type position: List[float]
        :param listenerThreshold: Distance from the listener to the informed
            source in meters where the informed source is active
        :type listenerThreshold: float
        :param hoaSourceThreshold: Distance from the HOASources to the informed
            source in meters where the informed source is active
        :type hoaSourceThreshold: float
        :param priorityValue: Determines the processing priority between informed
            sources that are close to each other.
        :type priorityValue: int
        """
        Entity.__init__(self, id)

        self.id  # Inherited by Entity
        self.position = position
        self.listenerThreshold = listenerThreshold
        self.hoaSourceThreshold = hoaSourceThreshold
        self.priorityValue = priorityValue
