# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from typing import List, Optional

from ..update import Update
from ..util import getAllChanges


def isEntityStatic(entityId: str, transformId: Optional[str], updates: List[Update]) -> bool:
    """Returns true if the entity never moves in space

    :param entityId: The string id of the entity to check
    :type entityId: str
    :param transformId: The string id of the parent transform of the entity
    :type transformId: Optional[str]
    :param updates: Checks this list of updates to see if the entity is moved
    :return: return false if the entity is dynamic
    :rtype: bool
    """
    changes = getAllChanges(updates)
    dynamicAttributes = ["position", "orientation", "cspace"]
    dynamicEntities = [c["id"] for c in changes if c["attr"] in dynamicAttributes]
    isEntityMovedByTransform = transformId in [ch["id"] for ch in changes]
    return entityId not in dynamicEntities and not isEntityMovedByTransform
