# This software module was originally developed by
#
# Koninklijke Philips N.V.
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Koninklijke Philips N.V. retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from . import validation
from .basetypes import Entity


class Portal(Entity):
    """A Portal based on an EIF Portal."""

    def __init__(
        self,
        id,
        portalExtent,
        portalNormal,
        acousticEnvironment1,
        acousticEnvironment2,
        material,
        portalPosition,
        portalSize,
        hasExplicitPortal,
    ):
        """Creates a Portal with EntityFactor. Do not construct without
        entityFactory.

        :param id: Unique Identifier
        :param portalExtent: reference to geometric element representing the portal.
        :param portalNormal: Normal Vector in cartesian coordinates expressed as a list of three
            floats [X, Y, Z]
        :param acousticenvironment1: reference to AcousticEnvironment ID for first associated AE.
        :param acousticenvironment2: reference to AcousticEnvironment ID for second associated AE.
        :param material: reference to matieral ID associated with the portal.
        """
        Entity.__init__(self, id)
        self.id = id
        self.portalExtent = portalExtent
        self.portalNormal = portalNormal
        self.acousticEnvironment1 = acousticEnvironment1
        self.acousticEnvironment2 = acousticEnvironment2
        self.material = material
        self.portalPosition = portalPosition
        self.portalSize = portalSize
        self.hasExplicitPortal = hasExplicitPortal
