# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from typing import List

from .basetypes import CSpace, Entity


class Transform(Entity):
    """An EIF Transform that is enhanced with extra metadata for the purposes
    of encoding to an MPEGI bitstream
    """

    def __init__(self, id: str, position: List[float], orientation: List[float], cspace: CSpace):
        """Creates a Transform. Typically, fromEIFTransform is used to
        instanciate this class instead

        :param id: Unique Indentifier
        :type id: str
        :param position: Spatial position in cartesian coordinates expressed as
            a list of three floats [X, Y, Z]
        :type position: List[float]
        :param orientation: Spatial orientation in cartesian coordinates
            expressed as a list of three floats [Yaw, Pitch, Roll]
        :type orientation: List[float]
        :param cspace: The frame of reference for coordinate spaces (user/relative)
        :type cspace: eifplus.CSpace
        """
        Entity.__init__(self, id)

        self.id  # Inherited by Entity

        self.position = position
        self.orientation = orientation
        self.cspace = cspace
