# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from typing import List

from .basetypes import Entity, Geometry, GeometryType, Location, CSpace


class UserRegion(Entity, Geometry):
    def __init__(
        self,
        id: str,
        position: List[float],
        orientation: List[float],
        size: List[float],
        geoType: GeometryType,
    ):
        """Represents where a user can reach within a scene. The position, orientation, size
        can have different semantics dependant on the geoType. This regions' geoType can be any
        primitive, i.e. a box, sphere, or cylinder.

        :param id: The ID of the user region
        :type id: str
        :param position: Spatial position in cartesian coordinates expressed as a list of three
            floats [X, Y, Z]
        :type position: List[float]
        :param orientation: Spatial orientation in cartesian coordinates expressed as a list of
            three floats [Yaw, Pitch, Roll]
        :type orientation: List[float]
        :param size: Size of dimensions in x, y, z
            expressed as a list of three floats [Yaw, Pitch, Roll]
        :type size: List[float]
        :param geoType: Either box, cylinder, or sphere
        """
        Entity.__init__(self, id)
        Geometry.__init__(self, Location(position, orientation, CSpace.Relative), geoType)

        self.id = id
        self.position = position
        self.orientation = orientation
        self.size = size
        self.geoType = geoType

        assert self.geoType != GeometryType.Mesh, "User Regions cannot be meshes"
