# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
from typing import List

from .environments import AcousticEnvironment
from .update import Update
from .util import getAllChanges


def raiseExceptionIfAcousticEnvironmentIsMovedByUpdates(
    updates: List[Update], environments: List[AcousticEnvironment]
) -> None:
    changes = getAllChanges(updates)

    regionAttributes = ["position", "orientation", "cspace", "size"]
    updatedRegions = [c["id"] for c in changes if c["attr"] in regionAttributes]

    for env in environments:
        if env.region in updatedRegions:
            raise Exception(
                "Invalid Update - Cannot Move Acoustic Environments: "
                f"The acoustic environment region '{env.region}' referred "
                f"to by the environment '{env.id}' has its position, "
                "orientation, or cspace changed in an update"
            )


def raiseExceptionIfDuplicateUpdateIndiciesExist(updates: List[Update]) -> None:
    updateIndices = [u.index for u in updates if u.index != None]
    for updateIndex in updateIndices:
        if updateIndices.count(updateIndex) > 1:
            raise Exception(
                f"Duplicate <Update> index value of '{updateIndex}': "
                "All updates must have unique index values"
            )
