# This software module was originally developed by
#
# Koninklijke Philips N.V.
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Koninklijke Philips N.V. retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.

import mpegi.metadata.reverb.constants as constants
import math

# Calculate the bounding volume of a mesh
def calculateBoundingVolumeOfMesh(mesh):
    x = [0, 0]
    y = [0, 0]
    z = [0, 0]

    for v in mesh.vertices:
        x[0] = min(x[0], v[0])
        x[1] = max(x[1], v[0])
        y[0] = min(y[0], v[1])
        y[1] = max(y[1], v[1])
        z[0] = min(z[0], v[2])
        z[1] = max(z[1], v[2])

    x = abs(x[0]) + abs(x[1])
    y = abs(y[0]) + abs(y[1])
    z = abs(z[0]) + abs(z[1])

    return [x, y, z]


# Calculate the predelay as it should be specified in the EIF from the room bounding volume.
def calculateEIFpredelay(bounds):
    return 4 * max(bounds) / constants.lsdf_speed_of_sound


# Calculate the predelay as it is used in the renderer (2 * shortest room dimension)
def calculateRendererPredelay(roomBounds, acousticPredelay):
    rendererPredelay = acousticPredelay / 4
    # minLoopdelay should be rounded to nearest prime number of samples to be 100% accurate to the renderer code but we work in seconds here.
    minLoopDelay = min(roomBounds) * 2 / constants.lsdf_speed_of_sound
    rendererPredelay = max(rendererPredelay, minLoopDelay)
    return rendererPredelay


# Calculate the DSR scaling as will be applied in the renderer
def calculateDSRscaling(rt60, rendererPredelay, acousticPredelay):
    num = -6.907755278982137 * -1
    alpha = num / rt60
    preDelayDelta = acousticPredelay - rendererPredelay
    if preDelayDelta != 0:
        revEnergyFactor = math.exp(2 * alpha * preDelayDelta)
    else:
        revEnergyFactor = 1.0

    return revEnergyFactor
