# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import numpy as np


def voxelCoordinateToVoxelIndex(voxelGrid, voxelCoordinate):
    """For an arbitrary Trimesh.VoxelGrid, compute the unique integer index for a three-integer voxel coordinate"""
    return voxelCoordinate[0] + voxelGrid.shape[0] * (
        voxelCoordinate[1] + voxelGrid.shape[1] * voxelCoordinate[2]
    )


def voxelCoordinateToPoint(voxelGrid, voxelCoordinate):
    """For an arbitrary Trimesh.VoxelGrid, compute the voxel center point for a three-integer voxel coordinate"""
    # pos = np.matmul(voxelGrid.transform, np.array([[voxelCoordinate[0]], [voxelCoordinate[1]], [voxelCoordinate[2]], [1]]))
    # return pos[0:3].reshape((3,))
    return voxelGrid.indices_to_points(voxelCoordinate)


def pointToVoxelCoordinate(voxelGrid, pos):
    """For an arbitrary Trimesh.VoxelGrid, compute the three-integer voxel coordinate for a Cartesian coordinate"""
    return voxelGrid.points_to_indices(np.array(pos))


def voxelIndexToVoxelCoordinate(voxelGrid, voxelIndex):
    """For an arbitrary Trimesh.VoxelGrid, compute the three-integer voxel coordinate for a unique integer index"""
    if voxelIndex < 0 or voxelIndex >= maxVoxelIndex(voxelGrid):
        raise ValueError("Voxel index out-of-range")

    vc = np.zeros((3,), dtype=np.int_)
    r = voxelIndex
    vc[0] = r % voxelGrid.shape[0]
    r /= voxelGrid.shape[0]
    vc[1] = r % voxelGrid.shape[1]
    vc[2] = r / voxelGrid.shape[1]
    return vc


def voxelIndexToPoint(voxelGrid, voxelIndex):
    """For an arbitrary Trimesh.VoxelGrid, compute the voxel center point in Cartesian coordinates for a unique integer index"""
    return voxelCoordinateToPoint(voxelGrid, voxelIndexToVoxelCoordinate(voxelGrid, voxelIndex))


def pointToVoxelIndex(voxelGrid, pos):
    """For an arbitrary Trimesh.VoxelGrid, compute the unique integer index for a point in Cartesian coordinates"""
    return voxelCoordinateToVoxelIndex(voxelGrid, pointToVoxelCoordinate(voxelGrid, pos))


def maxVoxelIndex(voxelGrid):
    """For an arbitrary Trimesh.VoxelGrid, compute the maximum unique integer index"""
    return voxelGrid.shape[0] * voxelGrid.shape[1] * voxelGrid.shape[2]
