# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import unittest

from mpegi.metadata import eifplus


class TestUnusedSource(unittest.TestCase):
    def setUp(self) -> None:
        alignment = eifplus.Alignment.fromString("object")
        positioning = eifplus.Positioning.fromString("L C R")
        inputLayout = eifplus.InputLayout(alignment, positioning)
        self.objectSource = eifplus.ObjectSource(
            id="objectSource",
            position=[0.0, 0.0, 0.0],
            orientation=[0.0, 0.0, 0.0],
            cspace=eifplus.CSpace.fromString("relative"),
            active=True,
            gainDb=0.0,
            refDistance=1.0,
            signal="dummystream",
            extent="",
            directivity="",
            directiveness=1.0,
            mode="continuous",
            play=True,
            inputLayout=inputLayout,
            aparams=None,
            transform="",
            anchor="",
            vdlMethod=0,
            hasReverbGain=False,
            reverbGainDb=0.0,
            noReverb=False,
            granularPos=[0.0, 0.0, 0.0],
        )

        self.toggledObjectSource = eifplus.ObjectSource(
            id="toggledObjectSource",
            position=[0.0, 0.0, 0.0],
            orientation=[0.0, 0.0, 0.0],
            cspace=eifplus.CSpace.fromString("relative"),
            active=False,
            gainDb=0.0,
            refDistance=1.0,
            signal="dummystream",
            extent="",
            directivity="",
            directiveness=1.0,
            mode="continuous",
            play=True,
            inputLayout=inputLayout,
            aparams=None,
            transform="",
            anchor="",
            vdlMethod=0,
            hasReverbGain=False,
            reverbGainDb=0.0,
            noReverb=False,
            granularPos=[0.0, 0.0, 0.0],
        )

        self.unusedObjectSource = eifplus.ObjectSource(
            id="unusedObjectSource",
            position=[0.0, 0.0, 0.0],
            orientation=[0.0, 0.0, 0.0],
            cspace=eifplus.CSpace.fromString("relative"),
            active=False,
            gainDb=0.0,
            refDistance=1.0,
            signal="dummystream",
            extent="",
            directivity="",
            directiveness=1.0,
            mode="continuous",
            play=True,
            inputLayout=inputLayout,
            aparams=None,
            transform="",
            anchor="",
            vdlMethod=0,
            hasReverbGain=False,
            reverbGainDb=0.0,
            noReverb=False,
            granularPos=[0.0, 0.0, 0.0],
        )
        mod = eifplus.Modification("toggledObjectSource", 1.0, {"active": "true"})
        updateObjectSource = eifplus.Update(
            "ToggleObjSource", None, 1.0, None, None, None, "Timed", [mod]
        )
        entities = [
            self.objectSource,
            self.toggledObjectSource,
            self.unusedObjectSource,
            updateObjectSource,
        ]
        self.metadata = eifplus.SceneMetadata()

        self.metadata.analyze(entities, [], "")

    def test_usedObjectSource(self):
        isRendered = self.metadata.isAudioSourceRendered(self.objectSource.id)
        self.assertEqual(isRendered, True)

    def test_usedObjectSourceViaUpdate(self):
        # An update has activated an object source that was inactive upon initialisation
        isRendered = self.metadata.isAudioSourceRendered(self.toggledObjectSource.id)
        self.assertEqual(isRendered, True)

    def test_unusedObjectSource(self):
        isRendered = self.metadata.isAudioSourceRendered(self.unusedObjectSource.id)
        self.assertEqual(isRendered, False)


if __name__ == "__main__":
    unittest.main()
