# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import unittest

from mpegi.metadata import eifplus


class TestUnusedStream(unittest.TestCase):
    def setUp(self) -> None:
        self.toggledStream = eifplus.AudioStream(
            "toggledStream", isLocallyCaptured=False, file="", aepInputChannels=[0]
        )
        self.unusedStream = eifplus.AudioStream(
            "unusedstream", isLocallyCaptured=False, file="", aepInputChannels=[0]
        )
        self.stream = eifplus.AudioStream(
            "mystream", isLocallyCaptured=False, file="", aepInputChannels=[0]
        )
        objectSource = eifplus.ObjectSource(
            "objectSource",
            [0.0, 0.0, 0.0],
            [0.0, 0.0, 0.0],
            eifplus.CSpace.fromString("relative"),
            True,
            0.0,
            1.0,
            self.stream.id,
            "",
            "",
            1.0,
            "continuous",
            True,
            None,
            None,
            "",
            "",
            0,
            False,
            0.0,
            False,
            [0.0, 0.0, 0.0],
        )
        toggledObjectSource = eifplus.ObjectSource(
            "toggledObjectSource",
            [0.0, 0.0, 0.0],
            [0.0, 0.0, 0.0],
            eifplus.CSpace.fromString("relative"),
            True,
            0.0,
            1.0,
            self.stream.id,
            "",
            "",
            1.0,
            "continuous",
            True,
            None,
            None,
            "",
            "",
            0,
            False,
            0.0,
            False,
            [0.0, 0.0, 0.0],
        )
        mod = eifplus.Modification("toggledSignalInStream", 1.0, {"signal": "toggledStream"})
        updateObjectSource = eifplus.Update(
            "toggledObjectSource", None, 1.0, None, None, None, "Timed", [mod]
        )
        entities = [
            objectSource,
            toggledObjectSource,
            self.stream,
            self.unusedStream,
            self.toggledStream,
            updateObjectSource,
        ]

        self.sceneMetadata = eifplus.SceneMetadata()

        self.sceneMetadata.analyze(entities, [], "")

    def test_usedStream(self):
        self.assertEqual(self.sceneMetadata.isAudioStreamUsed(self.stream.id), True)

    def test_toggledStream(self):
        # An updated switched an object source to use another stream instead
        self.assertEqual(self.sceneMetadata.isAudioStreamUsed(self.toggledStream.id), True)

    def test_unusedStream(self):
        self.assertEqual(self.sceneMetadata.isAudioStreamUsed(self.unusedStream.id), False)


if __name__ == "__main__":
    unittest.main()
