# This software module was originally developed by
#
# Dolby International AB
#
# in the course of development of ISO/IEC 23090-4 for reference purposes and
# its performance may not have been optimized. This software module is an
# implementation of one or more tools as specified by the ISO/IEC 23090-4
# standard. ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
# copyright license to copy, distribute, and make derivative works of this
# software module or modifications thereof for use in implementations or
# products claiming conformance to ISO/IEC standards incorporating ISO/IEC
# 23090-4 reference software and which satisfy any specified conformance
# criteria of that standard.
#
# Those intending to use this software module in products are advised that its
# use may infringe existing patents and that the license in the previous
# paragraph grants no licenses under such patents. ISO/IEC have no liability
# for use of this software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Dolby International AB
# retains full right to modify and use the code for its own purpose,
# assign or donate the code to a third party and to inhibit third parties
# from using the code for products that do not conform to ITU Recommendations
# and/or ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative works.
#
# Copyright (c) ISO/IEC 2022.
import pathlib
import pytest


from typing import List
from mpegi.framework import BitstreamComposer, tokens, BitstreamDecomposer


def test_writePackedVoxelCord():
    voxelDimensions3D = [64, 32, 16]
    voxelCord3D = [3, 21, 5]

    syntax: List[tokens.Token] = [
        tokens.StartMarker("start"),
        tokens.PackedVoxelCoord("Cord1", dimensions=voxelDimensions3D),
        tokens.EndMarker("end"),
    ]

    composer = BitstreamComposer(syntax)
    composer.push(voxelCord3D)

    bitstream = composer.get_bitstream()

    decomposer = BitstreamDecomposer(bitstream, syntax)
    decomposer.extract_between_markers()

    assert composer.scope == decomposer.scope

    voxelDimensions2D = [64, 32]
    voxelCord2D = [3, 21]

    syntax: List[tokens.Token] = [
        tokens.StartMarker("start"),
        tokens.PackedVoxelCoord("Cord2", dimensions=voxelDimensions2D),
        tokens.EndMarker("end"),
    ]

    composer = BitstreamComposer(syntax)
    composer.push(voxelCord2D)

    bitstream = composer.get_bitstream()

    decomposer = BitstreamDecomposer(bitstream, syntax)
    decomposer.extract_between_markers()

    assert composer.scope == decomposer.scope


def main():
    test_writePackedVoxelCord()


if __name__ == "__main__":
    main()
