# This software module was originally developed by
#
# Nokia
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Nokia retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2024.

from mpegi.metadata.reverb.los_helpers import get_los_factor_p2p_multi
from mpegi.metadata.reverb.reverbutils import populate_box_positions
from mpegi.metadata.reverb.portal_los_analyzer import PortalLosAnalyzer
from trimesh import creation, transformations
import numpy as np
import xml.etree.ElementTree as ET
from tests.ref_los_values.ref_los_values import ref_vals


def test_vox_to_mesh():
    patientroom2corridor = creation.box(
        extents=[2.4, 2, 0.06], transform=transformations.translation_matrix([0, 1, 2.6])
    )

    corr_geom = creation.box(
        extents=[40, 2.500, 2.200], transform=transformations.translation_matrix([0, 1.250, 3.700])
    )

    pla = PortalLosAnalyzer([], [], [], [], "", False, "")

    # Hospital voxel blocks
    subscene = ET.Element(
        "VoxelScene",
        {
            "id": "vox:Hospital0",
            "vox_resolution": "40 168 12",
            "position_s": "-4 -21 -0.25",
            "position_e": "6 21 2.75",
        },
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "28 5 1", "grid_coordinate_e": "35 164 1"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 60 1", "grid_coordinate_e": "26 69 1"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 71 1", "grid_coordinate_e": "26 98 1"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "27 62 2", "grid_coordinate_e": "27 66 9"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "20 70 2", "grid_coordinate_e": "24 70 9"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "28 5 12", "grid_coordinate_e": "35 164 12"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 60 12", "grid_coordinate_e": "26 98 12"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "27 4 2", "grid_coordinate_e": "27 61 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "28 4 2", "grid_coordinate_e": "36 4 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "36 5 2", "grid_coordinate_e": "36 165 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "6 59 2", "grid_coordinate_e": "6 99 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 59 2", "grid_coordinate_e": "26 59 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "27 67 2", "grid_coordinate_e": "27 79 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 70 2", "grid_coordinate_e": "19 70 5"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "25 70 2", "grid_coordinate_e": "26 70 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "27 90 2", "grid_coordinate_e": "27 165 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 99 2", "grid_coordinate_e": "26 99 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "28 165 2", "grid_coordinate_e": "35 165 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "7 70 6", "grid_coordinate_e": "8 70 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "17 70 6", "grid_coordinate_e": "19 70 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "27 62 10", "grid_coordinate_e": "27 66 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "9 70 10", "grid_coordinate_e": "16 70 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "20 70 10", "grid_coordinate_e": "24 70 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "27 80 10", "grid_coordinate_e": "27 89 11"}
    )
    ET.SubElement(
        subscene, "VoxelBlock", {"grid_coordinate_s": "9 70 6", "grid_coordinate_e": "16 70 9"}
    )
    voxel_blocks = subscene.findall("VoxelBlock")

    trimesh_vox_scene = []
    for v_b in voxel_blocks:
        block_coord_s = [int(val) for val in v_b.attrib["grid_coordinate_s"].split()]
        block_coord_e = [int(val) for val in v_b.attrib["grid_coordinate_e"].split()]
        block_coord_s = [block_coord_s[1], block_coord_s[2], block_coord_s[0]]
        block_coord_e = [block_coord_e[1], block_coord_e[2], block_coord_e[0]]

        vox_block_mesh = pla.vox_subscene_as_mesh(subscene, block_coord_s, block_coord_e)
        trimesh_vox_scene.append(vox_block_mesh)

    x_extent = corr_geom.extents[0]

    c_x, c_y, c_z = [8.07852206e-16, 1.25000000e00, 3.70000000e00]
    ray_cast_positions = [
        [-5, c_y, 4.2],
        [0, c_y, 4.2],
        [5, c_y, 4.2],
        [-2, c_y, 3],
        [0, c_y, 3],
        [2, c_y, 3],
    ]

    blocking_mesh = creation.box(
        extents=[x_extent, 0.5, 0.5],
        transform=transformations.translation_matrix(
            [corr_geom.centroid[0], corr_geom.centroid[1], corr_geom.centroid[2]]
        ),
    )

    meshes_to_raycast = [["", blocking_mesh]]
    for mesh in trimesh_vox_scene:
        meshes_to_raycast.append(["", mesh])

    exptected_los_truths = [False, False, False, True, True, True]

    target_vertices = []
    for i in range(len(ray_cast_positions)):
        face = []
        for v in patientroom2corridor.vertices:
            face.append(v)
        target_vertices.append(face)

    portal_los_data = get_los_factor_p2p_multi(
        ray_cast_positions, meshes_to_raycast, target_vertices, 0.05
    )
    portal_los_truths = []
    for los_ratio in portal_los_data:
        # los = los_ratio >= 0.5 as defined in reverb constants, dont use reference here
        portal_los_truths.append(los_ratio >= 0.5)

    assert portal_los_truths == exptected_los_truths


def test_los():
    corr_geom = creation.box(
        extents=[40, 2.500, 2.200], transform=transformations.translation_matrix([0, 1.250, 3.700])
    )
    nurseroom_geom = creation.box(
        extents=[2.700, 2.500, 5.200],
        transform=transformations.translation_matrix([-4.950, 1.250, 0.000]),
    )
    patientroom_geom = creation.box(
        extents=[7.200, 2.500, 5.200],
        transform=transformations.translation_matrix([0.000, 1.250, 0.000]),
    )

    aes = [
        ["Corridor", corr_geom],
        ["NurseRoom", nurseroom_geom],
        ["PatientRoom", patientroom_geom],
    ]
    positions = {}

    for ae_id, ae in aes:
        positions[ae_id] = populate_box_positions(0.5, ae)

    nurseroom2corridor = creation.box(
        extents=[1.2, 2, 0.06], transform=transformations.translation_matrix([-5.2, 1, 2.6])
    )
    nurseroom2patientroom = creation.box(
        extents=[1.2, 2, 0.06], transform=transformations.translation_matrix([-3.6, 1, 1.4])
    )
    nurseroom2patientroom.apply_transform(
        transformations.rotation_matrix(np.deg2rad(90), [0, 1, 0], nurseroom2patientroom.centroid)
    )
    patientroom2corridor = creation.box(
        extents=[2.4, 2, 0.06], transform=transformations.translation_matrix([0, 1, 2.6])
    )

    ae_portals = {
        "NurseRoom": {
            "prtl1:nurseroom2corridor": nurseroom2corridor,
            "prtl2:nurseroom2patientroom": nurseroom2patientroom,
        },
        "PatientRoom": {
            "prtl3:patientroom2corridor": patientroom2corridor,
            "prtl2:nurseroom2patientroom": nurseroom2patientroom,
        },
        "Corridor": {
            "prtl1:nurseroom2corridor": nurseroom2corridor,
            "prtl3:patientroom2corridor": patientroom2corridor,
        },
    }

    meshes_per_ae = {}
    for ae_id, ae_geom in aes:
        x_extent = ae_geom.extents[0]
        mesh = creation.box(
            extents=[x_extent, 0.5, 0.5],
            transform=transformations.translation_matrix(
                [ae_geom.centroid[0], ae_geom.centroid[1], ae_geom.centroid[2]]
            ),
        )
        meshes_per_ae[ae_id] = ["mesh_id", mesh]

    for ae_id, pos in positions.items():
        ae_meshes = [meshes_per_ae[ae_id]]
        portals = ae_portals[ae_id]

        for portal_id, portal_geom in portals.items():
            target_vertices = []
            for i in range(len(pos)):
                face = []
                for v in portal_geom.vertices:
                    face.append(v)
                target_vertices.append(face)

            expected_los_values = None
            for ref_val in ref_vals:
                prefix = f"Hospital_{ae_id}_{portal_id}_los_data"
                try:
                    expected_los_values = ref_val[prefix]
                except KeyError:
                    continue

            if expected_los_values is None:
                raise Exception("No ref values found")

            portal_los_data = get_los_factor_p2p_multi(pos, ae_meshes, target_vertices, 0.05)
            assert np.array_equal(expected_los_values, portal_los_data)


if __name__ == "__main__":
    test_los()
