# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
import unittest

import numpy as np

from mpegi.geometry import transforms


class TestTransforms(unittest.TestCase):
    def test_simple_wo_rotation(self):
        a_pos = np.array([1, 0, 0])
        a_ori = np.array([0, 0, 0])
        t_pos = np.array([0, 1, 0])
        t_ori = np.array([0, 0, 0])

        T = transforms.transformMatrixTRS_4x4(t_pos, t_ori, [1, 1, 1])
        v = transforms.applyTransformMatrix4x4(T, a_pos)

        self.assertTrue(np.allclose([1, 1, 0], v))

    def test_simple_w_rotation(self):
        a_pos = np.array([1, 0, 0])
        a_ori = np.array([0, 0, 0])
        t_pos = np.array([0, 1, 0])
        t_ori = np.deg2rad(np.array([90, 0, 0]))

        T = transforms.transformMatrixTRS_4x4(t_pos, t_ori, [1, 1, 1])
        v = transforms.applyTransformMatrix4x4(T, a_pos)

        self.assertTrue(np.allclose([0, 1, -1], v))

    def test_extraction_of_offset_and_angles(self):
        pos = np.array([1, -2, 1.5])
        ori = np.deg2rad(np.array([-170, 23, -45]))

        T = transforms.transformMatrixTRS_4x4(pos, ori, [1, 1, 1])
        pos_new, ori_new = transforms.getOffsetAndAnglesFromMatrixTRS(T)

        self.assertTrue(np.allclose(pos_new, pos))
        self.assertTrue(np.allclose(ori_new, ori))

    def test_combining_two_transforms(self):
        a_pos = np.array([1, 0, 0])
        a_ori = np.deg2rad(np.array([0, 90, 0]))
        t_pos = np.array([0, 1, 0])
        t_ori = np.deg2rad(np.array([90, 0, 0]))

        Ta = transforms.transformMatrixTRS_4x4(a_pos, a_ori, [1, 1, 1])
        Tt = transforms.transformMatrixTRS_4x4(t_pos, t_ori, [1, 1, 1])
        T = np.matmul(Tt, Ta)
        pos_new, ori_new = transforms.getOffsetAndAnglesFromMatrixTRS(T)

        self.assertTrue(np.allclose(pos_new, [0, 1, -1]))
        self.assertTrue(np.allclose(ori_new, np.deg2rad([90, 90, 0])))


if __name__ == "__main__":
    unittest.main()
