/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_FILTER_DESIGNER_INTERFACE
#define EAR_FILTER_DESIGNER_INTERFACE

#include <cstddef>
#include <memory>

#include <EAR/RealBuffer.h>

namespace EAR {

//! Interface for filter designers
/*!
 * Generates an FIR filter from a given magnitude response
 */
class FilterDesigner
{
public:
    //! Constructor
    /*!
     * \param f     float pointer to array of length H containing the frequency values at which the H magnitudes describe the desired frequency response
     * \param H     number of supporting magnitude values
     * \param N     length of the resulting FIR filter in taps
     */
    FilterDesigner(const float* f, std::size_t H, std::size_t N)
        : H(H)
        , N(N)
        , centerFreqs(H)
    {
        centerFreqs.write(f, 0, H);
    };

    //! Destructor
    virtual ~FilterDesigner(){};

    //! design a FIR filter with a given magnitude response
    /*!
     * \param m       float pointer to array of length H containing the desired magnitude values at the previously defined frequencies
     * \param result  reference to RealBuffer where the resulting filter is written
     *                (Caution: assumed to already have the correct size (N) to avoid the expensive resizing in time-critical situations)
     */
    virtual void generate(const float* m, RealBuffer& result) = 0;

protected:
    std::size_t H, N;

    RealBuffer centerFreqs;
};

} // namespace EAR

#endif /* EAR_FILTER_DESIGNER_INTERFACE */
