/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_GEOMETRYPREPROCESSOR
#define EAR_GEOMETRYPREPROCESSOR

#include <EAR/Geometry.h>

#include <ostream>
#include <vector>

// #include <cmath>

namespace EAR {

// TODO: the unified config infrastructure will make these defines obsolete
// #define EAR_ROUND_EDGE_DIST_THRESH 0.001f // distance threshold (currently unused)
#define EAR_ROUND_EDGE_ALPHA_MIN                                                                   \
    0.1f // lower limit of the detection range in degree for round edges (angle)
#define EAR_ROUND_EDGE_ALPHA_MAX                                                                   \
    31.f // upper limit of the detection range in degree for round edges (angle)
#define EAR_ROUND_EDGE_RADIUS_MIN                                                                  \
    0.60f // lower limit for the detection range in m for round edges (radius)

template <typename T> int EAR_SGN(T val)
{
    return (T(0) < val) - (val < T(0));
}

namespace GeometryPreprocessor {

struct EdgeInfo
{
    EdgeInfo()
        : num_round_edges(0){};

    std::vector<int> left; //!< ID of the left-hand triangle
    std::vector<int> right; //!< ID of the right-hand triangle
    std::vector<int> entry; //!< vertex indices
    unsigned int num_round_edges; //!< number of round edges
};

void getRoundStructure(std::vector<Mesh::RoundEdge>& roundEdges,
    std::vector<Mesh::RoundCorner>& roundCorners, const Mesh* mesh);

} // namespace GeometryPreprocessor

std::ostream& operator<<(std::ostream& os, GeometryPreprocessor::EdgeInfo& e);

} // namespace EAR

#endif /* EAR_GEOMETRYPREPROCESSOR */
