/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_LIMITER
#define EAR_LIMITER

#include <EAR/RealFrame.h>

#include <cstddef>
#include <vector>

namespace EAR {
class Limiter
{
public:
    //! Destructor
    virtual ~Limiter(){};

    //! Reset state of the limiter
    virtual void clear() = 0;
};

class LimiterInPlace
{
public:
    //! Process a block of samples - modify in place.
    /*!
     * \param inOut     sample values that will be processed and memory location to write result to
     */
    virtual void process(float** inOut) = 0;
};

class LimiterCopy
{
public:
    //! Process a block of samples - copy to output buffer.
    /*!
     * \param input     sample values that will be processed
     * \param output    memory location to write result to
    */
    virtual void process(float** input, float** output) = 0;
};

} // namespace EAR

#endif /* EAR_LIMITER */
