/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_LIMITER_IVAS
#define EAR_LIMITER_IVAS

#include "ivas_limiter.h"
#include <EAR/Limiter.h>

#include <cstddef>

namespace EAR {

class LimiterIvas : public Limiter, public LimiterInPlace
{
public:
    //! Constructor
    /*!
     * \param channels        number of channels
     * \param sampleRate      sampling rate in Hz
     * \param blocksize       number of samples to process
     * \param attackSeconds   attack time in seconds
     * \param threshold       limiting threshold [0.0 - 1.0]
     */
    LimiterIvas(std::size_t channels, std::size_t sampleRate, std::size_t blocksize,
        float attackSeconds, float threshold);

    //! Constructor using IVAS defaults for attackSeconds (0.005f) and threshold (32729.0f/32767.0f):
    /*!
     * \param channels        number of channels
     * \param sampleRate      sampling rate in Hz
     * \param blocksize       number of samples to process
     */
    LimiterIvas(std::size_t channels, std::size_t sampleRate, std::size_t blocksize);

    //! Destructor
    ~LimiterIvas();

    //! Inherited from Limiter
    void clear() override;

    //! Inherited from LimiterInPlace
    void process(float** inOut) override;

private:
    float limiter_mem[2];
    std::size_t channels;
    std::size_t sampleRate;
    std::size_t blocksize;
    float attackSeconds;
    float threshold;
};

} // namespace EAR

#endif /* EAR_LIMITER_IVAS */
