/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_MPEGIDECORRELATOR
#define EAR_MPEGIDECORRELATOR

#include <EAR/ComplexIIRFilter.h>

#include <cstddef>
#include <memory>
#include <vector>

namespace EAR {
class RealBuffer;
class RealFrame;
class ComplexBuffer;
class RFFT;
class IRFFT;

class MpegiDecorrelator
{
public:
    //! Constructor
    /*!
     * \param B 			block size in samples
     */
    MpegiDecorrelator(std::size_t B);

    //! process a mono signal with the decorrelator
    /*!
	 *
     * \param in 			single-channel input signal
     * \param out 			two-channel decorrelated output
     */
    void process(const float* in, float** out);

private:
    const int hB;
    int B, K, numToProcess;
    bool noneProcessed;
    //! read & write cursor index for the overlap buffer and delay buffer
    std::size_t inRC, inWC, outRC, outWC, delayRC, delayWC;
    // transient detection variables
    bool preIsTrans, writeDirect, writeProcessed;
    int transCount, noTransCount, inhibitCount;
    float preTranSum;

    std::unique_ptr<EAR::RFFT> decoFFT;
    std::unique_ptr<EAR::IRFFT> decoIFFT;
    // time-domain
    std::unique_ptr<EAR::RealBuffer> ringBuffer;
    std::unique_ptr<EAR::RealBuffer> window;
    std::unique_ptr<EAR::RealBuffer> curInData;
    std::unique_ptr<EAR::RealBuffer> windowedData;
    std::unique_ptr<EAR::RealBuffer> preDecorData;
    std::unique_ptr<EAR::RealBuffer> tmpOutput;
    std::unique_ptr<EAR::RealFrame> tmpFrame;
    std::unique_ptr<EAR::RealFrame> outFrame;
    // frequency-domain
    std::unique_ptr<EAR::ComplexBuffer> freqData;
    std::unique_ptr<EAR::RealBuffer> direcFreqData;
    std::unique_ptr<EAR::RealBuffer> preDirecEnergy;
    std::unique_ptr<EAR::RealBuffer> preDecorEnergy;

    std::vector<EAR::ComplexBuffer> delayStorage;
    std::vector<EAR::ComplexIIRFilter> aPFilters;
};

} //namespace EAR

#endif
