/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_AIRFLOW_PLANE
#define EAR_AIRFLOW_PLANE

#include <EAR/AirflowGeneratorBase.h>

namespace EAR {

class PlaneAirflowGenerator : public AirflowGeneratorBase
{
private:
    //! Constructor for a Plane airflow generator (only accessible through SceneObjectFactory)
    /*!
    * \param oid        fixed object ID (always assigned by factory method!)
    * \param name       arbitrary name of object
    * \param pos        position of the generator
    */
    PlaneAirflowGenerator(ObjectID oid, const std::string& name = "", Position pos = Position());

    friend SceneObjectFactory<PlaneAirflowGenerator>;

public:
    //! Inherited from AirflowGeneratorBase
    Type getType() const override { return Type::Plane; };

    Vector3 getAirspeed(const Vector3& listenerPosition) const override;

    std::atomic<float> width = 1.0f;
    std::atomic<float> height = 1.0f;
    std::atomic<float> edgeRollOff =
        50.0f; // Size of the edge rolloff area in percentage relative to the height and width. Airspeed will smoothly drop to 0 in this area.
};

} /* namespace EAR*/

#endif /* EAR_AIRFLOW_PLANE */
