/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_REALBUFFER
#define EAR_REALBUFFER

#include <EAR/Buffer.h>

#include <cstddef>

namespace EAR {

//! Specialization of Buffer with float values.
/*!
 * Data management etc. is inherited from its base class Buffer,
 * while additional functionality specific to audio samples values is implemented here.
 */
class RealBuffer : public Buffer<float>
{
public:
    //! Create an empty buffer
    RealBuffer();

    //! Create a buffer initialized with zeros
    /*!
     * \param length    buffer  [samples]
     */
    RealBuffer(std::size_t length);

    //! Copy constructor
    RealBuffer(const Buffer<float>& other);

    //! Copy constructor from a raw array pointer
    /*!
     * \param data      pointer to sample data to copy
     * \param length    buffer size [samples]
     */
    RealBuffer(const float* data, std::size_t length);

    //! Destructor
    virtual ~RealBuffer();

    //! Write into float RealBuffer from double samples
    /*!
     * \param src     double pointer to read from
     * \param offset  first sample to write into
     * \param count   number of samples to write
     */
    void writeDbl(const double* src, std::size_t offset, std::size_t count);

    //! Read from float RealBuffer into double pointer
    /*!
     * \param dst     double pointer to write to (memory allocation has to be done beforehand)
     * \param offset  first sample to read
     * \param count   number of samples to read
     */
    void readDbl(double* dst, std::size_t offset, std::size_t count);

    //! Return the minimum value in the buffer
    float mini() const;

    //! Return the maximum value in the buffer
    float maxi() const;

    //! Return the maximum absolute value in the buffer
    float peak() const;

    //! Return the root-mean-square value of the whole signal
    float rms() const;
};

} // namespace EAR

#endif /* EAR_REALBUFFER */
