/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_TIME_MEASURE
#define EAR_TIME_MEASURE

#include <chrono>

namespace EAR {

// TODO: make cross-platform

//! Abstraction of the steady_clock of std::chrono to measure the interval between startTimer() and stopTimer().
class TimeMeasure
{
public:
    //! To be called before the operation under test.
    static void startTimer();

    //! To be called after the operation under test.
    static void stopTimer();

    //! Return microseconds between startTimer() and stopTimer() calls
    static double getElapsedMicroseconds();

    //! Return nanoseconds between startTimer() and stopTimer() calls
    static double getElapsedNanoseconds();

    //! Print microseconds between startTimer() and stopTimer() calls to stdout
    static void printElapsedMicroseconds();

private:
    //! Prohibit direct instanciation of static class
    TimeMeasure(){};
    static std::chrono::high_resolution_clock::time_point start, end;
};

} // namespace EAR

#endif /* EAR_TIME_MEASURE */
