/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <EAR/BaseTypes.h>
#include <utility>

#ifndef _USE_MATH_DEFINES
#define _USE_MATH_DEFINES
#endif // !_USE_MATH_DEFINES
#include <cmath>
#include <iostream>

namespace EAR {

#define LOCATION_EPSILON 0.001 // 1 mm

std::ostream& operator<<(std::ostream& os, const CSpace& cs)
{
    switch (cs) {
    case CSpace::WCS:
        return os << "CSpace::WCS";

    case CSpace::RCS:
        return os << "CSpace::RCS";

    case CSpace::LCS:
        return os << "CSpace::LCS";
    }

    return os;
}

/*Vector3& Vector3::operator=(const Vector3& other) { // copy
    val[0] = other.val[0];
    val[1] = other.val[1];
    val[2] = other.val[2];

    return *this;
}

Vector3& Vector3::operator=(Vector3&& other) { // move
    val[0] = other.val[0];
    val[1] = other.val[1];
    val[2] = other.val[2];

    return *this;
}*/

void Vector3::set(float _x, float _y, float _z)
{
    val[0] = _x;
    val[1] = _y;
    val[2] = _z;
    val[3] = 1; // homogeneous coordinate
}

void Vector3::set(float _val[3])
{
    val[0] = _val[0];
    val[1] = _val[1];
    val[2] = _val[2];
    val[3] = 1; // homogeneous coordinate
}

Vector3& Vector3::operator+=(const Vector3& rhs)
{
    val[0] += rhs.val[0];
    val[1] += rhs.val[1];
    val[2] += rhs.val[2];

    return *this;
}

Vector3& Vector3::operator-=(const Vector3& rhs)
{
    val[0] -= rhs.val[0];
    val[1] -= rhs.val[1];
    val[2] -= rhs.val[2];

    return *this;
}

Vector3& Vector3::operator*=(const Vector3& rhs)
{
    float a = val[1] * rhs.val[2] - val[2] * rhs.val[1];
    float b = val[2] * rhs.val[0] - val[0] * rhs.val[2];
    float c = val[0] * rhs.val[1] - val[1] * rhs.val[0];

    val[0] = a;
    val[1] = b;
    val[2] = c;

    return *this;
}

Vector3& Vector3::operator*=(float rhs)
{
    val[0] = val[0] * rhs;
    val[1] = val[1] * rhs;
    val[2] = val[2] * rhs;

    return *this;
}

Vector3 Vector3::cross(const Vector3& rhs) const
{
    return *this * rhs;
}

float Vector3::dot(const Vector3& rhs) const
{
    return val[0] * rhs.val[0] + val[1] * rhs.val[1] + val[2] * rhs.val[2];
}

Vector3 Vector3::normalize() const
{
    float abs = this->abs();
    return Vector3(val[0] / abs, val[1] / abs, val[2] / abs);
}

float Vector3::abs() const
{
    return std::sqrt(val[0] * val[0] + val[1] * val[1] + val[2] * val[2]);
}

Vector3 Vector3::mirror(const Vector3& N0, float d) const
{
    float lambda = d - dot(N0);
    return *this + (2.f * lambda) * N0;
}

Vector3 operator+(Vector3 lhs, const Vector3& rhs)
{
    lhs += rhs;
    return lhs;
}
Vector3 operator-(Vector3 lhs, const Vector3& rhs)
{
    lhs -= rhs;
    return lhs;
}
Vector3 operator*(Vector3 lhs, const Vector3& rhs)
{
    lhs *= rhs;
    return lhs;
}
Vector3 operator*(Vector3 lhs, float rhs)
{
    lhs *= rhs;
    return lhs;
}

Vector3 operator*(float lhs, const Vector3& rhs)
{
    Vector3 res = rhs * lhs;
    return res;
}
Vector3 operator-(const Vector3& rhs)
{
    return Vector3(-rhs.val[0], -rhs.val[1], -rhs.val[2]);
}

std::ostream& operator<<(std::ostream& os, const Vector3& vec)
{
    return os << "Vector3(" << vec[0] << ", " << vec[1] << ", " << vec[2] << ")";
}
//! boolean compare for Vector3
bool operator==(const Vector3& lhs, const Vector3& rhs)
{
    return (fabs(lhs[0] - rhs[0]) < LOCATION_EPSILON) && (fabs(lhs[1] - rhs[1]) < LOCATION_EPSILON)
        && (fabs(lhs[2] - rhs[2]) < LOCATION_EPSILON);
}

//! boolean compare for Vector3
bool operator!=(const Vector3& lhs, const Vector3& rhs)
{
    return (fabs(lhs[0] - rhs[0]) >= LOCATION_EPSILON)
        || (fabs(lhs[1] - rhs[1]) >= LOCATION_EPSILON)
        || (fabs(lhs[2] - rhs[2]) >= LOCATION_EPSILON);
}

//! boolean compare for Vector3 with tolerance value
bool Vector3::isClose(const Vector3& rhs, float tolerance) const
{
    return !((fabs(val[0] - rhs[0]) >= tolerance) || (fabs(val[1] - rhs[1]) >= tolerance)
        || (fabs(val[2] - rhs[2]) >= tolerance));
}

void Euler::set(float y, float p, float r)
{
    angles[0] = y;
    angles[1] = p;
    angles[2] = r;
}

void Euler::set(float _angles[3])
{
    angles[0] = _angles[0];
    angles[1] = _angles[1];
    angles[2] = _angles[2];
}

std::ostream& operator<<(std::ostream& os, const Euler& eul)
{
    return os << "Euler(" << eul[0] << "deg, " << eul[1] << "deg, " << eul[2] << "deg)";
}

//! boolean compare for Euler
bool operator==(const Euler& lhs, const Euler& rhs)
{
    // TODO: Should use a threshold rather than float compare.
    return lhs[0] == rhs[0] && lhs[1] == rhs[1] && lhs[2] == rhs[2];
}

//! boolean compare for Euler
bool operator!=(const Euler& lhs, const Euler& rhs)
{
    // TODO: Should use a threshold rather than float compare.
    return lhs[0] != rhs[0] || lhs[1] != rhs[1] || lhs[2] != rhs[2];
}

float Euler::angleBetween(const Euler& rhs) const
{
    const auto thisHomogeneous = HomogenMatrix({ 0, 0, 0 }, *this);
    const auto rhsHomogeneous = HomogenMatrix({ 0, 0, 0 }, rhs);

    const auto thisToRhs = thisHomogeneous.inverseGeometric() * rhsHomogeneous;

    // Use the property that |theta| = arccos((tr(R) - 1) / 2) to compute the angle from the trace of the rotation matrix.

    // HomogeneousMatrix is 4x4, but the rotation matrix that is relevant is only the top-left 3x3 corner (the rest are zeros).
    const float trace = thisToRhs[0] + thisToRhs[5] + thisToRhs[10];

    float cosAngle = (trace - 1) / 2;

    // Due to numerical inaccuracies, cosAngle may be outside the [-1,1] range, in which case std::acos will return NaN. We therefore clip cosAngle to the valid range to prevent this.
    cosAngle = std::min(1.f, std::max(-1.f, cosAngle));

    const float angleRadians = std::acos(cosAngle);

    const float angleDegrees = angleRadians * 180.f / M_PI;

    return angleDegrees;
}

//! boolean compare for Euler with tolerance value
bool Euler::isClose(const Euler& rhs, float tolerance) const
{
    return angleBetween(rhs) < tolerance;
}

Euler Euler::operator+(const Euler& rhs)
{
    const Quaternion quat1 = Quaternion(*this);
    const Quaternion quat2 = Quaternion(rhs);

    const Quaternion composed = quat1 * quat2;
    return composed.getEuler();
}

void Quaternion::set(float y, float p, float r)
{
    float cy = static_cast<float>(std::cos(y / 360.f * M_PI));
    float sy = static_cast<float>(std::sin(y / 360.f * M_PI));
    float cp = static_cast<float>(std::cos(p / 360.f * M_PI));
    float sp = static_cast<float>(std::sin(p / 360.f * M_PI));
    float cr = static_cast<float>(std::cos(r / 360.f * M_PI));
    float sr = static_cast<float>(std::sin(r / 360.f * M_PI));

    this->q.w = cy * cp * cr + sy * sp * sr;
    this->q.x = cy * sp * cr + sy * cp * sr;
    this->q.y = sy * cp * cr - cy * sp * sr;
    this->q.z = cy * cp * sr - sy * sp * cr;

    normalize();
}

void Quaternion::set(const Euler& angles)
{
    set(angles[0], angles[1], angles[2]);
}

void Quaternion::set(float w, float x, float y, float z)
{
    this->q.w = w;
    this->q.x = x;
    this->q.y = y;
    this->q.z = z;
}

Vector3 Quaternion::rotate(const Vector3& v) const
{
    Vector3 u(q.x, q.y, q.z);
    return 2.0f * u.dot(v) * u + (q.w * q.w - u.dot(u)) * v + 2.0f * q.w * u.cross(v);
}

Euler Quaternion::getEuler() const
{
    Euler res;

    float sqw = q.w * q.w;
    float sqx = q.x * q.x;
    float sqy = q.y * q.y;
    float sqz = q.z * q.z;

    float norm = sqx + sqy + sqz + sqw; // needed to handle non-normalized quaternion
    float poleTest = (q.w * q.x - q.y * q.z) / norm;

    if (poleTest > 0.4999) { // singularity at north pole
        res[0] = 2.f * std::atan2(q.y, q.w) / M_PI * 180.f;
        res[1] = 90.f;
        res[2] = 0.f;
    } else if (poleTest < -0.4999) { // singularity at south pole
        res[0] = -2.f * std::atan2(q.y, q.w) / M_PI * 180.f;
        res[1] = -90.f;
        res[2] = 0.f;
    } else {
        res[0] = std::atan2(2 * (q.x * q.z + q.w * q.y), sqw - sqx - sqy + sqz) / M_PI * 180.f;
        res[1] = std::asin(2 * poleTest) / M_PI * 180.f;
        res[2] = std::atan2(2 * (q.x * q.y + q.w * q.z), sqw - sqx + sqy - sqz) / M_PI * 180.f;
    }

    return res;
}

Quaternion Quaternion::conj() const
{
    return Quaternion(q.w, -q.x, -q.y, -q.z);
}

void Quaternion::normalize()
{
    float norm = std::sqrt(q.x * q.x + q.y * q.y + q.z * q.z + q.w * q.w);
    q.w /= norm;
    q.x /= norm;
    q.y /= norm;
    q.z /= norm;
}

float Quaternion::dot(const Quaternion& other) const
{
    return q.x * other.q.x + q.y * other.q.y + q.z * other.q.z + q.w * other.q.w;
}

Quaternion& Quaternion::operator*=(float rhs)
{

    this->q.w *= rhs;
    this->q.x *= rhs;
    this->q.y *= rhs;
    this->q.z *= rhs;

    return *this;
}

Quaternion& Quaternion::operator*=(const Quaternion& rhs)
{

    float a = this->q.w * rhs.q.w - this->q.x * rhs.q.x - this->q.y * rhs.q.y - this->q.z * rhs.q.z;
    float b = this->q.w * rhs.q.x + this->q.x * rhs.q.w - this->q.y * rhs.q.z + this->q.z * rhs.q.y;
    float c = this->q.w * rhs.q.y + this->q.x * rhs.q.z + this->q.y * rhs.q.w - this->q.z * rhs.q.x;
    float d = this->q.w * rhs.q.z - this->q.x * rhs.q.y + this->q.y * rhs.q.x + this->q.z * rhs.q.w;

    this->q.w = a;
    this->q.x = b;
    this->q.y = c;
    this->q.z = d;

    return *this;
}

Quaternion slerp(const Quaternion& lhs, const Quaternion& rhs, float t)
{
    double d = lhs.dot(rhs);

    Quaternion rhs2 = rhs;
    if (d < 0.0f) {
        rhs2 = -1 * rhs2;
        d = -d;
    }

    if (d > 0.9995) {
        // linearly interpolate to avoid undefined acos
        Quaternion res = (1 - t) * lhs + t * rhs2;
        res.normalize();
        return res;
    }

    float th0 = std::acos(d);
    float th = th0 * t;
    float sinth = std::sin(th);
    float sinth0 = sin(th0);

    float s0 = std::cos(th) - d * sinth / sinth0;
    float s1 = sinth / sinth0;

    return (s0 * lhs) + (s1 * rhs2);
}

Quaternion operator*(const Quaternion& lhs, const Quaternion& rhs)
{
    Quaternion res;
    res[0] = lhs.q.w * rhs.q.w - lhs.q.x * rhs.q.x - lhs.q.y * rhs.q.y - lhs.q.z * rhs.q.z;
    res[1] = lhs.q.w * rhs.q.x + lhs.q.x * rhs.q.w - lhs.q.y * rhs.q.z + lhs.q.z * rhs.q.y;
    res[2] = lhs.q.w * rhs.q.y + lhs.q.x * rhs.q.z + lhs.q.y * rhs.q.w - lhs.q.z * rhs.q.x;
    res[3] = lhs.q.w * rhs.q.z - lhs.q.x * rhs.q.y + lhs.q.y * rhs.q.x + lhs.q.z * rhs.q.w;
    return res;
}

Quaternion operator*(Quaternion lhs, float rhs)
{
    lhs[0] *= rhs;
    lhs[1] *= rhs;
    lhs[2] *= rhs;
    lhs[3] *= rhs;

    return lhs;
}

Quaternion operator*(float lhs, const Quaternion& rhs)
{
    Quaternion res = rhs;
    res[0] *= lhs;
    res[1] *= lhs;
    res[2] *= lhs;
    res[3] *= lhs;
    return res;
}

Quaternion operator-(Quaternion lhs, const Quaternion& rhs)
{
    lhs[0] -= rhs[0];
    lhs[1] -= rhs[1];
    lhs[2] -= rhs[2];
    lhs[3] -= rhs[3];

    return lhs;
}

Quaternion operator+(Quaternion lhs, const Quaternion& rhs)
{
    lhs[0] += rhs[0];
    lhs[1] += rhs[1];
    lhs[2] += rhs[2];
    lhs[3] += rhs[3];

    return lhs;
}

std::ostream& operator<<(std::ostream& os, const Quaternion& qua)
{
    return os << "Quaternion(" << qua.q.w << ", " << qua.q.x << ", " << qua.q.y << ", " << qua.q.z
              << ")";
}

//! boolean compare
bool operator==(const Quaternion& lhs, const Quaternion& rhs)
{
    return lhs.q.w == rhs.q.w && lhs.q.x == rhs.q.x && lhs.q.y == rhs.q.y && lhs.q.z == rhs.q.z;
}

//! boolean compare
bool operator!=(const Quaternion& lhs, const Quaternion& rhs)
{
    return lhs.q.w != rhs.q.w || lhs.q.x != rhs.q.x || lhs.q.y != rhs.q.y || lhs.q.z != rhs.q.z;
}

void Polar::set(float a, float e, float d)
{
    values[0] = a;
    values[1] = e;
    values[2] = d;
}

void Polar::set(float _values[3])
{
    values[0] = _values[0];
    values[1] = _values[1];
    values[2] = _values[2];
}

//! boolean compare for Polar
bool operator==(const Polar& lhs, const Polar& rhs)
{
    return lhs[0] == rhs[0] && lhs[1] == rhs[1] && lhs[2] == rhs[2];
}

//! boolean compare for Polar
bool operator!=(const Polar& lhs, const Polar& rhs)
{
    return lhs[0] != rhs[0] || lhs[1] != rhs[1] || lhs[2] != rhs[2];
}

std::ostream& operator<<(std::ostream& os, const Polar& pol)
{
    return os << "Polar(" << pol[0] << "deg, " << pol[1] << "deg, " << pol[2] << "m)";
}

std::ostream& operator<<(std::ostream& os, const Position& pos)
{
    return os << "Position(" << pos.location << ", " << pos.orientation << ", " << pos.cSpace
              << ")";
}

Vector3 polarToCartesian(const Polar& pol)
{
    Vector3 res;
    res[0] = -1.f * pol[2] * static_cast<float>(std::sin(pol[0] * M_PI / 180.0f))
        * static_cast<float>(std::cos(pol[1] * M_PI / 180.0f));
    res[1] = pol[2] * static_cast<float>(std::sin(pol[1] * M_PI / 180.0f));
    res[2] = -1.f * pol[2] * static_cast<float>(std::cos(pol[0] * M_PI / 180.0f))
        * static_cast<float>(std::cos(pol[1] * M_PI / 180.0f));

    return res;
}

Polar cartesianToPolar(const Vector3& cart)
{
    if (cart.abs() == 0.f) {
        return Polar(0, 0, 0);
    }

    Polar res;
    res[0] = static_cast<float>(std::atan2(-1 * cart[0], -1 * cart[2])) / static_cast<float>(M_PI)
        * 180.0f;
    res[2] = cart.abs();
    res[1] = static_cast<float>(std::asin(cart[1] / res[2])) / static_cast<float>(M_PI) * 180.0f;
    // normalize by abs(cart) to stay in valid range of arcsin

    return res;
}

HomogenMatrix::HomogenMatrix()
    : matrix{ 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 }
{}

HomogenMatrix::HomogenMatrix(const Position& pos)
    : HomogenMatrix()
{
    setTranslation(pos.location);
    setRotation(pos.orientation);
}

HomogenMatrix::HomogenMatrix(const Vector3& loc, const Euler& ori)
    : HomogenMatrix()
{
    setTranslation(loc);
    setRotation(ori);
}

HomogenMatrix::HomogenMatrix(const Vector3& x, const Vector3& y, const Vector3& z, const Vector3& o)
    : matrix{ x[0], y[0], z[0], o[0], x[1], y[1], z[1], o[1], x[2], y[2], z[2], o[2], 0, 0, 0, 1 }
{}

void HomogenMatrix::setRotation(const Euler& rot)
{
    float yawRad = rot[0] * static_cast<float>(M_PI) / 180.0f;
    float pitchRad = rot[1] * static_cast<float>(M_PI) / 180.0f;
    float rollRad = rot[2] * static_cast<float>(M_PI) / 180.0f;

    float rotYaw[3][3] = { { 0.0f } };
    float rotPitch[3][3] = { { 0.0f } };
    float rotRoll[3][3] = { { 0.0f } };

    float tmpM[3][3];

    rotYaw[0][0] = rotYaw[2][2] = std::cos(yawRad);
    rotYaw[0][2] = std::sin(yawRad);
    rotYaw[1][1] = 1.0f;
    rotYaw[2][0] = -rotYaw[0][2];

    rotPitch[0][0] = 1.0f;
    rotPitch[1][1] = rotPitch[2][2] = std::cos(pitchRad);
    rotPitch[2][1] = std::sin(pitchRad);
    rotPitch[1][2] = -rotPitch[2][1];

    rotRoll[0][0] = rotRoll[1][1] = std::cos(rollRad);
    rotRoll[1][0] = std::sin(rollRad);
    rotRoll[0][1] = -rotRoll[1][0];
    rotRoll[2][2] = 1.0f;

    // yaw*pitch
    for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            tmpM[i][j] = 0.0f;

            for (int k = 0; k < 3; k++) {
                tmpM[i][j] += rotYaw[i][k] * rotPitch[k][j];
            }
        }
    }

    // (yaw*pitch)*roll
    for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            matrix[i * 4 + j] = 0.0f;

            for (int k = 0; k < 3; k++) {
                matrix[i * 4 + j] += tmpM[i][k] * rotRoll[k][j];
            }
        }
    }

    // TODO: speed up with SIMD?
}

void HomogenMatrix::setTranslation(const Vector3& tra)
{
    matrix[3] = tra[0];
    matrix[7] = tra[1];
    matrix[11] = tra[2];
}

Euler HomogenMatrix::getRotation() const
{
    Euler rot;

    rot[0] = std::atan2(matrix[2], matrix[10]) / static_cast<float>(M_PI) * 180.0f;
    rot[1] = -1 * std::asin(matrix[6]) / static_cast<float>(M_PI) * 180.0f;
    rot[2] = std::atan2(matrix[4], matrix[5]) / static_cast<float>(M_PI) * 180.0f;

    return rot;
}

std::vector<Vector3> HomogenMatrix::getBasis() const
{

    HomogenMatrix transform = this->inverse();

    std::vector<Vector3> basis{ transform * Vector3(1, 0, 0), transform * Vector3(0, 1, 0),
        transform * Vector3(0, 0, 1) };

    return basis;
}

Vector3 HomogenMatrix::getTranslation() const
{
    return Vector3(matrix[3], matrix[7], matrix[11]);
}

Vector3 HomogenMatrix::transform(const Vector3& rhs) const
{
    Vector3 result;

    for (int i = 0; i < 4; i++) {
        // result[i] assumed to be = 0
        for (int j = 0; j < 4; j++) {
            result[i] += rhs[j] * matrix[i * 4 + j];
        }
    }

    result[3] = 1; // because this edge case is not considered above

    // TODO: speed up with SIMD?

    return result;
}

Vector3 HomogenMatrix::operator*(const Vector3& rhs) const
{
    return transform(rhs);
}

HomogenMatrix HomogenMatrix::operator*(const float& rhs) const
{
    HomogenMatrix h;

    for (int i = 0; i < 16; i++) {
        h.matrix[i] = this->matrix[i] * rhs;
    }

    return h;
}

HomogenMatrix HomogenMatrix::concatenate(const HomogenMatrix& rhs) const
{
    HomogenMatrix result;

    for (int i = 0; i < 4; i++) {
        for (int j = 0; j < 4; j++) {
            result[i * 4 + j] = 0;

            for (int k = 0; k < 4; k++) {
                result[i * 4 + j] += matrix[i * 4 + k] * rhs[k * 4 + j];
            }
        }
    }

    // TODO: speed up with SIMD?

    return result;
}

HomogenMatrix HomogenMatrix::operator*(const HomogenMatrix& rhs) const
{
    return concatenate(rhs);
}

bool HomogenMatrix::isIdentical(const HomogenMatrix& rhs, float threshold) const
{
    bool result = true;

    for (int i = 0; i < 16; i++) {
        if (std::abs(matrix[i] - rhs[i]) >= threshold) {
            result = false;
            break;
        }
    }

    return result;
}

bool HomogenMatrix::operator==(const HomogenMatrix& rhs) const
{
    return isIdentical(rhs);
}

HomogenMatrix HomogenMatrix::inverseGeometric() const
{
    HomogenMatrix inverse;

    // invert rotation part (= transpose top-left 3x3 part of matrix)
    for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            inverse[j * 4 + i] = matrix[i * 4 + j];
        }
    }

    // invert translation part
    Vector3 invTrans(matrix[3], matrix[7], matrix[11]);
    invTrans = inverse * invTrans;

    inverse.matrix[3] = -1 * invTrans[0];
    inverse.matrix[7] = -1 * invTrans[1];
    inverse.matrix[11] = -1 * invTrans[2];

    return inverse;
}

HomogenMatrix HomogenMatrix::inverse3x3() const
{

    HomogenMatrix ret;

    float scale = 1.f
        / (matrix[0] * matrix[10] * matrix[5] - matrix[0] * matrix[6] * matrix[9]
            - matrix[1] * matrix[10] * matrix[4] + matrix[1] * matrix[6] * matrix[8]
            + matrix[2] * matrix[4] * matrix[9] - matrix[2] * matrix[5] * matrix[8]);

    ret.matrix[0] = scale * (matrix[10] * matrix[5] - matrix[6] * matrix[9]);
    ret.matrix[1] = scale * (matrix[2] * matrix[9] - matrix[1] * matrix[10]);
    ret.matrix[2] = scale * (matrix[1] * matrix[6] - matrix[2] * matrix[5]);
    ret.matrix[3] = 0.f;

    ret.matrix[4] = scale * (matrix[6] * matrix[8] - matrix[10] * matrix[4]);
    ret.matrix[5] = scale * (matrix[0] * matrix[10] - matrix[2] * matrix[8]);
    ret.matrix[6] = scale * (matrix[2] * matrix[4] - matrix[0] * matrix[6]);
    ret.matrix[7] = 0.f;

    ret.matrix[8] = scale * (matrix[4] * matrix[9] - matrix[5] * matrix[8]);
    ret.matrix[9] = scale * (matrix[1] * matrix[8] - matrix[0] * matrix[9]);
    ret.matrix[10] = scale * (matrix[0] * matrix[5] - matrix[1] * matrix[4]);
    ret.matrix[11] = 0.f;

    ret.matrix[12] = 0.f;
    ret.matrix[13] = 0.f;
    ret.matrix[14] = 0.f;
    ret.matrix[15] = 1.f;

    return ret;
}

std::vector<Matrix3_3> HomogenMatrix::getSubmatrices() const
{
    std::vector<Matrix3_3> splitted(16);

    const int dimension = 4;

    // get submatrix for each element
    for (int i = 0; i < dimension; i++) {
        for (int j = 0; j < dimension; j++) {

            //extract submatrices, by extracting matrix values and fill the matrix which represents
            //the submatrix corresponding to the specific tuple (row, column)
            int rowCount = 0;
            int columnCount = 0;

            for (int p = 0; p < dimension; p++) {
                for (int q = 0; q < dimension; q++) {

                    //  Copying into temporary matrix only those element
                    //  which are not in given row and column
                    if (i != p && j != q) {
                        splitted[i * dimension + j][rowCount][columnCount++] =
                            matrix[p * dimension + q];
                    }

                    if (columnCount == dimension - 1) {
                        //one row of the matrix was filled
                        columnCount = 0;
                        rowCount++;
                    }
                }
            }
        }
    }

    return splitted;
}

HomogenMatrix HomogenMatrix::getAdjugate() const
{
    HomogenMatrix adj;

    std::vector<Matrix3_3> submatrices = this->getSubmatrices();
    int sign = 1;

    //calculate adjugate
    for (unsigned int i = 0; i < submatrices.size(); i++) {
        int row = int(i / 4);
        int col = i % 4;
        sign = ((row + col) % 2 == 0) ? 1 : -1;
        adj[i] = sign * submatrices[i].getDeterminant();
    }

    adj = adj.T();

    return adj;
}

HomogenMatrix HomogenMatrix::T() const
{
    HomogenMatrix trans;
    int dimension = 4;

    for (int i = 0; i < 16; i++) {
        int row = int(i / 4);
        int col = i % 4;
        trans[dimension * col + row] = this->matrix[i];
    }

    return trans;
}

float HomogenMatrix::det() const
{
    float det = 0;

    std::vector<Matrix3_3> submatrices = this->getSubmatrices();

    det = submatrices[0].getDeterminant() * this->matrix[0]
        - submatrices[1].getDeterminant() * this->matrix[1]
        + submatrices[2].getDeterminant() * this->matrix[2]
        - submatrices[3].getDeterminant() * this->matrix[3];

    return det;
}

HomogenMatrix HomogenMatrix::inverse() const
{
    HomogenMatrix inv;
    float det = this->det();

    if (det == 0) {
        //inverse does not exist
        EAR_CHECK_REL(det != 0, "Inverse does not exist, determinant is 0");
    }

    inv = this->getAdjugate() * (1 / det);

    return inv;
}

std::ostream& operator<<(std::ostream& os, const HomogenMatrix& mat)
{
    os << "HomogenMatrix(" << std::endl;
    os << "              " << mat[0] << ", " << mat[1] << ", " << mat[2] << ", " << mat[3]
       << std::endl;
    os << "              " << mat[4] << ", " << mat[5] << ", " << mat[6] << ", " << mat[7]
       << std::endl;
    os << "              " << mat[8] << ", " << mat[9] << ", " << mat[10] << ", " << mat[11]
       << std::endl;
    os << "              " << mat[12] << ", " << mat[13] << ", " << mat[14] << ", " << mat[15]
       << ")";
    return os;
}

Matrix3_3& Matrix3_3::operator+=(const Matrix3_3& rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_colSize; j++) {
            m_matrix[i][j] += rhs[i][j];
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator-=(const Matrix3_3& rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_colSize; j++) {
            m_matrix[i][j] -= rhs[i][j];
        }
    }

    return *this;
}

std::array<float, 3> Matrix3_3::getRow(const int idx) const
{
    return m_matrix[idx];
}

std::array<float, 3> Matrix3_3::getColumn(const int idx) const
{
    return { m_matrix[0][idx], m_matrix[1][idx], m_matrix[2][idx] };
}

Matrix3_3& Matrix3_3::operator*=(const Matrix3_3& rhs)
{

    std::array<float, 3> col0 = rhs.getColumn(0);
    std::array<float, 3> col1 = rhs.getColumn(1);
    std::array<float, 3> col2 = rhs.getColumn(2);

    for (unsigned int k = 0; k < m_rowSize; k++) {
        float a = std::inner_product(col0.begin(), col0.end(), this->getRow(k).begin(), 0.0f);
        float b = std::inner_product(col1.begin(), col1.end(), this->getRow(k).begin(), 0.0f);
        float c = std::inner_product(col2.begin(), col2.end(), this->getRow(k).begin(), 0.0f);

        m_matrix[k][0] = a;
        m_matrix[k][1] = b;
        m_matrix[k][2] = c;
    }

    return *this;
}

Matrix3_3 Matrix3_3::getTranspose() const
{
    std::array<float, 3> col0 = this->getColumn(0);
    std::array<float, 3> col1 = this->getColumn(1);
    std::array<float, 3> col2 = this->getColumn(2);

    std::array<std::array<float, 3>, 3> transposed = { { col0, col1, col2 } };

    return Matrix3_3(transposed);
}

float Matrix3_3::getDeterminant() const
{

    return m_matrix[0][0] * (m_matrix[1][1] * m_matrix[2][2] - m_matrix[1][2] * m_matrix[2][1])
        - m_matrix[0][1] * (m_matrix[1][0] * m_matrix[2][2] - m_matrix[1][2] * m_matrix[2][0])
        + m_matrix[0][2] * (m_matrix[1][0] * m_matrix[2][1] - m_matrix[1][1] * m_matrix[2][0]);
}

Matrix3_3 Matrix3_3::getInverse() const
{
    float det = this->getDeterminant();

    if (det == 0) {
        EAR_CHECK_REL(det != 0, "Inverse does not exist, determinant is 0");
    }

    Matrix3_3 inverse({ { m_matrix[1][1] * m_matrix[2][2] - m_matrix[1][2] * m_matrix[2][1],
                            m_matrix[1][2] * m_matrix[2][0] - m_matrix[2][2] * m_matrix[1][0],
                            m_matrix[1][0] * m_matrix[2][1] - m_matrix[1][1] * m_matrix[2][0] },

        { -(m_matrix[0][1] * m_matrix[2][2] - m_matrix[0][2] * m_matrix[2][1]),
            m_matrix[0][0] * m_matrix[2][2] - m_matrix[0][2] * m_matrix[2][0],
            -(m_matrix[0][0] * m_matrix[2][1] - m_matrix[0][1] * m_matrix[2][0]) },

        { m_matrix[0][1] * m_matrix[1][2] - m_matrix[0][2] * m_matrix[1][1],
            -(m_matrix[0][0] * m_matrix[1][2] - m_matrix[0][2] * m_matrix[1][0]),
            m_matrix[0][0] * m_matrix[1][1] - m_matrix[0][1] * m_matrix[1][0] } });

    inverse *= 1 / det;
    return inverse.getTranspose();
}

Matrix3_3& Matrix3_3::operator+=(const float rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_colSize; j++) {
            m_matrix[i][j] += rhs;
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator-=(const float rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_rowSize; j++) {
            m_matrix[i][j] -= rhs;
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator-(const Matrix3_3& rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_rowSize; j++) {
            m_matrix[i][j] -= rhs[i][j];
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator+(const Matrix3_3& rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_rowSize; j++) {
            m_matrix[i][j] += rhs[i][j];
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator*=(const float rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_colSize; j++) {
            m_matrix[i][j] *= rhs;
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator/=(const float rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_colSize; j++) {
            m_matrix[i][j] /= rhs;
        }
    }

    return *this;
}

Matrix3_3& Matrix3_3::operator*(const float rhs)
{
    for (unsigned int i = 0; i < m_rowSize; i++) {
        for (unsigned int j = 0; j < m_colSize; j++) {
            m_matrix[i][j] *= rhs;
        }
    }

    return *this;
}

float Matrix3_3::getTrace() const
{
    return m_matrix[0][0] + m_matrix[1][1] + m_matrix[2][2];
}

Vector3 operator*(const Matrix3_3 lhs, const Vector3 rhs)
{
    Vector3 res;

    for (unsigned int i = 0; i < 3; i++) {
        for (unsigned int j = 0; j < 3; j++) {
            res[i] += lhs[i][j] * rhs[j];
        }
    }

    return res;
}

EAR::Matrix3_3& EAR::Matrix3_3::operator=(const EAR::Matrix3_3& rhs)
{
    for (int i = 0; i < 4; i++) {
        for (int j = 0; j < 4; j++) {
            m_matrix[i][j] = rhs[i][j];
        }
    }
    return *this;
}

std::ostream& operator<<(std::ostream& os, const Matrix3_3& matrix)
{
    os << "Matrix3_3(" << std::endl;
    os << "              " << matrix[0][0] << ", " << matrix[0][1] << ", " << matrix[0][2]
       << std::endl;
    os << "              " << matrix[1][0] << ", " << matrix[1][1] << ", " << matrix[1][2]
       << std::endl;
    os << "              " << matrix[2][0] << ", " << matrix[2][1] << ", " << matrix[2][2] << ")";
    return os;
}

HomogenMatrix rotateVectorTo(const Vector3 v1, const Vector3 v2)
{
    // code based on discussion in https://iquilezles.org/articles/noacos/
    const EAR::Vector3 axis = v1.cross(v2);

    const float cosA = v1.dot(v2);
    float k;
    if (cosA < -0.9999f) {
        k = 0.0f;
    } else {
        k = 1.0f / (1.0f + cosA);
    }

    const auto ax = axis.data()[0];
    const auto ay = axis.data()[1];
    const auto az = axis.data()[2];

    auto x = EAR::Vector3((ax * ax * k) + cosA, (ax * ay * k) + az, (ax * az * k) - ay);
    auto y = EAR::Vector3((ay * ax * k) - az, (ay * ay * k) + cosA, (ay * az * k) + ax);
    auto z = EAR::Vector3((az * ax * k) + ay, (az * ay * k) - ax, (az * az * k) + cosA);
    auto o = EAR::Vector3(0.0f, 0.0f, 0.0f);

    return EAR::HomogenMatrix(x, y, z, o);
}

} // namespace EAR
