/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <EAR/PointAirflowGenerator.h>

#include <cmath>

namespace EAR {

PointAirflowGenerator::PointAirflowGenerator(ObjectID oid, const std::string& name, Position pos)
    : AirflowGeneratorBase(oid, name, pos)
{}

Vector3 PointAirflowGenerator::getAirspeed(const Vector3& listenerPosition) const
{

    if (hasLimits) {
        // Get location of listener relative to the generator
        const Polar relativeListenerPosition =
            cartesianToPolar(getRelativePosition(getGlobalPosition(), listenerPosition));

        if (std::abs(relativeListenerPosition[0]) > aziRange
            || std::abs(relativeListenerPosition[1]) > elevRange) {
            // Listener is outside the azi or elevation range: 0 airflow is generated.
            return { 0, 0, 0 };
        }
    }

    const Vector3 listenerOffset = listenerPosition
        - getLocation(); // offset from generator to listener. I.e. in the direction of the airflow.

    float airspeed =
        getSpeed() * (1 / std::pow((listenerOffset.abs() + 1), 2 * getDistanceAttenuationFactor()));
    if (airspeed > getSpeed()) {
        airspeed = getSpeed();
    }

    return airspeed * listenerOffset.normalize();
}

void PointAirflowGenerator::setLimits(float aziRange, float elevRange)
{
    this->aziRange = aziRange;
    this->elevRange = elevRange;
    this->hasLimits = true;
}

void PointAirflowGenerator::removeLimits()
{
    this->aziRange = 360.f;
    this->elevRange = 360.f;
    this->hasLimits = false;
}

} /* namespace EAR */
