/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <EAR/RealBuffer.h>

#include <algorithm>
#include <cmath>
#include <numeric>
#include <utility>
#include <vector>

#include "AlignedAllocator.h"

namespace EAR {

RealBuffer::RealBuffer()
    : Buffer()
{}

RealBuffer::RealBuffer(std::size_t length)
    : Buffer(length)
{}

RealBuffer::RealBuffer(const Buffer<float>& other)
    : Buffer(other)
{}

RealBuffer::RealBuffer(const float* data, std::size_t length)
    : Buffer(data, length)
{}

RealBuffer::~RealBuffer() {}

void RealBuffer::writeDbl(const double* src, std::size_t offset, std::size_t count)
{
    std::transform(src, src + count, buffer->data() + offset,
        [](const double& v) { return static_cast<float>(v); });
}

void RealBuffer::readDbl(double* dst, std::size_t offset, std::size_t count)
{
    std::transform(buffer->data() + offset, buffer->data() + offset + count, dst,
        [](const float& v) { return static_cast<double>(v); });
}

float RealBuffer::mini() const
{
    return *std::min_element(buffer->begin(), buffer->end());
}

float RealBuffer::maxi() const
{
    return *std::max_element(buffer->begin(), buffer->end());
}

float RealBuffer::peak() const
{
    auto minmax = std::minmax_element(buffer->begin(), buffer->end());

    *minmax.first = std::abs(*minmax.first);
    *minmax.second = std::abs(*minmax.second);

    return *minmax.first > *minmax.second ? *minmax.first : *minmax.second;
}

float RealBuffer::rms() const
{
    float rms = std::inner_product(buffer->begin(), buffer->end(), buffer->begin(),
        0.f); // multiply-add buffer values with itselves
    return std::sqrt(rms / static_cast<float>(buffer->size()));
}

} // namespace EAR
