/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <EAR/SceneObjectFactory.h>

#include <EAR/AnchorObject.h>
#include <EAR/ConeAirflowGenerator.h>
#include <EAR/Geometry.h>
#include <EAR/GlobalAirflowGenerator.h>
#include <EAR/HOASource.h>
#include <EAR/Listener.h>
#include <EAR/ListenerVoiceSource.h>
#include <EAR/LoudspeakerLayout.h>
#include <EAR/PlaneAirflowGenerator.h>
#include <EAR/PointAirflowGenerator.h>
#include <EAR/PointSource.h>
#include <EAR/Portal.h>
#include <EAR/PortalReverbExtent.h>

namespace EAR {

template <class T> SceneObjectFactory<T>* SceneObjectFactory<T>::instance = nullptr;

template <class T> SceneObjectFactory<T>* SceneObjectFactory<T>::getInstance()
{
    if (!instance) {
        instance = new SceneObjectFactory<T>();
    }

    return instance;
}

template <class T> SceneObjectFactory<T>::SceneObjectFactory() {}

template <class T> ObjectID SceneObjectFactory<T>::getNextOID()
{
    return counter++;
}

template class SceneObjectFactory<AnchorObject>;
template class SceneObjectFactory<Box>;
template class SceneObjectFactory<Cylinder>;
template class SceneObjectFactory<HOASource>;
template class SceneObjectFactory<Listener>;
template class SceneObjectFactory<ListenerVoiceSource>;
template class SceneObjectFactory<LoudspeakerLayout>;
template class SceneObjectFactory<Mesh>;
template class SceneObjectFactory<PointSource>;
template class SceneObjectFactory<Portal>;
template class SceneObjectFactory<PortalReverbExtent>;
template class SceneObjectFactory<Sphere>;
template class SceneObjectFactory<GlobalAirflowGenerator>;
template class SceneObjectFactory<ConeAirflowGenerator>;
template class SceneObjectFactory<PlaneAirflowGenerator>;
template class SceneObjectFactory<PointAirflowGenerator>;

} // namespace EAR
