/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <EAR/SofaHRIRProvider.h>

#include <EAR/Exception.h>
#include <EAR/RealFrame.h>

namespace EAR {

SofaHRIRProvider::SofaHRIRProvider(const std::string& path)
    : reader(path)
{
    unsigned long temp;
    reader.getMetaData<unsigned long>("Data.SamplingRate", &temp);
    fs = static_cast<float>(temp);

    EAR_CHECK(reader.getDimension("R") == 2,
        "Can't do BInaural Rendering without two receivers (-> ears) in the SOFA dataset.");
}

SofaHRIRProvider::~SofaHRIRProvider() {}

void SofaHRIRProvider::getNearestIRForLocation(RealFrame& ir, Vector3 loc) const
{
    reader.getNearestValueForLocation(ir, loc);
}

std::unique_ptr<const float* []> SofaHRIRProvider::getIRPointerForLocation(
    Vector3 loc) const { return reader.getValuePointerForLocation(loc); }

std::unique_ptr<const float* []> SofaHRIRProvider::getIRPointerForIndex(
    int index) const { return reader.getValuePointerForIndex(index); }

std::vector<int> SofaHRIRProvider::getAllIndicesInRange(
    const EAR::Polar& bottomLeft, const EAR::Polar& topRight) const
{
    // TODO: this a crude way of implementing this, a kd tree range search would be much more efficient
    std::vector<int> result;
    const float* directions = reader.getDirectionsBuffer();
    unsigned int M = irCount();
    for (unsigned int i = 0; i < M; i++) {
        if (isInAngularRange(directions[i * 3], bottomLeft[0], topRight[0])
            && isInAngularRange(directions[i * 3 + 1], bottomLeft[1], topRight[1])) {
            result.push_back(i);
        }
    }

    return result;
}

float SofaHRIRProvider::getDistanceToNearestIndex(Vector3& loc) const
{
    return reader.getDistanceToNearestIndex(loc);
}

Vector3 SofaHRIRProvider::getNearestLocation(Vector3& loc) const
{
    return reader.getNearestLocation(loc);
}

Vector3 SofaHRIRProvider::getLocationForIndex(int idx) const
{
    return reader.getLocationForIndex(idx);
}

int SofaHRIRProvider::irLength() const
{
    return static_cast<int>(reader.getDimension("N"));
}

int SofaHRIRProvider::irCount() const
{
    return static_cast<int>(reader.getDimension("M"));
}

float SofaHRIRProvider::getSampleRate() const
{
    return static_cast<float>(fs);
}

} // namespace EAR
