/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef EAR_VDL_DEFINITIONS
#define EAR_VDL_DEFINITIONS

// 16th order IIR anti-imaging lowpass filter
// Sampling rate 44.1kHz, passband 18kHz, passband ripple <= 1/4 dB, stopband attenuation >=92 dB
const float LPAA_IIR_44100_18000_2x_GROUPDELAY = 3.928f;
const int LPAA_IIR_44100_18000_2x_LENGTH = 17;
const float LPAA_IIR_44100_18000_2x[2][17] = {
    { 0.00260010f, 0.01705314f, 0.06462728f, 0.17537751f, 0.37308982f, 0.65058980f, 0.95411688f,
        1.19433072f, 1.28610827f, 1.19433072f, 0.95411688f, 0.65058980f, 0.37308982f, 0.17537751f,
        0.06462728f, 0.01705314f, 0.00260010f },
    { 1.00000000f, -0.58054780f, 2.90756803f, -0.88764659f, 3.17164013f, -0.26376425f, 1.75787674f,
        0.18160235f, 0.56694791f, 0.13444087f, 0.11266939f, 0.03200992f, 0.01299998f, 0.00310939f,
        0.00067415f, 0.00009173f, 0.00000679f },
};

// 5th order IIR anti-imaging lowpass filter
// Sampling rate 44.1kHz, passband 18kHz, passband ripple <= 1/4 dB, stopband attenuation >=92 dB
const float LPAA_IIR_44100_18000_1x_GROUPDELAY = 0.371f;
const int LPAA_IIR_44100_18000_1x_LENGTH = 6;
const float LPAA_IIR_44100_18000_1x[2][6] = {
    { 0.49319651f, 2.46064922f, 4.91597664f, 4.91597664f, 2.46064922f, 0.49319651f },
    { 1.00000000f, 3.59427259f, 5.32121200f, 4.02803016f, 1.55288726f, 0.24324273f },
};

// 14th order IIR anti-imaging lowpass filter
// Sampling rate 48kHz, passband 18kHz, passband ripple <= 1/4 dB, stopband attenuation >=92 dB
const float LPAA_IIR_48000_18000_2x_GROUPDELAY = 4.159f;
const int LPAA_IIR_48000_18000_2x_LENGTH = 15;
const float LPAA_IIR_48000_18000_2x[2][15] = {
    { 0.00213613f, 0.01156903f, 0.03790756f, 0.08969363f, 0.16721597f, 0.25545786f, 0.32701586f,
        0.35466388f, 0.32701586f, 0.25545786f, 0.16721597f, 0.08969363f, 0.03790756f, 0.01156903f,
        0.00213613f },
    { 1.00000000f, -1.66780092f, 3.49047369f, -3.33760717f, 3.61841796f, -2.17731446f, 1.52993298f,
        -0.57082374f, 0.28603693f, -0.05744148f, 0.02337504f, -0.00138820f, 0.00076350f,
        0.00002609f, 0.00000574f },
};

// 4th order IIR anti-imaging lowpass filter
// Sampling rate 48kHz, passband 18kHz, passband ripple <= 1/4 dB, stopband attenuation >=92 dB
const float LPAA_IIR_48000_18000_1x_GROUPDELAY = 0.391f;
const int LPAA_IIR_48000_18000_1x_LENGTH = 5;
const float LPAA_IIR_48000_18000_1x[2][5] = {
    { 0.47349705f, 1.89172541f, 2.83645808f, 1.89172541f, 0.47349705f },
    { 1.00000000f, 2.53312005f, 2.58616279f, 1.22340938f, 0.22421078f },
};

// 3rd order modified Farrow structure FIR fractional delay filters for Lagrange and Spline interpolation
const int FD_FARROW_3_GD = 1;
const int FD_FARROW_3_N = 4;
const float FD_FARROW_LAGRANGE_3_C[4][4] = {
    { 0.00000000f, 1.00000000f, 0.00000000f, 0.00000000f }, //C_0(z)
    { -0.33333333f, -0.50000000f, 1.00000000f, -0.16666667f }, //C_1(z)
    { 0.50000000f, -1.00000000f, 0.50000000f, 0.00000000f }, //C_2(z)
    { -0.16666667f, 0.50000000f, -0.50000000f, 0.16666667f }, //C_3(z)
};

const float FD_FARROW_SPLINE_3_C[4][4] = {
    { 0.16666667f, 0.66666668, 0.16666667f, 0.00000000f }, // C_0(z)
    { -0.50000000f, 0.00000000f, 0.50000000f, 0.00000000f }, // C_1(z)
    { 0.50000000f, -1.00000000f, 0.50000000f, 0.00000000f }, // C_2(z)
    { -0.16666667f, 0.50000000f, -0.50000000f, 0.16666667f }, // C_3(z)
    // h[0]          h[-1]         h[-2]        h[-3]
};

// 1st order (linear) modified Farrow structure FIR fractional delay filters for Lagrange interpolation
const int FD_FARROW_1_GD = 0;
const int FD_FARROW_1_N = 2;
const float FD_FARROW_LAGRANGE_1_C[2][2] = {
    { 1.f, 0.f },
    { -1.f, 1.f },
};

// 1st order Lagrange FIR fractional delay filters, 1/129 sample delays. (The greater the quantization steps, the
// greater the existing aliasing, Source: The Effects of Quantizating the Fractional Interbal in Interpolation Filter, Jussi Vesma et al., )
const int FD_LAGRANGE_1_GD = 0;
const int FD_LAGRANGE_1_N = 2;
const int FD_LAGRANGE_1_M = 129;

// Due to symmetry, only the first coefficient is tabulated. c[1] = 1 - c[0]
const float FD_LAGRANGE_16_1_C[129] = { 1.000000f, 0.992188f, 0.984375f, 0.976562f, 0.968750f,
    0.960938f, 0.953125f, 0.945312f, 0.937500f, 0.929688f, 0.921875f, 0.914062f, 0.906250f,
    0.898438f, 0.890625f, 0.882812f, 0.875000f, 0.867188f, 0.859375f, 0.851562f, 0.843750f,
    0.835938f, 0.828125f, 0.820312f, 0.812500f, 0.804688f, 0.796875f, 0.789062f, 0.781250f,
    0.773438f, 0.765625f, 0.757812f, 0.750000f, 0.742188f, 0.734375f, 0.726562f, 0.718750f,
    0.710938f, 0.703125f, 0.695312f, 0.687500f, 0.679688f, 0.671875f, 0.664062f, 0.656250f,
    0.648438f, 0.640625f, 0.632812f, 0.625000f, 0.617188f, 0.609375f, 0.601562f, 0.593750f,
    0.585938f, 0.578125f, 0.570312f, 0.562500f, 0.554688f, 0.546875f, 0.539062f, 0.531250f,
    0.523438f, 0.515625f, 0.507812f, 0.500000f, 0.492188f, 0.484375f, 0.476562f, 0.468750f,
    0.460938f, 0.453125f, 0.445312f, 0.437500f, 0.429688f, 0.421875f, 0.414062f, 0.406250f,
    0.398438f, 0.390625f, 0.382812f, 0.375000f, 0.367188f, 0.359375f, 0.351562f, 0.343750f,
    0.335938f, 0.328125f, 0.320312f, 0.312500f, 0.304688f, 0.296875f, 0.289062f, 0.281250f,
    0.273438f, 0.265625f, 0.257812f, 0.250000f, 0.242188f, 0.234375f, 0.226562f, 0.218750f,
    0.210938f, 0.203125f, 0.195312f, 0.187500f, 0.179688f, 0.171875f, 0.164062f, 0.156250f,
    0.148438f, 0.140625f, 0.132812f, 0.125000f, 0.117188f, 0.109375f, 0.101562f, 0.093750f,
    0.085938f, 0.078125f, 0.070312f, 0.062500f, 0.054688f, 0.046875f, 0.039062f, 0.031250f,
    0.023438f, 0.015625f, 0.007812f, 0.f };

// 3rd order Lagrange FIR fractional delay filters, 1/64 sample delays. (The greater the quantization steps, the
// greater the existing aliasing, Source: The Effects of Quantizating the Fractional Interbal in Interpolation Filter, Jussi Vesma et al.)
const int FD_LAGRANGE_3_GD = 1;
const int FD_LAGRANGE_3_N = 4;
const int FD_LAGRANGE_3_M = 65;

// Due to symmetry, only half of the coefficients are tabulated. c[3] = 1 - c[0], c[2] = 1 - c[1]
const float FD_LAGRANGE_16_3_C[2][65] = { {
                                              -0.000000f,
                                              -0.005087f,
                                              -0.009933f,
                                              -0.014544f,
                                              -0.018921f,
                                              -0.023069f,
                                              -0.026993f,
                                              -0.030695f,
                                              -0.034180f,
                                              -0.037451f,
                                              -0.040512f,
                                              -0.043367f,
                                              -0.046021f,
                                              -0.048475f,
                                              -0.050735f,
                                              -0.052805f,
                                              -0.054688f,
                                              -0.056387f,
                                              -0.057907f,
                                              -0.059252f,
                                              -0.060425f,
                                              -0.061430f,
                                              -0.062271f,
                                              -0.062952f,
                                              -0.063477f,
                                              -0.063848f,
                                              -0.064072f,
                                              -0.064150f,
                                              -0.064087f,
                                              -0.063887f,
                                              -0.063553f,
                                              -0.063089f,
                                              -0.062500f,
                                              -0.061789f,
                                              -0.060959f,
                                              -0.060015f,
                                              -0.058960f,
                                              -0.057798f,
                                              -0.056534f,
                                              -0.055170f,
                                              -0.053711f,
                                              -0.052160f,
                                              -0.050522f,
                                              -0.048800f,
                                              -0.046997f,
                                              -0.045118f,
                                              -0.043167f,
                                              -0.041147f,
                                              -0.039062f,
                                              -0.036917f,
                                              -0.034714f,
                                              -0.032457f,
                                              -0.030151f,
                                              -0.027800f,
                                              -0.025406f,
                                              -0.022974f,
                                              -0.020508f,
                                              -0.018011f,
                                              -0.015488f,
                                              -0.012941f,
                                              -0.010376f,
                                              -0.007795f,
                                              -0.005203f,
                                              -0.002604f,
                                              -0.000000f,
                                          },
    {
        1.000000f,
        0.991945f,
        0.983414f,
        0.974417f,
        0.964966f,
        0.955072f,
        0.944748f,
        0.934004f,
        0.922852f,
        0.911303f,
        0.899368f,
        0.887060f,
        0.874390f,
        0.861368f,
        0.848007f,
        0.834318f,
        0.820312f,
        0.806002f,
        0.791397f,
        0.776510f,
        0.761353f,
        0.745935f,
        0.730270f,
        0.714369f,
        0.698242f,
        0.681902f,
        0.665359f,
        0.648626f,
        0.631714f,
        0.614634f,
        0.597397f,
        0.580015f,
        0.562500f,
        0.544863f,
        0.527115f,
        0.509268f,
        0.491333f,
        0.473322f,
        0.455246f,
        0.437117f,
        0.418945f,
        0.400743f,
        0.382523f,
        0.364294f,
        0.346069f,
        0.327860f,
        0.309677f,
        0.291533f,
        0.273438f,
        0.255404f,
        0.237442f,
        0.219564f,
        0.201782f,
        0.184107f,
        0.166550f,
        0.149122f,
        0.131836f,
        0.114702f,
        0.097733f,
        0.080938f,
        0.064331f,
        0.047922f,
        0.031723f,
        0.015745f,
        0.000000f,
    } };

// third order Lagrange interpolation in Newton interpolator structure
const int FD_NEWTON_3_GD = 1;
const int FD_NEWTON_3_N = 4;
const float FD_NEWTON_LAGRANGE_3_C[2] = { 0.50000000f, 0.16666667f };

// third order Spline interpolation in Newton interpolator structure
const float FD_NEWTON_SPLINE_3_C[2] = { 0.50000000f, 0.16666667f };

//Correction filter for spline interpolation. Applying this filter transforms spline approximation into spline interpolation
const int CF_FIR_GROUPDELAY = 5;
const int CF_FIR_LENGTH = 10;
const float CF_FIR[10] = { 0.00347238328, 0.00150651130, -0.0233071767, 0.109949641, -0.444294684,
    1.72920318, -0.480775334, 0.138894453, -0.0441908685, 0.0171125385 };

#endif /* EAR_VDL_DEFINTIONS */
