/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2024.
***************************************************************************/

#include <EAR/Biquad.h>
#include <EAR/HeadphoneEQ.h>
#include <doctest/doctest.h>
#include <iostream>
#include <vector>

TEST_CASE("Biquad filter response - 1/3rd Octave  wide 1kHz bandpass filter")
{
    float fs = 48000;
    float cf = 1000;
    float ThirdOctaveQ = 4.318473046963144;

    Biquad peakFilter = Biquad(bq_type_bandpass, cf / fs, ThirdOctaveQ, 0.f);

    std::vector<double> expected_a = { 0.014887551161843841, 0, -0.014887551161843841 };
    std::vector<double> expected_b = { 1.0, -1.9533693505519216, 0.9702248976763125 };

    CHECK(expected_a[0] == doctest::Approx(peakFilter.a0).epsilon(0.0001));
    CHECK(expected_a[1] == doctest::Approx(peakFilter.a1).epsilon(0.0001));
    CHECK(expected_a[2] == doctest::Approx(peakFilter.a2).epsilon(0.0001));
    CHECK(expected_b[1] == doctest::Approx(peakFilter.b1).epsilon(0.0001));
    CHECK(expected_b[2] == doctest::Approx(peakFilter.b2).epsilon(0.0001));

    static const float ThirdOctaveCenterFreqs[] = { // [Hz]
        20, 25, 31.5, 40, 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250,
        1600, 2000, 2500, 3150, 4000, 5000, 6300, 8000, 10000, 12500, 16000, 20000
    };

    std::vector<float> expected_response = { // [dB]
        -46.6950421600830, -44.7549417138905, -42.7444284396103, -40.6643033685665,
        -38.7184915874007, -36.6986434072852, -34.6030355030331, -32.6342155645893,
        -30.6479666972077, -28.4179383969305, -26.3544054222262, -24.2168976269386,
        -21.8746608003150, -19.2177032325184, -16.3458688499972, -12.5909148875754,
        -6.80716399387534, 2.50725141303854e-14, -6.81623352969476, -12.7668777552783,
        -16.3914600345554, -19.2927555710445, -21.9147679789004, -24.4168846503643,
        -26.6709833548664, -29.0019875575429, -31.4952131910136, -34.0137434462088,
        -36.9130025748850, -41.1351166330094, -47.8123599939438
    };

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        float response = 20 * std::log10(peakFilter.calcResponse(ThirdOctaveCenterFreqs[idx] / fs));
        CHECK(expected_response[idx] == doctest::Approx(response).epsilon(0.0001));
    }
}

TEST_CASE("Biquad filter response - lowpass, 50Hz")
{
    float fs = 48000;
    float cf = 50;
    float Q = 0.707;

    Biquad peakFilter = Biquad(bq_type_lowpass, cf / fs, Q, 0.f);

    std::vector<double> expected_a = { 0.00001065982712403427, 0.00002131965424806854,
        0.00001065982712403427 };
    std::vector<double> expected_b = { 1.0, -1.9907426744222796, 0.9907853137307757 };

    CHECK(expected_a[0] == doctest::Approx(peakFilter.a0).epsilon(0.0001));
    CHECK(expected_a[1] == doctest::Approx(peakFilter.a1).epsilon(0.0001));
    CHECK(expected_a[2] == doctest::Approx(peakFilter.a2).epsilon(0.0001));
    CHECK(expected_b[1] == doctest::Approx(peakFilter.b1).epsilon(0.0001));
    CHECK(expected_b[2] == doctest::Approx(peakFilter.b2).epsilon(0.0001));

    static const float ThirdOctaveCenterFreqs[] = { // [Hz]
        20, 25, 31.5, 40, 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250,
        1600, 2000, 2500, 3150, 4000, 5000, 6300, 8000, 10000, 12500, 16000, 20000
    };

    std::vector<float> expected_response = { // [dB]
        -0.110188149512283, -0.263904000129455, -0.636215614188181, -1.49214360112258,
        -3.01161172406207, -5.46722033466966, -8.78251294582964, -12.3052816507502,
        -16.0281072304110, -20.2480418336797, -24.1004211806228, -27.9673341257205,
        -31.9788420522012, -36.1286070884832, -40.0066006800146, -44.0247972523666,
        -48.1806989538175, -52.0660012291145, -55.9563720870062, -60.2696076923220,
        -64.1819597821609, -68.1147473396456, -72.2221597067963, -76.5268909876874,
        -80.6360431541067, -85.0403000648176, -89.8623673953980, -94.8040113790705,
        -100.542585335064, -108.947217504823, -122.282694304362
    };

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        float response = 20 * std::log10(peakFilter.calcResponse(ThirdOctaveCenterFreqs[idx] / fs));
        CHECK(expected_response[idx] == doctest::Approx(response).epsilon(0.0001));
    }
}

TEST_CASE("Biquad filter response - lowshelf, 100Hz")
{
    float fs = 48000;
    float cf = 100;
    float Q = 1;
    float gain = 4;

    Biquad peakFilter = Biquad(bq_type_lowshelf, cf / fs, Q, gain);

    std::vector<double> expected_a = { 1.0023993932002757, -1.9814388594703927,
        0.9793085390910392 };
    std::vector<double> expected_b = { 1.0, -1.9814885091445735, 0.9816582826171343 };

    CHECK(expected_a[0] == doctest::Approx(peakFilter.a0).epsilon(0.0001));
    CHECK(expected_a[1] == doctest::Approx(peakFilter.a1).epsilon(0.0001));
    CHECK(expected_a[2] == doctest::Approx(peakFilter.a2).epsilon(0.0001));
    CHECK(expected_b[1] == doctest::Approx(peakFilter.b1).epsilon(0.0001));
    CHECK(expected_b[2] == doctest::Approx(peakFilter.b2).epsilon(0.0001));

    static const float ThirdOctaveCenterFreqs[] = { // [Hz]
        20, 25, 31.5, 40, 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250,
        1600, 2000, 2500, 3150, 4000, 5000, 6300, 8000, 10000, 12500, 16000, 20000
    };

    std::vector<float> expected_response = { // [dB]
        3.9958, 3.9898, 3.9744, 3.9343, 3.8435, 3.6289, 3.1651, 2.4451, 1.5816, 0.79232, 0.36996,
        0.16083, 0.06549, 0.025452, 0.010462, 0.0041543, 0.0015966, 0.00065278, 0.00026655,
        9.8735e-05, 4.0109e-05, 1.6217e-05, 6.2983e-06, 2.3375e-06, 9.0748e-07, 3.2916e-07,
        1.0844e-07, 3.4757e-08, 9.2722e-09, 1.3388e-09, 6.2112e-11
    };

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        float response = 20 * std::log10(peakFilter.calcResponse(ThirdOctaveCenterFreqs[idx] / fs));
        CHECK(expected_response[idx] == doctest::Approx(response).epsilon(0.0001));
    }
}

TEST_CASE("Biquad filter response - highpass, 7kHz")
{
    float fs = 48000;
    float cf = 7000;
    float Q = 0.707;

    Biquad peakFilter = Biquad(bq_type_highpass, cf / fs, Q, 0.f);

    std::vector<double> expected_a = { 0.515275153074994, -1.030550306149988, 0.515275153074994 };
    std::vector<double> expected_b = { 1.0, -0.7799282923184014, 0.2811723199815747 };

    CHECK(expected_a[0] == doctest::Approx(peakFilter.a0).epsilon(0.0001));
    CHECK(expected_a[1] == doctest::Approx(peakFilter.a1).epsilon(0.0001));
    CHECK(expected_a[2] == doctest::Approx(peakFilter.a2).epsilon(0.0001));
    CHECK(expected_b[1] == doctest::Approx(peakFilter.b1).epsilon(0.0001));
    CHECK(expected_b[2] == doctest::Approx(peakFilter.b2).epsilon(0.0001));

    static const float ThirdOctaveCenterFreqs[] = { // [Hz]
        20, 25, 31.5, 40, 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250,
        1600, 2000, 2500, 3150, 4000, 5000, 6300, 8000, 10000, 12500, 16000, 20000
    };

    std::vector<float> expected_response = { // [dB]
        -103.041445231759, -99.1650391409789, -95.1502082443474, -91.0002156976682,
        -87.1237928993152, -83.1089347363916, -78.9588970794684, -75.0824075054057,
        -71.2058679072959, -66.9172226959754, -63.0404668982548, -59.1635123560601,
        -55.1477891450512, -50.9963277886279, -47.1177478716549, -43.0994150538801,
        -38.9437536198482, -35.0592263964154, -31.1708263595584, -26.8632625982863,
        -22.9640262498215, -19.0635583587951, -15.0403488962399, -10.9605434433479,
        -7.36836888774632, -4.17708359176956, -1.85463883467070, -0.685009647246013,
        -0.193823340112423, -0.0286571534402545, -0.00136961779660575
    };

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        float response = 20 * std::log10(peakFilter.calcResponse(ThirdOctaveCenterFreqs[idx] / fs));
        CHECK(expected_response[idx] == doctest::Approx(response).epsilon(0.0001));
    }
}

TEST_CASE("Biquad filter response - highshelf, 10Hz")
{
    float fs = 48000;
    float cf = 12000;
    float Q = 1;
    float gain = 6;

    Biquad peakFilter = Biquad(bq_type_highshelf, cf / fs, Q, gain);

    std::vector<double> expected_a = { 1.462384228965804, -0.5830111659899267, 0.2921998122779325 };
    std::vector<double> expected_b = { 1.0, -1.300707181133076e-16, 0.17157287525380988 };

    CHECK(expected_a[0] == doctest::Approx(peakFilter.a0).epsilon(0.0001));
    CHECK(expected_a[1] == doctest::Approx(peakFilter.a1).epsilon(0.0001));
    CHECK(expected_a[2] == doctest::Approx(peakFilter.a2).epsilon(0.0001));
    CHECK(expected_b[1] == doctest::Approx(peakFilter.b1).epsilon(0.0001));
    CHECK(expected_b[2] == doctest::Approx(peakFilter.b2).epsilon(0.0001));

    static const float ThirdOctaveCenterFreqs[] = { // [Hz]
        20, 25, 31.5, 40, 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250,
        1600, 2000, 2500, 3150, 4000, 5000, 6300, 8000, 10000, 12500, 16000, 20000
    };

    std::vector<float> expected_response = { // [dB]
        3.801e-11, 9.2801e-11, 2.339e-10, 6.0818e-10, 1.4848e-09, 3.7425e-09, 9.7312e-09,
        2.3758e-08, 5.8006e-08, 1.5572e-07, 3.802e-07, 9.2834e-07, 2.3403e-06, 6.0873e-06,
        1.4869e-05, 3.7509e-05, 9.7663e-05, 0.00023892, 0.00058515, 0.0015794, 0.0038864, 0.0096017,
        0.024651, 0.065892, 0.16642, 0.43469, 1.1331, 2.4734, 4.2889, 5.662, 5.9833
    };

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        float response = 20 * std::log10(peakFilter.calcResponse(ThirdOctaveCenterFreqs[idx] / fs));
        CHECK(expected_response[idx] == doctest::Approx(response).epsilon(0.0001));
    }
}

TEST_CASE("Convert headphone data parametric filters - 1 lowshelf")
{
    EAR::HeadphoneEQData headphoneData;
    headphoneData.specType = EAR::HeadphoneSpecType::parametricfilters;
    headphoneData.centerFreq = { 100 };
    headphoneData.q = { 1 };
    headphoneData.gain = { 4 };
    headphoneData.filterType = {
        EAR::HeadphoneFilterType::lowshelf,
    };

    float fs = 48000;
    float nTargetFreqs = 1024;

    std::vector<float> expected_response = { 1.584893, 1.583452, 1.562689, 1.487955, 1.360709,
        1.233912, 1.143201, 1.087532, 1.054867, 1.035573, 1.023866, 1.016527, 1.011772, 1.008596,
        1.006416, 1.004882, 1.003779, 1.002969, 1.002365, 1.001906, 1.001553, 1.001278, 1.001062,
        1.000889, 1.000750, 1.000637, 1.000544, 1.000468, 1.000405, 1.000352, 1.000307, 1.000269,
        1.000237, 1.000210, 1.000186, 1.000166, 1.000148, 1.000133, 1.000119, 1.000107, 1.000097,
        1.000088, 1.000080, 1.000073, 1.000066, 1.000060, 1.000055, 1.000051, 1.000047, 1.000043,
        1.000040, 1.000037, 1.000034, 1.000031, 1.000029, 1.000027, 1.000025, 1.000023, 1.000022,
        1.000020, 1.000019, 1.000018, 1.000017, 1.000016, 1.000015, 1.000014, 1.000013, 1.000012,
        1.000012, 1.000011, 1.000010, 1.000010, 1.000009, 1.000009, 1.000008, 1.000008, 1.000007,
        1.000007, 1.000007, 1.000006, 1.000006, 1.000006, 1.000005, 1.000005, 1.000005, 1.000005,
        1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000003, 1.000003, 1.000003,
        1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000002, 1.000002, 1.000002, 1.000002,
        1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000 };
    std::vector<float> gains;
    const auto [freqs, gains2] =
        EAR::HeadphoneEQ::getGainsFromBiquad(headphoneData, fs, nTargetFreqs);
    gains = gains2;

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        CHECK(expected_response[idx] == doctest::Approx(gains.at(idx)).epsilon(0.0001));
    }
}

TEST_CASE("Convert headphone data parametric filters - 1 lowshelf")
{
    EAR::HeadphoneEQData headphoneData;
    headphoneData.specType = EAR::HeadphoneSpecType::parametricfilters;
    headphoneData.centerFreq = { 100, 250 };
    headphoneData.q = { 1, 2 };
    headphoneData.gain = { 4, 3 };
    headphoneData.filterType = {
        EAR::HeadphoneFilterType::lowshelf,
        EAR::HeadphoneFilterType::peak,
    };

    float fs = 48000;
    float nTargetFreqs = 1024;

    std::vector<float> expected_response = { 1.584893, 1.585213, 1.569959, 1.504767, 1.391207,
        1.283850, 1.223020, 1.215423, 1.258415, 1.342733, 1.424145, 1.430183, 1.361672, 1.278370,
        1.211000, 1.162324, 1.127792, 1.102969, 1.084715, 1.070967, 1.060375, 1.052048, 1.045381,
        1.039959, 1.035486, 1.031750, 1.028595, 1.025906, 1.023592, 1.021586, 1.019834, 1.018294,
        1.016934, 1.015724, 1.014644, 1.013674, 1.012801, 1.012012, 1.011295, 1.010642, 1.010046,
        1.009500, 1.008998, 1.008536, 1.008109, 1.007715, 1.007349, 1.007009, 1.006692, 1.006397,
        1.006122, 1.005864, 1.005622, 1.005395, 1.005182, 1.004982, 1.004793, 1.004614, 1.004446,
        1.004287, 1.004136, 1.003993, 1.003857, 1.003729, 1.003606, 1.003490, 1.003379, 1.003274,
        1.003173, 1.003077, 1.002986, 1.002898, 1.002814, 1.002734, 1.002657, 1.002584, 1.002513,
        1.002445, 1.002380, 1.002318, 1.002258, 1.002200, 1.002144, 1.002091, 1.002039, 1.001990,
        1.001942, 1.001896, 1.001851, 1.001808, 1.001767, 1.001727, 1.001688, 1.001651, 1.001614,
        1.001579, 1.001545, 1.001512, 1.001481, 1.001450, 1.001420, 1.001391, 1.001363, 1.001335,
        1.001309, 1.001283, 1.001258, 1.001234, 1.001210, 1.001188, 1.001165, 1.001144, 1.001123,
        1.001102, 1.001082, 1.001063, 1.001044, 1.001025, 1.001007, 1.000990, 1.000973, 1.000956,
        1.000940, 1.000924, 1.000909, 1.000894, 1.000879, 1.000865, 1.000851, 1.000837, 1.000824,
        1.000811, 1.000798, 1.000786, 1.000774, 1.000762, 1.000750, 1.000739, 1.000728, 1.000717,
        1.000706, 1.000696, 1.000685, 1.000676, 1.000666, 1.000656, 1.000647, 1.000638, 1.000629,
        1.000620, 1.000611, 1.000603, 1.000595, 1.000587, 1.000579, 1.000571, 1.000563, 1.000556,
        1.000548, 1.000541, 1.000534, 1.000527, 1.000520, 1.000514, 1.000507, 1.000501, 1.000494,
        1.000488, 1.000482, 1.000476, 1.000470, 1.000464, 1.000459, 1.000453, 1.000448, 1.000442,
        1.000437, 1.000432, 1.000427, 1.000422, 1.000417, 1.000412, 1.000407, 1.000402, 1.000398,
        1.000393, 1.000389, 1.000384, 1.000380, 1.000376, 1.000371, 1.000367, 1.000363, 1.000359,
        1.000355, 1.000351, 1.000348, 1.000344, 1.000340, 1.000336, 1.000333, 1.000329, 1.000326,
        1.000322, 1.000319, 1.000316, 1.000312, 1.000309, 1.000306, 1.000303, 1.000300, 1.000297,
        1.000294, 1.000291, 1.000288, 1.000285, 1.000282, 1.000279, 1.000276, 1.000274, 1.000271,
        1.000268, 1.000266, 1.000263, 1.000261, 1.000258, 1.000256, 1.000253, 1.000251, 1.000248,
        1.000246, 1.000244, 1.000241, 1.000239, 1.000237, 1.000235, 1.000233, 1.000230, 1.000228,
        1.000226, 1.000224, 1.000222, 1.000220, 1.000218, 1.000216, 1.000214, 1.000212, 1.000210,
        1.000208, 1.000207, 1.000205, 1.000203, 1.000201, 1.000199, 1.000198, 1.000196, 1.000194,
        1.000193, 1.000191, 1.000189, 1.000188, 1.000186, 1.000185, 1.000183, 1.000181, 1.000180,
        1.000178, 1.000177, 1.000175, 1.000174, 1.000172, 1.000171, 1.000170, 1.000168, 1.000167,
        1.000165, 1.000164, 1.000163, 1.000161, 1.000160, 1.000159, 1.000158, 1.000156, 1.000155,
        1.000154, 1.000153, 1.000151, 1.000150, 1.000149, 1.000148, 1.000147, 1.000145, 1.000144,
        1.000143, 1.000142, 1.000141, 1.000140, 1.000139, 1.000138, 1.000137, 1.000136, 1.000135,
        1.000133, 1.000132, 1.000131, 1.000130, 1.000129, 1.000128, 1.000127, 1.000127, 1.000126,
        1.000125, 1.000124, 1.000123, 1.000122, 1.000121, 1.000120, 1.000119, 1.000118, 1.000117,
        1.000117, 1.000116, 1.000115, 1.000114, 1.000113, 1.000112, 1.000112, 1.000111, 1.000110,
        1.000109, 1.000108, 1.000108, 1.000107, 1.000106, 1.000105, 1.000104, 1.000104, 1.000103,
        1.000102, 1.000102, 1.000101, 1.000100, 1.000099, 1.000099, 1.000098, 1.000097, 1.000097,
        1.000096, 1.000095, 1.000095, 1.000094, 1.000093, 1.000093, 1.000092, 1.000091, 1.000091,
        1.000090, 1.000089, 1.000089, 1.000088, 1.000088, 1.000087, 1.000086, 1.000086, 1.000085,
        1.000085, 1.000084, 1.000084, 1.000083, 1.000082, 1.000082, 1.000081, 1.000081, 1.000080,
        1.000080, 1.000079, 1.000079, 1.000078, 1.000078, 1.000077, 1.000077, 1.000076, 1.000076,
        1.000075, 1.000075, 1.000074, 1.000074, 1.000073, 1.000073, 1.000072, 1.000072, 1.000071,
        1.000071, 1.000070, 1.000070, 1.000069, 1.000069, 1.000068, 1.000068, 1.000067, 1.000067,
        1.000067, 1.000066, 1.000066, 1.000065, 1.000065, 1.000064, 1.000064, 1.000064, 1.000063,
        1.000063, 1.000062, 1.000062, 1.000062, 1.000061, 1.000061, 1.000060, 1.000060, 1.000060,
        1.000059, 1.000059, 1.000059, 1.000058, 1.000058, 1.000057, 1.000057, 1.000057, 1.000056,
        1.000056, 1.000056, 1.000055, 1.000055, 1.000055, 1.000054, 1.000054, 1.000054, 1.000053,
        1.000053, 1.000053, 1.000052, 1.000052, 1.000052, 1.000051, 1.000051, 1.000051, 1.000050,
        1.000050, 1.000050, 1.000049, 1.000049, 1.000049, 1.000048, 1.000048, 1.000048, 1.000048,
        1.000047, 1.000047, 1.000047, 1.000046, 1.000046, 1.000046, 1.000046, 1.000045, 1.000045,
        1.000045, 1.000044, 1.000044, 1.000044, 1.000044, 1.000043, 1.000043, 1.000043, 1.000043,
        1.000042, 1.000042, 1.000042, 1.000041, 1.000041, 1.000041, 1.000041, 1.000040, 1.000040,
        1.000040, 1.000040, 1.000039, 1.000039, 1.000039, 1.000039, 1.000039, 1.000038, 1.000038,
        1.000038, 1.000038, 1.000037, 1.000037, 1.000037, 1.000037, 1.000036, 1.000036, 1.000036,
        1.000036, 1.000036, 1.000035, 1.000035, 1.000035, 1.000035, 1.000034, 1.000034, 1.000034,
        1.000034, 1.000034, 1.000033, 1.000033, 1.000033, 1.000033, 1.000033, 1.000032, 1.000032,
        1.000032, 1.000032, 1.000032, 1.000031, 1.000031, 1.000031, 1.000031, 1.000031, 1.000030,
        1.000030, 1.000030, 1.000030, 1.000030, 1.000030, 1.000029, 1.000029, 1.000029, 1.000029,
        1.000029, 1.000028, 1.000028, 1.000028, 1.000028, 1.000028, 1.000028, 1.000027, 1.000027,
        1.000027, 1.000027, 1.000027, 1.000027, 1.000026, 1.000026, 1.000026, 1.000026, 1.000026,
        1.000026, 1.000025, 1.000025, 1.000025, 1.000025, 1.000025, 1.000025, 1.000025, 1.000024,
        1.000024, 1.000024, 1.000024, 1.000024, 1.000024, 1.000024, 1.000023, 1.000023, 1.000023,
        1.000023, 1.000023, 1.000023, 1.000023, 1.000022, 1.000022, 1.000022, 1.000022, 1.000022,
        1.000022, 1.000022, 1.000021, 1.000021, 1.000021, 1.000021, 1.000021, 1.000021, 1.000021,
        1.000020, 1.000020, 1.000020, 1.000020, 1.000020, 1.000020, 1.000020, 1.000020, 1.000019,
        1.000019, 1.000019, 1.000019, 1.000019, 1.000019, 1.000019, 1.000019, 1.000018, 1.000018,
        1.000018, 1.000018, 1.000018, 1.000018, 1.000018, 1.000018, 1.000018, 1.000017, 1.000017,
        1.000017, 1.000017, 1.000017, 1.000017, 1.000017, 1.000017, 1.000017, 1.000016, 1.000016,
        1.000016, 1.000016, 1.000016, 1.000016, 1.000016, 1.000016, 1.000016, 1.000016, 1.000015,
        1.000015, 1.000015, 1.000015, 1.000015, 1.000015, 1.000015, 1.000015, 1.000015, 1.000015,
        1.000014, 1.000014, 1.000014, 1.000014, 1.000014, 1.000014, 1.000014, 1.000014, 1.000014,
        1.000014, 1.000013, 1.000013, 1.000013, 1.000013, 1.000013, 1.000013, 1.000013, 1.000013,
        1.000013, 1.000013, 1.000013, 1.000013, 1.000012, 1.000012, 1.000012, 1.000012, 1.000012,
        1.000012, 1.000012, 1.000012, 1.000012, 1.000012, 1.000012, 1.000012, 1.000011, 1.000011,
        1.000011, 1.000011, 1.000011, 1.000011, 1.000011, 1.000011, 1.000011, 1.000011, 1.000011,
        1.000011, 1.000010, 1.000010, 1.000010, 1.000010, 1.000010, 1.000010, 1.000010, 1.000010,
        1.000010, 1.000010, 1.000010, 1.000010, 1.000010, 1.000010, 1.000009, 1.000009, 1.000009,
        1.000009, 1.000009, 1.000009, 1.000009, 1.000009, 1.000009, 1.000009, 1.000009, 1.000009,
        1.000009, 1.000009, 1.000009, 1.000008, 1.000008, 1.000008, 1.000008, 1.000008, 1.000008,
        1.000008, 1.000008, 1.000008, 1.000008, 1.000008, 1.000008, 1.000008, 1.000008, 1.000008,
        1.000008, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007,
        1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007, 1.000007,
        1.000007, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006,
        1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006, 1.000006,
        1.000006, 1.000006, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005,
        1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005,
        1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000005, 1.000004, 1.000004, 1.000004,
        1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004,
        1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004, 1.000004,
        1.000004, 1.000004, 1.000004, 1.000004, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003,
        1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003,
        1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003,
        1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000003, 1.000002, 1.000002,
        1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002,
        1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002,
        1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002,
        1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002, 1.000002,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001, 1.000001,
        1.000001, 1.000001, 1.000001, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000, 1.000000,
        1.000000, 1.000000 };
    std::vector<float> gains;
    const auto [freqs, gains2] =
        EAR::HeadphoneEQ::getGainsFromBiquad(headphoneData, fs, nTargetFreqs);
    gains = gains2;

    for (int idx = 0; idx < expected_response.size(); ++idx) {
        CHECK(expected_response[idx] == doctest::Approx(gains.at(idx)).epsilon(0.0001));
    }
}
TEST_CASE("Convert headphone data parametric filters")
{
    EAR::HeadphoneEQData headphoneData;
    headphoneData.specType = EAR::HeadphoneSpecType::parametricfilters;
    headphoneData.centerFreq = { 100, 250, 500, 1000, 2000, 4000, 8000, 12000 };
    headphoneData.q = { 1, 2, 3, 4, 3, 2, 1, 1 };
    headphoneData.gain = { 4, 3, 2, 3, 4, 5, 6, 6 };
    headphoneData.filterType = {
        EAR::HeadphoneFilterType::lowshelf,
        EAR::HeadphoneFilterType::peak,
        EAR::HeadphoneFilterType::peak,
        EAR::HeadphoneFilterType::peak,
        EAR::HeadphoneFilterType::peak,
        EAR::HeadphoneFilterType::peak,
        EAR::HeadphoneFilterType::peak,
        EAR::HeadphoneFilterType::highshelf,
    };

    float fs = 48000;
    float nTargetFreqs = 1024;

    std::vector<float> expected_response = { 1.584893, 1.585402, 1.570717, 1.506424, 1.393984,
        1.287958, 1.228841, 1.223615, 1.270033, 1.359337, 1.447391, 1.460737, 1.399624, 1.324915,
        1.268913, 1.235977, 1.223515, 1.229566, 1.253322, 1.292032, 1.334523, 1.358601, 1.347890,
        1.310908, 1.267739, 1.230693, 1.203038, 1.184086, 1.172299, 1.166356, 1.165385, 1.168938,
        1.176938, 1.189634, 1.207574, 1.231569, 1.262605, 1.301592, 1.348813, 1.402871, 1.459228,
        1.509285, 1.542113, 1.549818, 1.532708, 1.498624, 1.457485, 1.416824, 1.380700, 1.350588,
        1.326537, 1.307984, 1.294197, 1.284469, 1.278191, 1.274874, 1.274133, 1.275677, 1.279290,
        1.284820, 1.292163, 1.301257, 1.312075, 1.324615, 1.338898, 1.354960, 1.372852, 1.392629,
        1.414346, 1.438054, 1.463782, 1.491533, 1.521267, 1.552884, 1.586206, 1.620956, 1.656742,
        1.693044, 1.729204, 1.764442, 1.797878, 1.828585, 1.855654, 1.878279, 1.895832, 1.907937,
        1.914503, 1.915724, 1.912050, 1.904122, 1.892699, 1.878585, 1.862565, 1.845360, 1.827599,
        1.809807, 1.792406, 1.775719, 1.759986, 1.745373, 1.731991, 1.719903, 1.709139, 1.699699,
        1.691570, 1.684720, 1.679114, 1.674708, 1.671456, 1.669313, 1.668232, 1.668169, 1.669083,
        1.670932, 1.673680, 1.677291, 1.681734, 1.686979, 1.692997, 1.699764, 1.707257, 1.715454,
        1.724334, 1.733880, 1.744073, 1.754898, 1.766337, 1.778376, 1.790999, 1.804191, 1.817937,
        1.832221, 1.847026, 1.862335, 1.878130, 1.894392, 1.911099, 1.928229, 1.945759, 1.963661,
        1.981908, 2.000469, 2.019311, 2.038399, 2.057695, 2.077159, 2.096747, 2.116414, 2.136112,
        2.155790, 2.175397, 2.194878, 2.214177, 2.233236, 2.251999, 2.270405, 2.288397, 2.305917,
        2.322908, 2.339315, 2.355086, 2.370170, 2.384521, 2.398096, 2.410859, 2.422774, 2.433816,
        2.443960, 2.453192, 2.461500, 2.468878, 2.475330, 2.480861, 2.485483, 2.489214, 2.492077,
        2.494097, 2.495307, 2.495739, 2.495431, 2.494422, 2.492754, 2.490470, 2.487613, 2.484227,
        2.480358, 2.476049, 2.471343, 2.466284, 2.460912, 2.455268, 2.449390, 2.443314, 2.437077,
        2.430709, 2.424244, 2.417709, 2.411133, 2.404539, 2.397953, 2.391394, 2.384884, 2.378440,
        2.372078, 2.365813, 2.359659, 2.353627, 2.347728, 2.341972, 2.336366, 2.330918, 2.325634,
        2.320519, 2.315578, 2.310814, 2.306230, 2.301828, 2.297611, 2.293578, 2.289731, 2.286069,
        2.282593, 2.279302, 2.276194, 2.273269, 2.270524, 2.267959, 2.265571, 2.263358, 2.261317,
        2.259446, 2.257743, 2.256204, 2.254827, 2.253609, 2.252547, 2.251638, 2.250878, 2.250266,
        2.249796, 2.249467, 2.249276, 2.249218, 2.249291, 2.249491, 2.249816, 2.250262, 2.250826,
        2.251505, 2.252296, 2.253195, 2.254200, 2.255307, 2.256514, 2.257817, 2.259214, 2.260701,
        2.262275, 2.263934, 2.265675, 2.267495, 2.269391, 2.271360, 2.273400, 2.275508, 2.277681,
        2.279916, 2.282211, 2.284564, 2.286971, 2.289430, 2.291940, 2.294496, 2.297097, 2.299741,
        2.302424, 2.305145, 2.307902, 2.310691, 2.313512, 2.316361, 2.319236, 2.322135, 2.325057,
        2.327999, 2.330958, 2.333933, 2.336923, 2.339924, 2.342935, 2.345954, 2.348979, 2.352008,
        2.355040, 2.358073, 2.361104, 2.364133, 2.367157, 2.370175, 2.373186, 2.376186, 2.379177,
        2.382154, 2.385118, 2.388066, 2.390998, 2.393912, 2.396806, 2.399679, 2.402530, 2.405358,
        2.408162, 2.410939, 2.413690, 2.416413, 2.419108, 2.421772, 2.424405, 2.427007, 2.429576,
        2.432111, 2.434612, 2.437077, 2.439507, 2.441900, 2.444255, 2.446573, 2.448852, 2.451091,
        2.453291, 2.455451, 2.457570, 2.459648, 2.461685, 2.463679, 2.465632, 2.467542, 2.469410,
        2.471234, 2.473016, 2.474754, 2.476449, 2.478100, 2.479708, 2.481272, 2.482793, 2.484270,
        2.485704, 2.487095, 2.488442, 2.489746, 2.491008, 2.492226, 2.493402, 2.494536, 2.495628,
        2.496678, 2.497687, 2.498654, 2.499581, 2.500467, 2.501314, 2.502120, 2.502888, 2.503617,
        2.504307, 2.504960, 2.505576, 2.506154, 2.506696, 2.507203, 2.507673, 2.508110, 2.508512,
        2.508880, 2.509215, 2.509518, 2.509788, 2.510027, 2.510236, 2.510414, 2.510562, 2.510682,
        2.510772, 2.510836, 2.510871, 2.510880, 2.510864, 2.510821, 2.510754, 2.510663, 2.510547,
        2.510409, 2.510249, 2.510066, 2.509862, 2.509638, 2.509393, 2.509129, 2.508845, 2.508544,
        2.508224, 2.507887, 2.507534, 2.507164, 2.506778, 2.506377, 2.505961, 2.505532, 2.505088,
        2.504632, 2.504162, 2.503681, 2.503187, 2.502682, 2.502167, 2.501641, 2.501105, 2.500559,
        2.500004, 2.499441, 2.498869, 2.498289, 2.497701, 2.497107, 2.496505, 2.495897, 2.495282,
        2.494662, 2.494036, 2.493405, 2.492769, 2.492128, 2.491483, 2.490833, 2.490180, 2.489523,
        2.488863, 2.488200, 2.487534, 2.486865, 2.486194, 2.485520, 2.484845, 2.484167, 2.483488,
        2.482807, 2.482125, 2.481441, 2.480757, 2.480072, 2.479385, 2.478698, 2.478011, 2.477323,
        2.476634, 2.475946, 2.475257, 2.474568, 2.473879, 2.473190, 2.472501, 2.471812, 2.471124,
        2.470435, 2.469747, 2.469060, 2.468372, 2.467685, 2.466999, 2.466312, 2.465627, 2.464941,
        2.464256, 2.463572, 2.462887, 2.462204, 2.461520, 2.460837, 2.460154, 2.459472, 2.458789,
        2.458107, 2.457425, 2.456744, 2.456062, 2.455381, 2.454699, 2.454017, 2.453336, 2.452654,
        2.451972, 2.451289, 2.450607, 2.449923, 2.449240, 2.448556, 2.447871, 2.447185, 2.446499,
        2.445812, 2.445124, 2.444435, 2.443745, 2.443054, 2.442362, 2.441668, 2.440973, 2.440277,
        2.439579, 2.438879, 2.438178, 2.437475, 2.436770, 2.436063, 2.435354, 2.434643, 2.433930,
        2.433215, 2.432497, 2.431777, 2.431054, 2.430329, 2.429601, 2.428870, 2.428137, 2.427400,
        2.426661, 2.425919, 2.425173, 2.424424, 2.423672, 2.422917, 2.422158, 2.421396, 2.420630,
        2.419861, 2.419088, 2.418311, 2.417530, 2.416746, 2.415957, 2.415165, 2.414368, 2.413568,
        2.412763, 2.411954, 2.411141, 2.410323, 2.409501, 2.408675, 2.407844, 2.407009, 2.406169,
        2.405324, 2.404475, 2.403621, 2.402763, 2.401900, 2.401032, 2.400159, 2.399281, 2.398398,
        2.397511, 2.396618, 2.395721, 2.394818, 2.393911, 2.392998, 2.392081, 2.391158, 2.390231,
        2.389298, 2.388360, 2.387417, 2.386469, 2.385516, 2.384558, 2.383594, 2.382626, 2.381652,
        2.380673, 2.379689, 2.378700, 2.377705, 2.376706, 2.375701, 2.374692, 2.373677, 2.372657,
        2.371632, 2.370602, 2.369567, 2.368527, 2.367482, 2.366432, 2.365377, 2.364317, 2.363252,
        2.362182, 2.361108, 2.360028, 2.358944, 2.357855, 2.356761, 2.355662, 2.354559, 2.353451,
        2.352338, 2.351221, 2.350099, 2.348973, 2.347842, 2.346707, 2.345567, 2.344424, 2.343275,
        2.342123, 2.340966, 2.339805, 2.338640, 2.337471, 2.336298, 2.335121, 2.333940, 2.332755,
        2.331566, 2.330373, 2.329177, 2.327977, 2.326773, 2.325565, 2.324355, 2.323140, 2.321922,
        2.320701, 2.319477, 2.318249, 2.317018, 2.315784, 2.314547, 2.313307, 2.312064, 2.310817,
        2.309568, 2.308317, 2.307062, 2.305805, 2.304545, 2.303283, 2.302018, 2.300751, 2.299481,
        2.298209, 2.296934, 2.295658, 2.294379, 2.293098, 2.291816, 2.290531, 2.289244, 2.287956,
        2.286666, 2.285374, 2.284080, 2.282785, 2.281488, 2.280190, 2.278890, 2.277589, 2.276287,
        2.274983, 2.273679, 2.272373, 2.271066, 2.269758, 2.268449, 2.267140, 2.265829, 2.264518,
        2.263206, 2.261894, 2.260581, 2.259267, 2.257953, 2.256639, 2.255324, 2.254009, 2.252694,
        2.251378, 2.250063, 2.248747, 2.247432, 2.246116, 2.244801, 2.243486, 2.242171, 2.240856,
        2.239542, 2.238228, 2.236914, 2.235601, 2.234289, 2.232977, 2.231666, 2.230356, 2.229046,
        2.227737, 2.226429, 2.225122, 2.223816, 2.222511, 2.221207, 2.219905, 2.218603, 2.217303,
        2.216004, 2.214706, 2.213409, 2.212114, 2.210821, 2.209529, 2.208239, 2.206950, 2.205663,
        2.204377, 2.203093, 2.201811, 2.200531, 2.199253, 2.197977, 2.196702, 2.195430, 2.194160,
        2.192891, 2.191625, 2.190361, 2.189100, 2.187840, 2.186583, 2.185328, 2.184076, 2.182826,
        2.181578, 2.180333, 2.179090, 2.177850, 2.176612, 2.175378, 2.174145, 2.172916, 2.171689,
        2.170465, 2.169244, 2.168026, 2.166810, 2.165597, 2.164388, 2.163181, 2.161977, 2.160777,
        2.159579, 2.158385, 2.157193, 2.156005, 2.154820, 2.153638, 2.152459, 2.151284, 2.150112,
        2.148943, 2.147778, 2.146616, 2.145457, 2.144302, 2.143151, 2.142002, 2.140858, 2.139717,
        2.138579, 2.137445, 2.136315, 2.135188, 2.134065, 2.132945, 2.131830, 2.130718, 2.129609,
        2.128505, 2.127404, 2.126307, 2.125214, 2.124125, 2.123040, 2.121958, 2.120881, 2.119807,
        2.118737, 2.117672, 2.116610, 2.115552, 2.114499, 2.113449, 2.112403, 2.111362, 2.110325,
        2.109291, 2.108262, 2.107237, 2.106216, 2.105200, 2.104187, 2.103179, 2.102175, 2.101175,
        2.100179, 2.099188, 2.098201, 2.097218, 2.096240, 2.095266, 2.094296, 2.093330, 2.092369,
        2.091412, 2.090460, 2.089512, 2.088569, 2.087629, 2.086695, 2.085764, 2.084839, 2.083917,
        2.083000, 2.082088, 2.081180, 2.080276, 2.079378, 2.078483, 2.077593, 2.076708, 2.075827,
        2.074951, 2.074079, 2.073212, 2.072349, 2.071492, 2.070638, 2.069789, 2.068945, 2.068106,
        2.067271, 2.066441, 2.065615, 2.064794, 2.063978, 2.063166, 2.062359, 2.061557, 2.060759,
        2.059966, 2.059178, 2.058394, 2.057616, 2.056841, 2.056072, 2.055307, 2.054547, 2.053792,
        2.053042, 2.052296, 2.051555, 2.050818, 2.050087, 2.049360, 2.048638, 2.047921, 2.047208,
        2.046501, 2.045798, 2.045100, 2.044406, 2.043718, 2.043034, 2.042355, 2.041681, 2.041012,
        2.040347, 2.039687, 2.039032, 2.038382, 2.037737, 2.037096, 2.036461, 2.035830, 2.035204,
        2.034583, 2.033966, 2.033355, 2.032748, 2.032146, 2.031549, 2.030957, 2.030370, 2.029787,
        2.029210, 2.028637, 2.028069, 2.027506, 2.026947, 2.026394, 2.025846, 2.025302, 2.024763,
        2.024229, 2.023700, 2.023176, 2.022657, 2.022142, 2.021633, 2.021128, 2.020628, 2.020133,
        2.019643, 2.019157, 2.018677, 2.018202, 2.017731, 2.017265, 2.016804, 2.016348, 2.015897,
        2.015451, 2.015009, 2.014573, 2.014141, 2.013714, 2.013293, 2.012876, 2.012463, 2.012056,
        2.011654, 2.011256, 2.010864, 2.010476, 2.010093, 2.009715, 2.009342, 2.008974, 2.008610,
        2.008252, 2.007898, 2.007550, 2.007206, 2.006867, 2.006533, 2.006204, 2.005880, 2.005560,
        2.005246, 2.004936, 2.004631, 2.004331, 2.004036, 2.003746, 2.003461, 2.003181, 2.002905,
        2.002635, 2.002369, 2.002108, 2.001852, 2.001601, 2.001355, 2.001114, 2.000877, 2.000646,
        2.000419, 2.000197, 1.999980, 1.999768, 1.999561, 1.999359, 1.999161, 1.998969, 1.998781,
        1.998598, 1.998420, 1.998247, 1.998079, 1.997916, 1.997757, 1.997604, 1.997455, 1.997311,
        1.997173, 1.997039, 1.996909, 1.996785, 1.996666, 1.996551, 1.996442, 1.996337, 1.996237,
        1.996142, 1.996052, 1.995966, 1.995886, 1.995811, 1.995740, 1.995674, 1.995613, 1.995557,
        1.995506, 1.995460, 1.995418, 1.995382, 1.995350, 1.995323, 1.995301, 1.995284, 1.995272,
        1.995265, 1.995262 };
    std::vector<float> gains;
    const auto [freqs, gains2] =
        EAR::HeadphoneEQ::getGainsFromBiquad(headphoneData, fs, nTargetFreqs);
    gains = gains2;
    for (int idx = 0; idx < expected_response.size(); ++idx) {
        CHECK(expected_response[idx] == doctest::Approx(gains.at(idx)).epsilon(0.0001));
    }
}
