/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2024.
***************************************************************************/

#ifndef DATA_FILE
#define DATA_FILE

#include <cstddef>
#include <memory>
#include <vector>

#include <fstream>
#include <iostream>
#include <string>

#include <EAR/ComplexBuffer.h>
#include <EAR/RealBuffer.h>

class DataFile
{
private:
    DataFile(){};

public:
    static void write(const std::string& filename, const EAR::RealBuffer& y, const double fs)
    {
        // buffer stored as a single channel
        const long long M = 1;

        // write header: samplerate (double) and number of channels (int64)
        std::ofstream oStrm(filename.c_str(), std::ios::binary);
        oStrm.write(reinterpret_cast<const char*>(&fs), sizeof(double));
        oStrm.write(reinterpret_cast<const char*>(&M), sizeof(long long));

        // append interlaced data
        for (int n = 0; n < y.length(); n++) {
            double sample = y[n];
            oStrm.write(reinterpret_cast<const char*>(&sample), sizeof(double));
        }

        // close file
        oStrm.close();
    }

    static void write(const std::string& filename, const EAR::ComplexBuffer& y, const double fs)
    {
        // real part in first channel, imag part in second channel
        const long long M = 2;

        // write header: samplerate (double) and number of channels (int64)
        std::ofstream oStrm(filename.c_str(), std::ios::binary);
        oStrm.write(reinterpret_cast<const char*>(&fs), sizeof(double));
        oStrm.write(reinterpret_cast<const char*>(&M), sizeof(long long));

        // append interlaced data
        for (int n = 0; n < y.length(); n++) {
            double re = y[n].real();
            oStrm.write(reinterpret_cast<const char*>(&re), sizeof(double));
            double im = y[n].imag();
            oStrm.write(reinterpret_cast<const char*>(&im), sizeof(double));
        }

        // close file
        oStrm.close();
    }
};

#endif
