/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <doctest/doctest.h>

#include <EAR/FIRFilter.h>
#include <EAR/RealFrame.h>

#include <EAR/StereoFIRFilter.h>

TEST_CASE("FIR filter test")
{

    SUBCASE("Length 4")
    {
        const float in1[] = { 1, 1, 1, 1 };
        const float in2[] = { 0, 0, 0, 0 };
        const float ref[] = { 1, 2, 3, 4, 3, 2, 1, 0 };

        EAR::FIRFilter conv(4, 4);
        float out[8];

        conv.setFilter(in1);
        conv.process(in1, out);
        conv.process(in2, out + 4);

        for (int i = 0; i < 8; i++) {
            CHECK(out[i] == doctest::Approx(ref[i]));
        }
    }

    SUBCASE("Length 6 and 8")
    {
        const float in1[] = { 1, 2, 3, 4, 5, 6, 7, 8 };
        const float in2[] = { -2, -1, 0, 1, 2, 3 };
        const float zeros[] = { 0, 0, 0, 0, 0, 0 };
        const float ref[] = { -2, -5, -8, -10, -10, -7, -4, -1, 20, 34, 40, 37, 24, 0, 0, 0, 0, 0,
            0 };

        EAR::FIRFilter conv(6, 8);
        float out[18];

        conv.setFilter(in1);
        conv.process(in2, out);
        conv.process(zeros, out + 6);
        conv.process(zeros, out + 12);

        for (int i = 0; i < 18; i++) {
            CHECK(out[i] == doctest::Approx(ref[i]));
        }
    }
}

TEST_CASE("Stereo FIR filter test")
{

    SUBCASE("Length 4")
    {
        const float in1[] = { 1, 1, 1, 1 };
        const float in2[] = { 0, 0, 0, 0 };
        const float ref[] = { 1, 2, 3, 4, 3, 2, 1, 0 };

        EAR::RealFrame filter(2, 4);
        filter.ch(0).write(in1, 0, 4);
        filter.ch(1).write(in1, 0, 4);
        filter.ch(1).mul(0.5);

        EAR::RealFrame output(2, 8);
        auto out = output.data();

        EAR::StereoFIRFilter conv(4, 4);

        conv.setFilter(filter.data().get());
        conv.process(in1, out.get());

        out[0] += 4;
        out[1] += 4;

        conv.process(in2, out.get());

        for (int i = 0; i < 8; i++) {
            CHECK(output[0][i] == doctest::Approx(ref[i]));
            CHECK(output[1][i] == doctest::Approx(0.5 * ref[i]));
        }
    }
}

TEST_CASE("FIR filter latency test")
{

    SUBCASE("Length 4")
    {
        const float in1[] = { 1, 0, 0, 0 };
        const float in2[] = { 0, 0, 0, 0 };
        const float in3[] = { 0, 0, 0, 0 };
        const float in4[] = { 0, 0, 0, 0 };
        const float b[] = { 1, 0, 0, 0, 0, 0, 0, 0, 0 };
        const float ref[] = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

        EAR::FIRFilter conv(4, 8);
        float out[16];

        conv.setFilter(b);
        conv.process(in1, out);
        conv.process(in2, out + 4);
        conv.process(in3, out + 8);
        conv.process(in4, out + 12);

        for (int i = 0; i < 16; i++) {
            CHECK(out[i] == doctest::Approx(ref[i]));
        }
    }
}
