/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2024.
***************************************************************************/

#include <doctest/doctest.h>

#include "DataFile.h"
#include <EAR/FIR2MinphaseFilterDesigner.h>
#include <EAR/FIRFilter.h>
#include <EAR/PchipMinphaseFilterDesigner.h>
#include <EAR/RealFrame.h>

#define WRITE_TO_FILE false

TEST_CASE("PchipMinphaseFilterDesigner test")
{

    SUBCASE("Low pass")
    {
        const float fs = 48000;
        const float fc = 250;
        const size_t Ntaps = 512;
        const float f[] = { 0.0f, fc, fc, 0.5f * fs }; // frequencies
        const float k[] = { 1.0f, 1.0f, 0.0f, 0.0f }; // gains
        const int Nfreqs = 4;

        std::unique_ptr<EAR::RealBuffer> b = std::make_unique<EAR::RealBuffer>(Ntaps);

        std::unique_ptr<EAR::FilterDesigner> fdsgn =
            std::make_unique<EAR::PchipMinphaseFilterDesigner>(f, Nfreqs, Ntaps, fs);

        fdsgn->generate(k, *b);

        EAR::RealBuffer frq = EAR::RealBuffer(Nfreqs);
        EAR::RealBuffer gains = EAR::RealBuffer(Nfreqs);
        EAR::RealBuffer bM = EAR::RealBuffer(Ntaps);
        frq.write(f, 0, Nfreqs);
        gains.write(k, 0, Nfreqs);

        EAR::FIR2MinphaseFilterDesigner fdsgnM = EAR::FIR2MinphaseFilterDesigner(Ntaps, fs);
        fdsgnM.design(frq, gains, bM);

#ifdef WRITE_TO_FILE
        DataFile::write("C:\\Trash\\pchipfir_lp.dat", *b, fs);
        DataFile::write("C:\\Trash\\fir2design_lp.dat", bM, fs);
#endif
    }

    SUBCASE("High pass")
    {
        const float fs = 48000;
        const float fc = 250;
        const size_t Ntaps = 512;
        const float f[] = { 0.0f, fc, fc, 0.5f * fs }; // frequencies
        const float k[] = { 0.0f, 0.0f, 1.0f, 1.0f }; // gains
        const int Nfreqs = 4;

        std::unique_ptr<EAR::RealBuffer> b = std::make_unique<EAR::RealBuffer>(Ntaps);

        std::unique_ptr<EAR::FilterDesigner> fdsgn =
            std::make_unique<EAR::PchipMinphaseFilterDesigner>(f, Nfreqs, Ntaps, fs);

        fdsgn->generate(k, *b);

        EAR::RealBuffer frq = EAR::RealBuffer(Nfreqs);
        EAR::RealBuffer gains = EAR::RealBuffer(Nfreqs);
        EAR::RealBuffer bM = EAR::RealBuffer(Ntaps);
        frq.write(f, 0, Nfreqs);
        gains.write(k, 0, Nfreqs);

        EAR::FIR2MinphaseFilterDesigner fdsgnM = EAR::FIR2MinphaseFilterDesigner(Ntaps, fs);
        fdsgnM.design(frq, gains, bM);

#ifdef WRITE_TO_FILE
        DataFile::write("C:\\Trash\\pchipfir_hp.dat", *b, fs);
        DataFile::write("C:\\Trash\\fir2design_hp.dat", bM, fs);
#endif
    }
}
