/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2023.
***************************************************************************/

#include <EAR/Box.h>
#include <EAR/Mesh.h>
#include <EAR/Portal.h>
#include <EAR/PortalReverbExtent.h>
#include <MPEGI/Bitstream/PortalPayload.h>
#include <MPEGI/Bitstream/ReverbPayload.h>

#include <MPEGI/Renderer/RenderItem.h>
#include <MPEGI/Renderer/RenderItemStore.h>
#include <MPEGI/Renderer/RenderStage.h>
#include <MPEGI/Stream/StreamManager.h>

#include <EAR/RealFrame.h>

#include <doctest/doctest.h>
#include <memory>
#include <mpegcore/datatypes_reverb.hpp>
#include <string>

namespace MPEGI {

static const unsigned int DEFAULT_AE_ID = 99;
static const std::string DEFAULT_AE_ID_NAME{ "default" };

class DummyPortalPayload : public PortalPayload
{
public:
    DummyPortalPayload() = default;
    DummyPortalPayload(const std::string& listenerAe, const std::string& itemAe);
    DummyPortalPayload& operator=(const DummyPortalPayload&) = delete;
    DummyPortalPayload(const DummyPortalPayload& other) = default;
    virtual ~DummyPortalPayload() {}
    void addPortal(std::unique_ptr<Portal>&& portal) { portals.push_back(std::move(portal)); };
};

class DummySceneState : public SceneState
{
public:
    DummySceneState(bool hasDefaultAe);
    DummySceneState& operator=(const DummySceneState&) = delete;
    DummySceneState(const DummySceneState&) = delete;
    virtual ~DummySceneState() {}
    void addAcousticEnvironment(std::optional<std::string> aeId, EAR::Geometry* enclosure);
    void createBox2(const EIFPayload::BoxDesc& box);
    void createPortal2(MPEGI::Portal* portal);
    void createMesh2(const EIFPayload::MeshDesc& mesh);

private:
    std::vector<std::unique_ptr<ReverbEnv>> dummyReverbEnvs;
};

} // namespace MPEGI