/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <doctest/doctest.h>

#include <EAR/BaseTypes.h>
#include <EAR/Variant.h>

#include <string>
#include <vector>

TEST_CASE("Variant")
{

    SUBCASE("serialize bool")
    {
        EAR::Variant testVar(true);
        std::string testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        CHECK(static_cast<bool>(testVar) == static_cast<bool>(resVar));
    }

    SUBCASE("serialize int")
    {
        EAR::Variant testVar(123);
        std::string testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        CHECK(static_cast<int>(testVar) == static_cast<int>(resVar));
    }

    SUBCASE("serialize float")
    {
        EAR::Variant testVar(-234.567f);
        std::string testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        CHECK(static_cast<float>(testVar) == static_cast<float>(resVar));
    }

    SUBCASE("serialize EAR::Euler")
    {
        EAR::Variant testVar(EAR::Euler(123.23, 32.43, -123.24));
        std::string testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        CHECK(static_cast<EAR::Euler>(testVar) == static_cast<EAR::Euler>(resVar));
    }

    SUBCASE("serialize EAR::Vector3")
    {
        EAR::Variant testVar(EAR::Vector3(27, 0.43, -1.24));
        std::string testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        CHECK(static_cast<EAR::Vector3>(testVar) == static_cast<EAR::Vector3>(resVar));
    }

    SUBCASE("serialize std::string")
    {
        EAR::Variant testVar(
            "I'm a particularly nasty (string), because there are a few special characters @:\"");
        std::string testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        CHECK(static_cast<const std::string&>(testVar) == static_cast<const std::string&>(resVar));
    }

    SUBCASE("serialize empty std::vector<float>")
    {
        EAR::Variant testVar{ std::vector<float>() };
        auto testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        auto const& vec = static_cast<std::vector<float> const&>(resVar);
        CHECK(vec.size() == 0);
    }

    SUBCASE("serialize std::vector<float> with a single value")
    {
        EAR::Variant testVar{ std::vector<float>{ -3.14F } };
        auto testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        auto const& vec = static_cast<std::vector<float> const&>(resVar);
        CHECK(vec.size() == 1);
        CHECK(vec[0] == -3.14F);
    }

    SUBCASE("serialize std::vector<float>")
    {
        EAR::Variant testVar{ std::vector<float>{ 1, 2, 3.4F, -4.19F, -5.01F, 10 } };
        auto testStr = testVar.serialize();

        EAR::Variant resVar;
        resVar.deserialize(testStr);

        auto const& vec = static_cast<std::vector<float> const&>(resVar);
        CHECK(vec.size() == 6);
        CHECK(vec[0] == 1.F);
        CHECK(vec[1] == 2.F);
        CHECK(vec[2] == 3.4F);
        CHECK(vec[3] == -4.19F);
        CHECK(vec[4] == -5.01F);
        CHECK(vec[5] == 10.F);
    }
}
