/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <EAR/RealBuffer.h>
#include <EAR/SofaReader.h>

#include <iostream>

using namespace EAR;
using namespace std;

int main(int argc, char* argv[])
{
    SofaReader<float, RealBuffer> sofa(
        "C:/code/ear/test/files/FABIAN_HRIR_measured_HATO_0_48k_DFE.sofa");

    RealBuffer* cache = sofa.getCacheBuffer();

    cout << "/*" << endl;
    cout << " * This dataset is taken from the FABIAN head-related transfer function database. "
         << endl;
    cout << " * History: Resampled at 48kHz; Diffuse-field equalized using the filter provided in "
            "FABIAN_CTF_measured_inverted_smoothed.sofa; Processed by Ericsson;"
         << endl;
    cout << "* License: Creative Commons (CC-BY) Visit http://creativecommons.org/licenses/by/4.0/ "
            "for licence details."
         << endl;
    cout << "* Organization: Audio Communication Group, TU Berlin, Germany (www.ak.tu-berlin.de)"
         << endl;
    cout << "* Origin : HRIRs were acoustically measured in an anechoic chamber using swept sines,"
            "and spectral deconvolution."
         << endl;
    cout
        << "* References : F.Brinkmann et al.(2017) : "
           "\"A High Resolution and Full-Spherical Head-Related Transfer Function Database for \""
           "\"Different Head-Above-Torso Orientations.\" J.Audio Eng.Soc.65(10) : 841 - 848. DOI : "
           "10.17743/ jaes .2017.0033"
        << endl;
    cout << " * Dataset properties: 11950 measurements, uniformly spaced on a sphere with radius "
            "of 1.7 meters."
         << endl;
    cout << "* Title: FABIAN HRIRs. Head-above-torso-orientation (HATO) = 0 degree" << endl;
    cout << " */" << endl;
    cout << endl;
    cout << "#ifndef EAR_FABIAN_FIXDEF" << endl;
    cout << "#define EAR_FABIAN_FIXDEF" << endl;
    cout << endl;
    cout << "namespace EAR {" << endl;
    cout << endl << endl;

    const int hrirCount = 11950;
    const int hrirSize = 256;
    cout << "const float FABIAN_DIFFEQ_HRIRS[" << hrirCount << "][2][" << hrirSize << "] = {"
         << endl;

    for (int i = 0; i < hrirCount; i++) {
        cout << "{";

        // left ear
        cout << "{";
        cout << cache->at(i * 2 * hrirSize);
        for (int k = 1; k < hrirSize; k++) {
            cout << ", " << cache->at(i * 2 * hrirSize + k);
        }
        // right ear
        cout << "}, {";
        cout << cache->at(i * 2 * hrirSize + hrirSize);
        for (int k = 1; k < hrirSize; k++) {
            cout << ", " << cache->at(i * 2 * hrirSize + hrirSize + k);
        }
        cout << "}";

        cout << "}";
        if (i != hrirCount - 1) {
            cout << ", ";
        }
    }

    cout << endl;
    cout << "};" << endl << endl;

    cout << "const float FABIAN_DIFFEQ_POLAR_COORDS[" << hrirCount << "][3] = {" << endl;

    const float* directions = sofa.getDirectionsBuffer();
    for (int i = 0; i < hrirCount; i++) {
        cout << "{" << directions[i * 3] << ", " << directions[i * 3 + 1] << ", "
             << directions[i * 3 + 2] << "}";
        if (i != hrirCount - 1) {
            cout << ", ";
        }
        cout << endl; //REMOVE ME!
    }

    cout << endl;
    cout << "};" << endl << endl;

    cout << "const unsigned long FABIAN_DIFFEQ_N = " << hrirSize << ";" << endl;
    cout << "const unsigned long FABIAN_DIFFEQ_M = " << hrirCount << ";" << endl;
    cout << "const float FABIAN_DIFFEQ_FS = 48000.f;" << endl;
    cout << endl;
    cout << "}" << endl;
    cout << endl;
    cout << "#endif /* EAR_FABIAN_FIXDEF */" << endl;
}
