#pragma once
//--------------------------------------------------------------------------------------------------
#include <vector>
//--------------------------------------------------------------------------------------------------
template<typename T>
struct accumulator
{
    using value_type = T;
    using const_iterator = typename std::vector<T>::const_iterator;

    const_iterator begin() const { return std::cbegin(_accu); }
    const_iterator end() const { return std::cend(_accu); }

    accumulator(std::size_t capacity)
        : _accu(capacity, T { 0 })
    {
    }

    ~accumulator() = default;
    accumulator(accumulator const&) = delete;
    accumulator& operator=(accumulator const&) = delete;

    void operator()(T value)
    {
        _accu[_pos++] = value;
        _pos %= get_window_size();
    }

    std::size_t get_window_size() const { return _accu.size(); }

protected:
    std::vector<T> _accu;
    std::size_t _pos { 0 };
};