#pragma once
//--------------------------------------------------------------------------------------------------
#include <chrono>
//--------------------------------------------------------------------------------------------------
template<
    typename T,
    typename Callback
    >
struct chrono_timer
{
    chrono_timer(Callback &callback)
        : _begin(std::chrono::high_resolution_clock::now())
        , _callback(callback)
    {
    }

    virtual ~chrono_timer()
    {
        auto const duration =
            std::chrono::duration_cast<T>(std::chrono::high_resolution_clock::now() - _begin);

        _callback(duration);
    }

    chrono_timer(chrono_timer const&) = delete;
    chrono_timer& operator=(chrono_timer const&) = delete;

    chrono_timer& operator=(chrono_timer&&) = delete;
    chrono_timer(chrono_timer &&other)
        : _begin(std::move(other._begin))
        , _callback(other._callback)
    {
    }

protected:
    std::chrono::time_point<std::chrono::high_resolution_clock> _begin;
    Callback &_callback;
};

//--------------------------------------------------------------------------------------------------
