/***************************************************************************
This software module was originally developed by

Dolby International AB

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Dolby International AB retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/
#pragma once

#include <algorithm>
#include <bitset>
#include <fstream>

//--------------------------------------------------------------------------------------------------
namespace MPEGI {
/// Data structures and classes for bit stream writing

class binaryStream
{
public:
    binaryStream(std::string out_file_path)
        : bitpos(7)
        , bits(0)
    {
        outstream.open(out_file_path, std::ios::binary);
    }

    ~binaryStream() { outstream.close(); }

    bool good() { return outstream.good(); }

    void writeBitsUInt(unsigned int n_bits, unsigned int value)
    {
        for (auto i = 0; i < n_bits; i++) {
            bits[bitpos] = value >> (n_bits - 1 - i) & 1;

            if (bitpos == 0) {
                char byte = (char)bits.to_ulong();
                outstream.write(&byte, 1);
                bitpos = 7;
            } else {
                bitpos--;
            }
        }
    }

    void writeEscapedUInt(std::size_t l1, std::size_t l2, std::size_t l3, unsigned int value)
    {
        unsigned int tmp;

        tmp = std::min(value, ((1U << l1) - 1));
        /* value */
        writeBitsUInt(l1, tmp);
        value -= tmp;

        if (tmp == (1U << l1) - 1) {
            tmp = std::min(value, ((1U << l2) - 1));
            /* valueAdd */
            writeBitsUInt(l2, tmp);
            value -= tmp;

            if (tmp == (1U << l2) - 1) {
                /* valueAdd */
                writeBitsUInt(l3, value);
            }
        }
    }

    void flush()
    {
        if (bitpos < 7) {
            writeBitsUInt(7 - bitpos, 0);
        }
    }

private:
    std::ofstream outstream;
    unsigned int bitpos;
    std::bitset<8> bits;
};

//--------------------------------------------------------------------------------------------------
} // namespace MPEGI
//--------------------------------------------------------------------------------------------------