/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DIRECTIVITY
#define MPEGI_DIRECTIVITY

#include <EAR/DirectivityModel.h>

#include <MPEGI/DirectivityCoding.h>

#include <vector>

namespace MPEGI {

//! Specialization of EAR::DirectivityModel for compressed directivities in the MPEG-I bitstream
class CompressedDirectivity : public EAR::DirectivityModel
{
public:
    //! Constructor
    /*!
     * \param directivityData     raw pointer to the compressed directivity blob (decoder does not guarantee const access)
     * \param length              length of the data in directivityData in bytes
     */
    CompressedDirectivity(std::uint8_t* directivityData, std::size_t length);

    //! Destructor
    ~CompressedDirectivity(){};

    //! Inherited from EAR::DirectivityModel
    float get(float frequency, const EAR::Polar& coordinates, bool shouldInterpolateFreq = true,
        bool shouldInterpolateSpace = true) override;

    const DirectivityCoverList& GetDirectivityCoverList() const override { return coverList; }

private:
    DirectivityCoverList coverList;
    std::vector<float> frequencies;
};

} // namespace MPEGI

#endif // MPEGI_DIRECTIVITY
