/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_EIF_PAYLOAD_HANDLER
#define MPEGI_EIF_PAYLOAD_HANDLER

#include <MPEGI/Bitstream/Bitstream.h>
#include <MPEGI/Bitstream/EIFPayload.h>

#include <memory>
#include <string>

namespace MPEGI {

// TODO: Move out of includes

class EIFPayloadHandler : public Bitstream::PayloadHandler
{
public:
    //! Constructor
    /*!
     * \param basePath      path to the loaded bitstream (required for checking relative files,
     *                      e.g. wav or directivity – will not be needed eventually)
     * \param checkFiles    whether or not to throw an exception when a linked file is not found (default: true)
     */
    EIFPayloadHandler(const std::string& basePath, bool checkFiles = true);

    //! Destructor
    ~EIFPayloadHandler(){};

    //! Inherited from Bitstream::PayloadHandler
    virtual std::shared_ptr<const Bitstream::Payload> unpack(
        const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig) override;

private:
    std::string basePath;
    bool checkFiles;
    std::shared_ptr<EIFPayload> payload;

    // helper functions to parse bitstream
    void createSceneData(const Bitstream::RawPayload& cursor);
    void createAcousticEnvironments(
        const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createAcousticMaterials(
        const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createAirflowGenerators(
        const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createAudioStreams(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createGranularSound(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createAnchors(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createChannelSources(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createDirectivities(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createPrimitives(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createMeshes(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createInformedSource(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig,
        EIFPayload::InformedSourceDesc& src);
    void createHoaGroups(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createHoaSources(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createObjectSources(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);
    void createTransforms(const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig);

    EIFPayload::AnchorDesc* findAnchor(int anchorId);
    EIFPayload::SourceDirectivityDesc* findDirectivity(int directivityId);
    EIFPayload::GeometryDesc* findGeometry(int geometryId);
    EIFPayload::HoaGroupDesc* findHoaGroupAndRegister(
        int hoaGroupId, EIFPayload::HoaSourceDesc* hoaSourcePtr);
    EIFPayload::AcousticMaterialDesc* findMaterial(int materialId);
    EIFPayload::ObjectSourceDesc* findObjectSource(int objectSourceId);
    EIFPayload::AudioStreamDesc* findStream(int streamId);
    EIFPayload::GranularSoundDesc* findGranularSound(int granularSoundId);
    EIFPayload::TransformDesc* findTransform(int transformId);
    inline std::string checkIfFileExistsRelatively(
        const std::string& relativeFile, const std::string& id) const;
};

} /* namespace MPEGI */

#endif // MPEGI_EIF_PAYLOAD_HANDLER
