/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_PORTAL_PAYLOAD
#define MPEGI_PORTAL_PAYLOAD

#include <MPEGI/Bitstream/Bitstream.h>
#include <MPEGI/Helpers/Types.h>
#include <algorithm>
#include <map>
#include <memory>
#include <set>

namespace MPEGI {

class PortalPayload : public Bitstream::Payload
{
public:
    //! Destructor
    ~PortalPayload(){};

    //! Returns all portals from the payload
    inline const std::vector<std::unique_ptr<Portal>>& getPortals() const { return portals; };

    //! Get source ID for given portal ID, returns empty-string on error
    // inline std::string getSourceId(int portalId) const
    // {
    //     auto const it = std::find_if(std::cbegin(portals), std::cend(portals),
    //         [portalId](auto const& p) { return p->portalId == portalId; });

    //     return (it == std::cend(portals)) ? "" : (*it)->sourceId;
    // }

    //! Returns the map of which portals are in which AE from the payload
    inline std::map<std::string, std::set<int>> getPortalMap() const { return portalMap; };

    //! Returns line of sight areas for each portal in each acoustic environment
    inline const std::map<unsigned int, std::shared_ptr<LineOfSightArea>>
    getLineOfSightAreas() const
    {
        return aeLosAreas;
    };

    //! Returns line of sight areas for each portal in areas outside any defined acoustic environments
    inline const std::vector<std::shared_ptr<LineOfSightArea>> getOutsideLineOfSightAreas() const
    {
        return outsideLosAreas;
    };

    inline bool getExplicitPortalMode() const { return isExplicitPortalMode; };
    inline std::map<std::pair<int, int>, float> getPortal2PortalFactors() const
    {
        return portal2portalFactors;
    };

protected:
    //! Handler is allowed to create new payloads and fill in data
    friend class PortalPayloadHandler;

    //! Constructor (only visible to PortalPayloadHandler and derived classes)
    PortalPayload() = default;

    std::vector<std::unique_ptr<Portal>> portals;
    std::map<std::string, std::set<int>> portalMap;
    std::map<std::pair<int, int>, float> portal2portalFactors;

    //! Line of sight areas ordered by acoustic environment id
    std::map<unsigned int, std::shared_ptr<LineOfSightArea>> aeLosAreas;

    //! Outside line of sight areas
    std::vector<std::shared_ptr<LineOfSightArea>> outsideLosAreas;

    bool isExplicitPortalMode{ false };
};

} // namespace MPEGI

#endif // MPEGI_PORTAL_PAYLOAD
