/***************************************************************************
This software module was originally developed jointly by

Nokia Technologies Ltd. and Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. and Koninklijke Philips N.V. retain full right to
modify and use the code for their own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/
#include <MPEGI/Bitstream/Bitstream.h>
#include <cstdint>

// Forward declared so that rans headers don't need to be included with this header
class RansDecoder;

namespace MPEGI {

class PortalPayloadHandler : public Bitstream::PayloadHandler
{
public:
    //! Constructor
    PortalPayloadHandler();

    //! Destructor
    ~PortalPayloadHandler();

    //! Inherited from Bitstream::PayloadHandler
    virtual std::shared_ptr<const Bitstream::Payload> unpack(
        const Bitstream::RawPayload& cursor, const BitstreamConfig& bsConfig) override;

    Bitstream::PayloadHandlerAndType makePortalPayloadHandler();

private:
    //! Get one symbol from rANS encoded mask
    std::int32_t getSymbol(std::int32_t index);

    //! Read a rANS encoded mask from and return a bitstream as a vector of boolean values
    std::vector<bool> decodeMask(std::int32_t length, const MPEGI::Bitstream::RawPayload& cursor);

    //! Read coordinates, dimensions and line of sight voxels of an area
    std::unique_ptr<LineOfSightArea> readLineOfSightArea(
        const Bitstream::RawPayload& cursor, bool isSmallScene);

    //! Initialization vectors, their sizes and offsets for the rANS decoder
    std::vector<std::int32_t> m_cdfSizes;
    std::vector<std::int32_t> m_offsets;
    std::vector<std::vector<std::int32_t>> m_cdfs;

    //! Pointer to rANS decoder
    std::unique_ptr<RansDecoder> m_ransDecoder;
};

} // namespace MPEGI
