/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#pragma once

#include <MPEGI/Bitstream/Bitstream.h>
#include <MPEGI/Helpers/ReverbConstants.h>
#include <MPEGI/Helpers/ReverbParameters.h>
#include <mpegcore/datatypes_reverb.hpp>

namespace MPEGI {

class ReverbPayload : public Bitstream::Payload
{
public:
    //! Destructor
    ~ReverbPayload(){};

    const std::unordered_map<unsigned int, std::shared_ptr<const ReverbParameters>>&
    GetReverbParameters() const
    {
        return reverbParameters;
    }

    const std::vector<std::shared_ptr<FreqGrid>>& GetFreqGrid() const { return freqGridData; }

    float reverbFadeInDistance{ DEFAULT_REVERB_FADE_IN_DISTANCE };
    float reverbMinimumDistance{ DEFAULT_REVERB_MINIMUM_DISTANCE };

    float reverbDistanceGainDropDb{ DEFAULT_REVERB_DISTANCE_GAIN_DROP_DB };
    bool headTrackingEnabled{ false };
    bool fadeInEnabled{ true };
    bool revDefaultAEPresent{ false };
    float revDefaultAEGain{ DEFAULT_REVERB_GAIN_IN_DEFAULT_AE };
    float revDefaultAEDelayRatio{ DEFAULT_REVERB_DELAY_RATIO_IN_DEFAULT_AE };
    float revDefaultAEDelayMinMs{ DEFAULT_REVERB_DELAY_MINIMUM_MS_IN_DEFAULT_AE };
    float revDefaultAEDistanceGainDropDb{ DEFAULT_REVERB_DISTANCE_GAIN_DROP_DB_IN_DEFAULT_AE };
    float revDefaultAEMinimumDistance{ DEFAULT_REVERB_MINIMUM_DISTANCE_IN_DEFAULT_AE };

    float revLsdfTheoreticalRDRtoActualRDRdB{ DEFAULT_REVERB_RDR_THEORETICAL_TO_RDR_ACTUAL };
    float revLsdfRDRtoActualRDRdB{ DEFAULT_REVERB_RDR_LSDF_TO_RDR_ACTUAL };
    bool revOverrideLSDF_RDR{ DEFAULT_REVERB_LSDF_RDR_OVERRIDE };

protected:
    //! Constructor (only visible to ReverbPayloadHandler derived classes)
    ReverbPayload() = default;

    //! Handler is allowed to create new payloads and fill in data
    friend class ReverbPayloadHandler;

    std::vector<std::shared_ptr<FreqGrid>> freqGridData;
    std::unordered_map<unsigned int, std::shared_ptr<const ReverbParameters>> reverbParameters;
};

} // namespace MPEGI
