/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <MPEGI/Bitstream/Globals.h>

using namespace MPEGI;

const std::unordered_map<std::string, cwLUT> LUTs::lut{
    { "deciSecondsCode",
        { { { 0.0f }, { 0.1f }, { 0.2f }, { 0.3f }, { 0.4f }, { 0.5f }, { 0.6f }, { 0.7f },
              { 0.8f }, { 0.9f }, { 1.0f } },
            { 6, 4, 5, 6, 7, 7, 4, 5, 2, 3, 0 }, { 3, 3, 3, 4, 4, 3, 4, 4, 4, 4, 3 },
            { "", "", "", "", "", "", "", "", "", "", "" } } },
    { "microsecondsCode",
        { { { 0.00001f }, { 0.00002f }, { 0.00003f }, { 0.00004f }, { 0.00005f }, { 0.00006f },
              { 0.00007f }, { 0.00008f }, { 0.00009f }, { 0.0001f }, { 0.00011f }, { 0.00012f },
              { 0.00013f }, { 0.00014f }, { 0.00015f }, { 0.00016f }, { 0.00017f }, { 0.00018f },
              { 0.00019f }, { 0.0002f }, { 0.00021f }, { 0.00022f }, { 0.00023f }, { 0.00024f },
              { 0.00025f }, { 0.00026f }, { 0.00027f }, { 0.00028f }, { 0.00029f }, { 0.0003f },
              { 0.00031f }, { 0.00032f }, { 0.00033f }, { 0.00034f }, { 0.00035f }, { 0.00036f },
              { 0.00037f }, { 0.00038f }, { 0.00039f }, { 0.0004f }, { 0.00041f }, { 0.00042f },
              { 0.00043f }, { 0.00044f }, { 0.00045f }, { 0.00046f }, { 0.00047f }, { 0.00048f },
              { 0.00049f }, { 0.0005f }, { 0.00051f }, { 0.00052f }, { 0.00053f }, { 0.00054f },
              { 0.00055f }, { 0.00056f }, { 0.00057f }, { 0.00058f }, { 0.00059f }, { 0.0006f },
              { 0.00061f }, { 0.00062f }, { 0.00063f }, { 0.00064f }, { 0.00065f }, { 0.00066f },
              { 0.00067f }, { 0.00068f }, { 0.00069f }, { 0.0007f }, { 0.00071f }, { 0.00072f },
              { 0.00073f }, { 0.00074f }, { 0.00075f }, { 0.00076f }, { 0.00077f }, { 0.00078f },
              { 0.00079f }, { 0.0008f }, { 0.00081f }, { 0.00082f }, { 0.00083f }, { 0.00084f },
              { 0.00085f }, { 0.00086f }, { 0.00087f }, { 0.00088f }, { 0.00089f }, { 0.0009f },
              { 0.00091f }, { 0.00092f }, { 0.00093f }, { 0.00094f }, { 0.00095f }, { 0.00096f },
              { 0.00097f }, { 0.00098f }, { 0.00099f } },
            { 444, 18, 445, 19, 894, 16, 895, 17, 892, 22, 893, 23, 434, 20, 435, 21, 432, 10, 433,
                11, 438, 8, 439, 9, 436, 14, 437, 15, 410, 12, 411, 13, 408, 2, 409, 3, 414, 0, 415,
                1, 412, 6, 413, 7, 402, 4, 403, 5, 400, 26, 401, 27, 406, 24, 407, 25, 404, 30, 405,
                31, 426, 28, 427, 29, 424, 18, 425, 19, 430, 16, 431, 17, 428, 22, 429, 23, 418, 20,
                419, 21, 416, 58, 417, 59, 422, 56, 423, 57, 420, 62, 421, 63, 442, 60, 443, 61,
                440, 24, 441 },
            { 9, 5, 9, 5, 10, 5, 10, 5, 10, 5, 10, 5, 9, 5, 9, 5, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9,
                6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9,
                6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9,
                6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 5, 9 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "milliSecondsCode",
        { { { 0.0f }, { 0.001f }, { 0.002f }, { 0.003f }, { 0.004f }, { 0.005f }, { 0.006f },
              { 0.007f }, { 0.008f }, { 0.009f }, { 0.01f }, { 0.011f }, { 0.012f }, { 0.013f },
              { 0.014f }, { 0.015f }, { 0.016f }, { 0.017f }, { 0.018f }, { 0.019f }, { 0.02f },
              { 0.021f }, { 0.022f }, { 0.023f }, { 0.024f }, { 0.025f }, { 0.026f }, { 0.027f },
              { 0.028f }, { 0.029f }, { 0.03f }, { 0.031f }, { 0.032f }, { 0.033f }, { 0.034f },
              { 0.035f }, { 0.036f }, { 0.037f }, { 0.038f }, { 0.039f }, { 0.04f }, { 0.041f },
              { 0.042f }, { 0.043f }, { 0.044f }, { 0.045f }, { 0.046f }, { 0.047f }, { 0.048f },
              { 0.049f }, { 0.05f }, { 0.051f }, { 0.052f }, { 0.053f }, { 0.054f }, { 0.055f },
              { 0.056f }, { 0.057f }, { 0.058f }, { 0.059f }, { 0.06f }, { 0.061f }, { 0.062f },
              { 0.063f }, { 0.064f }, { 0.065f }, { 0.066f }, { 0.067f }, { 0.068f }, { 0.069f },
              { 0.07f }, { 0.071f }, { 0.072f }, { 0.073f }, { 0.074f }, { 0.075f }, { 0.076f },
              { 0.077f }, { 0.078f }, { 0.079f }, { 0.08f }, { 0.081f }, { 0.082f }, { 0.083f },
              { 0.084f }, { 0.085f }, { 0.086f }, { 0.087f }, { 0.088f }, { 0.089f }, { 0.09f },
              { 0.091f }, { 0.092f }, { 0.093f }, { 0.094f }, { 0.095f }, { 0.096f }, { 0.097f },
              { 0.098f }, { 0.099f } },
            { 122, 123, 120, 121, 126, 127, 124, 125, 114, 115, 25, 112, 113, 118, 119, 116, 117,
                42, 43, 40, 18, 41, 46, 47, 44, 45, 34, 35, 32, 33, 19, 38, 39, 36, 37, 58, 59, 56,
                57, 62, 16, 63, 60, 61, 50, 51, 48, 49, 54, 55, 17, 52, 53, 10, 11, 8, 9, 14, 15,
                12, 22, 13, 2, 3, 0, 1, 6, 7, 4, 5, 23, 26, 27, 24, 25, 30, 31, 28, 29, 18, 20, 19,
                16, 17, 22, 23, 20, 21, 106, 107, 21, 104, 105, 110, 111, 108, 109, 98, 99, 48 },
            { 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7,
                7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7,
                7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 5, 7, 7, 7, 7,
                7, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 6 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "samplesCode",
        { { { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f }, { 8.0f },
              { 9.0f }, { 10.0f }, { 11.0f }, { 12.0f }, { 13.0f }, { 14.0f }, { 15.0f }, { 16.0f },
              { 17.0f }, { 18.0f }, { 19.0f }, { 20.0f }, { 21.0f }, { 22.0f }, { 23.0f },
              { 24.0f }, { 25.0f }, { 26.0f }, { 27.0f }, { 28.0f }, { 29.0f }, { 30.0f },
              { 31.0f }, { 32.0f }, { 33.0f }, { 34.0f }, { 35.0f }, { 36.0f }, { 37.0f },
              { 38.0f }, { 39.0f }, { 40.0f }, { 41.0f }, { 42.0f }, { 43.0f }, { 44.0f },
              { 45.0f }, { 46.0f }, { 47.0f }, { 48.0f }, { 49.0f }, { 50.0f }, { 51.0f },
              { 52.0f }, { 53.0f }, { 54.0f }, { 55.0f }, { 56.0f }, { 57.0f }, { 58.0f },
              { 59.0f }, { 60.0f }, { 61.0f }, { 62.0f }, { 63.0f }, { 64.0f }, { 65.0f },
              { 66.0f }, { 67.0f }, { 68.0f }, { 69.0f }, { 70.0f }, { 71.0f }, { 72.0f },
              { 73.0f }, { 74.0f }, { 75.0f }, { 76.0f }, { 77.0f }, { 78.0f }, { 79.0f },
              { 80.0f }, { 81.0f }, { 82.0f }, { 83.0f }, { 84.0f }, { 85.0f }, { 86.0f },
              { 87.0f }, { 88.0f }, { 89.0f }, { 90.0f }, { 91.0f }, { 92.0f }, { 93.0f },
              { 94.0f }, { 95.0f } },
            { 24, 25, 62, 63, 60, 61, 18, 19, 16, 17, 22, 23, 20, 21, 10, 11, 8, 9, 14, 15, 12, 13,
                2, 3, 0, 1, 6, 7, 4, 5, 26, 27, 24, 25, 30, 31, 28, 29, 18, 19, 16, 17, 22, 23, 20,
                21, 26, 442, 443, 440, 441, 446, 447, 444, 445, 434, 435, 432, 433, 438, 439, 436,
                437, 458, 459, 456, 457, 462, 463, 460, 461, 450, 451, 448, 449, 454, 455, 452, 453,
                474, 475, 472, 473, 478, 479, 476, 477, 466, 467, 464, 465, 470, 471, 468, 469 },
            { 5, 5, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
                9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
                9, 9, 9, 9, 9, 9, 9, 9, 9, 9 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "" } } },
    { "secondsCode",
        { { { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f }, { 8.0f },
              { 9.0f }, { 10.0f }, { 11.0f }, { 12.0f }, { 13.0f }, { 14.0f }, { 15.0f }, { 16.0f },
              { 17.0f }, { 18.0f }, { 19.0f }, { 20.0f }, { 21.0f }, { 22.0f }, { 23.0f },
              { 24.0f }, { 25.0f }, { 26.0f }, { 27.0f }, { 28.0f }, { 29.0f }, { 30.0f } },
            { 3, 1, 0, 15, 13, 12, 11, 9, 8, 14, 13, 12, 11, 9, 8, 5, 29, 28, 21, 31, 30, 21, 9, 8,
                41, 41, 40, 81, 161, 160 },
            { 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 7, 7, 7, 8,
                8 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "" } } },
    { "metersCode",
        { { { 0.0f }, { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f },
              { 8.0f }, { 9.0f }, { 10.0f }, { 11.0f }, { 12.0f }, { 13.0f }, { 14.0f }, { 15.0f },
              { 16.0f }, { 17.0f }, { 18.0f }, { 19.0f }, { 20.0f }, { 21.0f }, { 22.0f },
              { 23.0f }, { 24.0f }, { 25.0f }, { 26.0f }, { 27.0f }, { 28.0f }, { 29.0f },
              { 30.0f }, { 31.0f }, { 32.0f }, { 33.0f }, { 34.0f }, { 35.0f }, { 36.0f },
              { 37.0f }, { 38.0f }, { 39.0f }, { 40.0f }, { 41.0f }, { 42.0f }, { 43.0f },
              { 44.0f }, { 45.0f }, { 46.0f }, { 47.0f }, { 48.0f }, { 49.0f }, { 50.0f },
              { 51.0f }, { 52.0f }, { 53.0f }, { 54.0f }, { 55.0f }, { 56.0f }, { 57.0f },
              { 58.0f }, { 59.0f }, { 60.0f }, { 61.0f }, { 62.0f }, { 63.0f }, { 64.0f },
              { 65.0f }, { 66.0f }, { 67.0f }, { 68.0f }, { 69.0f }, { 70.0f }, { 71.0f },
              { 72.0f }, { 73.0f }, { 74.0f }, { 75.0f }, { 76.0f }, { 77.0f }, { 78.0f },
              { 79.0f }, { 80.0f }, { 81.0f }, { 82.0f }, { 83.0f }, { 84.0f }, { 85.0f },
              { 86.0f }, { 87.0f }, { 88.0f }, { 89.0f }, { 90.0f }, { 91.0f }, { 92.0f },
              { 93.0f }, { 94.0f }, { 95.0f }, { 96.0f }, { 97.0f }, { 98.0f }, { 99.0f } },
            { 61, 50, 51, 48, 49, 54, 55, 52, 53, 10, 11, 8, 9, 14, 15, 12, 13, 2, 3, 0, 1, 6, 7, 4,
                5, 26, 27, 24, 25, 30, 31, 28, 29, 18, 19, 16, 17, 22, 23, 20, 21, 42, 43, 40, 41,
                46, 47, 44, 45, 16, 68, 69, 142, 143, 140, 141, 154, 155, 152, 153, 158, 159, 156,
                157, 146, 147, 144, 145, 150, 151, 148, 149, 234, 235, 232, 233, 238, 239, 236, 237,
                226, 227, 224, 225, 230, 231, 228, 229, 250, 251, 248, 249, 254, 255, 252, 253, 242,
                243, 240, 241 },
            { 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 7, 7, 8, 8, 8, 8, 8,
                8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "hectometersCode",
        { { { 0.0f }, { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f },
              { 8.0f }, { 9.0f } },
            { 0, 1, 6, 7, 4, 5, 6, 7, 4, 5 }, { 3, 3, 3, 3, 3, 3, 4, 4, 4, 4 },
            { "", "", "", "", "", "", "", "", "", "" } } },
    { "kilometersCode",
        { { { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f }, { 8.0f },
              { 9.0f }, { 10.0f } },
            { 2, 3, 1, 0, 7, 5, 4, 13, 25, 24 }, { 2, 3, 3, 3, 3, 4, 4, 4, 5, 5 },
            { "", "", "", "", "", "", "", "", "", "" } } },
    { "centimetersCode",
        { { { 0.0f }, { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f },
              { 8.0f }, { 9.0f }, { 10.0f }, { 11.0f }, { 12.0f }, { 13.0f }, { 14.0f }, { 15.0f },
              { 16.0f }, { 17.0f }, { 18.0f }, { 19.0f }, { 20.0f }, { 21.0f }, { 22.0f },
              { 23.0f }, { 24.0f }, { 25.0f }, { 26.0f }, { 27.0f }, { 28.0f }, { 29.0f },
              { 30.0f }, { 31.0f }, { 32.0f }, { 33.0f }, { 34.0f }, { 35.0f }, { 36.0f },
              { 37.0f }, { 38.0f }, { 39.0f }, { 40.0f }, { 41.0f }, { 42.0f }, { 43.0f },
              { 44.0f }, { 45.0f }, { 46.0f }, { 47.0f }, { 48.0f }, { 49.0f }, { 50.0f },
              { 51.0f }, { 52.0f }, { 53.0f }, { 54.0f }, { 55.0f }, { 56.0f }, { 57.0f },
              { 58.0f }, { 59.0f }, { 60.0f }, { 61.0f }, { 62.0f }, { 63.0f }, { 64.0f },
              { 65.0f }, { 66.0f }, { 67.0f }, { 68.0f }, { 69.0f }, { 70.0f }, { 71.0f },
              { 72.0f }, { 73.0f }, { 74.0f }, { 75.0f }, { 76.0f }, { 77.0f }, { 78.0f },
              { 79.0f }, { 80.0f }, { 81.0f }, { 82.0f }, { 83.0f }, { 84.0f }, { 85.0f },
              { 86.0f }, { 87.0f }, { 88.0f }, { 89.0f }, { 90.0f }, { 91.0f }, { 92.0f },
              { 93.0f }, { 94.0f }, { 95.0f }, { 96.0f }, { 97.0f }, { 98.0f }, { 99.0f } },
            { 50, 51, 48, 49, 54, 55, 52, 53, 42, 43, 40, 41, 46, 47, 44, 45, 34, 35, 32, 33, 38,
                39, 36, 37, 58, 59, 56, 57, 62, 63, 60, 61, 50, 51, 48, 49, 54, 55, 52, 53, 10, 11,
                8, 9, 14, 15, 12, 13, 2, 3, 0, 1, 6, 7, 4, 5, 26, 27, 24, 25, 30, 31, 28, 29, 18,
                19, 16, 17, 22, 23, 20, 21, 42, 43, 40, 41, 46, 47, 44, 45, 34, 35, 32, 33, 38, 39,
                36, 37, 122, 123, 120, 121, 126, 127, 124, 125, 58, 59, 56, 57 },
            { 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
                7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
                7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 6, 6, 6, 6 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "millimetersCode",
        { { { 0.0f }, { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f },
              { 8.0f }, { 9.0f } },
            { 0, 1, 6, 7, 4, 5, 6, 7, 4, 5 }, { 3, 3, 4, 4, 4, 4, 3, 3, 3, 3 },
            { "", "", "", "", "", "", "", "", "", "" } } },
    { "orientationCode",
        { { { 0.0f, 0.0f, 0.0f }, { 90.0f, 0.0f, 0.0f }, { 180.0f, 0.0f, 0.0f },
              { -90.0f, 0.0f, 0.0f }, { 0.0f, 90.0f, 0.0f }, { 0.0f, -90.0f, 0.0f },
              { 0.0f, 0.0f, 180.0f }, { -10006.0f, 0.0f, 0.0f }, { -10007.0f, 0.0f, 0.0f },
              { -10004.0f, 0.0f, 0.0f } },
            { 1, 2, 3, 0, 1, 10, 11, 6, 7, 4 }, { 2, 4, 4, 4, 4, 4, 4, 3, 3, 3 },
            { "Default orientation: (yaw, pitch, roll)", "Default orientation: (yaw, pitch, roll)",
                "Default orientation: (yaw, pitch, roll)",
                "Default orientation: (yaw, pitch, roll)",
                "Default orientation: (yaw, pitch, roll)",
                "Default orientation: (yaw, pitch, roll)",
                "Default orientation: (yaw, pitch, roll)", "Default angles transmitted",
                "Explicit angles transmitted", "High resolution angles transmitted" } } },
    { "defaultYawCode",
        { { { -135.0f }, { -90.0f }, { -45.0f }, { 0.0f }, { 45.0f }, { 90.0f }, { 135.0f },
              { 180.0f } },
            { 2, 3, 0, 1, 6, 7, 4, 5 }, { 3, 3, 3, 3, 3, 3, 3, 3 },
            { "", "", "", "", "", "", "", "" } } },
    { "defaultPitchCode",
        { { { -90.0f }, { -45.0f }, { 0.0f }, { 45.0f }, { 90.0f } }, { 3, 4, 0, 5, 1 },
            { 2, 3, 2, 3, 2 }, { "", "", "", "", "" } } },
    { "defaultRollCode",
        { { { -135.0f }, { -90.0f }, { -45.0f }, { 0.0f }, { 45.0f }, { 90.0f }, { 135.0f },
              { 180.0f } },
            { 2, 3, 0, 1, 6, 7, 4, 5 }, { 3, 3, 3, 3, 3, 3, 3, 3 },
            { "", "", "", "", "", "", "", "" } } },
    { "coarseAngleCode",
        { { { -180.0f }, { -175.0f }, { -170.0f }, { -165.0f }, { -160.0f }, { -155.0f },
              { -150.0f }, { -145.0f }, { -140.0f }, { -135.0f }, { -130.0f }, { -125.0f },
              { -120.0f }, { -115.0f }, { -110.0f }, { -105.0f }, { -100.0f }, { -95.0f },
              { -90.0f }, { -85.0f }, { -80.0f }, { -75.0f }, { -70.0f }, { -65.0f }, { -60.0f },
              { -55.0f }, { -50.0f }, { -45.0f }, { -40.0f }, { -35.0f }, { -30.0f }, { -25.0f },
              { -20.0f }, { -15.0f }, { -10.0f }, { -5.0f }, { 0.0f }, { 5.0f }, { 10.0f },
              { 15.0f }, { 20.0f }, { 25.0f }, { 30.0f }, { 35.0f }, { 40.0f }, { 45.0f },
              { 50.0f }, { 55.0f }, { 60.0f }, { 65.0f }, { 70.0f }, { 75.0f }, { 80.0f },
              { 85.0f }, { 90.0f }, { 95.0f }, { 100.0f }, { 105.0f }, { 110.0f }, { 115.0f },
              { 120.0f }, { 125.0f }, { 130.0f }, { 135.0f }, { 140.0f }, { 145.0f }, { 150.0f },
              { 155.0f }, { 160.0f }, { 165.0f }, { 170.0f }, { 175.0f } },
            { 18, 19, 16, 17, 22, 23, 20, 21, 58, 59, 56, 57, 62, 63, 60, 61, 50, 51, 48, 49, 54,
                55, 52, 53, 42, 43, 40, 41, 46, 47, 44, 45, 34, 35, 32, 33, 38, 39, 36, 37, 58, 59,
                56, 57, 62, 63, 60, 61, 50, 51, 48, 49, 54, 55, 52, 53, 10, 11, 8, 9, 14, 15, 12,
                13, 2, 3, 0, 1, 6, 7, 4, 5 },
            { 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "" } } },
    { "fineAngleCode",
        { { { 0.1f }, { 0.2f }, { 0.3f }, { 0.4f }, { 0.5f }, { 0.6f }, { 0.7f }, { 0.8f },
              { 0.9f }, { 1.0f }, { 1.1f }, { 1.2f }, { 1.3f }, { 1.4f }, { 1.5f }, { 1.6f },
              { 1.7f }, { 1.8f }, { 1.9f }, { 2.0f }, { 2.1f }, { 2.2f }, { 2.3f }, { 2.4f },
              { 2.5f }, { 2.6f }, { 2.7f }, { 2.8f }, { 2.9f }, { 3.0f }, { 3.1f }, { 3.2f },
              { 3.3f }, { 3.4f }, { 3.5f }, { 3.6f }, { 3.7f }, { 3.8f }, { 3.9f }, { 4.0f },
              { 4.1f }, { 4.2f }, { 4.3f }, { 4.4f }, { 4.5f }, { 4.6f }, { 4.7f }, { 4.8f },
              { 4.9f }, { 5.0f } },
            { 28, 29, 62, 63, 60, 61, 18, 19, 16, 17, 22, 23, 20, 21, 10, 11, 8, 9, 14, 15, 12, 13,
                2, 3, 0, 1, 6, 7, 4, 5, 26, 27, 24, 25, 30, 31, 28, 29, 18, 19, 16, 17, 22, 23, 20,
                21, 26, 27, 24, 25 },
            { 5, 5, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "" } } },
    { "coarseGainCode",
        { { { -90.0f }, { -60.0f }, { -30.0f }, { -24.0f }, { -18.0f }, { -15.0f }, { -12.0f },
              { -9.0f }, { -6.0f }, { -3.0f }, { 0.0f }, { 3.0f }, { 6.0f }, { 9.0f }, { 12.0f },
              { 15.0f }, { 24.0f }, { 36.0f } },
            { 219, 52, 53, 30, 31, 14, 12, 7, 4, 5, 0, 2, 13, 12, 110, 111, 108, 218 },
            { 9, 7, 7, 6, 6, 5, 4, 3, 3, 3, 2, 3, 4, 5, 8, 8, 8, 9 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "fineGainCode",
        { { { -29.0f }, { -28.0f }, { -27.0f }, { -26.0f }, { -25.0f }, { -24.0f }, { -23.0f },
              { -22.0f }, { -21.0f }, { -20.0f }, { -19.0f }, { -18.0f }, { -17.0f }, { -16.0f },
              { -15.0f }, { -14.0f }, { -13.0f }, { -12.0f }, { -11.0f }, { -10.0f }, { -9.0f },
              { -8.0f }, { -7.0f }, { -6.0f }, { -5.0f }, { -4.0f }, { -3.0f }, { -2.0f },
              { -1.0f } },
            { 6, 7, 4, 5, 18, 19, 16, 17, 22, 23, 20, 21, 10, 11, 8, 9, 14, 15, 12, 13, 26, 27, 24,
                25, 14, 15, 0, 2, 3 },
            { 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 5, 4, 2,
                2 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "" } } },
    { "countOrIndexLoCode",
        { { { 0.0f }, { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f },
              { 8.0f }, { 9.0f }, { 10.0f }, { 11.0f }, { 12.0f }, { 13.0f }, { 14.0f }, { 15.0f },
              { 16.0f }, { 17.0f }, { 18.0f }, { 19.0f }, { 20.0f }, { 21.0f }, { 22.0f },
              { 23.0f }, { 24.0f }, { 25.0f }, { 26.0f }, { 27.0f }, { 28.0f }, { 29.0f },
              { 30.0f }, { 31.0f }, { 32.0f }, { 33.0f }, { 34.0f }, { 35.0f }, { 36.0f },
              { 37.0f }, { 38.0f }, { 39.0f }, { 40.0f }, { 41.0f }, { 42.0f }, { 43.0f },
              { 44.0f }, { 45.0f }, { 46.0f }, { 47.0f }, { 48.0f }, { 49.0f }, { 50.0f },
              { 51.0f }, { 52.0f }, { 53.0f }, { 54.0f }, { 55.0f }, { 56.0f }, { 57.0f },
              { 58.0f }, { 59.0f }, { 60.0f }, { 61.0f }, { 62.0f }, { 63.0f } },
            { 7, 4, 12, 13, 10, 11, 8, 9, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, 63,
                62, 61, 60, 59, 58, 57, 56, 111, 110, 109, 108, 107, 106, 105, 104, 103, 102, 101,
                100, 99, 98, 97, 96, 95, 94, 93, 92, 91, 90, 89, 88, 87, 86, 85, 84, 83, 82, 81,
                80 },
            { 4, 3, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
                7, 7, 7, 7, 7, 7, 7 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "" } } },
    { "countOrIndexHiCode",
        { { { 1.0f }, { 2.0f }, { 3.0f }, { 4.0f }, { 5.0f }, { 6.0f }, { 7.0f }, { 8.0f },
              { 9.0f }, { 10.0f }, { 11.0f }, { 12.0f }, { 13.0f }, { 14.0f }, { 15.0f } },
            { 1, 0, 6, 5, 4, 7, 5, 15, 14, 13, 9, 8, 25, 49, 48 },
            { 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 6, 7, 7 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "frequencyCode",
        { { { 16.0f }, { 20.0f }, { 25.0f }, { 31.5f }, { 40.0f }, { 50.0f }, { 63.0f }, { 80.0f },
              { 100.0f }, { 125.0f }, { 160.0f }, { 200.0f }, { 250.0f }, { 315.0f }, { 400.0f },
              { 500.0f }, { 630.0f }, { 800.0f }, { 1000.0f }, { 1250.0f }, { 1600.0f },
              { 2000.0f }, { 2500.0f }, { 3150.0f }, { 4000.0f }, { 5000.0f }, { 6300.0f },
              { 8000.0f }, { 10000.0f }, { 12500.0f }, { 16000.0f }, { 20000.0f }, { 25000.0f },
              { 31500.0f }, { 40000.0f } },
            { 35, 14, 15, 9, 12, 13, 0, 26, 27, 1, 24, 25, 14, 30, 31, 15, 28, 29, 12, 18, 19, 13,
                16, 17, 10, 22, 23, 11, 20, 21, 2, 16, 138, 139, 68 },
            { 6, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6, 6, 4, 6,
                6, 4, 5, 8, 8, 7 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "dsrCode",
        { { { -150.0f }, { -149.0f }, { -148.0f }, { -147.0f }, { -146.0f }, { -145.0f },
              { -144.0f }, { -143.0f }, { -142.0f }, { -141.0f }, { -140.0f }, { -139.0f },
              { -138.0f }, { -137.0f }, { -136.0f }, { -135.0f }, { -134.0f }, { -133.0f },
              { -132.0f }, { -131.0f }, { -130.0f }, { -129.0f }, { -128.0f }, { -127.0f },
              { -126.0f }, { -125.0f }, { -124.0f }, { -123.0f }, { -122.0f }, { -121.0f },
              { -120.0f }, { -119.0f }, { -118.0f }, { -117.0f }, { -116.0f }, { -115.0f },
              { -114.0f }, { -113.0f }, { -112.0f }, { -111.0f }, { -110.0f }, { -109.0f },
              { -108.0f }, { -107.0f }, { -106.0f }, { -105.0f }, { -104.0f }, { -103.0f },
              { -102.0f }, { -101.0f }, { -100.0f }, { -99.0f }, { -98.0f }, { -97.0f }, { -96.0f },
              { -95.0f }, { -94.0f }, { -93.0f }, { -92.0f }, { -91.0f }, { -90.0f }, { -89.0f },
              { -88.0f }, { -87.0f }, { -86.0f }, { -85.0f }, { -84.0f }, { -83.0f }, { -82.0f },
              { -81.0f }, { -80.0f }, { -79.0f }, { -78.0f }, { -77.0f }, { -76.0f }, { -75.0f },
              { -74.0f }, { -73.0f }, { -72.0f }, { -71.0f }, { -70.0f }, { -69.0f }, { -68.0f },
              { -67.0f }, { -66.0f }, { -65.0f }, { -64.0f }, { -63.0f }, { -62.0f }, { -61.0f },
              { -60.0f }, { -59.0f }, { -58.0f }, { -57.0f }, { -56.0f }, { -55.0f }, { -54.0f },
              { -53.0f }, { -52.0f }, { -51.0f }, { -50.0f }, { -49.0f }, { -48.0f }, { -47.0f },
              { -46.0f }, { -45.0f }, { -44.0f }, { -43.0f }, { -42.0f }, { -41.0f }, { -40.0f },
              { -39.0f }, { -38.0f }, { -37.0f }, { -36.0f }, { -35.0f }, { -34.0f }, { -33.0f },
              { -32.0f }, { -31.0f }, { -30.0f }, { -29.0f }, { -28.0f }, { -27.0f }, { -26.0f },
              { -25.0f }, { -24.0f }, { -23.0f }, { -22.0f }, { -21.0f }, { -20.0f }, { -19.0f },
              { -18.0f }, { -17.0f }, { -16.0f }, { -15.0f }, { -14.0f }, { -13.0f }, { -12.0f },
              { -11.0f }, { -10.0f } },
            { 140, 141, 286, 287, 284, 285, 130, 131, 128, 129, 134, 135, 132, 133, 234, 235, 232,
                233, 238, 239, 236, 237, 226, 227, 224, 225, 230, 231, 228, 229, 250, 251, 248, 249,
                254, 255, 252, 253, 242, 243, 240, 241, 246, 247, 244, 245, 202, 203, 200, 201, 206,
                207, 204, 205, 194, 195, 192, 193, 198, 199, 196, 197, 218, 219, 216, 217, 222, 223,
                220, 221, 20, 21, 38, 39, 36, 37, 58, 59, 56, 57, 62, 63, 60, 61, 50, 51, 48, 49,
                54, 55, 52, 53, 10, 11, 8, 9, 14, 15, 12, 13, 2, 3, 0, 1, 6, 7, 4, 5, 42, 43, 40,
                41, 46, 47, 44, 45, 18, 19, 16, 17, 22, 210, 211, 208, 209, 214, 215, 212, 213, 186,
                187, 184, 185, 190, 191, 188, 189, 138, 139, 136, 137 },
            { 8, 8, 9, 9, 9, 9, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
                9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
                9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
                6, 6, 6, 6, 6, 6, 6, 6, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 8, 8, 8,
                8 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "", "", "", "", "", "", "", "" } } },
    { "frequencyHopCode",
        { { { 1.059463094f }, { 1.122462048f }, { 1.189207115f }, { 1.25992105f }, { 1.414213562f },
              { 2.0f }, { 4.0f } },
            { 2, 3, 0, 1, 1, 3, 2 }, { 4, 4, 4, 2, 4, 2, 2 }, { "", "", "", "", "", "", "" } } },
    { "fgdDefaultGrid",
        { {
              { 31.5f, 63.0f, 125.0f, 250.0f, 500.0f, 1000.0f, 2000.0f, 4000.0f, 8000.0f,
                  16000.0f },
              { 25.0f, 50.0f, 100.0f, 200.0f, 400.0f, 800.0f, 1600.0f, 3150.0f, 6300.0f, 12500.0f },
              { 20.0f, 25.0f, 31.5f, 40.0f, 50.0f, 63.0f, 80.0f, 100.0f, 125.0f, 160.0f, 200.0f,
                  250.0f, 315.0f, 400.0f, 500.0f, 630.0f, 800.0f, 1000.0f, 1250.0f, 1600.0f,
                  2000.0f, 2500.0f, 3150.0f, 4000.0f, 5000.0f, 6300.0f, 8000.0f, 10000.0f, 12500.0f,
                  16000.0f, 20000.0f },
              { 25.0f, 100.0f, 400.0f, 1600.0f, 6300.0f },
              { 125.0f, 250.0f, 500.0f, 1000.0f, 2000.0f, 4000.0f },
              { 25.0f, 250.0f, 2500.0f },
              { 27.0f, 56.0f, 89.0f, 126.0f, 168.0f, 214.0f, 265.0f, 323.0f, 387.0f, 459.0f, 539.0f,
                  628.0f, 727.0f, 839.0f, 963.0f, 1101.0f, 1256.0f, 1429.0f, 1621.0f, 1836.0f,
                  2077.0f, 2345.0f, 2644.0f, 2978.0f, 3351.0f, 3767.0f, 4232.0f, 4750.0f, 5329.0f,
                  5975.0f, 6697.0f, 7502.0f, 8401.0f, 9405.0f, 10525.0f, 11775.0f, 13171.0f,
                  14729.0f, 16468.0f, 18410.0f, 20577.0f },
              { 27.0f, 89.0f, 168.0f, 265.0f, 387.0f, 539.0f, 727.0f, 963.0f, 1256.0f, 1621.0f,
                  2077.0f, 2644.0f, 3351.0f, 4232.0f, 5329.0f, 6697.0f, 8401.0f, 10525.0f, 13171.0f,
                  16468.0f, 20577.0f },
              { 50.0f, 150.0f, 250.0f, 350.0f, 450.0f, 570.0f, 700.0f, 840.0f, 1000.0f, 1170.0f,
                  1370.0f, 1600.0f, 1850.0f, 2150.0f, 2150.0f, 2500.0f, 2900.0f, 3400.0f, 4000.0f,
                  4800.0f, 5800.0f, 7000.0f, 8500.0f, 10500.0f, 13500.0f },
          },
            { 0, 1, 2, 3, 4, 5, 6, 7, 8 }, { 4, 4, 4, 4, 4, 4, 4, 4, 4 },
            { "", "", "", "", "", "", "", "", "" } } },
    { "delayLineCountCode",
        { { { 3.0f }, { 7.0f }, { 15.0f }, { 31.0f }, { 63.0f } }, { 4, 3, 0, 1, 5 },
            { 3, 2, 2, 2, 3 }, { "", "", "", "", "", "" } } },
    { "decimalCode",
        { { { 0.0f }, { 0.1f }, { 0.2f }, { 0.3f }, { 0.4f }, { 0.5f }, { 0.6f }, { 0.7f },
              { 0.8f }, { 0.9f }, { 1.0f } },
            { 6, 4, 5, 6, 7, 7, 4, 5, 2, 3, 0 }, { 3, 3, 3, 4, 4, 3, 4, 4, 4, 4, 3 },
            { "", "", "", "", "", "", "", "", "", "", "" } } },
    { "portal2PortalFactorCode",
        { { { -6.0f }, { -7.0f }, { -8.0f }, { -9.0f }, { -10.0f }, { -11.0f }, { -12.0f },
              { -13.0f }, { -14.0f }, { -15.0f }, { -16.0f }, { -17.0f }, { -18.0f }, { -20.0f },
              { -22.0f }, { -24.0f }, { -27.0f }, { -30.0f }, { -33.0f }, { -36.0f }, { -39.0f },
              { -42.0f }, { -48.0f }, { -54.0f }, { -60.0f }, { -66.0f }, { -72.0f }, { -78.0f },
              { -90.0f } },
            { 8, 9, 10, 11, 8, 9, 14, 15, 12, 13, 2, 3, 0, 1, 6, 7, 4, 5, 26, 27, 24, 25, 30, 31,
                28, 29, 22, 23, 10 },
            { 4, 4, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
                4 },
            { "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
                "", "", "", "", "", "", "", "" } } }
};
