/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_EMBREE_HELPER
#define MPEGI_EMBREE_HELPER

#include <EAR/BaseTypes.h>
#include <EAR/Mesh.h>

#include <embree3/rtcore.h>

namespace MPEGI {

namespace EmbreeHelper {

// TODO: there's a lot of overlap with EAR::EmbreeTracer, we should try to combine both efforts

inline void setOrg(RTCRay& ray, const EAR::Vector3& org)
{
    ray.org_x = org.data()[0];
    ray.org_y = org.data()[1];
    ray.org_z = org.data()[2];
}

inline void setDir(RTCRay& ray, const EAR::Vector3& dir)
{
    ray.dir_x = dir.data()[0];
    ray.dir_y = dir.data()[1];
    ray.dir_z = dir.data()[2];
}

inline EAR::Vector3 hitPosition(const RTCRay& ray)
{
    return {
        ray.org_x + ray.dir_x * ray.tfar,
        ray.org_y + ray.dir_y * ray.tfar,
        ray.org_z + ray.dir_z * ray.tfar,
    };
}

inline float dot(const RTCRay* ray, const RTCHit* hit)
{
    return ray->dir_x * hit->Ng_x + ray->dir_y * hit->Ng_y + ray->dir_z * hit->Ng_z;
}

inline RTCGeometry addMesh(RTCScene embreeScene, const EAR::Mesh& mesh,
    RTCBuildQuality buildQuality = RTC_BUILD_QUALITY_MEDIUM)
{
    auto embreeDevice = rtcGetSceneDevice(embreeScene);

    auto embreeMesh = rtcNewGeometry(embreeDevice, RTC_GEOMETRY_TYPE_TRIANGLE);
    rtcSetSharedGeometryBuffer(embreeMesh, RTC_BUFFER_TYPE_VERTEX, 0, RTC_FORMAT_FLOAT3,
        mesh.getVertices(EAR::Mesh::CSpace::Global).data(), 0, sizeof(EAR::Vector3),
        mesh.getVertices().size());
    rtcSetSharedGeometryBuffer(embreeMesh, RTC_BUFFER_TYPE_INDEX, 0, RTC_FORMAT_UINT3,
        mesh.getTriangles().data(), 0, sizeof(EAR::Mesh::Triangle), mesh.getTriangles().size());

    // NB: per-geometry build quality only taken into account if scene uses RTC_BUILD_QUALITY_LOW
    rtcSetGeometryBuildQuality(embreeMesh, buildQuality);

    rtcCommitGeometry(embreeMesh);
    rtcAttachGeometry(embreeScene, embreeMesh);
    rtcReleaseDevice(embreeDevice);
    return embreeMesh;
}

} // namespace EmbreeHelper

} // namespace MPEGI

#endif /* MPEGI_EMBREE_HELPER */
