/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DIFFRACTION_EDGE
#define MPEGI_DIFFRACTION_EDGE

#include <EAR/Exception.h>
#include <EAR/Geometry.h>

namespace MPEGI {

// It provides diffraction edge information
class DiffractionEdge
{

public:
    // Constructor
    DiffractionEdge() = default;

    // Destructor
    ~DiffractionEdge() = default;

    // Return edge center position with offset as an option
    void getEdgeCenterLocation(float scale = 0.01f);

    int edgeID;

    const EAR::Mesh* parentMesh;
    int meshID;

    // Edge vtx IDs
    std::pair<unsigned int, unsigned int> vtxIDs;

    // Edge vtx's coordinates
    std::pair<EAR::Vector3, EAR::Vector3> vtxCoords;

    // Edge length
    float edgeLength;

    // Edge direction
    EAR::Vector3 edgeVector;

    // Normalized edge direction: it starts from the first vtx coordinate in vtxCoords
    EAR::Vector3 normalizedEdgeDirection;

    // Adjacent triangle IDs
    std::pair<unsigned int, unsigned int> adjacentTriIDs;

    // Adjacent triangles
    std::pair<const EAR::Mesh::Triangle*, const EAR::Mesh::Triangle*> adjTris;

    // Adjacent triangle's centroids
    std::pair<EAR::Vector3, EAR::Vector3> adjacentTriCentroids;

    // Adjacent triangle's normals
    std::pair<EAR::Vector3, EAR::Vector3> adjacentTriNormals;

    // The internal angle between two adjacent triangles around this edge
    float edgeAngle;

    // Flag which indicates if the edge is a sort of rounded edge or not: edgeAngle < 10 degree then will be considered rounded edge
    bool roundedEdge{ false };

    // edge center point coordinate
    EAR::Vector3 cp;

    // edge center point coordinate for visibility test
    EAR::Vector3 cp_vt;

    // Flag which indicates if this edge is less relevant with the diffraction with given voxel grid and source location
    bool relevant{ true };

    // Edge on static geometry
    bool isStatic{ true };
};
} // namespace MPEGI

#endif /* MPEGI_DIFFRACTION_EDGE */
