/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DIFFRACTION_PATH
#define MPEGI_DIFFRACTION_PATH

#include <string>

namespace MPEGI {

// This structure contains diffraction-edge relevant information necessary when buidling paths
struct DiffractionPoint
{
    // Do not change the order
    int edgeID;
    DiffractionEdge* edgePtr{ nullptr };
    int faceIndicator;
    float angle;
};

// It contains diffraction path information necessary when rendering a sound along the path
class DiffractionPath
{
public:
    DiffractionPath(const std::vector<DiffractionPoint> pointSequence)
        : pointSequence(pointSequence)
        , keyId(pointSequence.front().edgeID)
    {

        for (auto point : pointSequence) {
            ePtSequence.push_back(point.edgePtr);
            eIdSequence.push_back(point.edgeID);
            fInSequence.push_back(point.faceIndicator);
            angSequence.push_back(point.angle);
        }

        std::string pathNamePostFix;
        for (auto point : pointSequence) {
            pathNamePostFix.append("_" + std::to_string(point.edgeID) + "."
                + std::to_string(point.faceIndicator) + "." + std::to_string(int(point.angle)));
        }
        pathName = "PATH_" + pathNamePostFix;
    };

    // return point sequence
    inline const std::vector<DiffractionPoint>& getPointSequence(void) const
    {
        return pointSequence;
    }

    // return path name
    inline const std::string& getPathName(void) const { return pathName; }

    // return edge pointer sequence
    inline const std::vector<DiffractionEdge*>& getEdgePtrSequence(void) const
    {
        return ePtSequence;
    }

    // return edge Id sequence
    inline const std::vector<int>& getEdgeIDSequence(void) const { return eIdSequence; }

    // return face indicator sequence
    inline const std::vector<int>& getFaceIndicatorSequence(void) const { return fInSequence; }

    // return angle sequence
    inline const std::vector<float>& getAngleSequence(void) const { return angSequence; }

    // return key Id
    inline const int getKeyId(void) const { return keyId; }

private:
    std::string pathName;

    std::vector<DiffractionPoint> pointSequence;
    std::vector<DiffractionEdge*> ePtSequence;
    std::vector<int> eIdSequence;
    std::vector<int> fInSequence;
    std::vector<float> angSequence;

    int keyId;
};

} // namespace MPEGI

#endif /* MPEGI_DIFFRACTION_PATH */
