/***************************************************************************
This software module was originally developed by

Dolby International AB

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Dolby International AB retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/
#pragma once
//--------------------------------------------------------------------------------------------------
#include <memory>

#include "MPEGI/Renderer/RenderStage.h"

//--------------------------------------------------------------------------------------------------
namespace MPEGI {
//--------------------------------------------------------------------------------------------------
class RenderStageProxy : public RenderStage
{
protected:
    //@{
    /// Implementation of RendererStage Interface
    void processBlock(DspContext ctx) override { _impl->processBlock(ctx); };

    std::unique_ptr<RenderStage::DspData> update(RenderList& render_list) override
    {
        return _impl->update(render_list);
    }

    void initializeStage(const Bitstream& bitstream, RenderItemStore* itemStore,
        const StreamManager* streamManager, const SceneState* state) override
    {
        setup(bitstream);
        if (!_impl) {
            throw std::logic_error("RenderStageProxy::setup() failed");
        }

        _impl->initializeStage(bitstream, itemStore, streamManager, state);
    }

    void setObserver(RenderingStageObserver* renderStageObserver) override
    {
        _impl->setObserver(renderStageObserver);
    }

    //@}

protected:
    /**
     * Setup proxy based on the bitstream
     *
     * @param bitstream Bitstream structure
     */
    virtual void setup(const Bitstream&) = 0;

protected:
    using stage_ptr = std::shared_ptr<RenderStage>;
    stage_ptr _impl;
};

//--------------------------------------------------------------------------------------------------
} // namespace MPEGI
//--------------------------------------------------------------------------------------------------
