/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_SCENE_STATE_OBSERVER
#define MPEGI_SCENE_STATE_OBSERVER

// fwd declares
namespace EAR {
class SceneObject;
}

namespace MPEGI {

// fwd declares
class SceneState;

//! SceneStateObserver Interface
class SceneStateObserver
{
public:
    //! Destructor
    virtual ~SceneStateObserver(){};

    //! Registration callback
    /*!
     * \param sceneState         pointer to the SceneState that will be observed
     */
    virtual void sceneStateAttached(const SceneState* sceneState) = 0;

    //! Scene detachment callback
    virtual void sceneStateDetached() = 0;

    //! Notify observers of a newly added SceneObject
    /*!
     * This method is also called for all existing SceneObjects directly after a new observer registration to allow the observer to configure itself according to the current SceneState.
     *
     * \param object            pointer to new SceneObject (itself observable by a SceneObjectObserver)
     */
    virtual void sceneObjectAdded(EAR::SceneObject* object) = 0;

    //! Notify observers of a removed SceneObject
    /*!
     * \param object            pointer to the SceneObject that will be removed
     */
    virtual void sceneObjectRemoved(EAR::SceneObject* object) = 0;
};

} // namespace MPEGI

#endif /* MPEGI_SCENE_STATE_OBSERVER */
