/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_SCENE_UPDATE
#define MPEGI_SCENE_UPDATE

#include <EAR/Variant.h>

#include <string>
#include <vector>

namespace MPEGI {

//! An Update represents an atomic modification to the Scene
class Update
{
public:
    //! Constructor
    /*!
     * \paramn timestamp        timestamp of the Update, where values < 0 (or values smaller than the current scene time) indicate immediate execution (default -1)
     * \param interpolate       whether to interpolate values or crossfade into new state (default true)
     */
    Update(double timestamp = -1)
        : timestamp(timestamp){};

    //! Add a modification to the Update
    /*!
     * \param id                the id of the target entity that is being modified as given by the scene description
     * \param attr              the attribute name of the entity being modified
     * \param val               the new value to change the given attribute to
     * \param ip                interpolate the values? (default true)
     */
    void addModification(
        const std::string& id, const std::string& attr, const EAR::Variant& val, bool ip = true)
    {
        modifications.emplace_back(id, attr, val, ip);
    }

    double timestamp;

    struct Modification
    {
        //! The id of the target entity that is being modified as given by the scene description
        std::string entityId;
        //! The attribute name of the entity being modified
        std::string attributeName;
        //! The new value to change the given attribute to
        EAR::Variant targetValue;

        bool interpolate;

        Modification(
            const std::string& id, const std::string& attr, const EAR::Variant& val, bool ip)
            : entityId(id)
            , attributeName(attr)
            , targetValue(val)
            , interpolate(ip){};
    };

    //! List of all modifications to be executed at the given timestamp
    std::vector<Modification> modifications;
};

} // namespace MPEGI

#endif /* MPEGI_SCENE_UPDATE */
