/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_VST_STREAM_MANAGER
#define MPEGI_VST_STREAM_MANAGER

#include <MPEGI/Stream/StreamAccessBuffer.h>
#include <MPEGI/Stream/StreamManager.h>

#include <cstddef>
#include <memory>
#include <unordered_map>
#include <utility>
#include <vector>

// fwd declares
namespace EAR {
class RealBuffer;
class RealFrame;
} // namespace EAR

namespace MPEGI {

// fwd declares
class Bitstream;

//! The VSTStreamManager ingests double samples given by JUCE and provides a way to map them on sound sources.
class VSTStreamManager : public StreamManager
{
public:
    //! Constructor
    /*!
     * \param bitstream     Bitstream instance (the information about AudioStreams in the EIFPayload is used to load them)
     * \param fs            sample rate [Hz]
     * \param B             block size [samples]
     */
    VSTStreamManager(const Bitstream& bitstream, float fs, std::size_t B);

    //! Destructor
    ~VSTStreamManager();

    //! Inherited from StreamManager
    std::shared_ptr<StreamAccessBuffer> createAccessBuffer(
        const std::string& streamId, double t = 0.0) override;

    //! Inherited from StreamManager
    std::shared_ptr<StreamAccessBuffer> createAccessBuffer(
        const SceneState::AudioStream& stream) override;

    //! Inherited from StreamManager
    void removeAccessBuffer(StreamAccessBuffer* buf) override;

    //! Inherited from StreamManager
    void nextBlock() override;

    //! Inherited from StreamManager
    std::size_t getNbrOfChannels(const std::string& id) const override;

    //! Get total number of channels that will be ingested
    std::size_t getNbrOfChannels() const;

    //! Ingest a block of samples
    /*!
     * \param input       float samples as given by JUCE
     *                    (the channel order is expected to be corresponding to the "aepInputChannels" attribute in the EIF)
     */
    void ingest(const float* const* input);

private:
    //! internal buffers of all audio samples in all streams from the EIF
    std::unordered_map<std::string, std::unique_ptr<EAR::RealFrame>> buffers;
    //! list of all active StreamAccessBuffer instances (not as shared_ptr, so they are deleted when not used outside - the destructor removes the pointer from this list)
    std::vector<StreamAccessBuffer*> accessors;

    std::vector<std::pair<int, EAR::RealBuffer*>> inputChannelToBuffer;

    std::size_t highestChannel;
};

} // namespace MPEGI

#endif /* MPEGI_VST_STREAM_MANAGER */
