/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include "MPEGI/Helpers/RayPlaneIntersection.h"

#include "MPEGI/Renderer/PerceptualPointDistribution.h"
#include <EAR/BaseTypes.h>

namespace MPEGI {

std::pair<bool, EAR::Vector3> checkRayPlaneIntersection(
    EAR::Vector3 rayOrigin, EAR::Vector3 rayVector, IntersectPlane P)
{
    bool hit;
    EAR::Vector3 intersect;
    float d = P.planeCenter.dot(-P.planeNormal);
    float denom = rayVector.dot(P.planeNormal);
    if (std::abs(denom) < 1E-7f) {
        hit = false;
    } else {
        float t = -(d + rayOrigin.dot(P.planeNormal)) / denom;
        intersect = rayOrigin + t * rayVector;

        if (P.hasBounds) {

            bool insideX = P.u.dot(intersect) >= P.uDotLL && P.u.dot(intersect) <= P.uDotLR;
            bool insideY = P.v.dot(intersect) >= P.vDotLL && P.v.dot(intersect) <= P.vDotUL;

            hit = t > 0.0f && insideX && insideY;
        } else {
            hit = t > 0.0f;
        }
    }

    return std::pair{ hit, intersect };
}

std::vector<MPEGI::RayHit> checkRayHitsPlaneIntersection(std::vector<MPEGI::RayHit> rayHits,
    IntersectPlane P, EAR::Vector3 listenerPosition, EAR::Quaternion listenerRotation)
{

    const EAR::Vector3* rays = SESS_RAY_DISTRIBUTION;
    std::vector<MPEGI::RayHit> raysHitPlane;
    float d = P.planeCenter.dot(-P.planeNormal);
    float t;
    float listDotNormal = listenerPosition.dot(P.planeNormal);
    for (MPEGI::RayHit rh : rayHits) {
        bool hit;
        EAR::Vector3 intersect;
        EAR::Vector3 rayVector = listenerRotation.rotate(rays[rh.rayID]);
        float denom = rayVector.dot(P.planeNormal);
        if (std::abs(denom) < 1E-7f) {
            hit = false;
        } else {

            t = -(d + listDotNormal) / rayVector.dot(P.planeNormal);
            intersect = listenerPosition + t * rayVector;

            if (P.hasBounds) {

                bool insideX = P.u.dot(intersect) >= P.uDotLL && P.u.dot(intersect) <= P.uDotLR;
                bool insideY = P.v.dot(intersect) >= P.vDotLL && P.v.dot(intersect) <= P.vDotUL;

                hit = t > 0.0f && insideX && insideY;
            } else {
                hit = t > 0.0f;
            }
        }
        if (hit) {
            raysHitPlane.emplace_back(rh);
        }
    }

    return raysHitPlane;
}

} // namespace MPEGI