/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <MPEGI/Helpers/Types.h>

#include <algorithm>
#include <type_traits>

namespace MPEGI {

void ReverbEnv::append(float f, float rt, std::optional<float> rdr)
{
    //add new reflection frequency tuple to interpolator
    rtInterpolator->addSupportingPoint(f, rt);

    FrequencyDecay freqDecayData;
    freqDecayData.frequency = f;
    freqDecayData.rdr = rdr;
    freqDecayData.rt60 = rt;

    frequencyDecayResponse.push_back(freqDecayData);
    std::sort(frequencyDecayResponse.begin(), frequencyDecayResponse.end(),
        [](auto a, auto b) { return a.frequency < b.frequency; });
}

ItemPropertyProxy operator&(ItemProperty lhs, ItemProperty rhs)
{
    return { static_cast<ItemProperty>(static_cast<std::underlying_type<ItemProperty>::type>(lhs)
        & static_cast<std::underlying_type<ItemProperty>::type>(rhs)) };
}

ItemProperty operator|(ItemProperty lhs, ItemProperty rhs)
{
    return static_cast<ItemProperty>(static_cast<std::underlying_type<ItemProperty>::type>(lhs)
        | static_cast<std::underlying_type<ItemProperty>::type>(rhs));
}

ItemProperty operator~(ItemProperty prop)
{
    return static_cast<ItemProperty>(~static_cast<std::underlying_type<ItemProperty>::type>(prop));
}

ItemProperty& operator|=(ItemProperty& lhs, ItemProperty rhs)
{
    lhs = static_cast<ItemProperty>(static_cast<std::underlying_type<ItemProperty>::type>(lhs)
        | static_cast<std::underlying_type<ItemProperty>::type>(rhs));

    return lhs;
}

ItemPropertyProxy operator&=(ItemProperty& lhs, ItemProperty rhs)
{
    lhs = static_cast<ItemProperty>(static_cast<std::underlying_type<ItemProperty>::type>(lhs)
        & static_cast<std::underlying_type<ItemProperty>::type>(rhs));
    return { lhs };
}

bool operator&(AParam lhs, AParam rhs)
{
    return static_cast<bool>(static_cast<std::underlying_type<AParam>::type>(lhs)
        & static_cast<std::underlying_type<AParam>::type>(rhs));
}

AParam operator|(AParam lhs, AParam rhs)
{
    return static_cast<AParam>(static_cast<std::underlying_type<AParam>::type>(lhs)
        | static_cast<std::underlying_type<AParam>::type>(rhs));
}

AParam& operator|=(AParam& lhs, AParam rhs)
{
    lhs = static_cast<AParam>(static_cast<std::underlying_type<AParam>::type>(lhs)
        | static_cast<std::underlying_type<AParam>::type>(rhs));

    return lhs;
}

AParam& operator&=(AParam& lhs, AParam rhs)
{
    lhs = static_cast<AParam>(static_cast<std::underlying_type<AParam>::type>(lhs)
        & static_cast<std::underlying_type<AParam>::type>(rhs));

    return lhs;
}

void VoxelCoord::set(int _x, int _y, int _z)
{
    val[0] = _x;
    val[1] = _y;
    val[2] = _z;
}

void VoxelCoord::set(int _val[3])
{
    val[0] = _val[0];
    val[1] = _val[1];
    val[2] = _val[2];
}

VoxelCoord& VoxelCoord::operator+=(const VoxelCoord& rhs)
{
    val[0] += rhs.val[0];
    val[1] += rhs.val[1];
    val[2] += rhs.val[2];

    return *this;
}

VoxelCoord& VoxelCoord::operator-=(const VoxelCoord& rhs)
{
    val[0] -= rhs.val[0];
    val[1] -= rhs.val[1];
    val[2] -= rhs.val[2];

    return *this;
}

VoxelCoord operator+(VoxelCoord lhs, const VoxelCoord& rhs)
{
    lhs += rhs;
    return lhs;
}

VoxelCoord operator-(VoxelCoord lhs, const VoxelCoord& rhs)
{
    lhs -= rhs;
    return lhs;
}

VoxelCoord operator-(const VoxelCoord& rhs)
{
    return VoxelCoord(-rhs.val[0], -rhs.val[1], -rhs.val[2]);
}

std::ostream& operator<<(std::ostream& os, const VoxelCoord& vec)
{
    return os << "VoxelCoord(" << vec[0] << ", " << vec[1] << ", " << vec[2] << ")";
}
//! boolean compare for VoxelCoord
bool operator==(const VoxelCoord& lhs, const VoxelCoord& rhs)
{
    return lhs[0] == rhs[0] && lhs[1] == rhs[1] && lhs[2] == rhs[2];
}

//! boolean compare for VoxelCoord
bool operator!=(const VoxelCoord& lhs, const VoxelCoord& rhs)
{
    return lhs[0] != rhs[0] || lhs[1] != rhs[1] || lhs[2] != rhs[2];
}

} // namespace MPEGI
