/***************************************************************************
This software module was originally developed by

Dolby International AB

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Dolby International AB retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <algorithm>
#include <random>

#include <doctest/doctest.h>

#include <MPEGI/Bitstream/VoxDataPayload.h>
#include <MPEGI/Helpers/Config.h>
#include <MPEGI/Helpers/VoxelTypes.h>
#include <iostream>

using namespace MPEGI;

TEST_CASE("Projection map conversion")
{
    const int grid_size = 256;
    const int number_of_blocks = 32;

    Voxel::dmap_index_t projection_dimension = { grid_size, grid_size };

    std::vector<Voxel::Payload::block<2>> blocks_in;
    std::vector<Voxel::Payload::block<2>> blocks_out;

    for (auto i = 0; i < number_of_blocks; i++) {
        int mat_id = 1;
        std::array<Voxel::voxel_index_val_t, 2> ps;
        std::array<Voxel::voxel_index_val_t, 2> pe;

        for (auto j = 0; j < 2; j++) {

            Voxel::voxel_index_val_t A = rand() % grid_size;
            Voxel::voxel_index_val_t B = rand() % grid_size;

            ps[j] = std::min(A, B);
            pe[j] = std::max(A, B);
        }
        blocks_in.push_back(Voxel::Payload::block<2>(mat_id, ps, pe));
    }

    // set up reference projection_map
    Voxel::projection_map projection_map_ref(blocks_in, projection_dimension);
    // convert reference projection_map to blocks
    projection_map_ref.convert(blocks_out);
    // construct projection_map from blocks
    Voxel::projection_map projection_map_dut(blocks_out, projection_dimension);

    // compare reference projection_map with constructed projection_map
    for (auto x = 0; x < projection_dimension[0]; x++) {
        for (auto y = 0; y < projection_dimension[1]; y++) {
            CHECK_EQ(projection_map_ref.at(x, y), projection_map_dut.at(x, y));
        }
    }
}
