#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include <memory>
#include <vector>

#include <MPEGI/GeometryCoding.h>

namespace py = pybind11;

struct Coeffs
{
    Coeffs(float _c1, float _c2, float _c3, float _c4)
        : c1(_c1)
        , c2(_c2)
        , c3(_c3)
        , c4(_c4)
    {}

    float c1;
    float c2;
    float c3;
    float c4;
};

struct Material
{
    Material(int _freq_grid_index, std::vector<Coeffs> _coeffs)
        : freq_grid_index(_freq_grid_index)
        , coeffs(std::move(_coeffs))
    {}

    int freq_grid_index;
    std::vector<Coeffs> coeffs;
};

py::bytes encodeMaterials(const std::vector<Material>& mats, float matQntStep)
{
    auto geoData = std::make_unique<MPEGI::GeometryData>();

    geoData->setEncoderOptions(0.001, // not relevant for material coding
        0.0, // not relevant for material coding
        0.0, // not relevant for material coding
        matQntStep,
        false // not relevant for material coding
    );

    const float matCount = mats.size();
    for (int i = 0; i < matCount; i++) {
        const Material& mat = mats[i];
        const float freqCount = mat.coeffs.size();
        std::vector<float> c1;
        std::vector<float> c2;
        std::vector<float> c3;
        std::vector<float> c4;

        for (const auto& cfs : mat.coeffs) {
            c1.push_back(cfs.c1);
            c2.push_back(cfs.c2);
            c3.push_back(cfs.c3);
            c4.push_back(cfs.c4);
        }

        geoData->addAcousticMaterial(matCount, i, freqCount, mat.freq_grid_index, c1.data(),
            c2.data(), c3.data(), c4.data());
    }

    int requiredBufferSize = geoData->getRequiredBufferSize();
    auto buffer = std::make_unique<unsigned char[]>(requiredBufferSize);
    int actualBufferSize = geoData->encodeToBuffer(requiredBufferSize, buffer.get());
    const char* bufferPtr = (const char*)buffer.get();
    return py::bytes(bufferPtr, actualBufferSize);
}
