"""Flex Range Coding - Arithmetically Code Symbols."""

class Alphabet:
    """Alphabet to represent a range of symbols.

    This is a helper class for the encoding and decoding symbols with
    the coders.
    """

    name: str
    size: int
    max_freq: int

    def __init__(self, name: str, *, size: int, max_freq: int = ...) -> None:
        """Construct an alphabet.

        :param name: The name of this alphabet.
        :param size: The total number of symbols that this alphabet
            represents.
        :param max_freq: The maximum possible frequency of any one
            symbol, defaults to 32768
        """
        ...

class FlexEncoder:
    """Flex Range Encoder - Arithmetically Encode Symbols."""

    def __init__(self) -> None:
        """Construct Flex Range Encoder."""
        ...
    def encode_symbol(self, symbol: int, *, alpha: Alphabet) -> None:
        """Encode a symbol with a given alphbet.

        The flex range encoder will implement an adaptive estimator
        and change the probability of the symbol distribution upon
        every call. Each alphabet will have its own adaptive estimator.

        :param symbol: The symbol to encode.
        :param alpha: The alphabet in which the symbol belongs to.
        """
        ...
    def encode_symbol_uniform(self, symbol: int, *, alpha: Alphabet) -> None:
        """Encode a symbol with a given alphbet with uniform probability.

        :param symbol: The symbol to encode, up to 16 bits
        :param alpha: The alphabet in which the symbol belongs to.
        """
        ...
    def encode_symbol_uniform_long(self, symbol: int, *, alpha: Alphabet) -> None:
        """Encode a long symbol with uniform probability.

        :param symbol: The symbol to encode, up to 32 bits
        :param alpha: The alphabet in which the symbol belongs to.
        """
        ...
    def encode_bits(self, value: int, *, width: int) -> None:
        """Encode an int up to 16 bits.

        :param value: The value to encoder
        :param width: The width in bits to encode, cannot exceed 16
        """
        ...
    def encode_bits_long(self, value: int, *, width: int) -> None:
        """Encode an int up to 32 bits.

        :param value: The value to encoder
        :param width: The width in bits to encode, cannot exceed 32
        """
        ...
    def finish(self) -> bytes:
        """Declare end of encoding and return encoded blob."""
        ...

class FlexDecoder:
    """Flex Range Decoder - Arithmetically Decode Symbols."""

    def __init__(self, buffer: bytes) -> None:
        """Construct Flex Range Decoder."""
        ...
    def decode_symbol(self, symbol: int, *, alpha: Alphabet) -> None:
        """Decode a symbol with a given alphbet.

        The flex range decoder will implement an adaptive estimator
        and change the probability of the symbol distribution upon
        every call. Each alphabet will have its own adaptive estimator.

        :param alpha: The alphabet in which the symbol belongs to.
        :return: The decoded symbol
        """
        ...
    def decode_bits(self, *, width: int) -> int:
        """Decode an int up to 16 bits.

        :param width: The width in bits, cannot exceed 16
        :return: The decoded int
        """
        ...
    def decode_symbol_uniform(self, alpha: Alphabet) -> int
        """Decode a symbol with a given alphbet with uniform probability.

        :param alpha: The alphabet in which the symbol belongs to.
        :return: The decoded symbol
        """
        ...
    def decode_symbol_uniform_long(self, alpha: Alphabet) -> int:
        """Decode a long symbol with uniform probability.

        :param alpha: The alphabet in which the symbol belongs to.
        :return: The decoded symbol
        """
        ...
    def decode_bits_long(self, *, width: int) -> int:
        """Decode an int up to 32 bits.

        :param width: The width in bits, cannot exceed 32
        :return: The decoded int
        """
        ...
    def finish(self) -> int:
        """Declare end of decoding and return consumed bits."""
        ...
