"""Efficient Mesh Coding."""
from typing import List, Optional


class Vertex:
    """Vertex in cartesian space (m)"""

    x: float
    y: float
    z: float
    def __init__(self, *, x: float, y: float, z: float) -> None:
        """Construct a vertex."""
        ...

class Face:
    """A triangle of a mesh."""

    mat_idx: Optional[int]
    v1: int
    v2: int
    v3: int
    def __init__(self, *, mat_idx: Optional[int], v1: int, v2: int, v3: int) -> None:
        """Construct a face.

        :param mat_idx: The index for the material, use None if there
            is no material for this face.
        :param v1: The first vertex index.
        :param v2: The second vertex index.
        :param v3: The third vertex index.
        """
        ...

class Mesh:
    """Mesh representation"""

    vertices: List[Vertex]
    faces: List[Face]
    def __init__(self, *, vertices: List[Vertex], faces: List[Face]) -> None:
        """Construct a Mesh."""
        ...

def encode(
    mesh: Mesh,
    *,
    vertex_qnt_step: float,
    vertex_tolerance: float,
    face_tolerance: float,
    remove_faces_without_mats: bool
) -> bytes:
    """Encode a mesh into bytes.

    :param mesh: The mesh to encode.
    :param vertex_qnt_step: The quantization step size for vertex coords.
    :param vertex_tolerance: The tolerance for duplicate vertices.
    :param face_tolerance: The tolerance for duplicate faces.
    :param remove_faces_without_mats: If true, remove faces with no mats.
    :return: The encoded mesh in bytes.
    """
    ...
