from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import setup

mcode_ext = Pybind11Extension(
    "mcode",
    [
        "src/CodingUtils.cpp",
        "src/DirectivityCoding.cpp",
        "src/flex_range_encoder.cpp",
        "src/flex_range_decoder.cpp",
        "src/GeometryCoding.cpp",
        "python/pybinds.cpp",
    ],
    include_dirs=[
        "include",
    ],
    define_macros=[
        # To get M_PI on MSVC:
        ("_USE_MATH_DEFINES", None),
    ],
)


setup(
    name="mcode",
    version="0.0.0",
    description="Metadata Coding Library for MPEG-I",
    include_dirs=["../include/"],
    ext_modules=[mcode_ext],
    packages=["mcode"],
    package_dir={"": "python"},
    package_data={
        "mcode": [
            "__init__.pyi",
            "directivity.pyi",
            "flex.pyi",
            "material.pyi",
            "mesh.pyi",
            "py.typed",
        ]
    },
    cmdclass={"build_ext": build_ext},
    python_requires=">=3.6",
    extras_require={"test": "pytest"},
)
