/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <MPEGI/DirectivityCoding.h>
#include <MPEGI/DirectivityEncodingWrapper.h>
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <new>

using namespace std;
using namespace MPEGI;

void* dewCreateInstance()
{
    DirectivityCoverList* dcl = NULL;

    try {
        dcl = new DirectivityCoverList;
    } catch (const bad_alloc&) {
        // NULL pointer will be returned on memory allocation failure
    }

    return dcl;
}

void dewAddDirectivityCover(
    void* instance, float freq, float deg_step_req, int cover_size, float* cover, float db_step_req)
{
    DirectivityCoverList* dcl = (DirectivityCoverList*)instance;

    dcl->addDirectivityCover(freq, deg_step_req, cover_size, cover, db_step_req);
}

int dewGetRequiredBufferSize(void* instance)
{
    DirectivityCoverList* dcl = (DirectivityCoverList*)instance;

    return dcl->getRequiredBufferSize();
}

int dewEncodeToBuffer(void* instance, int buffer_size, unsigned char* buffer_data)
{
    DirectivityCoverList* dcl = (DirectivityCoverList*)instance;

    return dcl->encodeToBuffer(buffer_size, buffer_data);
}

void dewDestroyInstance(void* instance)
{
    DirectivityCoverList* dcl = (DirectivityCoverList*)instance;

    delete dcl;
}

void dewEnableAutomatedTextDump(void* instance, char* file_prefix)
{
    DirectivityCoverList* dcl = (DirectivityCoverList*)instance;

    dcl->enableAutomatedTextDump(file_prefix);
}
