/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <MPEGI/GeometryCoding.h>
#include <MPEGI/GeometryEncodingWrapper.h>
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <new>

using namespace std;
using namespace MPEGI;

void* gewCreateInstance()
{
    GeometryData* gd = NULL;

    try {
        gd = new GeometryData;
    } catch (const bad_alloc&) {
        // NULL pointer will be returned on memory allocation failure
    }

    return gd;
}

void gewAddVertexList(void* instance, int ver_count, float* x, float* y, float* z)
{
    GeometryData* gd = (GeometryData*)instance;

    gd->addVertexList(ver_count, x, y, z);
}

void gewAddTriangleList(void* instance, int tri_count, int* v1_index, int* v2_index, int* v3_index,
    int shr_mat_count, int* mat_index)
{
    GeometryData* gd = (GeometryData*)instance;

    gd->addTriangleList(tri_count, v1_index, v2_index, v3_index, shr_mat_count, mat_index);
}

void gewAddAcousticMaterial(void* instance, int mat_count, int mat_index, int freq_count,
    int freq_grid_index, float* c1, float* c2, float* c3, float* c4)
{
    GeometryData* gd = (GeometryData*)instance;

    gd->addAcousticMaterial(mat_count, mat_index, freq_count, freq_grid_index, c1, c2, c3, c4);
}

void gewSetEncoderOptions(void* instance, float quant_step, float vertex_tol, float triangle_tol,
    float mat_qnt_step, int tri_rm_nomat)
{
    GeometryData* gd = (GeometryData*)instance;

    gd->setEncoderOptions(quant_step, vertex_tol, triangle_tol, mat_qnt_step, tri_rm_nomat != 0);
}

int gewGetRequiredBufferSize(void* instance)
{
    GeometryData* gd = (GeometryData*)instance;

    return gd->getRequiredBufferSize();
}

int gewEncodeToBuffer(void* instance, int buffer_size, unsigned char* buffer_data)
{
    GeometryData* gd = (GeometryData*)instance;

    return gd->encodeToBuffer(buffer_size, buffer_data);
}

void gewDestroyInstance(void* instance)
{
    GeometryData* gd = (GeometryData*)instance;

    delete gd;
}

void gewEnableAutomatedTextDump(void* instance, char* file_prefix)
{
    GeometryData* gd = (GeometryData*)instance;

    gd->enableAutomatedTextDump(file_prefix);
}
