% This software module was originally developed by
%
% Fraunhofer IIS
%
% in the course of development of ISO/IEC 23090-4 for reference purposes
% and its performance may not have been optimized. This software module
% is an implementation of one or more tools as specified by the ISO/IEC
% 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
% non-exclusive, copyright license to copy, distribute, and make
% derivative works of this software module or modifications thereof for
% use in implementations or products claiming conformance to ISO/IEC
% standards incorporating ISO/IEC 23090-4 reference software and which
% satisfy any specified conformance criteria of that standard.
%
% Those intending to use this software module in products are advised that
% its use may infringe existing patents and that the license in the
% previous paragraph grants no licenses under such patents. ISO/IEC and
% the developers of this software module have no liability for use of this
% software module or modifications thereof.
%
% Copyright is not released for products that do not conform to an ISO/IEC
% standard.
%
% Fraunhofer IIS retains full right to modify and use the code for its own
% purpose, assign or donate the code to a third party without consent of
% and without accounting to third parties, and to inhibit third parties
% from using the code for products that do not conform to MPEG-related
% ISO/IEC International Standards.
%
% This copyright notice must be included in all copies or derivative
% works.
%
% Copyright (c) ISO/IEC 2022.

% read and visualize stage time measurement from mpeg-i cli-renderer

% filename without.txt extension
itemname = 'example';

T = readtable([itemname '.txt']);

t = [T.Update, T.ProcessBlock];
nBlocks = [T.numUpdate, T.numProcessBlock];
t = t./nBlocks;
stagenames = T.Stage;

% plot absolute times used per stage
figure
clf;hold on
bar([t;sum(t)]/1000)
box on
grid on
ylabel('avg. cpu time per stage [ms]')
legend(['update(): ' num2str(sum(t(:,1))/1000, '%1.2f') 'ms' ], ['processBlock(): ' num2str(sum(t(:,2))/1000, '%1.2f') 'ms' ], ...
        'location', 'northwest')
set(gca, 'xtick', 1:length(t)+1, 'xticklabel', [T.Stage; 'total']);
title(itemname, 'interpreter', 'none')
set(gca, 'xgrid', 'off')

% plot relative times used per stage
figure
clf;hold on
bar([t(:,1)/sum(t(:,1)), t(:,2)/sum(t(:,2))]*100)
box on
grid on
ylabel('relative distribution between stages [%]')
legend('update()', 'processBlock()', 'location', 'best')
title(itemname, 'interpreter', 'none')
set(gca, 'xtick', 1:length(t), 'xticklabel', T.Stage);

% plot estimated CPU load, based on different rates 
figure
clf;hold on
% estimate CPU load, assuming 60Hz update rate and 256 samples audio block length at Fs=48kHz
cpuLoad = [t(:,1)*60/1e6, t(:,2)*48000/256/1e6] * 100; 
bar([cpuLoad; sum(cpuLoad)], 'stacked')
box on
grid on
ylabel('estimated CPU load [%]')
legend('update()', 'processBlock()', 'location', 'best')
title(itemname, 'interpreter', 'none')
set(gca, 'xtick', 1:length(t)+1, 'xticklabel', [T.Stage; 'total']);
