% This software module was originally developed by
%
% Fraunhofer IIS
%
% in the course of development of ISO/IEC 23090-4 for reference purposes
% and its performance may not have been optimized. This software module
% is an implementation of one or more tools as specified by the ISO/IEC
% 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
% non-exclusive, copyright license to copy, distribute, and make
% derivative works of this software module or modifications thereof for
% use in implementations or products claiming conformance to ISO/IEC
% standards incorporating ISO/IEC 23090-4 reference software and which
% satisfy any specified conformance criteria of that standard.
%
% Those intending to use this software module in products are advised that
% its use may infringe existing patents and that the license in the
% previous paragraph grants no licenses under such patents. ISO/IEC and
% the developers of this software module have no liability for use of this
% software module or modifications thereof.
%
% Copyright is not released for products that do not conform to an ISO/IEC
% standard.
%
% Fraunhofer IIS retains full right to modify and use the code for its own
% purpose, assign or donate the code to a third party without consent of
% and without accounting to third parties, and to inhibit third parties
% from using the code for products that do not conform to MPEG-related
% ISO/IEC International Standards.
%
% This copyright notice must be included in all copies or derivative
% works.
%
% Copyright (c) ISO/IEC 2022.

% read and visualize frame-wise stage time measurements 
% from mpeg-i cli-renderer

function plot_item_stagetimes_framewise(itemName, renderStages)

plotFontSize = 20;
labelFontSize = 12;
isOctave = exist('OCTAVE_VERSION', 'builtin') ~= 0;
%%
B=256;Fs = 48000;

files = dir(['stagetimes_framewise' filesep itemName '*.txt']);
files2 = dir(['stagetimes_total' filesep itemName '*.txt']);

numRep = length(files);

stageTimesRep = [];

for rep = 1:numRep
    fileName = ['stagetimes_framewise' filesep files(rep).name];
    ttmp = importdata(fileName);
    stageTimesRep = cat(3, stageTimesRep, ttmp);
end

stageTimesRep(isnan(stageTimesRep)) = 0;
stageTimes = mean(stageTimesRep, 3, "omitnan");
stageNr = stageTimes(:,1);
stageThread = stageTimes(:,2);

stageTimes = stageTimes(:, 3:end);

stageTimesRep = stageTimesRep(:,3:end,:);

numStages = length(unique(stageNr));
numFrames = size(stageTimes,2);

%%
if (nargin < 2)
    if (~isempty(files2) && ~isOctave)
        T = readtable(['stagetimes_total' filesep files(1).name]);
        renderStages = T.Stage;
    else
        renderStages = arrayfun(@num2str, 1:numStages, 'UniformOutput', 0);
    end
end

%%
% plot sum of processing times
figure(1)

clf
hold on

cm = lines(64);

subplot(211)
hold on

% plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimes(1:2:end,:),1), '.', 'markersize', 5, 'color', cm(1,:))
for ctr = 1:numRep
    plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimesRep(1:2:end,:, ctr)), '.', 'markersize', 2, 'color', cm(1,:));
end

h2 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(sum(stageTimes(1:2:end,:),1),180), 'linewidth', 3, 'color', 'k');

sumUpdateTimes = sum(stageTimes(1:2:end,:),1);
avgUpdateTime = mean(sumUpdateTimes);
h3 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumUpdateTimes,180,1), 50), 'linewidth', 3, 'color', cm(5,:));
h4 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumUpdateTimes,180,99),50), 'linewidth', 3, 'color', cm(2,:));

xlabel('scene time [s]'); ylabel('processing time [us]'); grid on; box on
title([itemName ' - update()'], 'interpreter', 'none');
ylim([0 prctile(sum(stageTimesRep(1:2:end,:, ctr)),99.5)*2+1])

legend([h2, h3, h4], 'moving average', 'moving min (1% percentile)', 'moving max (99% percentile)','FontSize',20)

ax = gca;
ax.FontSize = plotFontSize;

subplot(212)
hold on
% plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimes(2:2:end,:),1), '.', 'markersize', 5, 'color', cm(1,:))
for ctr = 1:numRep
    plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimesRep(2:2:end,:, ctr)), '.', 'markersize', 2, 'color', cm(1,:));
end
h22 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(sum(stageTimes(2:2:end,:),1),180), 'linewidth', 3, 'color', 'k');

sumProcTimes = sum(stageTimes(2:2:end,:),1);
avgProcTime = mean(sumProcTimes);
h23 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumProcTimes,180,1), 90), 'linewidth', 3, 'color', cm(5,:));
h24 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumProcTimes,180,99),90), 'linewidth', 3, 'color', cm(2,:));

xlabel('scene time [s]'); ylabel('processing time [us]'); grid on; box on
title([itemName ' - processBlock()'], 'interpreter', 'none');
ylim([0 prctile(sum(stageTimesRep(2:2:end,:, ctr)),99.5)*2+1])

legend([h22, h23, h24], 'moving average', 'moving min (1% percentile)', 'moving max (99% percentile)')

ax = gca;
ax.FontSize = plotFontSize;

% [imind,cm] = rgb2ind(frame2im(getframe(gcf)),256);
% imwrite(imind,cm,['stagetimes_framewise' filesep itemName '_sum.png'],'png');
set(gcf, 'Position', get(0,'ScreenSize'));
print(gcf,['stagetimes_framewise' filesep itemName '_sum.png'],'-dpng', '-r0')

%%
% plot overall CPU load
figure(2)
clf
hold on

cm = lines(64);

subplot(211)
hold on

% plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimes(1:2:end,:),1), '.', 'markersize', 5, 'color', cm(1,:))
for ctr = 1:numRep
    plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimesRep(1:2:end,:, ctr))*60/1e6*100, '.', 'markersize', 2, 'color', cm(1,:));
end

h2 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(sum(stageTimes(1:2:end,:),1),180)*60/1e6*100, 'linewidth', 3, 'color', 'k');

sumUpdateTimes = sum(stageTimes(1:2:end,:),1);
avgUpdateTime = mean(sumUpdateTimes);
h5 = plot([0 numFrames*B/Fs], [1 1]*avgUpdateTime*60/1e6*100, 'c:', 'linewidth', 2);
h3 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumUpdateTimes,180,1), 90)*60/1e6*100, 'linewidth', 3, 'color', cm(5,:));
h4 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumUpdateTimes,180,99),90)*60/1e6*100, 'linewidth', 3, 'color', cm(2,:));

xlabel('scene time [s]'); ylabel('CPU Load [%]'); grid on; box on
title([itemName ' - update()'], 'interpreter', 'none');
ylim([0 prctile(sum(stageTimesRep(1:2:end,:, ctr)),99.5)*2+1]*60/1e6*100)

legend([h2, h3, h4, h5], 'moving average', 'moving min (1% percentile)', 'moving max (99% percentile)', ['average CPU load: ' num2str(avgUpdateTime*60/1e6*100, '%2.1f')])

ax = gca;
ax.FontSize = plotFontSize;

subplot(212)
hold on
% plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimes(2:2:end,:),1), '.', 'markersize', 5, 'color', cm(1,:))
for ctr = 1:numRep
    plot(linspace(0,numFrames*B/Fs, numFrames),sum(stageTimesRep(2:2:end,:, ctr))*48000/B/1e6*100, '.', 'markersize', 2, 'color', cm(1,:));
end
h22 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(sum(stageTimes(2:2:end,:),1),180)*48000/B/1e6*100, 'linewidth', 3, 'color', 'k');

sumProcTimes = sum(stageTimes(2:2:end,:),1);
avgProcTime = mean(sumProcTimes);
h25 = plot([0 numFrames*B/Fs], [1 1]*avgProcTime*48000/B/1e6*100, 'c:', 'linewidth', 2);
h23 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumProcTimes,180,1), 50)*48000/B/1e6*100, 'linewidth', 3, 'color', cm(5,:));
h24 = plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(sumProcTimes,180,99),180)*48000/B/1e6*100, 'linewidth', 3, 'color', cm(2,:));

xlabel('scene time [s]'); ylabel('CPU Load [%]'); grid on; box on
title([itemName ' - processBlock()'], 'interpreter', 'none');
ylim([0 prctile(sum(stageTimesRep(2:2:end,:, ctr)),99.5)*2+1]*48000/B/1e6*100)

legend([h22, h23, h24, h25], 'moving average', 'moving min (1% percentile)', 'moving max (99% percentile)', ['average CPU load: ' num2str(avgProcTime*48000/B/1e6*100, '%2.1f')])

ax = gca;
ax.FontSize = plotFontSize;

set(gcf, 'Position', get(0,'ScreenSize'));
print(gcf,['stagetimes_framewise' filesep itemName '_cpu_sum.png'],'-dpng', '-r0')

%%
% plot area graph of accumulated stage times
figure(3)
clf
hold on

stageTimesMovMean = zeros(size(stageTimes));
for ctr = 1:size(stageTimes,1)
    stageTimesMovMean(ctr,:) = movmean(stageTimes(ctr,:), 180);
end

cm = turbo(numStages);
subplot(211); hold on

if (~isOctave)
    % low res area plot to avoid performance issues in octave
    nPoints = 6*60;
    xLowres = linspace(0,numFrames*B/Fs, nPoints);
    yLowres = interp1(linspace(0,numFrames*B/Fs, numFrames),stageTimesMovMean(1:2:end,:)'*60/1e6*100, xLowres);
    area(xLowres, yLowres);
else
    area(linspace(0,numFrames*B/Fs, numFrames),stageTimesMovMean(1:2:end,:)'*60/1e6*100);
end


xlabel('scene time [s]'); ylabel('CPU Load [%]'); grid on; box on
title([itemName ' - update()'], 'interpreter', 'none');
ylim([0 prctile(sum(stageTimes(1:2:end,:)),99)*1.8+1]*60/1e6*100)
set(gca,'ColorOrder',cm);

ax = gca;
ax.FontSize = plotFontSize;

legend(renderStages, 'numColumns', 4, 'FontSize',labelFontSize)

subplot(212); hold on

if (~isOctave)
    % low res area plot to avoid performance issues in octave
    nPoints = 6*60;
    xLowres = linspace(0,numFrames*B/Fs, nPoints);
    yLowres = interp1(linspace(0,numFrames*B/Fs, numFrames),stageTimesMovMean(2:2:end,:)'*48000/B/1e6*100, xLowres);
    area(xLowres, yLowres);
else
    area(linspace(0,numFrames*B/Fs, numFrames),stageTimesMovMean(2:2:end,:)'*48000/B/1e6*100);
end

xlabel('scene time [s]'); ylabel('CPU Load [%]'); grid on; box on
title([itemName ' - processBlock()'], 'interpreter', 'none');
ylim([0 prctile(sum(stageTimes(2:2:end,:)),99)*1.8+1]*48000/B/1e6*100)

ax = gca;
ax.FontSize = plotFontSize;

set(gca,'ColorOrder',cm);
legend(renderStages, 'numColumns', 4,'FontSize', labelFontSize)
set(gcf, 'Position', get(0,'ScreenSize'));
print(gcf,['stagetimes_framewise' filesep itemName '_cpu_area.png'],'-dpng', '-r0')


%%
figure(4)
clf
cm = lines(64);
if(isOctave)
    numRows = 5;
    numCols = ceil(numStages/5);
else
    % tiled layout has nicer spacing than subplot, but is not supported by octave
    tiledlayout('flow','TileSpacing', 'tight', 'Padding', 'compact')
end

for ctr = 1:numStages
    if (isOctave)
        subplot(numRows,numCols,ctr)
    else
        nexttile
    end
    hold on
    idx = find(stageNr == ctr-1);

    if (~isOctave) % octave seems to have performance issues with that many scatter plots
        for ctrR = 1:numRep
            plot(linspace(0,numFrames*B/Fs, numFrames),squeeze(stageTimesRep(idx(1),:, ctrR))*60/1e6*100, '.', 'markersize', 3, 'color', cm(1,:))
        end
    end
    plot(linspace(0,numFrames*B/Fs, numFrames),movmean(stageTimes(idx(1), :), 180)*60/1e6*100, '-', 'linewidth', 2, 'Color', 'k')
    plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(stageTimes(idx(1),:),180,1), 90)*60/1e6*100, 'linewidth', 2, 'color', cm(5,:));
    plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(stageTimes(idx(1),:),180,99),90)*60/1e6*100, 'linewidth', 2, 'color', cm(2,:));

    xlabel('scene time [s]')
    ylabel('CPU Load [%]')

    title([renderStages{ctr} ' - update'], 'FontSize', labelFontSize);
    box on
    grid on
    axis tight
    ylim([0 prctile(stageTimes(idx(1), :),99.5)*2+1]*60/1e6*100)
end
set(gcf, 'Position', get(0,'ScreenSize'));
print(gcf,['stagetimes_framewise' filesep itemName '_cpu_update.png'],'-dpng', '-r0')

%%
figure(5)
clf
cm = lines(64);
if(isOctave)
    numRows = 5;
    numCols = ceil(numStages/5);
else
    % tiled layout has nicer spacing than subplot, but is not supported by octave
    tiledlayout('flow','TileSpacing', 'tight', 'Padding', 'compact')
end

for ctr = 1:numStages
    if (isOctave)
        subplot(numRows,numCols,ctr)
    else
        nexttile
    end
    hold on
    idx = find(stageNr == ctr-1);

    if (~isOctave) % octave seems to have performance issues with that many scatter plots
        for ctrR = 1:numRep
            plot(linspace(0,numFrames*B/Fs, numFrames),squeeze(stageTimesRep(idx(2),:, ctrR))*48000/B/1e6*100, '.', 'markersize', 3, 'color', cm(1,:))
        end
    end

    plot(linspace(0,numFrames*B/Fs, numFrames),movmean(stageTimes(idx(2), :), 180)*48000/B/1e6*100, '-', 'linewidth', 2, 'Color', 'k')
    plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(stageTimes(idx(2), :),180,1), 90)*48000/B/1e6*100, 'linewidth', 2, 'color', cm(5,:));
    plot(linspace(0,numFrames*B/Fs, numFrames),movmean(movprctile(stageTimes(idx(2), :),180,99),90)*48000/B/1e6*100, 'linewidth', 2, 'color', cm(2,:));
    
    xlabel('scene time [s]')
    ylabel('CPU Load [%]')
    
    title([renderStages{ctr} ' - process'],'FontSize', labelFontSize);
    box on
    grid on
    axis tight
    ylim([0 prctile(stageTimes(idx(2), :),99.5)*2+1]*48000/B/1e6*100)
end
set(gcf, 'Position', get(0,'ScreenSize'));
print(gcf,['stagetimes_framewise' filesep itemName '_cpu_process.png'],'-dpng', '-r0')

end

%%
function y = movprctile(x, wnd_len, perc)

x = x(:);
input_len = length(x);
xPad = [flipud(x(1:wnd_len/2)); x; flipud(x(end-wnd_len/2:end))];

shift_mat = zeros(wnd_len, input_len);
for n = 1:wnd_len
    shift_mat(n,:) = xPad(n:n+input_len-1);
end

sort_mat = sort(shift_mat);

idx = round(perc/100*wnd_len);
idx = max(idx,1);
y = sort_mat(idx, :);
end

%%
function y = movmean(x, wnd_len)

x = x(:);
input_len = length(x);
xPad = [flipud(x(1:wnd_len/2)); x; flipud(x(end-wnd_len/2:end))];

shift_mat = zeros(wnd_len, input_len);
for n = 1:wnd_len
    shift_mat(n,:) = xPad(n:n+input_len-1);
end

y = mean(shift_mat);
end