# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.
#
# convert trajectory format .log->.osc (AEP -> cli renderer)

import sys
import os

def myFunc(inputPath, inputFileName):

    inFilename = os.path.join(inputPath, inputFileName)
    outFilename = os.path.join('convertedTrajectories', os.path.splitext(os.path.basename(inFilename))[0] + '.osc')

    try:
        # Open input and output files
        with open(inFilename, 'r') as inputFile, open(outFilename, 'w') as outputFile:
            firstLine = True
            t = 0
            count = 0
            aT = 0
            newRecord = False
            
            # loop for reading line by line
            for line in inputFile:
                if firstLine:
                    firstLine = False
                    continue
                
                # check if there is a new recording to adjust the time
                if "/startVisuals" in line:
                    newRecord = True
                
                # take only lines that contain OSC_IN
                if "OSC_IN" in line:
                    # divide the line in time and the rest of the line
                    time, rest, values = line[11:].strip().split(",")[:3]
                    hh, mm, ss = map(float, time.strip().split(":"))
                    secs = hh * 3600 + mm * 60 + ss

                    if newRecord:
                        t += 0.01
                        aT = secs
                        newRecord = False
                    else:
                        if count == 0:
                            aT = secs
                        else:
                            t += secs - aT
                            aT = secs

                    if "/userPose" in line:
                        outputFile.write(f"Record(t={t},\nMessage(\"/userPose\",\n")
                        tokens = values.strip().split(" ")[1:]
                        for token in tokens:
                            outputFile.write(f"   float: {token},\n")
                        outputFile.write(")\n)\n")
                    
                    if "/interactionEvent" in line:
                        outputFile.write(f"Record(t={t},\nMessage(\"/interactionEvent\",\n")
                        tokens = values.strip().split(" ")[1:]
                        for i, token in enumerate(tokens):
                            if i == 0:
                                outputFile.write(f"   int: {token},\n")
                            else:
                                outputFile.write(f"   float: {token},\n")
                        outputFile.write(")\n)\n")
                    
                    count += 1

        print("\nTrajectory file converted from '.log' to '.osc'")

    except IOError as e:
        print("Failed to open file:", str(e))
        sys.exit(1)