// ----------- implementation ---------------------------------------
// pinkgen.cpp

#include "PinkNoise.h"
#include <cstdlib>

// Static class data
long int const PinkNoise::pA[5] = { 14055, 12759, 10733, 12273, 15716 };
short int const PinkNoise::pPSUM[5] = { 22347, 27917, 29523, 29942, 30007 };

// Set to zero state.
void PinkNoise::pinkclear()
{
    int i;
    for (i = 0; i < 5; ++i) {
        contrib[i] = 0L;
    }
    accum = 0L;
}

void PinkNoise::internal_clear(unsigned int seed)
{
    pinkclear();
    generator.seed(seed); // Setting the seed also resets the generator state
}

PinkNoise::PinkNoise(unsigned int seed)
{
    internal_clear(seed);
}

// Coding artifact for convenience. This obscures the fact that the
// 'break' violates the ordinary rules of well-structured C++ code!
// An exception could provide a better structure, but the overhead
// is not acceptable.
#define UPDATE_CONTRIB(n)                                                                          \
    {                                                                                              \
        accum -= contrib[n];                                                                       \
        contrib[n] = (long)randv * pA[n];                                                          \
        accum += contrib[n];                                                                       \
        break;                                                                                     \
    }

// Evaluate next randomized 'pink' number with uniform CPU loading.
short int PinkNoise::pinkrand()
{
    short int randu = std::abs(ugen(generator));
    //short int  randu = ugen.urand() & 0x7fff;     // U[0,32767]
    short int randv = ugen(generator);
    //short int  randv = (short int) ugen.urand();  // U[-32768,32767]

    // Structured block, at most one update is performed
    while (1) {
        if (randu < pPSUM[0])
            UPDATE_CONTRIB(0);
        if (randu < pPSUM[1])
            UPDATE_CONTRIB(1);
        if (randu < pPSUM[2])
            UPDATE_CONTRIB(2);
        if (randu < pPSUM[3])
            UPDATE_CONTRIB(3);
        if (randu < pPSUM[4])
            UPDATE_CONTRIB(4);
        break;
    }
    return (short int)(accum >> 16);
}

float PinkNoise::tick()
{
    return float(pinkrand()) / 32767.f;
}