// Pink noise class using the autocorrelated generator method.
// Method proposed and described by Larry Trammell "the RidgeRat" --
// see http://www.ridgerat-tech.us/pink/newpink.htm
//
// This code is submitted to the public domain -- USE AT YOUR OWN RISK.
//
// ------------------------------------------------------------------
//
// This is a canonical, 16-bit fixed-point implementation of the
// generator in 32-bit arithmetic. There are only a few system
// dependencies.
//
//   -- access to an allocator 'malloc' for operator new
//   -- access to definition of 'size_t'
//   -- assumes 32-bit two's complement arithmetic
//   -- assumes long int is 32 bits, short int is 16 bits
//   -- assumes that signed right shift propagates the sign bit
//
// It needs a separate URand class to provide uniform 16-bit random
// numbers on interval [1,65535]. The assumed class must provide
// methods to query and set the current seed value, establish a
// scrambled initial seed value, and evaluate uniform random values.

#pragma once

#include <stddef.h>

#include <random>

class PinkNoise
{
private:
    // Coefficients (fixed)
    static long int const pA[5];
    static short int const pPSUM[5];

    // Internal pink generator state
    long int contrib[5]; // stage contributions
    long int accum; // combined generators
    void internal_clear(unsigned int seed);

    // Include a UNoise component
    std::default_random_engine generator;
    std::uniform_int_distribution<short int> ugen{ -32768, 32767 };

public:
    PinkNoise(unsigned int seed);
    void pinkclear();
    short int pinkrand();

    // Convenience function to get float value. Calls pinkrand and converts to float.
    float tick();
};
