/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#pragma once
#include <MPEGI/Bitstream/Bitstream.h>
#include <MPEGI/Helpers/AirflowSource.h>
#include <unordered_map>
#include <vector>

//MountainCabin_simple
//Airflow Profile Parameters
std::array<float, 24> MountainCabin_FreqGrid_0{ 125, 250, 500, 1000, 2000, 4000 };
std::array<float, 24> MountainCabin_FreqGrid_1{ 125, 250, 500, 1000, 2000, 4000, 8000 };

std::vector<std::pair<float, float>> profile_gain{ std::make_pair(-24.0f, 0.f),
    std::make_pair(-22.0f, 0.f), std::make_pair(-20.0f, 0.f), std::make_pair(-20.0f, 0.f),
    std::make_pair(-22.0f, 0.f), std::make_pair(-24.0f, 0.f) };
std::vector<std::pair<float, float>> profile_q;
const std::vector<MPEGI::AirflowProfileParameters> MountainCabin_Gain_Profile(
    { MPEGI::AirflowProfileParameters(MPEGI::AirflowProfileParameters::ProfileType::Freq_Gain_Pairs,
        1u, MPEGI::AirflowProfileParameters::ProfileModification::speed, 1.95503426f, profile_gain,
        profile_q) });

std::vector<std::pair<float, float>> azi1_profile_gain{ std::make_pair(-24.0f, -20.f),
    std::make_pair(-22.0f, -18.f), std::make_pair(-20.0f, -16.f), std::make_pair(-20.0f, -16.f),
    std::make_pair(-22.0f, -18.f), std::make_pair(-24.0f, -20.f) };
std::vector<std::pair<float, float>> azi1_profile_q{ std::make_pair(1.9685f, 1.9685f),
    std::make_pair(1.9685f, 1.9685f), std::make_pair(1.9685f, 1.9685f),
    std::make_pair(1.9685f, 1.9685f), std::make_pair(1.9685f, 1.9685f),
    std::make_pair(1.9685f, 1.9685f) };
const std::vector<MPEGI::AirflowProfileParameters> MountainCabin_Azimuth_Profile(
    { MPEGI::AirflowProfileParameters(MPEGI::AirflowProfileParameters::ProfileType::Filter_Params,
        1u, MPEGI::AirflowProfileParameters::ProfileModification::azimuth, 330.0f,
        azi1_profile_gain, azi1_profile_q) });

//Airflow Source Parameters
const std::string generatorStringID = "globalAirflow";
unsigned int id = 6;
unsigned int generatorID = 7;
unsigned int defaultProfile = 0;
bool hasValidRegions = true;
std::vector<MPEGI::AirflowProfileParameters> customAirflowProfiles;
const bool activeInDefaultRegion = true;
std::vector<int> regionIDs{ 0u };

const MPEGI::AirflowSource MountainCabin_Airflow(id, generatorID, generatorStringID, defaultProfile,
    hasValidRegions, customAirflowProfiles, activeInDefaultRegion, regionIDs);
