/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DEBUG_PRINTING_STAGE
#define MPEGI_DEBUG_PRINTING_STAGE

#include <MPEGI/Renderer/RenderList.h>
#include <MPEGI/Renderer/RenderStage.h>

namespace MPEGI {

//! Print information about render items
/*! 
 * Inputs: 
 * - All item types
 *  
 * Outputs: 
 * - unchanged
 *  
 * How it works: 
 * - Prints debug information about the render items and stage function calls to stdout
 * 
 * Stages order:
 * - Required before: none
 * - Required after: none
 *  
 * Config options: 
 * - none
 *  
 */
class DebugPrintingStage : public RenderStage
{
public:
    std::unique_ptr<RenderStage::DspData> update(RenderList& renderList) override;
    void processBlock(DspContext ctx) override;

private:
    void initializeStage(const Bitstream& bitstream, RenderItemStore* itemStore,
        const StreamManager* streamManager, const SceneState* state) override;

    size_t processCounter = 1; // RenderList::id also starts at 1
};

} // namespace MPEGI

#endif /* MPEGI_DEBUG_PRINTING_STAGE */
