/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_ARBITRARYAXISROTATION
#define MPEGI_ARBITRARYAXISROTATION

#include <EAR/Exception.h>

#include <MPEGI/Helpers/Config.h>

namespace MPEGI {

// Possible values of one of input argument of ArbitraryAxisRotation::setRotationMatrix(): CoordAxis
enum class RotatingAxis
{
    X_AXIS = int(0),
    Y_AXIS,
    Z_AXIS
};

// Arbitrary axis rotation
class ArbitraryAxisRotation
{

public:
    // Constructor
    ArbitraryAxisRotation(void);

    // Destructor
    ~ArbitraryAxisRotation(void);

    // Set the matrixIndex-th matrix as identity
    void setMatrixIdentity(int matrixIndex);

    // Set all matrices as identity
    void setAllMatricesAsIdentity();

    // Set translation matrix and its inverse from a given translation vector
    void setTranslationMatrix(EAR::Vector3 translationVector);

    // Set rotation matrix and its inverse from a given rotating axis vector, axis flag and rotating angle
    void setRotationMatrix(EAR::Vector3 rotatingAxis, RotatingAxis CoordAxis, float alpha = 0.f);

    // Muliply matrix_transform matrix with matrix_phantom[matrixIndex]
    void MatMul(int matrixIndex);

    // Return transformed vector of p by multiplication with matrix_phantom[index]
    EAR::Vector3 MatVecMul(EAR::Vector3 p, int index);

    // Return transformation matrix
    void getTransformationMatrix(int phantomMatrixLastIndex);

private:
    // Dim: 7 x 4 x 4 matrix
    float matrix_phantom[7][4][4] = {
        0.f,
    }; // [1][][]: D, [2]: Rx, [3]: Ry, [4]: Rz, [5]:inv(Ry), [6]: inv(Rx) [7]: inv(D)

    // 4 x 4 transform matrix
    float matrix_transform[4][4] = {
        0.f,
    };
};

} // namespace MPEGI

#endif /* MPEGI_ARBITRARYAXISROTATION */
