/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DETAIL
#define MPEGI_DETAIL

#include <EAR/Geometry.h>

#include <MPEGI/Helpers/Config.h>
#include <MPEGI/Renderer/Udt.h>

namespace MPEGI {
namespace detail {

//! A-weights in the one-third octave band
//! usage: for audibility of valid diffraction paths
static const float Diffraction_A_Weighting[] = { -56.7, -50.5, -44.7, -39.4, -34.6, -30.2, -26.2,
    -22.5, -19.1, -16.1, -13.4, -10.9, -8.6, -6.6, -4.8, -3.2, -1.9, -0.8, 0, 0.6, 1, 1.2, 1.3, 1.2,
    1, 0.5, -0.1, -1.1, -2.5, -4.3, -6.6 };

//! IS_OCCLUDED_BY_A_GIVEN_MESH
/*!
		* \usage:							this function returns true if the specifically given mesh is the occluder between origin and target
											otherwise false is returned
		* \param tracer:
											type: EmbreeTracer
											EmbreeTracer initiated by all the meshes in the scene
		* \param origin:
											type: EAR::Vector3
											the point where a ray will start
		* \param target:
											type: EAR::Vector3
		* \param givenMesh:
											type: EAR::Mesh
											a mesh of interest
		* \param return:
											type: bool
											true if the specifically given mesh is the occluder, false otherwise
		*/
bool isOccludedByAGivenMesh(EAR::EmbreeTracer* tracer, const EAR::Vector3& origin,
    const EAR::Vector3& target, const EAR::Mesh* givenMesh);
} // namespace detail
} // namespace MPEGI

#endif /* MPEGI_DETAIL */
