/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_LISTENER_STRUCT
#define MPEGI_LISTENER_STRUCT

#include <MPEGI/Helpers/Types.h>
#include <MPEGI/Renderer/RenderList.h>

#include <EAR/Exception.h>
#include <EAR/Geometry.h>

#include <MPEGI/Bitstream/DiffractionPayload.h>

namespace MPEGI {

// Listener structure for diffraction
struct Listener
{
    // Listener location (EAR::Vector3)
    EAR::Vector3 listenerLocation;

    // Listener orientation (Euler)
    EAR::Euler listenerOrientation;

    // Direct path occluder kept here to deal with dynamic geometries
    const EAR::Mesh* listenerOccluder{ nullptr };

    // Flag which indicates if this listener structure is intialized or not
    bool listenerInitialized{ false }; // in order not to set the unchangeable parameters

    // Flag which indicates if this listesner relocates from the previous frame time
    bool listenerRelocated{ true };

    // A vector of visible edge id
    std::vector<int> visibleEdgeIdList;

    // Flag which indicates if visibleEdgeIdList is updated in the current time frame
    bool isUpdated;

    // Update the listener position
    void update(RenderList::ListenerState&);
};

} // namespace MPEGI

#endif /* MPEGI_LISTENER_STRUCT */
