/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_PATH_TO_VALIDATE
#define MPEGI_PATH_TO_VALIDATE

#include <EAR/Exception.h>
#include <EAR/Geometry.h>

namespace MPEGI {

//! VALIDATED PATH
/*!
	* \usage: this class provides the additional information for valid path which will be rendered in RT
	*/
class PathToValidate
{
public:
    //! Constructor
    PathToValidate(){};

    //! Destructor
    ~PathToValidate() {}

public:
    //! Diffraction path type: FO, HO, HHO, UNKNOWN
    DiffractionPathType pathType;

    //! diffraction angle along the first edge
    float diffractionAngleAtSource;

    //! diffraction angle along the final edge
    float diffractionAngleAtListener;

    //! the first edge ID in this path
    int initialEdgeID;

    //! the last edge ID in this path
    int lastEdgeID;

    //! local path index to fetch a diffraction path information from three different std::map containers
    //! used as the key to find the diffraction paths (by edges, triangles and angles) in map container
    float localPathIndex;

    //! Flag which indicates if this path is finally fetched into the spatializer or not
    bool activated{ false };

    //! propagation distance from source to the listener through this path
    float distance;

    //! Bitstream Id of direct path occluding geometry
    int geometryBitsId{ -1 };

    //! Mesh which occludes the propagation path along the precomputed diffraction path
    std::vector<const EAR::Mesh*> intermediatePathOccluder;

    //! Indicates which pair of edges is occluded by the indirectPathOccluder
    std::vector<int> whichPathOccluded; // int: tells which edge is occluded
    //-----------------------------------------------------------------------------------------------------------

private:
};

} // namespace MPEGI

#endif /* MPEGI_PATH_TO_VALIDATE */
