/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_SOURCE_STRUCT
#define MPEGI_SOURCE_STRUCT

#include <EAR/Exception.h>
#include <EAR/Geometry.h>

#include <MPEGI/Bitstream/DiffractionPayload.h>
#include <MPEGI/Helpers/Types.h>

#include "Definitions.h"
#include "ListenerStruct.h"

namespace MPEGI {

// Source structure for diffraction
struct Source
{
    // Constructor
    Source(void)
        : listener(nullptr){};

    Source(MPEGI::Listener* _listener)
        : listener(_listener)
    {}

    // Listener pointer
    MPEGI::Listener* listener;

    // Id of the source
    int sourceID;

    // Flag which indicates if this source relocates from the previous frame time
    bool sourceRelocated;

    // Source location (EAR::Vector3)
    EAR::Vector3 sourceLocation;

    // Source location (EAR::Vector3)
    EAR::Vector3 prevLocation;

    // Source speed
    float speed;

    // Source orientation (EAR::Euler)
    EAR::Euler sourceOrientation;

    // Source name in the string type
    std::string sourceObjectName;

    // Source status
    ItemStatus sourceStatus;

    // Source type
    ItemType sourceType;

    // Occluder between imaginary direct path between a source and the listener
    const EAR::Mesh* sourceOccluder{ nullptr };

    // Flag which indicates if this source structure is intialized or not
    bool isSourceInitialized{ false }; // in order not to set the unchangeable parameters

    // A vector of visible edge ids per source
    std::vector<int> visibleEdgeIdList;

    // A vector of path indices per source
    std::vector<std::pair<int, int>> pathIndexList;

    // IsOccluded and isOcclusionStageChanged flags for heavy/light updates
    bool isPreviouslyOccluded;
    bool isOcclusionStateChanged;

    // Flag which indicates this source is a type of extended source
    bool isExtendedSource;

    // Update source attributes
    Source& update(const RenderItem& _item, float B, float Fs);
};
} // namespace MPEGI

#endif /* MPEGI_SOURCE_STRUCT */
