/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DIRECTIVITY_STAGE
#define MPEGI_DIRECTIVITY_STAGE

#include <MPEGI/Bitstream/DirectivityPayload.h>
#include <MPEGI/Renderer/RenderList.h>
#include <MPEGI/Renderer/RenderStage.h>
#include <MPEGI/Renderer/RenderStageRegistry.h>

#include <map>
#include <vector>

namespace MPEGI {

//! Add EQ to render items based on their (optional) directivity
/*! 
 * Inputs: 
 * - Items with ItemType::Primary, ItemType::Representative, ItemType::Reflection or ItemType::Diffraction
 *  
 * Outputs: 
 * - Same RIs, but with potentially updated EQ field
 *  
 * How it works: 
 * - The angle between source and listener is calculated
 *   (for reflections, this should be the angle of a specular ray exiting the source, being reflected and then arriving at the listener)
 * - Based on this angle, the corresponding EQ is looked up in the RIs directivity data
 * - The directivity EQ is scaled by the "directiveness" field
 * 
 * Stages order:
 * - Required before: none
 * - Required after: EqStage
 *  
 * Config options: 
 * - none
 * 
 */
class DirectivityStage : public RenderStage
{
public:
    DirectivityStage();

    //! Destructor
    ~DirectivityStage();

    //! Inherited from RenderStage
    std::unique_ptr<RenderStage::DspData> update(RenderList& renderList) override;

private:
    friend class Renderer;

    //! Inherited from RenderStage
    void initializeStage(const Bitstream& bitstream, RenderItemStore* itemStore,
        const StreamManager* streamManager, const SceneState* state) override;

    struct ItemState : RenderItem::ExtraState
    {
        ItemState()
            : magnitudes(Config::getInt("EQNBands"))
        {}

        std::vector<float> magnitudes;
    };
};

} // namespace MPEGI

#endif /* MPEGI_DIRECTIVITY_STAGE */
