/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_LC_EARLY_REFLECTION_PATTERNPRODUCER
#define MPEGI_LC_EARLY_REFLECTION_PATTERNPRODUCER

#include <EAR/BaseTypes.h>

#include <mutex>
#include <optional>

#include <MPEGI/Helpers/Config.h>
#include <MPEGI/Helpers/ConfigObserver.h>

#include "LCERConfig.h"

namespace MPEGI {

class LCERPatternProducer : public ConfigObserver
{
public:
    static constexpr float PI{ 3.14159265358979323846f };

    struct ERPos
    {
        float azimuth; //in radians
        float elevation; //in radians
        float radius;
        float amplitude;

        float distReducFactor;

        EAR::Vector3 globalPosition;

        inline EAR::Polar polar()
        {
            return EAR::Polar(-azimuth * 180.0f / PI, elevation * 180.0f / PI, radius);
        }
    };

    struct Pattern
    {
        std::vector<ERPos> erpos;

        inline friend std::ostream& operator<<(std::ostream& os, const Pattern& sr)
        {
            os << "azimuth (rad), elevation (rad), radius, amplitude" << std::endl;
            for (auto& entry : sr.erpos) {
                os << std::to_string(entry.azimuth) << "," << std::to_string(entry.elevation) << ","
                   << std::to_string(entry.radius) << "," << std::to_string(entry.amplitude)
                   << std::endl;
            }
            os << std::endl;
            return os;
        }
    };

    //! Constructor
    LCERPatternProducer(LCERConfig* config);

    //! Destructor
    ~LCERPatternProducer();

    void updateConfig(LCERConfig* config, bool resetEnvCrossfadeFactor = true);

    //! process
    /*!
     * \param listenerLoc      current location of the listener in WCS
     * \param itemLoc          current location of the item in WCS
     */
    Pattern process(EAR::Vector3 listenerLoc, EAR::Vector3 itemLoc);

    void applyEnvCrossfadeFactor(float factor);

    size_t getNumberOfEarlyReflectionItems();

    //! Inherited from ConfigObserver
    void configValueChanged(const std::string& key) override;

private:
    Pattern spiral(EAR::Vector3 listenerLoc, EAR::Vector3 itemLoc);
    Pattern outdoor(EAR::Vector3 listenerLoc, EAR::Vector3 itemLoc);

private:
    LCERConfig* config;

    std::mutex changeParamsMutex;

    float envCrossfadeFactor;
};

} // namespace MPEGI

#endif // MPEGI_LC_EARLY_REFLECTION_PATTERNPRODUCER
