/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include "OneOverRDistance.h"

#include <cmath>

OneOverRDistance::OneOverRDistance(float _dampingStart, float roStart, float roEnd, float _alpha)
    : dampingStart(_dampingStart)
    , rollOffStart(roStart)
    , rollOffEnd(roEnd)
    , alpha(_alpha)
{}

float OneOverRDistance::getGainForDistance(float dist, float refDist)
{
    float gain = 0.f; // outside of rendering area

    // make sure that refererence distance is never smaller than the minimum distance
    if (refDist < dampingStart) {
        refDist = dampingStart;
    }

    if (dist < dampingStart) { // closer than the minimum distance
        // avoiding division by zero with a constant fraction expansion
        gain = std::pow((refDist + 0.0001f) / (dampingStart + 0.0001f), alpha);
    } else if (dist < rollOffStart) { // default 1/r case
        // avoiding division by zero with a constant fraction expansion
        gain = std::pow((refDist + 0.0001f) / (dist + 0.0001f), alpha);
    } else if (dist < rollOffEnd) { // roll-off
        float borderGain = std::pow((refDist + 0.0001f) / (rollOffStart + 0.0001f), alpha);

        if (borderGain > 1.f) {
            borderGain = 1.f;
        }

        // the gain in the roll off range is linearly going towards 0
        gain = borderGain * ((rollOffEnd - dist + 0.0001f) / (rollOffEnd - rollOffStart + 0.0001f));
    }

    return gain;
}
