/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

//! 1/r distance model
/*!
 * The 1/r model assumes that sources decay with gain 1/r, where r is the distance between source and receiver. This model handles all frequencies equally (AFAB - all frequencies are beautiful).
 *
 * 1/r theoretically allows very close sources to be infinitely loud and far away source to be still audible,
 * so we limit the gain to 1 and have a roll-off distance range, where sources are ramped to zero gain.
 */
class OneOverRDistance
{
public:
    //! Constructor.
    /*!
     * \param _dampingStart  the radius of the area where no damping is applied
     * \param roStart        distance where roll-off begins
     * \param roEnd          distance from where a source is set to zero -> inaudible
     * \param _alpha         alpha is the exponent in the equation (1/r)^a; the higher the alpha, the steeper attenuation curve
     */
    OneOverRDistance(float _dampingStart, float roStart, float roEnd, float _alpha);

    //! returns the 1/r gain within the pre-defined bounds
    /*!
     * \param dist      distance between listener and source [m]
     * \param refDist   reference distance, where gain shall be 1 [m]
     */
    float getGainForDistance(float dist, float refDist = 1.f);

private:
    const float dampingStart, rollOffStart, rollOffEnd, alpha;
};
