/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2023.
***************************************************************************/

#include "PortalTest.h"

// Implement Dummy classes needed for PortalTest

namespace MPEGI {

DummyBitstream::DummyBitstream(std::shared_ptr<DummyPortalPayload> pp)
{
    portalPayloads.push_back(pp);
}

void DummyBitstream::addPortalPayload(std::shared_ptr<DummyPortalPayload> pp)
{
    portalPayloads.push_back(pp);
}

std::vector<std::shared_ptr<const Bitstream::Payload>> DummyBitstream::getAllPayloadsOfType(
    PayloadType type) const
{
    switch (type) {
    case PLD_PORTAL:
        return portalPayloads;
    default:
        auto emptyPayload = std::vector<std::shared_ptr<const Bitstream::Payload>>();
        return emptyPayload;
    }
}

std::shared_ptr<const Bitstream::Payload> DummyBitstream::getOnlyPayloadOfType(
    PayloadType type) const
{
    if (type == PLD_PORTAL && portalPayloads.size() == 1) {
        return portalPayloads.front();
    }
    auto pldAsString = std::string{ payloadTypeToString(type) };
    EAR_FAIL("Zero payloads found for requested type: " + pldAsString);

    return nullptr;
}

void DummyBitstream::clearPayloads()
{
    portalPayloads.clear();
}

DummySceneState::DummySceneState(bool hasDefaultAe)
{
    if (hasDefaultAe) {
        addAcousticEnvironment(DEFAULT_AE_ID_NAME, nullptr);
    }
}

std::string DummySceneState::addMaterial(float transmissionCoeff, float couplingCoeff)
{
    const std::vector<float> thirdOctaveCenterFreqs{ MPEGI::Config::getFloatPtr("EQCenterFreqs"),
        MPEGI::Config::getFloatPtr("EQCenterFreqs") + MPEGI::Config::getInt("EQNBands") };

    EIFPayload::AcousticMaterialDesc description;
    for (const auto& f : thirdOctaveCenterFreqs) {
        EIFPayload::AcousticMaterialDesc::FrequencyDesc freqDesc;
        freqDesc.f = f;
        freqDesc.t = transmissionCoeff;
        freqDesc.c = couplingCoeff;
        description.frequencies.push_back(freqDesc);
    }

    description.bsId = materialBsId++;
    const std::string id = "DummyMaterial" + std::to_string(description.bsId);
    description.id = id;
    createMaterial(description);

    return id;
}

void DummySceneState::addAcousticEnvironment(
    std::optional<std::string> aeId, const EAR::Geometry* enclosure)
{
    ReverbEnv env = ReverbEnv(aeId.value_or("0"), enclosure);
    env.isLsdf = false;
    env.isSceneBasedAE = true;
    env.predelay = 0;
    if (aeId.has_value()) {
        if (aeId.value() == DEFAULT_AE_ID_NAME) {
            env.acousticEnvironmentBsId = DEFAULT_AE_ID;
        } else {
            env.acousticEnvironmentBsId = std::stoi(aeId.value_or("0"));
        }
    }

    dummyReverbEnvs.push_back(std::make_unique<ReverbEnv>(std::move(env)));
}

const std::vector<std::unique_ptr<ReverbEnv>>& DummySceneState::getReverbEnvs() const
{
    return dummyReverbEnvs;
}

// Create public method from protected method of base class
void DummySceneState::createBox2(const EIFPayload::BoxDesc& boxDesc)
{
    SceneState::createBox(boxDesc);
}

// Create public method from protected method of base class
void DummySceneState::createPortal2(MPEGI::Portal* portal)
{
    SceneState::createPortal(portal);
}

} // namespace MPEGI
