/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_DIRECTIVITY_SPECTRUM
#define MPEGI_DIRECTIVITY_SPECTRUM

#include <MPEGI/DirectivityCoding.h>

namespace MPEGI {

struct DirectivitySpectrum
{
    const float one_over_180 = 1.0f / 180.0f;

    std::vector<float> coefficients;
    std::vector<float> frequencies;

    DirectivitySpectrum(const DirectivityCoverList& coverList)
    {
        const auto& list = coverList.getDirectivityCoverList();
        coefficients.resize(list.size());
        frequencies.resize(list.size());

        for (int freqIdx = 0; freqIdx < list.size(); freqIdx++) {
            float meanDirGain = 0.0f;
            unsigned int nrCovers = 0;
            const auto& dirCover = (*list[freqIdx]);

            for (int eleIdx = 0; eleIdx < dirCover.el_cnt; eleIdx++) {
                float deltaEle = M_PI / (dirCover.el_cnt - 1);
                if (eleIdx == 0 || eleIdx == dirCover.el_cnt - 1) {
                    deltaEle /= 2.0f;
                }
                for (int aziIdx = 0; aziIdx < dirCover.az_cnt[eleIdx]; aziIdx++) {
                    float deltaAzi = 2 * M_PI / dirCover.az_cnt[eleIdx];

                    float elevation =
                        M_PI * (float)eleIdx / (float)(dirCover.el_cnt - 1) - (0.5 * M_PI);
                    // Calculating the surface area of this cover on the sphere
                    float coverSurfaceWeight =
                        std::max(std::cos(elevation) * deltaAzi * deltaEle, 0.0f);

                    int coverIdx = eleIdx * dirCover.cover_width + aziIdx;
                    // meanDirGain += coverSurfaceWeight; // Test case: Should sum up to approximately 4 * pi
                    // meanDirGain += 10^(cover/10) * coverSurfaceWeight
                    meanDirGain +=
                        std::pow(10, dirCover.cover[coverIdx] / 10.0f) * coverSurfaceWeight;
                    nrCovers++;
                }
            }

            coefficients[freqIdx] = std::sqrt(meanDirGain / (4 * M_PI));
            frequencies[freqIdx] = dirCover.freq;
        }
    }
};

} // namespace MPEGI

#endif /* MPEGI_DIRECTIVITY_SPECTRUM */
