/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_RENDERSTAGES_REVERB_REVERB_HPP
#define MPEGI_RENDERSTAGES_REVERB_REVERB_HPP

#include <cstddef>
#include <memory>
#include <optional>
#include <unordered_map>
#include <vector>

#include <EAR/BaseTypes.h>
#include <mpegcore/AudioBuffer.hpp>
#include <mpegcore/AudioBufferArray.hpp>
#include <mpegcore/RingBuffer.hpp>
#include <mpegcore/datatypes_reverb.hpp>
#include <mpegcore/frontend/FdnReverb.hpp>

namespace mpegcore::frontend {

class GraphicEqFilter;

} // namespace mpegcore::frontend

namespace MPEGI {

struct Reverb
{
    Reverb(mpegcore::ReverberatorParameters const& reverberatorParameters);

    void filterEQ() noexcept;
    void initialize(std::size_t B);
    void processBlock() noexcept;
    void initPredelayLine(std::size_t length);
    mpegcore::AudioBuffer& getInputBuffer(std::size_t length);

    struct PredelayData
    {
        std::size_t predelayInSamples;
        mpegcore::AudioBuffer inputAudio;
        mpegcore::AudioBuffer predelayedInputAudio;
        std::unique_ptr<mpegcore::RingBuffer> predelayBuffer;
    };

    size_t numberOfOutputs() const;

    mpegcore::ReverberatorParameters reverberatorParameters;
    std::optional<mpegcore::frontend::FdnReverb> fdn;
    std::vector<PredelayData> predelays;
    mpegcore::AudioBuffer mixedPredelayInput;
    mpegcore::AudioBufferArray outputAudio;
    std::vector<EAR::Position> outputPositions;
    std::unique_ptr<mpegcore::frontend::GraphicEqFilter> eqFilter;
    std::size_t B{ 0 };
};

} // namespace MPEGI

#endif // MPEGI_RENDERSTAGES_REVERB_REVERB_HPP
