/***************************************************************************
This software module was originally developed by

Koninklijke Philips N.V.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Koninklijke Philips N.V. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#pragma once

#include "../ReverbConfigCalc.h"
#include <array>
#include <unordered_map>

namespace ReverbRoomsHardcodedData {

struct RoomCorners
{
    std::array<float, 3> corner1;
    std::array<float, 3> corner2;
};

static inline std::unordered_map<unsigned int, RoomCorners> buildings{
    { 1,
        { { -44.626076f, -23.128301999999998f, -1.051419f },
            { 37.917406f, 23.331f, 19.896594999999998f } } },
    { 0, { { 14.837622f, 17.0159f, 0.057734f }, { 25.161942f, 29.275f, 8.712136f } } }
};
static inline SetBaseConfigStruct baseConfigCathedralRefenceRoomId62{ 15, 50.46, 1.5f, 48000, 2423,
    6384, 4.0f, new float[3]{ 10.32f, 12.26f, 8.65f },
    { 3631, 3449, 3299, 3163, 3037, 2927, 2837, 2753, 2687, 2621, 2579, 2531, 2477, 2447, 2423, 0,
        0, 0, 0, 0 } };

static inline SetBaseConfigStruct baseConfigCathedralRefenceRoomId63{ 15, 122, 3.94037604f, 48000,
    11100, 44400, 4.0f, new float[3]{ 82.55f, 46.45f, 20.95f },
    { 23099, 20509, 18253, 16273, 14563, 13063, 11777, 10631, 9643, 8779, 8017, 7369, 6793, 6299,
        5861, 0, 0, 0, 0, 0 } };

static inline T60Settings t60SettingsRoomId62()
{

    std::array<std::array<double, 18>, 15> t60FltCoeff;
    std::array<std::array<double, 9>, 15> bVector{ { {
                                                         0.37438384f,
                                                         -0.65214670f,
                                                         0.13865729f,
                                                         0.29753527f,
                                                         -0.20313936f,
                                                         0.03717649f,
                                                         0.02664555f,
                                                         -0.01573045f,
                                                         -0.00007859f,
                                                     },
        {
            0.39391816f,
            -0.72893143f,
            0.28170517f,
            0.19669022f,
            -0.19919877f,
            0.05753804f,
            0.01068103f,
            -0.00903382f,
            0.00036938f,
        },
        {
            0.37238774f,
            -0.01017907f,
            -0.75296313f,
            0.12945616f,
            0.43813184f,
            -0.15046100f,
            -0.04099595f,
            0.04511732f,
            -0.01861733f,
        },
        {
            0.38466319f,
            -0.03565059f,
            -0.75749183f,
            0.16683601f,
            0.41362515f,
            -0.15486440f,
            -0.03089865f,
            0.04005193f,
            -0.01726094f,
        },
        {
            0.40226212f,
            -0.04574882f,
            -0.78342205f,
            0.18439421f,
            0.42377958f,
            -0.16271272f,
            -0.03193721f,
            0.04075216f,
            -0.01707481f,
        },
        {
            0.41801673f,
            -0.05674525f,
            -0.80872315f,
            0.20080705f,
            0.43409964f,
            -0.16985027f,
            -0.03246691f,
            0.04152447f,
            -0.01717853f,
        },
        {
            0.46636102f,
            -0.94269246f,
            0.25202847f,
            0.52724564f,
            -0.28243634f,
            -0.06765267f,
            0.05083546f,
            -0.00154607f,
            -0.00035280f,
        },
        {
            0.47474310f,
            -0.61893421f,
            0.10344406f,
            0.06291853f,
            -0.03865606f,
            0.01397261f,
            0.00932039f,
            0.00574359f,
            0.00147860f,
        },
        {
            0.48384210f,
            -0.76486951f,
            0.27682805f,
            0.03855674f,
            -0.04844804f,
            0.01567920f,
            0.00555683f,
            0.00233001f,
            0.00066582f,
        },
        {
            0.46335596f,
            -0.45494252f,
            -0.22001149f,
            0.15871727f,
            0.02075374f,
            -0.00794579f,
            0.01585676f,
            0.00794849f,
            0.00099583f,
        },
        {
            0.47188464f,
            -0.79712301f,
            0.29734915f,
            0.00319524f,
            0.02968052f,
            -0.00429799f,
            -0.01616951f,
            0.00542930f,
            0.00146767f,
        },
        {
            0.18614361f,
            0.38668084f,
            -0.31396604f,
            -0.68472314f,
            0.22908811f,
            0.26077953f,
            -0.04726197f,
            0.00245836f,
            -0.00003979f,
        },
        {
            0.18188220f,
            0.36890912f,
            -0.33510324f,
            -0.65764755f,
            0.27175832f,
            0.23120868f,
            -0.04853847f,
            0.00311183f,
            -0.00006488f,
        },
        {
            0.31568104f,
            -0.48365039f,
            -0.48780555f,
            0.63900179f,
            0.34188786f,
            -0.20628013f,
            -0.10544202f,
            -0.00649639f,
            0.00026196f,
        },
        {
            0.35762563f,
            -0.55830455f,
            -0.37164035f,
            0.61366493f,
            0.19505453f,
            -0.16576606f,
            -0.05585283f,
            -0.00572065f,
            -0.00019228f,
        } } };
    std::array<std::array<double, 9>, 15> aVector{ { {
                                                         1.00000000f,
                                                         -2.17889380f,
                                                         0.97947824f,
                                                         0.90556771f,
                                                         -0.93180233f,
                                                         0.17218855f,
                                                         0.11690866f,
                                                         -0.07144042f,
                                                         0.01179433f,
                                                     },
        {
            1.00000000f,
            -2.26620364f,
            1.33843362f,
            0.47252578f,
            -0.79523683f,
            0.24856891f,
            0.03527229f,
            -0.03407582f,
            0.00498859f,
        },
        {
            1.00000000f,
            -0.61633110f,
            -1.86430454f,
            1.24448037f,
            1.03711212f,
            -0.82435846f,
            -0.10881914f,
            0.17809388f,
            -0.03237785f,
        },
        {
            1.00000000f,
            -0.67513788f,
            -1.75746584f,
            1.25773001f,
            0.90070140f,
            -0.76210374f,
            -0.07579216f,
            0.14942816f,
            -0.02715654f,
        },
        {
            1.00000000f,
            -0.66037965f,
            -1.75881577f,
            1.23537850f,
            0.90105611f,
            -0.74837255f,
            -0.07729553f,
            0.14638749f,
            -0.02637591f,
        },
        {
            1.00000000f,
            -0.65085447f,
            -1.76730502f,
            1.22269082f,
            0.90839911f,
            -0.74166071f,
            -0.07980561f,
            0.14510994f,
            -0.02593286f,
        },
        {
            1.00000000f,
            -2.36445379f,
            1.10219884f,
            1.20311570f,
            -1.03150368f,
            -0.12856473f,
            0.24869300f,
            -0.00783828f,
            -0.01964968f,
        },
        {
            1.00000000f,
            -1.63248515f,
            0.51500213f,
            0.22121263f,
            -0.12618998f,
            0.02180509f,
            0.01732412f,
            -0.00062835f,
            -0.00043623f,
        },
        {
            1.00000000f,
            -1.90405309f,
            0.95096761f,
            0.09139869f,
            -0.16679788f,
            0.02594231f,
            0.01472129f,
            -0.00063353f,
            -0.00029531f,
        },
        {
            1.00000000f,
            -1.17537177f,
            -0.32020992f,
            0.53676987f,
            0.02699973f,
            -0.06888118f,
            0.00920011f,
            0.00774006f,
            0.00065950f,
        },
        {
            1.00000000f,
            -1.89080036f,
            0.92595649f,
            0.01405615f,
            0.02103737f,
            -0.04896739f,
            -0.04028513f,
            0.02633420f,
            0.00215866f,
        },
        {
            1.00000000f,
            -0.31995192f,
            -1.55741119f,
            0.25738508f,
            0.82167387f,
            -0.03220529f,
            -0.15838429f,
            -0.00010275f,
            0.01012445f,
        },
        {
            1.00000000f,
            -0.44667763f,
            -1.45431244f,
            0.37283266f,
            0.73611653f,
            -0.06518830f,
            -0.13690218f,
            0.00277498f,
            0.00844624f,
        },
        {
            1.00000000f,
            -0.81677163f,
            -1.45094931f,
            1.02806520f,
            0.75584841f,
            -0.39616245f,
            -0.17584305f,
            0.04729500f,
            0.01639649f,
        },
        {
            1.00000000f,
            -1.09841216f,
            -1.03915954f,
            1.17423582f,
            0.40183854f,
            -0.38914648f,
            -0.09369805f,
            0.04311038f,
            0.01096312f,
        } } };

    for (int i = 0; i < aVector.size(); ++i) {
        for (int j = 0; j < aVector[i].size(); ++j) {
            t60FltCoeff[i][(2 * j) + 0] = bVector[i][j];
            t60FltCoeff[i][(2 * j) + 1] = aVector[i][j];
        }
    }

    T60Settings settings(IIRFilterType::Polynomial);
    settings.t60FltOrder = 8;
    for (int i = 0; i < 15; ++i)
        for (int j = 0; j < 18; ++j)
            settings.filters.t60FltCoeff[i][j] = t60FltCoeff[i][j];

    return settings;
}
static inline T60Settings t60SettingsRoomId63()
{
    std::array<std::array<double, 18>, 15> t60FltCoeff;
    std::array<std::array<double, 9>, 15> bVector{ { {
                                                         0.30300763f,
                                                         -0.67322576f,
                                                         0.83067429f,
                                                         -0.75377345f,
                                                         0.46102381f,
                                                         -0.22103092f,
                                                         0.07036843f,
                                                         -0.01623376f,
                                                         0.00175929f,
                                                     },
        {
            0.35051832f,
            -0.87995607f,
            1.04679298f,
            -0.84252846f,
            0.46154258f,
            -0.17240524f,
            0.04288705f,
            -0.00555651f,
            0.00015245f,
        },
        {
            0.39378801f,
            -1.00075030f,
            1.19330585f,
            -0.95181894f,
            0.51670408f,
            -0.19043528f,
            0.04667688f,
            -0.00608348f,
            0.00018809f,
        },
        {
            0.43556368f,
            -1.11413443f,
            1.32419384f,
            -1.04268563f,
            0.55776459f,
            -0.20070510f,
            0.04757486f,
            -0.00598682f,
            0.00017493f,
        },
        {
            0.47542632f,
            -1.22386348f,
            1.44423759f,
            -1.11320806f,
            0.57790792f,
            -0.19742702f,
            0.04332645f,
            -0.00466929f,
            0.00003135f,
        },
        {
            0.51247197f,
            -1.34746063f,
            1.61219084f,
            -1.24548519f,
            0.64990568f,
            -0.22367683f,
            0.04948081f,
            -0.00589345f,
            0.00017160f,
        },
        {
            0.54799992f,
            -1.45890009f,
            1.74951434f,
            -1.33904302f,
            0.69057983f,
            -0.23199801f,
            0.04905928f,
            -0.00567657f,
            0.00018530f,
        },
        {
            0.58093798f,
            -1.55146956f,
            1.84084117f,
            -1.37415957f,
            0.68613803f,
            -0.21850102f,
            0.04214286f,
            -0.00433490f,
            0.00011842f,
        },
        {
            0.61099541f,
            -1.63521433f,
            1.91419446f,
            -1.38739371f,
            0.66742402f,
            -0.19952820f,
            0.03389501f,
            -0.00281076f,
            0.00004594f,
        },
        {
            0.63862944f,
            -1.69122338f,
            1.95019948f,
            -1.38992238f,
            0.65260297f,
            -0.18436708f,
            0.02756068f,
            -0.00168511f,
            -0.00001387f,
        },
        {
            0.66337866f,
            -1.75226903f,
            2.01939774f,
            -1.44409442f,
            0.68134719f,
            -0.19283958f,
            0.02871874f,
            -0.00178196f,
            0.00000583f,
        },
        {
            0.68656522f,
            -1.81660879f,
            2.09664154f,
            -1.50017357f,
            0.70798093f,
            -0.20033942f,
            0.02975179f,
            -0.00182438f,
            0.00000442f,
        },
        {
            0.70598704f,
            -1.87011564f,
            2.16057968f,
            -1.54600918f,
            0.72938955f,
            -0.20614491f,
            0.03044186f,
            -0.00182933f,
            0.00000178f,
        },
        {
            0.72419131f,
            -1.91249692f,
            2.19317889f,
            -1.55355191f,
            0.72726363f,
            -0.20683560f,
            0.03214828f,
            -0.00218338f,
            -0.00001816f,
        },
        {
            0.74146849f,
            -1.96044326f,
            2.24925852f,
            -1.58905053f,
            0.73956513f,
            -0.20831496f,
            0.03177681f,
            -0.00209300f,
            -0.00001415f,
        } } };
    std::array<std::array<double, 9>, 15> aVector{ { {
                                                         1.00000000f,
                                                         -2.54185677f,
                                                         3.35302997f,
                                                         -3.14149666f,
                                                         2.05591154f,
                                                         -1.01674724f,
                                                         0.37246126f,
                                                         -0.09673183f,
                                                         0.02210611f,
                                                     },
        {
            1.00000000f,
            -2.77986145f,
            3.56280255f,
            -2.96279931f,
            1.68910241f,
            -0.65638077f,
            0.18069853f,
            -0.03930964f,
            0.00909369f,
        },
        {
            1.00000000f,
            -2.78135180f,
            3.54938793f,
            -2.92083931f,
            1.64160538f,
            -0.62481415f,
            0.16539088f,
            -0.03374221f,
            0.00767167f,
        },
        {
            1.00000000f,
            -2.77285123f,
            3.50516939f,
            -2.83863020f,
            1.56445134f,
            -0.58167738f,
            0.15034372f,
            -0.03023378f,
            0.00681275f,
        },
        {
            1.00000000f,
            -2.76503825f,
            3.45054650f,
            -2.73060751f,
            1.45781374f,
            -0.51502645f,
            0.12265705f,
            -0.02244516f,
            0.00526618f,
        },
        {
            1.00000000f,
            -2.80196142f,
            3.52699995f,
            -2.79296470f,
            1.49236906f,
            -0.53101563f,
            0.12859607f,
            -0.02416648f,
            0.00506364f,
        },
        {
            1.00000000f,
            -2.81617570f,
            3.53549838f,
            -2.76725769f,
            1.45292318f,
            -0.49968708f,
            0.11360760f,
            -0.02061534f,
            0.00447784f,
        },
        {
            1.00000000f,
            -2.80981827f,
            3.47973323f,
            -2.65521383f,
            1.34884739f,
            -0.44150907f,
            0.09372527f,
            -0.01738684f,
            0.00424691f,
        },
        {
            1.00000000f,
            -2.80315018f,
            3.41643405f,
            -2.52934790f,
            1.23627365f,
            -0.38244486f,
            0.07531555f,
            -0.01445957f,
            0.00376300f,
        },
        {
            1.00000000f,
            -2.76426935f,
            3.31028605f,
            -2.40650368f,
            1.14664841f,
            -0.33279186f,
            0.05471943f,
            -0.00805841f,
            0.00250529f,
        },
        {
            1.00000000f,
            -2.74676323f,
            3.27478623f,
            -2.38176012f,
            1.13738608f,
            -0.32832849f,
            0.05165356f,
            -0.00649628f,
            0.00209516f,
        },
        {
            1.00000000f,
            -2.74199414f,
            3.26404023f,
            -2.37125301f,
            1.13083863f,
            -0.32558078f,
            0.05088332f,
            -0.00633552f,
            0.00208603f,
        },
        {
            1.00000000f,
            -2.73767018f,
            3.25589442f,
            -2.36452579f,
            1.12717891f,
            -0.32430100f,
            0.05059474f,
            -0.00625558f,
            0.00209064f,
        },
        {
            1.00000000f,
            -2.72448587f,
            3.21149349f,
            -2.30651307f,
            1.08946466f,
            -0.31553036f,
            0.05433894f,
            -0.00880208f,
            0.00225877f,
        },
        {
            1.00000000f,
            -2.72113085f,
            3.20249701f,
            -2.29333210f,
            1.07959318f,
            -0.31225771f,
            0.05336140f,
            -0.00820258f,
            0.00219710f,
        } } };

    for (int i = 0; i < aVector.size(); ++i) {
        for (int j = 0; j < aVector[i].size(); ++j) {
            t60FltCoeff[i][(2 * j) + 0] = bVector[i][j];
            t60FltCoeff[i][(2 * j) + 1] = aVector[i][j];
        }
    }

    T60Settings settings(IIRFilterType::Polynomial);
    settings.t60FltOrder = 8;
    for (int i = 0; i < 15; ++i)
        for (int j = 0; j < 18; ++j)
            settings.filters.t60FltCoeff[i][j] = t60FltCoeff[i][j];

    return settings;
}
static inline ColorParams colorParamsRoomId62()
{
    ColorParams params(IIRFilterType::Polynomial);
    const float commonComponentClrFlt = 0.000192866821;

    const std::array<std::array<double, MAX_FILTERORDER + 1>, 2> colorFltCoeff{
        { { 1.56977999f, -3.37846398f, 3.67981935f, -2.39036918f, 0.82200825f, 0.00663162f,
              -0.17650270f, 0.09954567f, -0.03304559f, 0.00522437f, 0.00075236f, -0.00051182f,
              0.00019505f },
            { 1.00000000f, -1.52340734f, 1.52283287f, -0.83333981f, 0.25439593f, 0.02706663f,
                -0.06795805f, 0.03352074f, -0.00920692f, 0.00037425f, 0.00047724f, -0.00015859f,
                0.00002904f } }
    };

    for (int i = 0; i < 2; ++i)
        for (int j = 0; j < MAX_FILTERORDER + 1; ++j)
            params.filters.colorFltCoeff[i][j] = colorFltCoeff[i][j];

    params.commonComponentClrFlt = commonComponentClrFlt;
    return params;
}
static inline ColorParams colorParamsRoomId63()
{
    ColorParams params(IIRFilterType::Polynomial);
    const float commonComponentClrFlt = 0.000192866821;

    const std::array<std::array<double, MAX_FILTERORDER + 1>, 2> colorFltCoeff{
        { { 1.53658533f, -2.43377137f, -0.37636280f, 1.76913011f, -0.22227035f, -0.28361031f,
              0.00928612f, -0.04399678f, 0.04810081f, 0.02084525f, -0.01432534f, -0.00716719f,
              -0.00091302f },
            { 1.00000000f, -0.97358775f, -0.73242354f, 0.52819347f, 0.14003186f, 0.06183762f,
                0.04093996f, -0.05712020f, -0.01438896f, 0.00710952f, 0.00100256f, -0.00020575f,
                -0.00002651f } }
    };

    for (int i = 0; i < 2; ++i)
        for (int j = 0; j < MAX_FILTERORDER + 1; ++j)
            params.filters.colorFltCoeff[i][j] = colorFltCoeff[i][j];

    params.commonComponentClrFlt = commonComponentClrFlt;
    return params;
}
static inline lateReverb lateReverbRoomId62{
    { 19.6793251, 56.3104591, 72.4326553, 98.3410873, 107.463219, 116.839722, 126.477615,
        136.384262, 146.567108, 157.033844, 167.792450, 178.851013, 190.217896, 201.901718,
        213.911270, 226.255707, 238.944305, 251.986725, 265.392731, 279.172546, 293.336578,
        307.895508, 322.860321, 338.242432, 354.053436, 370.305237, 387.010193, 404.180878,
        421.830322, 439.971863, 458.619263, 477.786530, 497.488190, 517.739197, 538.554810,
        559.950867, 581.943420, 604.549133, 627.785217, 651.669128, 676.218933, 701.453186,
        727.391052, 754.052185, 781.456665, 809.625061, 838.579041, 868.340210, 898.931091,
        930.375000, 962.695618, 995.917419, 1030.06543, 1065.16553, 1101.24426, 1138.32886,
        1176.44763, 1215.62915, 1255.90308, 1297.29993, 1339.85095, 1383.58838, 1428.54529,
        1474.75562, 1522.25452, 1571.07764, 1621.26208, 1672.84583, 1725.86743, 1780.36768,
        1836.38757, 1893.96924, 1953.15637, 2013.99365, 2076.52734, 2140.80469, 2206.87402,
        2274.78540, 2344.59009, 2416.34131, 2490.09326, 2565.90112, 2643.82275, 2723.91699,
        2806.24463, 2890.86743, 2977.84985, 3067.25732, 3159.15771, 3253.62036, 3350.71655,
        3450.52026, 3553.10669, 3658.55298, 3766.93970, 3878.34863, 3992.86328, 4110.57129,
        4231.56055, 4355.92334, 4483.75391, 4615.14844, 4750.20654, 4889.03027, 5031.72461,
        5178.39746, 5329.15967, 5484.12549, 5643.41211, 5807.13916, 5975.43213, 6148.41748,
        6326.22559, 6508.99072, 6696.85205, 6889.95166, 7088.43457, 7292.45215, 7502.15723,
        7717.71045, 7939.27295, 8167.01270, 8401.10254, 8641.71875, 8889.04395, 9143.26562,
        9404.57520, 9907.86523, 10411.1562, 10914.4463, 11417.7373, 11921.0273, 12424.3184,
        12927.6094, 13430.8994, 13934.1895, 14437.4805, 14940.7715, 15444.0605, 15947.3516,
        16450.6426, 16953.9336, 17457.2227, 17960.5137, 18463.8047, 18967.0938, 19470.3867,
        19973.6758, 20476.9668, 20980.2578, 21483.5469, 21986.8379, 22490.1289, 22993.4180,
        23496.7109, 24000.0000 },
    { 4.16796064, 4.14593077, 3.94564295, 3.61067271, 3.56342030, 3.54302955, 3.54459286,
        3.56391692, 3.41089058, 3.48139691, 3.56842852, 3.66886258, 3.77959180, 3.89561534,
        4.00937223, 4.11214924, 4.19656420, 4.25847292, 4.29734373, 4.31539059, 4.31625319,
        4.30386782, 4.28177834, 4.25286055, 4.21928835, 4.18260956, 4.14387751, 4.10377645,
        4.06271267, 4.02093124, 3.97854733, 3.93562984, 3.89222503, 3.84838080, 3.80416608,
        3.75965381, 3.71492481, 3.67003870, 3.62504363, 3.57996941, 3.53482175, 3.48958540,
        3.44420266, 3.39856005, 3.35246801, 3.30565071, 3.25775146, 3.20836878, 3.15709805,
        3.10359550, 3.04763484, 2.98914361, 2.92822337, 2.86513376, 2.80025601, 2.73403907,
        2.66695380, 2.59944916, 2.53193331, 2.46476388, 2.39824891, 2.33265567, 2.26821184,
        2.20511675, 2.14354134, 2.08364153, 2.02556515, 1.96945322, 1.91544175, 1.86365104,
        1.81418025, 1.76708782, 1.72238827, 1.68004608, 1.63997626, 1.60205436, 1.56612265,
        1.53200197, 1.49950397, 1.46843982, 1.43862998, 1.40990913, 1.38213146, 1.35517061,
        1.32891798, 1.30327666, 1.27816129, 1.25348818, 1.22917306, 1.20513248, 1.18127799,
        1.15752411, 1.13378680, 1.10999286, 1.08608031, 1.06200743, 1.03775382, 1.01332450,
        0.988746881, 0.964067876, 0.939349353, 0.914662659, 0.890083075, 0.865686417, 0.841547847,
        0.817739964, 0.794332385, 0.771390438, 0.748973846, 0.727135837, 0.705922186, 0.685374081,
        0.665531158, 0.646435499, 0.628135383, 0.610685706, 0.594142020, 0.578552365, 0.563946962,
        0.550329626, 0.537674308, 0.525926352, 0.515011489, 0.504847586, 0.495355099, 0.486469895,
        0.478150278, 0.464202464, 0.452782571, 0.443731874, 0.436830550, 0.431756586, 0.428135097,
        0.425605297, 0.423862457, 0.422670186, 0.421856374, 0.421300203, 0.420918375, 0.420654804,
        0.420471787, 0.420343518, 0.420253217, 0.420188904, 0.420142949, 0.420109868, 0.420085669,
        0.420067936, 0.420054853, 0.420044988, 0.420037597, 0.420032024, 0.420027643, 0.420024157,
        0.420021594, 0.420019507 },
    { 2.06502227e-06, 2.05439392e-06, 1.94644122e-06, 1.72374723e-06, 1.69207635e-06,
        1.68689883e-06, 1.70577539e-06, 1.74400407e-06, 1.49876871e-06, 1.62644756e-06,
        1.80192387e-06, 1.99996157e-06, 2.18484706e-06, 2.32690218e-06, 2.41561838e-06,
        2.45538467e-06, 2.45417186e-06, 2.41881389e-06, 2.35635457e-06, 2.27528244e-06,
        2.18451851e-06, 2.09145423e-06, 2.00064915e-06, 1.91392746e-06, 1.83150848e-06,
        1.75317552e-06, 1.67880796e-06, 1.60840079e-06, 1.54213103e-06, 1.48066749e-06,
        1.42536737e-06, 1.37793802e-06, 1.33968410e-06, 1.31087984e-06, 1.29065859e-06,
        1.27738645e-06, 1.26919940e-06, 1.26442251e-06, 1.26176747e-06, 1.26035263e-06,
        1.25962526e-06, 1.25926124e-06, 1.25907968e-06, 1.25896941e-06, 1.25881240e-06,
        1.25837084e-06, 1.25712381e-06, 1.25410588e-06, 1.24785458e-06, 1.23658367e-06,
        1.21856749e-06, 1.19260517e-06, 1.15840373e-06, 1.11674149e-06, 1.06938455e-06,
        1.01878300e-06, 9.67610731e-07, 9.18277976e-07, 8.72557450e-07, 8.31427315e-07,
        7.95156268e-07, 7.63549053e-07, 7.36226582e-07, 7.12827614e-07, 6.93077823e-07,
        6.76741990e-07, 6.63516062e-07, 6.52918516e-07, 6.44229942e-07, 6.36499863e-07,
        6.28632108e-07, 6.19527441e-07, 6.08250730e-07, 5.94182382e-07, 5.77109404e-07,
        5.57250019e-07, 5.35201650e-07, 5.11827636e-07, 4.88101136e-07, 4.64940399e-07,
        4.43076232e-07, 4.22983646e-07, 4.04885270e-07, 3.88816431e-07, 3.74710510e-07,
        3.62471297e-07, 3.52009351e-07, 3.43239492e-07, 3.36053972e-07, 3.30287548e-07,
        3.25692412e-07, 3.21926905e-07, 3.18565299e-07, 3.15131786e-07, 3.11150700e-07,
        3.06208193e-07, 3.00008622e-07, 2.92415990e-07, 2.83469376e-07, 2.73373473e-07,
        2.62462862e-07, 2.51148151e-07, 2.39850664e-07, 2.28940252e-07, 2.18687262e-07,
        2.09239843e-07, 2.00626218e-07, 1.92777748e-07, 1.85562129e-07, 1.78817828e-07,
        1.72382599e-07, 1.66113324e-07, 1.59898931e-07, 1.53666988e-07, 1.47387723e-07,
        1.41075859e-07, 1.34789801e-07, 1.28625018e-07, 1.22700911e-07, 1.17139933e-07,
        1.12042571e-07, 1.07465993e-07, 1.03414031e-07, 9.98433052e-08, 9.66810774e-08,
        9.38489535e-08, 9.12847113e-08, 8.71606147e-08, 8.40638492e-08, 8.19802892e-08,
        8.07397171e-08, 8.00715014e-08, 7.97357700e-08, 7.95742565e-08, 7.94984629e-08,
        7.94633266e-08, 7.94470907e-08, 7.94395802e-08, 7.94360560e-08, 7.94343862e-08,
        7.94336188e-08, 7.94332422e-08, 7.94330575e-08, 7.94329225e-08, 7.94329011e-08,
        7.94328798e-08, 7.94328585e-08, 7.94328656e-08, 7.94328514e-08, 7.94328301e-08,
        7.94328443e-08, 7.94328443e-08, 7.94328443e-08, 7.94328372e-08, 7.94328372e-08,
        7.94328514e-08 },
    6384
};
static inline lateReverb lateReverbRoomId63{
    { 19.6793251, 56.3104591, 72.4326553, 98.3410873, 107.463219, 116.839722, 126.477615,
        136.384262, 146.567108, 157.033844, 167.792450, 178.851013, 190.217896, 201.901718,
        213.911270, 226.255707, 238.944305, 251.986725, 265.392731, 279.172546, 293.336578,
        307.895508, 322.860321, 338.242432, 354.053436, 370.305237, 387.010193, 404.180878,
        421.830322, 439.971863, 458.619263, 477.786530, 497.488190, 517.739197, 538.554810,
        559.950867, 581.943420, 604.549133, 627.785217, 651.669128, 676.218933, 701.453186,
        727.391052, 754.052185, 781.456665, 809.625061, 838.579041, 868.340210, 898.931091,
        930.375000, 962.695618, 995.917419, 1030.06543, 1065.16553, 1101.24426, 1138.32886,
        1176.44763, 1215.62915, 1255.90308, 1297.29993, 1339.85095, 1383.58838, 1428.54529,
        1474.75562, 1522.25452, 1571.07764, 1621.26208, 1672.84583, 1725.86743, 1780.36768,
        1836.38757, 1893.96924, 1953.15637, 2013.99365, 2076.52734, 2140.80469, 2206.87402,
        2274.78540, 2344.59009, 2416.34131, 2490.09326, 2565.90112, 2643.82275, 2723.91699,
        2806.24463, 2890.86743, 2977.84985, 3067.25732, 3159.15771, 3253.62036, 3350.71655,
        3450.52026, 3553.10669, 3658.55298, 3766.93970, 3878.34863, 3992.86328, 4110.57129,
        4231.56055, 4355.92334, 4483.75391, 4615.14844, 4750.20654, 4889.03027, 5031.72461,
        5178.39746, 5329.15967, 5484.12549, 5643.41211, 5807.13916, 5975.43213, 6148.41748,
        6326.22559, 6508.99072, 6696.85205, 6889.95166, 7088.43457, 7292.45215, 7502.15723,
        7717.71045, 7939.27295, 8167.01270, 8401.10254, 8641.71875, 8889.04395, 9143.26562,
        9404.57520, 9907.86523, 10411.1562, 10914.4463, 11417.7373, 11921.0273, 12424.3184,
        12927.6094, 13430.8994, 13934.1895, 14437.4805, 14940.7715, 15444.0605, 15947.3516,
        16450.6426, 16953.9336, 17457.2227, 17960.5137, 18463.8047, 18967.0938, 19470.3867,
        19973.6758, 20476.9668, 20980.2578, 21483.5469, 21986.8379, 22490.1289, 22993.4180,
        23496.7109, 24000.0000 },
    { 3.61048818, 3.63603687, 3.87425184, 4.33486652, 4.43669128, 4.51918411, 4.58756924,
        4.64566469, 4.78645611, 4.82461357, 4.85766315, 4.88810587, 4.91743803, 4.94649744,
        4.97576618, 5.00553799, 5.03599691, 5.06726456, 5.09941816, 5.13250208, 5.16655111,
        5.20157528, 5.23757410, 5.27451849, 5.31233168, 5.35080957, 5.38954878, 5.42787266,
        5.46484613, 5.49938297, 5.53041601, 5.55709362, 5.57889652, 5.59567690, 5.60762596,
        5.61516666, 5.61887026, 5.61933565, 5.61715555, 5.61284971, 5.60685110, 5.59949923,
        5.59102964, 5.58157301, 5.57114363, 5.55964851, 5.54689646, 5.53262281, 5.51653624,
        5.49835443, 5.47784567, 5.45485353, 5.42930555, 5.40121460, 5.37065840, 5.33776045,
        5.30266476, 5.26552582, 5.22648478, 5.18566322, 5.14316607, 5.09908581, 5.05351448,
        5.00656748, 4.95840740, 4.90926552, 4.85946465, 4.80939960, 4.75954103, 4.71037531,
        4.66238213, 4.61597586, 4.57147026, 4.52905798, 4.48879862, 4.45063400, 4.41440773,
        4.37988710, 4.34679413, 4.31483698, 4.28372335, 4.25317860, 4.22295952, 4.19285583,
        4.16269732, 4.13234806, 4.10171652, 4.07074451, 4.03942823, 4.00779057, 3.97589207,
        3.94382572, 3.91169405, 3.87960625, 3.84765267, 3.81591129, 3.78441525, 3.75317359,
        3.72215438, 3.69129682, 3.66051483, 3.62970519, 3.59875774, 3.56755495, 3.53598452,
        3.50394082, 3.47132301, 3.43804502, 3.40402508, 3.36919737, 3.33350873, 3.29692960,
        3.25945616, 3.22113013, 3.18204951, 3.14237142, 3.10230637, 3.06209421, 3.02197790,
        2.98216057, 2.94278550, 2.90392756, 2.86560607, 2.82781601, 2.79056621, 2.75392485,
        2.71805167, 2.65461016, 2.59979701, 2.55468726, 2.51937056, 2.49291563, 2.47378063,
        2.46028447, 2.45092225, 2.44448519, 2.44007659, 2.43705606, 2.43497944, 2.43354630,
        2.43254972, 2.43185258, 2.43136168, 2.43101287, 2.43076372, 2.43058491, 2.43045402,
        2.43035889, 2.43028831, 2.43023562, 2.43019581, 2.43016553, 2.43014193, 2.43012381,
        2.43010998, 2.43009853 },
    { 3.55829314e-07, 3.58634139e-07, 3.81016207e-07, 3.98638917e-07, 3.94260582e-07,
        3.87652705e-07, 3.80420062e-07, 3.73765687e-07, 3.53823850e-07, 3.36926973e-07,
        3.26521274e-07, 3.22327907e-07, 3.24021016e-07, 3.31430130e-07, 3.43878867e-07,
        3.60095157e-07, 3.78728572e-07, 3.98719692e-07, 4.19200433e-07, 4.39147072e-07,
        4.57251389e-07, 4.72265555e-07, 4.83522172e-07, 4.91147603e-07, 4.95845768e-07,
        4.98505472e-07, 4.99905639e-07, 5.00599072e-07, 5.00925694e-07, 5.01073771e-07,
        5.01138857e-07, 5.01166767e-07, 5.01178590e-07, 5.01183479e-07, 5.01185525e-07,
        5.01186435e-07, 5.01186776e-07, 5.01186889e-07, 5.01187003e-07, 5.01186889e-07,
        5.01186946e-07, 5.01186776e-07, 5.01185980e-07, 5.01180295e-07, 5.01154318e-07,
        5.01060470e-07, 5.00785632e-07, 5.00117039e-07, 4.98730628e-07, 4.96231280e-07,
        4.92238598e-07, 4.86495594e-07, 4.78958498e-07, 4.69844593e-07, 4.59619145e-07,
        4.48926215e-07, 4.38472142e-07, 4.28892861e-07, 4.20645733e-07, 4.13955320e-07,
        4.08824434e-07, 4.05090816e-07, 4.02502593e-07, 4.00786405e-07, 3.99693278e-07,
        3.99021701e-07, 3.98622376e-07, 3.98391450e-07, 3.98261307e-07, 3.98189258e-07,
        3.98149126e-07, 3.98123490e-07, 3.98094357e-07, 3.98033478e-07, 3.97885998e-07,
        3.97555880e-07, 3.96895217e-07, 3.95710430e-07, 3.93784944e-07, 3.90917535e-07,
        3.86961318e-07, 3.81854903e-07, 3.75633107e-07, 3.68420984e-07, 3.60411605e-07,
        3.51834984e-07, 3.42923812e-07, 3.33885254e-07, 3.24882507e-07, 3.16033777e-07,
        3.07426944e-07, 2.99143124e-07, 2.91276933e-07, 2.83946520e-07, 2.77282936e-07,
        2.71407146e-07, 2.66402992e-07, 2.62298101e-07, 2.59059249e-07, 2.56601027e-07,
        2.54804633e-07, 2.53538360e-07, 2.52674056e-07, 2.52097493e-07, 2.51709395e-07,
        2.51420744e-07, 2.51143149e-07, 2.50779493e-07, 2.50218591e-07, 2.49339109e-07,
        2.48020484e-07, 2.46159232e-07, 2.43685804e-07, 2.40574593e-07, 2.36846887e-07,
        2.32567473e-07, 2.27835926e-07, 2.22777118e-07, 2.17530456e-07, 2.12237296e-07,
        2.07024712e-07, 2.01987348e-07, 1.97174487e-07, 1.92589553e-07, 1.88203686e-07,
        1.83980490e-07, 1.79904859e-07, 1.72877478e-07, 1.67277364e-07, 1.63382779e-07,
        1.61019756e-07, 1.59732522e-07, 1.59081097e-07, 1.58766269e-07, 1.58618022e-07,
        1.58549142e-07, 1.58517281e-07, 1.58502473e-07, 1.58495610e-07, 1.58492355e-07,
        1.58490792e-07, 1.58489982e-07, 1.58489655e-07, 1.58489428e-07, 1.58489357e-07,
        1.58489286e-07, 1.58489257e-07, 1.58489286e-07, 1.58489257e-07, 1.58489229e-07,
        1.58489257e-07, 1.58489243e-07, 1.58489243e-07, 1.58489243e-07, 1.58489243e-07,
        1.58489257e-07 },
    44400
};
} // namespace ReverbRoomsHardcodedData