/***************************************************************************
This software module was originally developed by

Fraunhofer IIS

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Fraunhofer IIS retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_SIMPLE_CULLING_STAGE
#define MPEGI_SIMPLE_CULLING_STAGE

#include <MPEGI/Renderer/RenderStage.h>
#include <MPEGI/Renderer/RenderStageRegistry.h>

#include <MPEGI/Renderer/RenderList.h>

#include <map>

namespace MPEGI {

//! Deactivate render items when they are very low in volume
/*! 
 * Inputs: 
 * - All render items with ItemType::Primary, ItemType::Representative, ItemType::Reflection,
 *   ItemType::DirectionalReverb or ItemType::HOA
 *  
 * Outputs: 
 * - All render items that are below a threshold are deactivated
 *  
 * How it works: 
 * - If the maximum gain in any EQ band is below a configurable threshold, the respective
 *   render item is deactivated
 * 
 * Stages order:
 * - Required before: none
 * - Required after: FadeStage
 *  
 * Config options: 
 * - (not real-time adjustable) "SimpleCulling_DeactivationThreshold": when this gain threshold is not reached in any EQ band,
 *   the RI is deactivated (default: 0.001 -> -60dB)
 * 
 */
class SimpleCullingStage : public RenderStage
{
public:
    SimpleCullingStage();

    //! Destructor
    ~SimpleCullingStage();

    //! Inherited from RenderStage
    std::unique_ptr<RenderStage::DspData> update(RenderList& renderList) override;

private:
    friend class Renderer;

    //! Inherited from RenderStage
    void initializeStage(const Bitstream& bitstream, RenderItemStore* itemStore,
        const StreamManager* streamManager, const SceneState* state) override;

    RenderItemStore* itemStore;

    struct ItemState : RenderItem::ExtraState
    {
        bool deactivate = false;
        float eqPSUM = 0.f;
    };
    bool voxelEREnable = false;
};

} // namespace MPEGI

#endif /* MPEGI_SIMPLE_CULLING_STAGE */
