/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGI_RENDERSTAGES_SPHOASTAGE_HPP
#define MPEGI_RENDERSTAGES_SPHOASTAGE_HPP

#include <MPEGI/Renderer/RenderStage.h>
#include <MPEGI/Renderer/RenderStageRegistry.h>

#include <cstddef>
#include <map>
#include <memory>
#include <string>
#include <unordered_map>
#include <vector>

#include <mpegcore/Config.hpp>
#include <mpegcore/EsdConverter.hpp>
#include <mpegcore/HoaBus.hpp>
#include <mpegcore/HoaDecoder.hpp>

namespace EAR {

class HOASource;
class Listener;
class RealBuffer;

} // namespace EAR

namespace MPEGI {

//! ...
/*! 
 * Inputs: 
 * - 
 *  
 * Outputs: 
 * - 
 *  
 * How it works: 
 * - 
 * 
 * Stages order:
 * - Required before: ?
 * - Required after: ?
 *  
 * Config options: 
 * -
 * 
 */
class SphoaStage : public RenderStage
{
public:
    SphoaStage();
    ~SphoaStage();

    std::unique_ptr<RenderStage::DspData> update(RenderList& renderList) override;
    void processBlock(DspContext ctx) override;

private:
    friend class Renderer;

    void initializeStage(const Bitstream& bitstream, RenderItemStore* itemStore,
        const StreamManager* streamManager, const SceneState* state) override;

    void initializeHoaInputs(std::vector<std::unique_ptr<EAR::HOASource>> const& sphoaSources);

    struct InputAudio
    {
        mpegcore::AudioBuffer& audioBuffer;
    };

    struct HoaInput
    {
        mpegcore::HoaBus::InputBus& inputBus;
        std::vector<InputAudio> inputAudios;
        std::size_t signalChannelCount{ 0 };
    };

    HoaInput* hoaInputFor(RenderItem const& ri);
    void createHoaInput(std::string const& eifObjectId, std::size_t channelCount);

    struct DSPItem
    {
        HoaInput* hoaInput{ nullptr };
        EAR::RealFrame* signalFrame{ nullptr };
    };

    struct DspData : RenderStage::DspData
    {
        std::vector<DSPItem> dspItems;
    };

    std::vector<RenderItem*> outputRenderItems;

    mpegcore::Config config;
    mpegcore::HoaBus hoaBus;

    // Map EIF object IDs to HoaInputs. It would be nicer to use ObjectID as a key but making it
    // available when the inputs are accessed via RenderItems would require a bit too large of a
    // refectoring at this point.
    std::unordered_map<std::string, HoaInput> hoaInputs;

    mpegcore::HoaDecoder hoaDecoder;
    mpegcore::EsdConverter esdConverter;

    std::size_t B{ 0 };
    float sphoaGain{ 0 };

    bool suspendProcessing{ true };
};

} // namespace MPEGI

#endif // MPEGI_RENDERSTAGES_SPHOASTAGE_HPP
