#include "CornerFindingAlgorithmC.h"
#include "jps_pathfinder_impl.h"
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <sstream>

namespace py = pybind11;
//--------------------------------------------------------------------------------------------------
class corner_finding_algorithm_wrapper
{
public:
    MPEGI::Voxel::dmap_index_t _corner{ 0 };
    MPEGI::Voxel::projection_map _corner_map;

    corner_finding_algorithm_wrapper(MPEGI::Voxel::projection_map const& p_map)
        : _corner_map(make_corner_map(p_map))
    {}

    int operator()(std::size_t /* @todo: remove */, MPEGI::Voxel::path_2d const& path)
    {
        auto const& p = path.get();
        return MPEGI::Voxel::CornerFindingAlgorithm(
            p.size(), p.data(), _corner_map, _corner.data());
    }
};

//--------------------------------------------------------------------------------------------------
using const_at_fn = char const& (MPEGI::Voxel::projection_map::*)(std::size_t, std::size_t) const;

//--------------------------------------------------------------------------------------------------
PYBIND11_MODULE(vox_diffraction, m)
{
    m.doc() = "Voxel diffraction algorithms and classes",
    py::class_<MPEGI::Voxel::projection_map>(m, "projection_map")
        .def(py::init<MPEGI::Voxel::voxel_index_val_t const&,
            MPEGI::Voxel::voxel_index_val_t const&>())
        .def("get", static_cast<const_at_fn>(&MPEGI::Voxel::projection_map::at))
        .def("set", &MPEGI::Voxel::projection_map::set)
        .def("__str__",
            [](MPEGI::Voxel::projection_map const& pm) {
                std::stringstream ss;
                for (int y = pm.dim[1] - 1; y >= 0; --y) {
                    std::string sep = "";
                    for (auto x = 0U; x < pm.dim[0]; ++x) {
                        ss << sep << static_cast<int>(pm.at(x, y));
                        sep = " ";
                    }

                    ss << std::endl;
                }
                return ss.str();
            })
        .def_readonly("resolution", &MPEGI::Voxel::projection_map::dim);

    py::class_<MPEGI::Voxel::path_2d>(m, "path_2d")
        .def("__str__",
            [](MPEGI::Voxel::path_2d const& p) {
                std::stringstream ss;
                std::string sep = "";
                for (auto i = 0U; i < p.size(); ++i) {
                    auto const xy = p.get(i);
                    ss << sep << "(" << xy[0] << ", " << xy[1] << ")";
                    sep = " > ";
                }

                return ss.str();
            })
        .def("length", &MPEGI::Voxel::path_2d::size);

    py::class_<MPEGI::Voxel::i_pathfinder>(m, "pathfinder_ifc");
    py::class_<MPEGI::Voxel::jps_pathfinder, MPEGI::Voxel::i_pathfinder>(m, "pathfinder")
        .def(py::init<>())
        .def("init", &MPEGI::Voxel::i_pathfinder::init)
        .def("find_path", &MPEGI::Voxel::i_pathfinder::find_path)
        .def("test_direct_view", &MPEGI::Voxel::i_pathfinder::test_direct_view)
        .def("get_path", &MPEGI::Voxel::i_pathfinder::get_path)
        .def("get_path_length", &MPEGI::Voxel::i_pathfinder::get_path_length);

    py::class_<corner_finding_algorithm_wrapper>(m, "cornerfinder")
        .def(py::init<MPEGI::Voxel::projection_map const&>())
        .def("find_corner", &corner_finding_algorithm_wrapper::operator())
        .def_readonly("corner", &corner_finding_algorithm_wrapper::_corner)
        .def_readonly("corner_map", &corner_finding_algorithm_wrapper::_corner_map);
}