/***************************************************************************
This software module was originally developed by

Dolby International AB

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Dolby International AB retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/
#ifndef WOO_ALGORITHM_C_HPP
#define WOO_ALGORITHM_C_HPP

template <typename T>
void WooAlgorithm(unsigned int const gridSize_xyz[3], T size_vox, const T minBound[3],
    const T maxBound[3], const T user_xyz[3], const T object_xyz[3], const unsigned int user_idx[3],
    const unsigned int object_idx[3], int* num_indices_found, unsigned int** indices_found)
{
    static_assert(std::is_floating_point_v<T>, "Must be floating point");

    int step_xyz[3] = { 0, 0, 0 };

    double tVoxel_xyz[3] = { 0.0, 0.0, 0.0 };
    unsigned int xyz_idx[3];
    double vec_xyz[3];
    double dir_xyz[3];
    double voxelMax_xyz[3];
    double tMax_xyz[3];
    double tDelta[3];
    double norm;

    xyz_idx[0] = user_idx[0];
    xyz_idx[1] = user_idx[1];
    xyz_idx[2] = user_idx[2];

    vec_xyz[0] = object_xyz[0] - user_xyz[0];
    vec_xyz[1] = object_xyz[1] - user_xyz[1];
    vec_xyz[2] = object_xyz[2] - user_xyz[2];

    norm = sqrt(vec_xyz[0] * vec_xyz[0] + vec_xyz[1] * vec_xyz[1] + vec_xyz[2] * vec_xyz[2]);
    dir_xyz[0] = vec_xyz[0] / norm;
    dir_xyz[1] = vec_xyz[1] / norm;
    dir_xyz[2] = vec_xyz[2] / norm;

    for (int i = 0; i < 3; i++) {
        if (xyz_idx[i] == (gridSize_xyz[i] + 1))
            xyz_idx[i] = xyz_idx[i] - 1;

        if (dir_xyz[i] >= 0) {
            tVoxel_xyz[i] = (double)xyz_idx[i] / gridSize_xyz[i];
            step_xyz[i] = 1;
        } else {
            tVoxel_xyz[i] = (double)(xyz_idx[i] - 1) / gridSize_xyz[i];
            step_xyz[i] = -1;
        }
    }

    voxelMax_xyz[0] = minBound[0] + tVoxel_xyz[0] * (maxBound[0] - minBound[0]);
    voxelMax_xyz[1] = minBound[1] + tVoxel_xyz[1] * (maxBound[1] - minBound[1]);
    voxelMax_xyz[2] = minBound[2] + tVoxel_xyz[2] * (maxBound[2] - minBound[2]);

    tMax_xyz[0] = (voxelMax_xyz[0] - user_xyz[0]) / dir_xyz[0];
    tMax_xyz[1] = (voxelMax_xyz[1] - user_xyz[1]) / dir_xyz[1];
    tMax_xyz[2] = (voxelMax_xyz[2] - user_xyz[2]) / dir_xyz[2];

    tDelta[0] = size_vox / std::abs(dir_xyz[0]);
    tDelta[1] = size_vox / std::abs(dir_xyz[1]);
    tDelta[2] = size_vox / std::abs(dir_xyz[2]);

    *num_indices_found = 0;

    while ((xyz_idx[0] * step_xyz[0] <= object_idx[0] * step_xyz[0])
        && (xyz_idx[1] * step_xyz[1] <= object_idx[1] * step_xyz[1])
        && (xyz_idx[2] * step_xyz[2] <= object_idx[2] * step_xyz[2])
        && (xyz_idx[0] <= gridSize_xyz[0]) && (xyz_idx[0] >= 1) && (xyz_idx[1] <= gridSize_xyz[1])
        && (xyz_idx[1] >= 1) && (xyz_idx[2] <= gridSize_xyz[2]) && (xyz_idx[2] >= 1)) {

        indices_found[*num_indices_found][0] = xyz_idx[0];
        indices_found[*num_indices_found][1] = xyz_idx[1];
        indices_found[*num_indices_found][2] = xyz_idx[2];

        (*num_indices_found)++;

        if (tMax_xyz[0] < tMax_xyz[1]) {
            if (tMax_xyz[0] < tMax_xyz[2]) {
                xyz_idx[0] = xyz_idx[0] + step_xyz[0];
                tMax_xyz[0] = tMax_xyz[0] + tDelta[0];
            } else {
                xyz_idx[2] = xyz_idx[2] + step_xyz[2];
                tMax_xyz[2] = tMax_xyz[2] + tDelta[2];
            }
        } else {
            if (tMax_xyz[1] < tMax_xyz[2]) {
                xyz_idx[1] = xyz_idx[1] + step_xyz[1];
                tMax_xyz[1] = tMax_xyz[1] + tDelta[1];
            } else {
                xyz_idx[2] = xyz_idx[2] + step_xyz[2];
                tMax_xyz[2] = tMax_xyz[2] + tDelta[2];
            }
        }
    }
}
#endif