/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

//
// `AudioBufferArray` implements an "array" of `AudioBuffers`.
//

#ifndef MPEGCORE_AUDIOBUFFERARRAY_HPP
#define MPEGCORE_AUDIOBUFFERARRAY_HPP

#include <vector>

#include "AudioBuffer.hpp"

namespace mpegcore {

class AudioBufferArray
{
public:
    AudioBufferArray() noexcept {}
    AudioBufferArray(std::size_t bufferCount, std::size_t samplesPerBuffer) noexcept;

    void copyWithBufferOffset(
        AudioBufferArray const& source, std::size_t targetBufferOffset) noexcept;
    void fill(Sample value);
    bool init(std::size_t bufferCount, std::size_t samplesPerBuffer) noexcept;
    std::size_t size() const noexcept { return m_buffers.size(); }

    Sample const* const* rawPointerArray() const noexcept { return m_bufferPtrs.data(); }
    Sample** rawPointerArray() noexcept { return m_bufferPtrs.data(); }

    std::vector<AudioBuffer>::const_iterator begin() const noexcept { return m_buffers.cbegin(); }

    std::vector<AudioBuffer>::iterator begin() noexcept { return m_buffers.begin(); }

    std::vector<AudioBuffer>::const_iterator end() const noexcept { return m_buffers.cend(); }

    std::vector<AudioBuffer>::iterator end() noexcept { return m_buffers.end(); }

    AudioBuffer const& operator[](std::size_t idx) const noexcept { return m_buffers[idx]; }
    AudioBuffer& operator[](std::size_t idx) noexcept { return m_buffers[idx]; }

    // Return the number of samples per buffer or 0 if there are no buffers.
    std::size_t samplesPerBuffer() const noexcept;

private:
    std::vector<AudioBuffer> m_buffers;
    std::vector<Sample*> m_bufferPtrs;
};

void copy(AudioBufferArray const& source, AudioBufferArray& destination) noexcept;
void copy(AudioBufferArray const& source, Sample** destination) noexcept;

} // namespace mpegcore

#endif // MPEGCORE_AUDIOBUFFERARRAY_HPP
