/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_HOATRANSLATORCONFIG_HPP
#define MPEGCORE_HOATRANSLATORCONFIG_HPP

#include <string>

#include <hoatranslation/MultiPointHoaTranslator.hpp>

namespace mpegcore {

struct HoaParametersConfig
{
    bool isInputESD;
};

struct HoaTranslatorConfig
{
    bool isActive{ true };
    int hopSize{ 128 };
    int hybridMode{ 1 };
    int LDmode{ 0 };
    float interpolationEqGainMax{ 4.0F };
    int lowerBandFreqLimit{ 4000 };
    int upperBandFreqLimit{ 10000 };
    bool useDiffuseDecorrelation{ true };
    bool useEqualizer{ true };
    int nAnalysisChannels{ 4 };
    bool useEncodedValues{ true };
    std::string matlabHRTFFile{};
    std::string matlabDirsFile{};
    std::string matlabITDFile{};
    bool useMatlabFile{ false };
    std::string resourcePath{};
    bool isInputSN3D{ false };
    std::string spatialMetadataBinaryFile{};
    bool nonParametricOnly{ false };
    bool useImprovedExteriorRendering{ true };
    float exteriorProjectionRadius{ 4.0F };
    bool adaptiveExteriorProjectionRadius{ true };
    int groupId;
    bool useRecursiveAveraging{ true };
    float recursionSmoothingCoefficient{ 0.9F };
    DIFFUSE_METHOD_FOR_TARGET_SCM diffuseMethodForTargetSCM{ DIFFUSE_COVARIANCE_MATRIX };
    std::string sofaFile;
    bool useSphoa6DofExtension;
    float sphoa6DofMaxGainDb{ 9.0F };
    float sphoa6dofProjectionRadius;
    int NthSphoaInEncodedMetadata{ 1 };
    bool useSourceAwareMode{ false };
    bool useSourceAwarePostFilter{ true };
    bool useSourceAwareTimeAlignment{ true };
    SOURCE_RESIDUAL_MODE sourceResidualMode{ BOTH };
    TARGET_SCM_MODE targetScmMode{ SEPARATE_SCM };
    int maxSimulInformedSources;
};

} // namespace mpegcore

#endif // MPEGCORE_HOATRANSLATORCONFIG_HPP
