/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_RINGBUFFER_HPP
#define MPEGCORE_RINGBUFFER_HPP

#include <cstddef>

#include <mpegcore/AudioBuffer.hpp>

namespace mpegcore {

class RingBuffer
{
public:
    // `RingBuffer` implements a single channel sample ring buffer with fixed delay.
    //
    // - `writeSize` determines how many samples we want to write without reading.
    // - `delayInSamples` is the number of samples the data is delayed within the buffer between
    //   writes and reads.
    //
    // Write does not need to happen in a single operation. If read and write operations use
    // different buffer sizes caller needs to make sure `writeSize` is large enough to receive data
    // after read operations that do not exhaust data.
    RingBuffer(std::size_t writeSize, std::size_t delayInSamples);

    void read(AudioBuffer& destination);
    void write(AudioBuffer const& source);

private:
    AudioBuffer m_buffer;
    std::size_t m_readIndex{ 0 };
    std::size_t m_readSamples{ 0 };
    std::size_t m_writeIndex{ 0 };
    std::size_t m_writtenSamples{ 0 };
};

} // namespace mpegcore

#endif // MPEGCORE_RINGBUFFER_HPP
