/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_ALGEBRA_LERP_HPP
#define MPEGCORE_ALGEBRA_LERP_HPP

namespace mpegcore {
namespace algebra {

template <typename T> struct Domain
{
    T x0;
    T x1;
};

template <typename T> struct Range
{
    T y0;
    T y1;
};

// Return linear interpolation of value in `range` based on `value` in `domain`.
template <typename TD, typename TR> constexpr TR lerp(Range<TR> range, Domain<TD> domain, TD value)
{
    auto const x0 = domain.x0;
    auto const x1 = domain.x1;
    auto const y0 = range.y0;
    auto const y1 = range.y1;
    return static_cast<TR>((y0 * (x1 - value) + y1 * (value - x0)) / (x1 - x0));
}

} // namespace algebra
} // namespace mpegcore

#endif // MPEGCORE_ALGEBRA_LERP_HPP
