/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_DATATYPES_HPP
#define MPEGCORE_DATATYPES_HPP

#include <array>
#include <chrono>
#include <cstddef>
#include <cstdint>
#include <string>
#include <vector>

#include <mpegcore/datatypes_reverb.hpp>

namespace mpegcore {

struct NeuralNetworkParameters
{
    using Vector = std::vector<double>;
    using Matrix = std::vector<Vector>;

    Vector theta1;
    Vector theta2;
    Vector theta3;
    Vector tMin;
    Vector tMax;
    Vector xMin;
    Vector xMax;
    Matrix W1;
    Matrix W2;
    Matrix W3;

    static Matrix makeMatrix(std::size_t rowCount, std::size_t columnCount, Vector data);
};

struct AcousticParameters
{
    struct FrequencyData
    {
        float frequency;
    };

    struct FrequencyDecay : public FrequencyData
    {
        float rt60;
        std::optional<float> rdr;
    };

    struct FrequencyGain : public FrequencyData
    {
        float gain;
    };

    struct DirectionalGain
    {
        float azim;
        float elev;
        std::vector<FrequencyGain> frequencyResponse;
    };

    std::optional<double> preDelaySec;

    //! Sorted by frequency in ascending order.
    std::vector<FrequencyDecay> frequencyDecayResponse;

    //! Direction- and frequency-dependent gain
    std::optional<std::vector<DirectionalGain>> directionalFrequencyResponse;

    //! Whether these parameters were populated from RIR analysis
    bool isFromRIR{ false };
};

} // namespace mpegcore

#endif // MPEGCORE_DATATYPES_HPP
