/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_SPDLOG_HPP
#define MPEGCORE_SPDLOG_HPP

// SPDLOG_ACTIVE_LEVEL sets the level that is allowed by the logging macros. If
// we always use `spdlog::set_level` to allow `spdlog::level::trace` and then
// use the logging macros (e.g. SPDLOG_DEBUG) to log then SPDLOG_ACTIVE_LEVEL
// controls which logging gets compiled in.
//
// For changing of logging levels at runtime we'd need to use the `spdlog`
// functions directly instead of the logging macros so that individual loggers'
// levels could be set.

#ifdef NDEBUG
#define SPDLOG_ACTIVE_LEVEL SPDLOG_LEVEL_INFO
#else
#define SPDLOG_ACTIVE_LEVEL SPDLOG_LEVEL_TRACE
#endif

// Include any spdlog headers having compiler warnings so that they can be
// marked as system headers and warnings from them can be suppressed.
#if defined(__clang__)
#pragma clang system_header
#endif // defined(__clang__)

#if defined(WIN32)
// Without this Windows compilation will get confused about byte type.
#define WIN32_LEAN_AND_MEAN
#endif // defined(WIN32)

// Include the main spdlog.h first, otherwise Windows linking will get errors
// about multiply defined symbols.
#include <spdlog/spdlog.h>

#include <spdlog/fmt/ostr.h>
#include <spdlog/sinks/stdout_color_sinks.h>

#endif // MPEGCORE_SPDLOG_HPP
