/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <algorithm>

#include <hoatranslation/MultiPointHoaTranslator.hpp>

void example_of_execution() {

    hoaTranslatorData *h = new hoaTranslatorData;

    int frameSize = 2048;
    int sampleRate = 48000;
    int hopSize = 128;
    int hybridMode = 1;
    int LDmode = 0;
    int outChannels = 2;
    int nMicrophoneArrays = 5;
    int channelsPerArray = 16;
    float interpolationEqGainMax = 4.0f;
    int nAnalysisChannels = 4;
    BINAURAL_AMBI_DECODER_METHODS decodingMethod = BINAURAL_DECODER_MAGLS;
    bool useEncodedValues = false;
    std::string resourcePath = "/Users/lapajune/CLionProjects/test_hoa_translator/hoa_translator/resources/";   // <-- change this accordingly
    DIFFUSE_METHOD_FOR_TARGET_SCM diffuseMethodForTargetSCM = DIFFUSE_COVARIANCE_MATRIX;
    std::vector<std::vector<float>> arrayPositionsOpenGL(nMicrophoneArrays, std::vector<float>(3, 0.0F));
    std::vector<std::vector<float>> arrayOrientationsOpenGL(nMicrophoneArrays, std::vector<float>(3, 0.0F));
    int groupId = 0;

    inputFilesAndFlags inputFiles{"hrtfs/hrtf_dirs_from_matlab.txt",
                                  "hrtfs/hrtfs_from_matlab.txt",
                                  "hrtfs/itds_from_matlab.txt",
                                  "",
                                  "data/test_6dof_metadata.bin",
                                  false };

    optimalMixingParameters optimalMixingParams{4000,
                                                10000,
                                                true,
                                                true,
                                                false,
                                                0.9f};

    exteriorRenderingParameters exteriorRenderingParams{true,
                                                        4.0f};

    sphoa6DofExtensionParameters sphoaParameters{false, 0.0f, 0.0f, 1};

    micArrayPositionsAndOrientations micPosAndOrientations{arrayPositionsOpenGL, arrayOrientationsOpenGL};

    std::vector<float> linearSourceGains(nMicrophoneArrays, 1.0f);

    std::vector<int> complexityReductionParameters{};

    sourceAwareParameters sourceAwareParams{false, true, true, BOTH, SEPARATE_SCM, {}, 10};

    std::vector<sourceAwareParametersPerSource> sourceAwareParamsPerSource{};

    create_hoa_translator(&h,
                          frameSize,
                          sampleRate,
                          hopSize,
                          hybridMode,
                          LDmode,
                          outChannels,
                          channelsPerArray,
                          nAnalysisChannels,
                          resourcePath,
                          diffuseMethodForTargetSCM,
                          micPosAndOrientations,
                          read_file_to_vector(resourcePath + inputFiles.spatialMetadataBinaryFile),
                          sourceAwareParams,
                          sourceAwareParamsPerSource);

    initialize_hoa_translator(&h,
                              groupId,
                              useEncodedValues,
                              interpolationEqGainMax,
                              decodingMethod,
                              inputFiles,
                              optimalMixingParams,
                              exteriorRenderingParams,
                              sphoaParameters,
                              linearSourceGains,
                              complexityReductionParameters);

    //// Process loop
    // Execute below actions to a single input frame

    // Read one frame from the input signal (fill inputSignalBuffer somehow, e.g. read from .wav file or copy from another source)
    // NOTE: the input should be in N3D ACN
    for (int mic = 0; mic < h->nMicrophoneArrays; ++mic) {
        int currentSHOrder = 0;
        for (int ch = 0; ch < h->channelsPerArray; ++ch) {
            if (std::floor(std::sqrt(ch)) >= (currentSHOrder + 1)) {
                ++currentSHOrder;
            }
            for (int n = 0; n < h->frameSize; ++n) {
                h->timeDomainInputFrame[mic][ch][n] = h->inputSignalBuffer[n * h->channelsPerArray + ch];
            }
        }
    }

    if (!h->useSphoa6DofExtensionMode) {
        // Collect the interpolation weights corresponding to this frame/listener position
        pre_process_positions(h);

        // Save the current weights and triangle to the beginning of the track record
        save_tracked_values(h);

        // Select active triangle and interpolation weights from a set of current
        // and previous frames
        choose_active_triangle_from_track_record(h);

        check_for_crossfade_start(h);

        determine_active_arrays(h);
    }

    // Transform to time-frequency domain
    transform_to_time_frequency_domain(h);

    // Obtain DOA angles, DTR, and energy values (metadata)
    if (h->useEncodedValues) {
        read_spatial_metadata_from_binary(h);
    } else {
        spatial_analysis(h);
    }

    //// Metadata interpolator
    // NOTE: in this example, the yaw, pitch, and roll angles are not updated.
    interpolate_mic_metadata(h);

    //// Signal interpolator
    interpolate_signals(h);

    //// Synthesis processor
    generate_prototype_signals(h);
    optimal_mixing(h);

    // Transform back to time-domain
    transform_to_time_domain(h, h->outputTFDProcessingFrame);

    // Write the current frame to output
    for (int ch = 0; ch < h->outChannels; ++ch) {
        for (int n = 0; n < h->frameSize; ++n) {
            h->outputSignalBuffer[n * h->outChannels + ch] = h->timeDomainOutputFrame[ch][n];
        }
    }

    // Save values from this frame, check if crossfade can be stopped, increase the counter for processed frames
    post_rendering_actions(h);

    //// process loop

    destroy_hoa_translator(&h);
    delete h;
}
