#!/bin/bash

set -e

if [[ $# -eq 0 ]]; then
    cat <<EOT
Build MPEG HOA Translation

Usage:

  $(basename $0) build_type [test] [-- [cmake_arg ...]]

  The build_type argument has to be "release", "debug".
  If "test" argument is given then tests will be built.
  Any arguments after -- are passed to Cmake configuration as such.

  Examples:
      $(basename $0) release
      $(basename $0) debug test

EOT
    exit 1
fi

prg=$0
if [ ! -e "$prg" ]; then
  case $prg in
    (*/*) exit 1;;
    (*) prg=$(command -v -- "$prg") || exit;;
  esac
fi
prg_dir=$(
  cd -P -- "$(dirname -- "$prg")" && pwd -P
) || exit
project_dir=$(dirname -- "$prg_dir") || exit

if [ "$1" == "release" ]; then
    build_type=${1}
elif [ "$1" == "debug" ]; then
    build_type=${1}
else
    echo "Error: the build_type argument has to be \"release\" or \"debug\""
    exit 1
fi
shift

build_os="unknown"
if [[ "$OSTYPE" == "linux-gnu" ]]; then
    build_os="linux"
elif [[ "$OSTYPE" == "darwin"* ]]; then
    build_os="macos"
fi
compiler=`cmake --system-information \
    | grep -E 'CMAKE_CXX_COMPILER_ID ".*"$' \
    | sed 's/^.* "\(.*\)"/\L\1/'`
build_dir=build-mpeg-hoa-translator-${build_os}-${compiler}-${build_type}

cmake_options=()
if [[ "${1}" == "test" ]]; then
    build_dir="${build_dir}-test"
    cmake_options+=(-DMPEG_HOA_TRANSLATOR_TESTS=ON)
    shift
fi

if [[ "${1}" == "--" ]]; then
    shift
    cmake_options+=("$@")
fi

cmake ${project_dir} -B${build_dir} -DCMAKE_BUILD_TYPE=${build_type} "${cmake_options[@]}"
cmake --build ${build_dir} --config ${build_type} -- -j8

set +e
