@echo off
setlocal EnableDelayedExpansion

if [%1] == [] (
    @echo(
    @echo Build mpeg-i-cfp-hoa-translation for Windows with MSVC
    @echo(
    @echo Usage:
    @echo   %0 build_type lib_path [test]
    @echo(
    @echo Where
    @echo     build_type is release or debug.
    @echo     lib_path is path to dependencies libs.
    @echo     test if given then tests will be built.
    @echo(
    @echo  NOTE: all paths are relative to the mpeg-i-cfp-hoa-translation directory.
    goto :EOF
)

if [%2] == [] (
  @echo Missing lib_path
  (call) && rem Call used to set error level to 1
  goto :error
)
set "MPEG_I_HOA_TRANSLATION_LIBRARY_PATH=%~dp0\..\%~2"
set "TINYXML2_PATH=%~dp0\..\%~2\tinyxml2"

call :getVisualStudioGenerator

if "%1" == "release" (
    set build_folder=build-vs-win-release
    if [%3] == [test] (
        cmake . -B!build_folder! -G %vs_generator% -DCMAKE_BUILD_TYPE=Release -DMPEG_HOA_TRANSLATOR_TESTS=ON || goto :error
    ) else (
        cmake . -B!build_folder! -G %vs_generator% -DCMAKE_BUILD_TYPE=Release || goto :error
    )
    cmake --build !build_folder! --config Release || goto :error
) else if "%1" == "debug" (
    set build_folder=build-vs-win-debug
    if [%3] == [test] (
        cmake . -B!build_folder! -G %vs_generator% -DCMAKE_BUILD_TYPE=Debug -DMPEG_HOA_TRANSLATOR_TESTS=ON || goto :error
    ) else (
        cmake . -B!build_folder! -G %vs_generator% -DCMAKE_BUILD_TYPE=Debug || goto :error
    )
    @echo GOING TO BUILD
    cmake --build !build_folder! --config Debug || goto :error
) else (
  @echo Unknown build_type
  rem Call used to set error level to 1
  (call)
  goto :error
)

goto :EOF

:error
echo Failed with error #%errorlevel%.
exit /b %errorlevel%

:getVisualStudioGenerator
set arrayLen=0
set vs_versions[0]=0
for /f "tokens=*" %%i in ('"C:\Program Files (x86)\Microsoft Visual Studio\Installer\vswhere.exe"') do (
    set temp_output_line=%%i
    set trimmed_token=!temp_output_line:catalog_productLineVersion: =!
    if NOT "!temp_output_line!"=="!trimmed_token!" (
        set vs_versions[!arrayLen!]=!trimmed_token!
        set /a "arrayLen+=1"
        if !trimmed_token! == 2017 (
            set vs_version=2017
        )
    )
)
if NOT !vs_version! == 2017 (
    for /l %%n in (0,1,%array_len%+1) do (
        if !vs_versions[%%n]! == 2019 (
            set vs_version=2019
        )
    )
)
if !vs_version! == 2017 (
    set vs_generator="Visual Studio 15 2017 Win64"
) else if !vs_version! == 2019 (
    set vs_generator="Visual Studio 16 2019"
) else (
  @echo Visual Studio version needs to be either 2017 or 2019
  (call) && rem Call used to set error level to 1
  goto :error
)
exit /b 0