/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef HOA_TRANSLATION_METADATAINTERPOLATOR_HPP
#define HOA_TRANSLATION_METADATAINTERPOLATOR_HPP

#include "Utilities.hpp"

/* Vectorize the spatial metadata as (in OpenGL):
 *      tempV1 (right) = dtr * -sin(azi) * cos(elev)
 *      tempV2 (up)    = dtr * sin(elev)
 *      tempV3 (back)  = dtr * -cos(azi) * cos(elev)
 */
void vectorize_metadata(hoaTranslatorData *h,
                        bool interpolateResidual);

/* Rotate the metadata vectors based on head and mic array orientations
 *
 * Output:
 *      rotated tempV1, tempV2 and tempV3
 */
void rotate_metadata_vectors(hoaTranslatorData *h);

/* Interpolate the metadata parameters (DOA angles, DTR and energy).
 *
 * The spatial metadata for each mic array can be expressed in vector form as (in OpenGL):
 *
 *      v1 (right) = dtr * -sin(azi) * cos(elev)
 *      v2 (up)    = dtr * sin(elev)
 *      v3 (back)  = dtr * -cos(azi) * cos(elev)
 *
 * The vectors are then averaged for the active triangle as:
 *
 *      v1 = (w1 * v1(mic1)) + (w2 * v1(mic2)) + (w3 * v1(mic3))
 *      v2 = (w1 * v2(mic1)) + (w2 * v2(mic2)) + (w3 * v2(mic3))
 *      v3 = (w1 * v3(mic1)) + (w2 * v3(mic2)) + (w3 * v3(mic3))
 *
 * , where wx is the interpolation weight for a mic array.
 * The interpolated metadata is obtained with:
 *
 *      interpolated_azi    = atan2(-v1, -v3)
 *      interpolated_elev   = atan2(v2, sqrt(v1^2 + v3^2))
 *      interpolated_dtr    = sqrt(v1^2 + v2^2 + v3^2)
 *      interpolated_energy = (w1 * energy1) + (w2 * energy2) + (w3 * energy3)
 *
 */
void interpolate_metadata(hoaTranslatorData *h, bool interpolateResidual);

// Interpolate the metadata obtained from the mic arrays
void interpolate_mic_metadata(hoaTranslatorData *h);

// Interpolate the residual metadata (only used in source aware mode)
void interpolate_residual_metadata(hoaTranslatorData *h);

#endif //HOA_TRANSLATION_METADATAINTERPOLATOR_HPP
