#ifndef HOA_TRANSLATION_OPTIMALMIXING_HPP
#define HOA_TRANSLATION_OPTIMALMIXING_HPP

#include "Utilities.hpp"

/* Formulate mixing matrix M and the residual of the mixing matrix Cr from binaural input based on [1].
 *
 * Input:
 *      Cx                  - estimate for the target covariance matrix (2x2)
 *      Cy                  - the target covariance matrix (2x2)
 *      Q                   - prototype signal matrix (identity matrix, 2x2)
 *      useEnergyFlag       - flag to indicate if energy compensation is applied to M
 *      regularizationCoeff - The higher the coefficient, the less boost to noise but also higher decorrelation. Low values
 *                            should be OK, if the input has high SNR. (default 0.2)
 *      calculateCr         - flag to indicate if Cr is calculated
 *
 * Output:
 *      M                   - mixing matrix (2x2)
 *      Cr                  - residual of the mixing matrix M (2x2)
 *
 * [1] Vilkamo, J., Bäckström, T., & Kuntz, A. (2013). Optimized
 *     covariance domain framework for time--frequency processing of
 *     spatial audio. Journal of the Audio Engineering Society, 61(6),
 *     403-411.
 */
void formulate_M_and_Cr_2x2(hoaTranslatorData *h,
                            const hoatMatrixCmplx2D &Cx,
                            const hoatMatrixCmplx2D &Cy,
                            const hoatMatrix2D &Q,
                            bool useEnergyFlag,
                            float regularizationCoeff,
                            hoatMatrixCmplx2D &M,
                            hoatMatrixCmplx2D &Cr,
                            bool calculateCr);

#endif //HOA_TRANSLATION_OPTIMALMIXING_HPP
