/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef HOA_TRANSLATION_ROTATOR_HPP
#define HOA_TRANSLATION_ROTATOR_HPP

#include "Utilities.hpp"

/* Get rotation matrix R.
 *
 * NOTE: when applying the matrix R to a vector (x, y, z) in OpenGL convention, it is assumed the axes follow:
 *      - x (right)
 *      - y (up)
 *      - z (back)
 * Input:
 *      - yawDeg,   yaw rotation in degrees (right-handed rotation around y-axis in OpenGL)
 *      - pitchDeg, pitch rotation in degrees (right-handed rotation around x-axis in OpenGL)
 *      - rollDeg,  roll rotation in degrees (right-handed rotation around z-axis in OpenGL)
 *
 * Output:
 *      - R, rotation matrix
 */
void get_rotation_matrix_OpenGL(float yawDeg,
                                float pitchDeg,
                                float rollDeg,
                                float R[3][3]);

/* Get rotation matrix R for Euler convention, where the axis follow:
 *      - x (front)
 *      - y (left)
 *      - z (up)
 *
 * and the rotations:
 *      - yaw (right-handed rotation around z-axis)
 *      - pitch (right-handed rotation around y-axis)
 *      - roll (right-handed rotation around x-axis)
 */
void get_rotation_matrix_classic(float yawDeg,
                                 float pitchDeg,
                                 float rollDeg,
                                 float R[3][3]);

/* Get spherical harmonics rotation matrix, that is used to rotate the prototype signal.
 *
 * OpenGL and ACN/SAF coordinate axes and orientations differ, see below. Because of the different rotations,
 * pitch and roll rotation angles need to be reverted here.
 *
 * ACN/SAF axes: x (front), y (left), z (up)
 * OpenGL axes:  x (right), y (up),   z (back)
 *
 *    OpenGL:                     ACN/SAF:                        Result:
 *    * yaw around up-axis        * yaw around up-axis (+)        * +yaw
 *    * pitch around right-axis   * pitch around left-axis (-)    * -pitch
 *    * roll around back-axis     * roll around front-axis (-)    * -roll
 *
 * Output:
 *      currentSHrotationMatrix    - spherical harmonics rotation matrix for the current closest mic array signal
 *      previousSHrotationMatrix   - spherical harmonics rotation matrix for the previous closest mic array signal
 *                                   (output only if crossfade is in progress)
 */
void get_sh_rotation_matrix(hoaTranslatorData *h);


#endif //HOA_TRANSLATION_ROTATOR_HPP
