/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef HOA_TRANSLATION_SIGNALINTERPOLATOR_HPP
#define HOA_TRANSLATION_SIGNALINTERPOLATOR_HPP

#include "Utilities.hpp"

/* Interpolate the time-frequency signals based on the interpolation data (parametric approach) as:
 *
 *      interpolated_signal = eqGains * closest_mic_array_signal
 *
 * , where equalization gains are obtained from:
 *
 *      eqGains = min( sqrt( ((w1 * E1) + (w2 * E2) + (w3 * E3)) / E_closest_mic_array ), interpolationEqGainMax )
 *
 * Here wx and Ex correspond to the interpolation weight and energy of a mic array in a triangle, respectively.
 * interpolationEqGainMax is some maximum multiplier, so that the closest array signal doesn't get unexpected
 * infinite multiplication.
 *
 * NOTE: Energy calculated from single frequency bins for now (no frequency band or temporal averaging, this might need some improvement).
 *
 * Output:
 *      interpolatedTFDFrame    - interpolated time-frequency domain signal
 */
void interpolate_signals(hoaTranslatorData *h);

/* Interpolate the time-frequency signals (non-parametric approach) as:
 *
 *      interpolated_signal = (w1 * sig1) + (w2 * sig2) + (w3 * sig3)
 *
 * , where wx corresponds to the interpolation weight, and sigx to the time-frequency signal of a mic array in a triangle.
 * This approach is non-parametric, since it doesn't consider signal energies, that are obtained from the spatial analysis
 * of the mic array signals.
 *
 * NOTE: Energy calculated from single frequency bins for now (no frequency band or temporal averaging, this might need some improvement).
 *
 * Output:
 *      interpolatedTFDFrame    - interpolated time-frequency domain signal
 */
void interpolate_three_closest_arrays(hoaTranslatorData *h);

#endif //HOA_TRANSLATION_SIGNALINTERPOLATOR_HPP
