/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef HOA_TRANSLATION_SOURCEAWARE_HPP
#define HOA_TRANSLATION_SOURCEAWARE_HPP

#include "Utilities.hpp"

/* Get beamformed source signals from the closest mic arrays to the source positions.
 *
 * Output:
 *      TFDSourceFrame   - beamformed source signals in TF-domain (nSources x nFrames x nBands)
 */
void get_beamformed_source_signals(hoaTranslatorData *h);

/* Time-align source signals at each mic array position and at the listener position.
 *
 * Output:
 *      TFDMicSrcTAFrame        - time-aligned source signals (from mics to sources), (nMicrophones x nSources x nFrames x nBands)
 *      TFDListenerSrcTAFrame   - time-aligned source signals (from listener to sources), (nSources x nFrames x nBands)
 */
void time_align_source_signals(hoaTranslatorData *h);

/* Estimate spatial covariance matrices (SCM) for mic array source signals
 *
 * Output:
 *      Cam_src - ambisonics (FOA) model SCMs for the source signals (from mics to sources), (nMicrophoneArrays x nBands x 4 x 4)
 */
void estimate_mic_model_SCM(hoaTranslatorData *h);

/* Estimate SCMs for listener source signals.
 *
 * Output:
 *      Cao_src - ambisonics (FOA) model SCMs for the source signals (from listener to sources), (nBands x 4 x 4)
 *      Cyo_src - binaural model SCMs for the source signals (from listener to sources), (nBands x 2 x 2)
 */
void estimate_listener_model_SCM(hoaTranslatorData *h);

/* Spatial analysis for the mic array source signals (DOA angles, DTR, energy).
 * Note: the energy data is for a single sub-frame.
 *
 * Output:
 *      srcAzimuthData      - azimuth data   (nMicrohoneArrays x nBands)
 *      srcElevationData    - elevation data (nMicrohoneArrays x nBands)
 *      srcDTRData          - DTR data       (nMicrohoneArrays x nBands)
 *      srcEnergyData       - energy data    (nMicrohoneArrays x nBands)
 */
void spatial_analysis_for_mic_model_SCM(hoaTranslatorData *h);

// Build target SCM (Cy) by first combining source and residual parameters, and then calculating the target SCM from the
// combined parameters.
void build_target_SCM_from_combined_parameters(hoaTranslatorData *h);

// Build target SCM (Cy) by combining source and residual SCMs.
void build_target_SCM_from_separate_SCMs(hoaTranslatorData *h);

// Determine which informed sources are active for the source aware processing. Sources beyond maxSrcToListDist limit are
// turned off. If the mic array closest to the source has not been processed long enough (i.e. the user has just moved to
// the vicinity of the source), the source is also ignored for the source aware processing.
void determine_active_sources_and_update_track_record(hoaTranslatorData *h);

// Check if source aware mode should be activated or deactivated. If there are no active sources (i.e. the listener is far
// away from all the informed sources), turn the source aware mode off.
void check_for_source_aware_mode(hoaTranslatorData *h);

// Processing
void pre_process_source_aware(hoaTranslatorData *h);
void source_aware_processing(hoaTranslatorData *h);


#endif //HOA_TRANSLATION_SOURCEAWARE_HPP
