/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef HOA_TRANSLATION_STFTTRANSFORM_HPP
#define HOA_TRANSLATION_STFTTRANSFORM_HPP

#include "Utilities.hpp"

/* Transform input signals from time domain to time-frequency domain using the afSTFT from SAF
 *
 * Output:
 *      TFDInputFrame    - input signals in time-frequency domain
 */
void transform_to_time_frequency_domain(hoaTranslatorData *h);

/* Transform output signal from time-frequency domain to time domain using the inverse afSTFT from SAF
 *
 * Input:
 *      processedTFDFrame        - output signal in time-frequency domain
 *
 * Output:
 *      timeDomainOutputFrame    - output signal in time domain
 */
void transform_to_time_domain(hoaTranslatorData *h,
                              float_complex ***processedTFDFrame);

// Same as above, but with custom matrix input
void transform_to_time_domain(hoaTranslatorData *h,
                              const hoatMatrixCmplx3D &processedTFDMatrix);

#endif //HOA_TRANSLATION_STFTTRANSFORM_HPP
