/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_AUDIOVECTORBUFFER_HPP
#define MPEGCORE_AUDIOVECTORBUFFER_HPP

// `AudioVectorBuffer` implements a vector sample buffer for several channels of
// audio.

#include <array>
#include <cstddef>
#include <vector>

#ifndef MPEGI_ARM_BUILD
#include <vectorclass/vectorclass.h>
#else
#include "VclCompatibilityLayer.hpp"
#endif

#include <mpegcore/AudioBuffer.hpp>

namespace mpegcore {

using AudioVectorBuffer = std::vector<vcl::Vec8f>;

constexpr std::size_t AUDIO_VECTOR_BUFFER_WIDTH{ sizeof(AudioVectorBuffer::value_type)
    / sizeof(float) };
static_assert(AUDIO_VECTOR_BUFFER_WIDTH == 8);

using AudioVectorAudioBufferPointers = std::array<AudioBuffer*, AUDIO_VECTOR_BUFFER_WIDTH>;

// Interleave samples from `source` buffers into `target`.
void interleave(AudioVectorAudioBufferPointers const& source, AudioVectorBuffer& target);

// Deinterleave samples from `source` buffer into `target` buffers.
void deinterleave(AudioVectorBuffer const& source, AudioVectorAudioBufferPointers& target);

} // namespace mpegcore

#endif // MPEGCORE_AUDIOVECTORBUFFER_HPP
