/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_DIRECTIONALDECAY_HPP
#define MPEGCORE_DIRECTIONALDECAY_HPP

#include <cstdint>
#include <optional>
#include <string>

#include "ShOrder.hpp"
#include <mpegcore/Hoa.hpp>

#include <../src/rirlib_internal.h>
#include <rirlib.h>

#include <EAR/AudioFile.h>
#include <EAR/Geometry.h>

namespace mpegcore {

// For storing init parameters, pointer to the analysis object, creating and destroying it
struct SHDecay
{
    std::string srirPath;
    HoaNormalization normalization{ HoaNormalization::n3d };
    std::optional<EAR::Position> sourcePosition;
    std::optional<EAR::Position> micPosition;
    int srcDirectivityFlag{ 1 }; // 0: omni, 1: "regular" lousdpeaker directivity
    void* pRirLib{ nullptr };

    EAR::AudioFile srir; // Set by init()

    void init(std::size_t const numDelayLines);
    // To perform analysis on a file read from disk
    void setRIR_file();
    // To perform analysis on an internal buffer, e.g. the FDN's IR
    void setRIR_buffer(const float** bufPtr, std::size_t const nChan, std::size_t const nSamp,
        std::uint32_t const sampleRate);
    bool isInitialized();
    void analyze(ANALYSIS_STAGE endStage);

    ~SHDecay();
};

} // namespace mpegcore

#endif // MPEGCORE_DIRECTIONALDECAY_HPP
