/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_ESDCONVERTER_HPP
#define MPEGCORE_ESDCONVERTER_HPP

#include <cstddef>
#include <cstdint>
#include <vector>

#include <mpegcore/AudioBuffer.hpp>

namespace mpegcore {

class EsdConverter
{
public:
    EsdConverter() = default;
    EsdConverter(std::uint32_t samplesPerBuffer);
    ~EsdConverter();

    bool convertEsdToHoa(std::size_t order);
    bool convertHoaToEsd(std::size_t order);
    void initialize(std::uint32_t samplesPerBuffer);

    float* inputAudioData(std::size_t channelIndex) noexcept;

    void copyToInput(AudioBuffer const& source, std::size_t channelIndex);
    void copyToInput(float const* data, std::size_t sampleCount, std::size_t channelIndex);

    float const* getOutputData(std::size_t channelIndex) const noexcept;
    float** getOutputPointerArray() { return m_output; };

    bool isInitialized() { return m_isInitialized; };
    std::size_t maxConversionOrder() { return m_maxConversionOrder; };
    std::uint32_t samplesPerConversion() { return m_samplesPerConversion; };

private:
    bool m_isInitialized{ false };
    std::size_t m_maxConversionOrder{ 4 };
    std::vector<std::size_t> m_channelsPerConversion;
    std::uint32_t m_samplesPerConversion;
    std::vector<float**> m_EsdtoHoaMatrices;
    std::vector<float**> m_HoatoEsdMatrices;

    float** m_input{ nullptr };
    float** m_output{ nullptr };
};

/* Get the ESD to HOA conversion matrix. More details about the conversion below in
 * convert_esd_to_hoa().
 *
 * Input:
 *      nChannels   - number of channels in one HOA input signal (= (order+1)^2)
 *      M_ESDtoHOA  - pre-allocated handle for the ESD to HOA conversion matrix (size: nChannels *
 *                    nChannels)
 *
 * Output:
 *      M_ESDtoHOA  - ESD to HOA conversion matrix
 *
 * Return:
 *      true if the matrix was generated succesfully; false otherwise
 */
bool get_esd_conversion_matrix(const int nChannels, float* M_ESDtoHOA);

} // namespace mpegcore

#endif // MPEGCORE_ESDCONVERTER_HPP
