/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_HOABUS_HPP
#define MPEGCORE_HOABUS_HPP

#include <memory>
#include <vector>

#include <mpegcore/Config.hpp>
#include <mpegcore/Rotation.hpp>

#include "ShOrder.hpp"
#include <mpegcore/AudioBufferArray.hpp>

namespace mpegcore {

class HoaBus
{
public:
    class InputBus
    {
        friend class HoaBus;

    public:
        InputBus(AudioBufferArray* buffers, Rotation* rotation);

        // NOTE(mikkogy,20190917) reference is valid for the whole lifetime of
        // HoaBus.
        AudioBufferArray& buffers();

        void setRotation(Rotation const& rotation);

    private:
        AudioBufferArray* m_buffers;
        Rotation* m_rotation;
    };

    class OutputBus
    {
        friend class HoaBus;

    public:
        struct Hoa
        {
            AudioBufferArray const* buffers;
            Rotation const* rotation;
        };

        OutputBus(Hoa&& hoa);
        // NOTE(mikkogy,20190917) reference is valid for the whole lifetime of
        // HoaBus::OutputBus.
        HoaBus::OutputBus::Hoa const& getHoa() const;

        ShOrder order() const noexcept;

    private:
        Hoa m_hoa;
    };

    HoaBus(Config const& config);

    HoaBus::InputBus& createInputBus(ShOrder order);

    // NOTE(mikkogy,20190923) iterators to output buses may be invalidated if
    // createInputBus is called.
    std::vector<HoaBus::OutputBus const*> const& getOutputBuses() const;

private:
    struct BusData
    {
        AudioBufferArray audioBufferArray;
        Rotation rotation;
        InputBus inputBus;
        OutputBus outputBus;
        BusData(AudioBufferArray&& audioBufferArray, Rotation&& rotation, ShOrder order);
    };

    std::vector<std::unique_ptr<BusData>> m_busData;
    Config const m_config;
    std::vector<HoaBus::OutputBus const*> m_outputBuses;
};

} // namespace mpegcore

#endif // MPEGCORE_HOABUS_HPP
