/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_HOADECODER_HPP
#define MPEGCORE_HOADECODER_HPP

#include <memory>
#include <optional>
#include <vector>

#include <mpegcore/Config.hpp>

#include "HoaBus.hpp"
#include "ShBinDecoder.hpp"
#include "ShLsDecoder.hpp"
#include "ShRotator.hpp"
#include <mpegcore/AudioBufferArray.hpp>

namespace mpegcore {

class HoaDecoder
{
public:
    // Initialize this decoder. Sets up the decoder data structures based on the
    // current state of the `hoaBus`. All input buses whose output buses are to
    // be decoded by this `HoaDecoder` have to have been created using `hoaBus`
    // before calling this. If new input buses are created with `hoaBus` after
    // calling this then the output buses corresponding to the newly created
    // input buses won't be decoded by this `HoaDecoder`. Returns true on
    // success and false otherwise.
    bool initialize(Config const& config, HoaBus const& hoaBus);
    void render();

    AudioBufferArray const& outputBuffer() const;

private:
    struct Decoder
    {
        std::optional<ShBinDecoder> binDecoder;
        std::optional<ShLsDecoder> lsDecoder;
        AudioBufferArray rotatedBuffers;
        ShRotator rotator;
        HoaBus::OutputBus::Hoa const& hoa;

        Decoder(std::optional<ShBinDecoder>&& binDec, std::optional<ShLsDecoder>&& lsDec,
            AudioBufferArray&& rotatedBuffers, ShRotator&& rot, HoaBus::OutputBus::Hoa const& hoa);

        Decoder() = delete;
        Decoder(Decoder const& decoder) = delete;
    };

    std::vector<std::unique_ptr<Decoder>> m_decoders;
    AudioBufferArray m_outputBuffer;
    AudioBufferArray m_renderBuffer;
};

} // namespace mpegcore

#endif // MPEGCORE_HOADECODER_HPP
