/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_REVERBERATORPARAMETERS_HPP
#define MPEGCORE_REVERBERATORPARAMETERS_HPP

#include <cstddef>
#include <cstdint>
#include <optional>
#include <unordered_map>

#include <EAR/BaseTypes.h>
#include <EAR/GeometryBase.h>

#include <mpegcore/AudioBufferArray.hpp>
#include <mpegcore/datatypes_reverb.hpp>

namespace mpegcore {

constexpr std::size_t NUM_SUPPORTED_DELAY_LINE_COUNTS = 5;

struct AcousticParameters;
struct NeuralNetworkParameters;
using NeuralNets = std::unordered_map<std::size_t, NeuralNetworkParameters const*>;
using OutputMapping = std::unordered_map<std::string, std::vector<std::size_t>>;
using OutputDirection = EAR::Polar;

// Creates a vector of output directions (azi, ele) for numDirections delay lines
// ordered from shortest to the longest delay.
void createOutputDirections(std::size_t numDirections,
    std::vector<OutputDirection>& outputDirections, bool forceLegacyDirections = false);
void createOutputMapping(const std::string reverbID,
    const std::vector<EAR::Vector3>& reverbDirections_xyz,
    const std::vector<EAR::Vector3>& outputDirections_xyz, OutputMapping& outputMapping);

std::optional<ReverberatorParameters> createReverbParamsFromAcousticParams(
    AcousticParameters const& acousticParameters, EAR::Geometry::BoundingBox boundingBox,
    std::uint32_t sampleRateHz, std::size_t numDelayLines);

std::size_t getFDNDiffuseOnsetIdx(AudioBufferArray& multichanFdnIR,
    ReverberatorParameters const& reverbParams, std::uint32_t sampleRateHz);
std::array<size_t, NUM_SUPPORTED_DELAY_LINE_COUNTS> getSupportedDelayLineCounts();
std::size_t complexityLevelToNumDelayLines(std::size_t complexityLevel);
template <typename T> std::vector<size_t> sortIndices(const std::vector<T>& valuesToSort);

} // namespace mpegcore

#endif // MPEGCORE_REVERBERATORPARAMETERS_HPP
