/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_SHBINDECODER_HPP
#define MPEGCORE_SHBINDECODER_HPP

#include <cstdint>
#include <optional>
#include <string>

#include "ShOrder.hpp"
#include <mpegcore/Hoa.hpp>

#include "DECMTX.h"

namespace mpegcore {

class AudioBufferArray;

struct BinAmbiParams
{
    ShOrder order;
    std::uint32_t sampleRateHz;
    std::string hrirSofaFile;
    HoaNormalization normalization{ HoaNormalization::n3d };
};

bool operator==(BinAmbiParams const& a, BinAmbiParams const& b);

struct BinAmbi
{
    BinAmbiParams params;
    void* ambi{ nullptr };
    bool isInUse{ false };

    bool init();
    void deinit(std::size_t clearBufferRounds, std::size_t clearBufferSize);
    void destroy();

    void ShBinDecoder_initCodec(void* const hAmbi);
};

class ShBinDecoder
{
public:
    ShBinDecoder(BinAmbiParams params);
    ShBinDecoder(ShBinDecoder&& other) noexcept;
    ShBinDecoder(ShBinDecoder const& other) = delete;
    ~ShBinDecoder();

    void decode(AudioBufferArray const& inputs, AudioBufferArray& outputs);
    bool isInitialized();

private:
    std::optional<std::size_t> m_ambiHandle;
};

} // namespace mpegcore

#endif // MPEGCORE_SHBINDECODER_HPP
