/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include "ShOrder.hpp"

#include <cmath>

#include <mpegcore/spdlog.hpp>

namespace mpegcore {

std::size_t shChannelCount(mpegcore::ShOrder order)
{
    auto nOrder = static_cast<std::size_t>(order);
    return (nOrder + 1) * (nOrder + 1);
}

ShOrder shOrderFromChannelCount(std::size_t channelCount) noexcept
{
    auto order = static_cast<ShOrder>(static_cast<std::size_t>(std::sqrt(channelCount) - 1));

    if (order > ShOrder::max) {
        SPDLOG_WARN("Channel count ({}) too high to convert to ShOrder. "
                    "Returning maximum order",
            channelCount);
        return ShOrder::max;
    }

    if (shChannelCount(order) != channelCount) {
        SPDLOG_WARN("Channel count mismatch mapping channelCount ({}) to "
                    "ShOrder. Returning next higher order or maximum order",
            channelCount);
        if (order < ShOrder::max) {
            order = static_cast<ShOrder>(static_cast<std::size_t>(order) + 1);
        }
        return order;
    }

    return order;
}

SH_ORDERS toSafShOrders(ShOrder order) noexcept
{
    std::unordered_map<ShOrder, SH_ORDERS> orderMap{
        // SAF doesn't support zeroth order, so we convert it.
        { ShOrder::zeroth, SH_ORDER_FIRST },
        { ShOrder::first, SH_ORDER_FIRST },
        { ShOrder::second, SH_ORDER_SECOND },
        { ShOrder::third, SH_ORDER_THIRD },
        { ShOrder::fourth, SH_ORDER_FOURTH },
        { ShOrder::fifth, SH_ORDER_FIFTH },
        { ShOrder::sixth, SH_ORDER_SIXTH },
        { ShOrder::seventh, SH_ORDER_SEVENTH },
    };
    if (auto const it = orderMap.find(order); it != orderMap.end()) {
        return it->second;
    }
    assert(false && "should not reach this");
    return SH_ORDER_FOURTH;
}

} // namespace mpegcore
