/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2023.
***************************************************************************/
/**************************************************************************
Class names, class method signatures and function signatures in this header
match partially those of the Vectorclass library. Full Vectorclass license
is included in this repository in the following directory:
vendor/vectorclass/vectorclass/LICENSE
***************************************************************************/

#include <array>

// Compatibility layer for ARM64 and other platforms that Vectorclass does not support.
// Only the minimal subset of methods required by mpegcore is implemented.
namespace vcl {

class Vec4f
{
public:
    float extract(int index) const { return m_payload[index]; }

    Vec4f(float f0, float f1, float f2, float f3)
    {
        m_payload = std::array<float, 4>{ f0, f1, f2, f3 };
    }

private:
    std::array<float, 4> m_payload;
};

class Vec8f
{
public:
    Vec8f() {}

    Vec8f(float f) { m_payload = std::array<float, 8>{ f, f, f, f, f, f, f, f }; }

    Vec8f const insert(int index, float value)
    {
        m_payload[index] = value;
        return *this;
    }

    Vec8f(float f0, float f1, float f2, float f3, float f4, float f5, float f6, float f7)
    {
        m_payload = std::array<float, 8>{ f0, f1, f2, f3, f4, f5, f6, f7 };
    }

    float extract(int index) const { return (*this)[index]; }

    // Member functions to split into two Vec4f:
    Vec4f get_low() const { return Vec4f(m_payload[0], m_payload[1], m_payload[2], m_payload[3]); }
    Vec4f get_high() const { return Vec4f(m_payload[4], m_payload[5], m_payload[6], m_payload[7]); }

    float operator[](int index) const { return m_payload[index]; }

private:
    std::array<float, 8> m_payload;
};

// Element-wise arithmetic operations
static inline Vec8f operator*(Vec8f const a, Vec8f const b)
{
    return Vec8f(a[0] * b[0], a[1] * b[1], a[2] * b[2], a[3] * b[3], a[4] * b[4], a[5] * b[5],
        a[6] * b[6], a[7] * b[7]);
}

static inline Vec8f operator-(Vec8f const a, Vec8f const b)
{
    return Vec8f(a[0] - b[0], a[1] - b[1], a[2] - b[2], a[3] - b[3], a[4] - b[4], a[5] - b[5],
        a[6] - b[6], a[7] - b[7]);
}

static inline Vec8f operator+(Vec8f const a, Vec8f const b)
{
    return Vec8f(a[0] + b[0], a[1] + b[1], a[2] + b[2], a[3] + b[3], a[4] + b[4], a[5] + b[5],
        a[6] + b[6], a[7] + b[7]);
}

static inline Vec8f operator/(Vec8f const a, Vec8f const b)
{
    return Vec8f(a[0] / b[0], a[1] / b[1], a[2] / b[2], a[3] / b[3], a[4] / b[4], a[5] / b[5],
        a[6] / b[6], a[7] / b[7]);
}

static inline Vec8f& operator+=(Vec8f& a, Vec8f const b)
{
    a = a + b;
    return a;
}

static inline Vec8f& operator-=(Vec8f& a, Vec8f const b)
{
    a = a - b;
    return a;
}

static inline Vec8f& operator*=(Vec8f& a, Vec8f const b)
{
    a = a * b;
    return a;
}

static inline Vec8f& operator/=(Vec8f& a, Vec8f const b)
{
    a = a / b;
    return a;
}

} // namespace vcl
