/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_CICP_LOUDSPEAKERLAYOUT_HPP
#define MPEGCORE_CICP_LOUDSPEAKERLAYOUT_HPP

#include <cstddef>
#include <cstdint>
#include <vector>

namespace mpegcore {
namespace cicp {

using LayoutIndex = std::uint8_t;

enum class ChannelType : std::uint8_t
{
    normal,
    lfe,
};

struct Channel
{
    float azimuthDeg{ 0.F };
    float elevationDeg{ 0.F };
    ChannelType type{ ChannelType::normal };
};

using ChannelConfiguration = std::vector<Channel>;

constexpr std::size_t CHANNEL_CONFIGURATION_COUNT{ 20 };

} // namespace cicp

// Returns 0 if `inputLayout` is not valid. May return 0 even if `inputLayout`
// is valid.
std::size_t channelCount(cicp::LayoutIndex inputLayout) noexcept;

// Returns 0 if `inputLayout` is not valid. May return 0 even if `inputLayout`
// is valid.
std::size_t lfeCount(cicp::LayoutIndex inputLayout) noexcept;

std::vector<std::size_t> lfeIndices(cicp::LayoutIndex inputLayout) noexcept;

// Returns an empty vector if `inputLayout` is not valid. May return an empty
// vector even if `inputLayout` is valid.
cicp::ChannelConfiguration const& channelConfiguration(cicp::LayoutIndex inputLayout) noexcept;

bool isValid(cicp::LayoutIndex inputLayout) noexcept;

} // namespace mpegcore

#endif // MPEGCORE_CICP_LOUDSPEAKERLAYOUT_HPP
