/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#include <mpegcore/datatypes_reverb.hpp>

#include <algorithm>
#include <iostream>
#include <mpegcore/chrono.hpp>
#include <spdlog/fmt/fmt.h>
#include <utility>

namespace mpegcore {

std::chrono::nanoseconds minDelay(
    ReverberatorCommon::DelayLineGroup const& lines, std::uint32_t sampleRateHz) noexcept
{
    std::size_t minDelay = minDelaySamp(lines);
    return samplesToNanoseconds(static_cast<double>(minDelay), sampleRateHz);
}

std::size_t minDelaySamp(ReverberatorCommon::DelayLineGroup const& lines) noexcept
{
    auto minDelaySamp = lines[0].delayInSamples;
    for (auto const& line : lines) {
        minDelaySamp = std::min(minDelaySamp, line.delayInSamples);
    }
    return minDelaySamp;
}

GraphicEqParameters::GraphicEqParameters()
    : GraphicEqParameters(Type::uninitialized, 0)
{}

GraphicEqParameters::GraphicEqParameters(Type type, std::size_t bandCount)
    : type{ type }
{
    a[0].resize(bandCount, 0.F);
    a[1].resize(bandCount, 0.F);
    b[0].resize(bandCount, 0.F);
    b[1].resize(bandCount, 0.F);
    stageF.resize(bandCount, 1.F);
}

std::size_t GraphicEqParameters::bandCount() const noexcept
{
    return a[0].size();
}

// En/disable status posting for feedback during render or debugging
// Note RIR analysis status/debugging can be set by LOG_LEVEL_RUNTIME in rirlib.h
bool postStatus = false;

void printStatus(const std::string& message)
{
    if (postStatus) {
        std::cout << message << std::endl;
    }
}

template <typename... Args> void printStatus(const std::string& formatMessage, Args&&... args)
{
    if (postStatus) {
        std::cout << fmt::format(formatMessage, std::forward<Args>(args)...) << std::endl;
    }
}
// Explicit instantiation for specific types
template void printStatus(const std::string& message, bool& flag);
template void printStatus(const std::string& message, const bool& flag);
template void printStatus(const std::string& message, float& value);
template void printStatus(const std::string& message, const float& value);
template void printStatus(const std::string& message, float&& value1, float&& value2);

} // namespace mpegcore
