/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.
***************************************************************************/

#ifndef MPEGCORE_PROCESSING_HPP
#define MPEGCORE_PROCESSING_HPP

#include <cstddef>
#include <cstdint>

#include "gain.hpp"
#include <mpegcore/AudioBuffer.hpp>

namespace mpegcore {

class AudioBufferArray;

void processGain(AudioBuffer& buffer, GainLinear gain);

void processMute(AudioBuffer& buffer);

void add(AudioBuffer const& source, AudioBuffer& target);
void add(AudioBufferArray const& source, AudioBufferArray& target);

void toMono(AudioBufferArray& bufferArray);

enum class FadeDirection : std::uint8_t
{
    // Would like this to be just IN (and OUT) but that won't compile with MSVC.
    fadeIn,
    fadeOut,
};

// Fades audio `buffer` in `direction` during the buffer's duration.
void fade(FadeDirection direction, AudioBuffer& buffer);

struct PartialFade
{
    std::size_t totalLength;
    std::size_t processLength;
    std::size_t processStartIndex;
};

// Fades audio `buffer` in `direction` during `totalLength` samples starting
// from `begin`. Wraps around at `buffer`'s end if needed. `processLength`
// samples are processed and processing is started from `processStartIndex`.
void fade(FadeDirection direction, AudioBuffer& buffer, std::size_t begin,
    PartialFade const& partialFade);

} // namespace mpegcore

#endif // MPEGCORE_PROCESSING_HPP
