/***************************************************************************
This software module was originally developed by

Nokia Technologies Ltd.

in the course of development of ISO/IEC 23090-4 for reference purposes and
its performance may not have been optimized.  This software module is an
implementation of one or more tools as specified by the ISO/IEC 23090-4
standard.  ISO/IEC gives you a royalty-free, worldwide, non-exclusive,
copyright license to copy, distribute, and make derivative works of this
software module or modifications thereof for use in implementations or
products claiming conformance to ISO/IEC standards incorporating ISO/IEC
23090-4 reference software and which satisfy any specified conformance
criteria of that standard.

Those intending to use this software module in products are advised that its
use may infringe existing patents and that the license in the previous
paragraph grants no licenses under such patents.  ISO/IEC and the developers
of this software module have no liability for use of this software module or
modifications thereof.

Copyright is not released for products that do not conform to an ISO/IEC
standard.

Nokia Technologies Ltd. retains full right to
modify and use the code for its own purpose, assign or donate the code to
a third party without consent of and without accounting to third parties,
and to inhibit third parties from using the code for products that do not
conform to MPEG-related ISO/IEC International Standards.

This copyright notice must be included in all copies or derivative works.

Copyright (c) ISO/IEC 2022.

This software module derives its structure, with substantial modification of
functionality, from the HO-SIRR library released under the following license:

BSD 3-Clause License

Copyright (c) 2019, Leo McCormack
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***************************************************************************/

#include "rirlib_internal.h"

void loadSphDesignPreset(
    SPHDESIGN_PRESETS preset, float dirsDeg[RIRLIB_MAX_NUM_OUTPUTS][2], int* nChanOut)
{
    int ch, i, nCH;

    switch (preset) {
    default:
    case SPHDESIGN_PRESET_DEFAULT:
    case SPHDESIGN_PRESET_T_DESIGN_15:
        nCH = 15;
        // clang-format off
        float dirs_t_15_deg[15][2] = { // Sloane t-design 3.15.6
            {   90.000000000000000f,   0.000000000000000f },
            {   90.000000000000000f,  72.000000000000000f },
            {  -90.000000000000000f, -36.000000000000000f },
            {  -90.000000000000000f,  36.000000000000000f },
            {   90.000000000000000f, -72.000000000000000f },
            {   45.000000000000000f,   0.000000000000000f },
            {   17.172037850513838f,  42.260195269907655f },
            {  -38.973447336865647f, -24.558803945449998f },
            {  -38.973447336865647f,  24.558803945449998f },
            {   17.172037850513838f, -42.260195269907655f },
            {  135.000000000000000f,   0.000000000000000f },
            {  162.827962149486183f, -42.260195269907655f },
            { -141.026552663134368f,  24.558803945449998f },
            { -141.026552663134368f, -24.558803945449998f },
            {  162.827962149486183f,  42.260195269907655f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_15_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_FWPAIR_15:
        nCH = 15;
        // clang-format off
        // Sloane t-design 3.15.6, rotated such that a pair of vertices
        // lies on the horizontal plane, bisected by the forward axis.
        // Reordered so that point ordering forms approx longest path.
        // longest path:
        //    14 7 12 8 1 9 11 4 5 2 3 15 10 13 6
        // path length:
        //    2125.0884
        float dirs_t_fwpair_15_deg[15][2] = {
            { 82.658851354417308f, 45.776701836081472f },
            { -103.211797148889076f, -20.996660207713223f },
            { 62.447049548527559f, 0.000000000000002f },
            { 172.658851354417294f, -0.000000000000009f },
            { -5.115301174902946f, -55.075721661607460f },
            { -139.498775509469226f, 12.794209241377812f },
            { 14.604676412414083f, -12.794209241377812f },
            { -119.778797922152194f, 55.075721661607460f },
            { 97.341148645582692f, -30.446384317065231f },
            { -62.447049548527559f, 0.000000000000002f },
            { 137.764752257362176f, 30.446384317065224f },
            { -21.682301948166053f, 20.996660207713226f },
            { 152.447049548527559f, -45.776701836081472f },
            { -11.552377423280149f, 69.731081550693503f },
            { -113.341721673774956f, -69.731081550693489f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_fwpair_15_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_SPH_COVER_15:
        nCH = 15;
        // clang-format off
        float dirs_sphcov_15_deg[15][2] = { // a scrambled version of __SphCovering_15_dirs_deg
            {   38.2804562082806f, -30.8503395210240f  },
            {  -52.6920636292062f,   5.49340474815531f },
            {  107.773361264108f,  -72.5078089738057f  },
            {   59.1464331913549f,  28.4645432620993f  },
            { -161.528261603282f,   16.8449591768462f  },
            { -142.534710694695f,  -43.2348222627768f  },
            {  156.658120344670f,  -21.2555882837633f  },
            {  131.132850571592f,   27.4269166951174f  },
            {    1.54996542738790f, -1.22343677994285f },
            { -102.771439712616f,   43.3889479096670f  },
            {   95.6495743191396f, -14.9478959171680f  },
            {  147.645494227262f,   72.1869526085324f  },
            { -104.931490600259f,  -11.1594989757630f  },
            {  -10.9944871307654f,  52.3288083870932f  },
            {  -44.6563305524964f, -50.0312476286186f  } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_sphcov_15_deg[ch][i];
        // dirs_deg[ch][i] = __SphCovering_15_dirs_deg[ch][i];
        break;
    case SPHDESIGN_MPEGFDN_15:
        nCH = 15;
        // clang-format off
        // A set of dirs not contained in SAF, used by the FDN as defaults
        float dirs_fdn_deg[15][2] = {
            {   90,   0 },
            {  -90,   0 },
            {  114,  20 },
            {  -60,  -6 },
            {   85, -44 },
            { -130, -21 },
            {   49,  41 },
            {  -67,  52 },
            {  154,  -9 },
            {  -48, -55 },
            {   19, -31 },
            { -162,  21 },
            {  151,  73 },
            {   -9,  20 },
            {  180, -63 }
        };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_fdn_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_16:
        nCH = 16;
        // clang-format off
        float dirs_t_16_deg[16][2] = { // Sloane t-design 3.16.5
            {   45.000000000000000f,  35.264389682754654f },
            {  -45.000000000000000f, -35.264389682754654f },
            {  135.000000000000000f, -35.264389682754654f },
            { -135.000000000000000f,  35.264389682754654f },
            { -168.562313494910768f, -22.882599322073684f },
            {  168.562313494910768f,  22.882599322073684f },
            {  -11.437686505089228f,  22.882599322073684f },
            {   11.437686505089228f, -22.882599322073684f },
            { -113.297204593018279f, -10.526857218370301f },
            {  -66.702795406981735f,  10.526857218370301f },
            {   66.702795406981735f, -10.526857218370301f },
            {  113.297204593018279f,  10.526857218370301f },
            { -115.166239760545622f, -64.556226870393346f },
            {   64.833760239454406f, -64.556226870393346f },
            {  115.166239760545622f,  64.556226870393346f },
            {  -64.833760239454406f,  64.556226870393346f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_16_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_FWPAIR_16:
        nCH = 16;
        // clang-format off
        // Sloane t-design 3.16.5, rotated such that a pair of vertices
        // lies on the horizontal plane, bisected by the forward axis.
        // Reordered so that point ordering forms approx longest path.
        // longest path:
        //    7 6 8 15 13 1 9 12 10 3 2 4 14 16 11 5
        // path length:
        //    2224.8527
        float dirs_t_fwpair_16_deg[16][2] = {
            { -25.090255472603932f, -0.000000000000007f },
            { 129.428412924164178f, 39.396588351997721f },
            { -25.284474196937136f, -50.887278623655888f },
            { 56.041528780243532f, 41.694942683699189f },
            { -146.205877422017039f, -30.542883102391645f },
            { 25.090255472603918f, -0.000000000000003f },
            { -131.140593919862653f, 21.528331275621177f },
            { 91.352611572339157f, -1.882393520164967f },
            { -76.610968511589363f, 19.332594756429710f },
            { 136.959191161008818f, -26.505516873070622f },
            { -87.485447409704364f, -29.741420914715203f },
            { -149.979586507246751f, 70.453536241936888f },
            { 162.599660128156273f, -74.957980508180469f },
            { -21.747408485195972f, 53.974412189020235f },
            { 56.041528780243539f, -41.694942683699182f },
            { 174.401119378988568f, 8.118556123479774f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_fwpair_16_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_SPH_COVER_16:
        nCH = 16;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __SphCovering_16_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_4:
        nCH = 4;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __Tdesign_degree_2_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_12:
        nCH = 12;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __Tdesign_degree_4_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_24:
        nCH = 24;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __Tdesign_degree_6_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_31:
        nCH = 31;
        // clang-format off
        float dirs_t_31_deg[31][2] = { // Sloane t-design 3.31.6
            {   11.641966039969587f,  51.975005482280878f },
            { -132.358033860679797f,  51.975005478973856f },
            {   83.641966129358394f,  51.975005449598946f },
            {  155.641966065849061f,  51.975005439457696f },
            {  -60.358033891428725f,  51.975005385981589f },
            {   37.297898758502392f,  25.347387788316219f },
            { -178.702101187289884f,  25.347387753357463f },
            { -106.702101237677269f,  25.347387735934770f },
            {  109.297898839779492f,  25.347387724015125f },
            {  -34.702101108459964f,  25.347387701093936f },
            {   73.932689321756726f,   9.754846270811861f },
            {  -70.067310618572250f,   9.754846239373542f },
            { -142.067310697216243f,   9.754846218261742f },
            {  145.932689435319531f,   9.754846160875539f },
            {    1.932689429929676f,   9.754846124032284f },
            { -101.792532458706575f,  89.999999984650273f },
            {  118.065648503139855f, -14.592870472580216f },
            {  -97.934351456055822f, -14.592870489853011f },
            {   46.065648569640487f, -14.592870532146506f },
            {  -25.934351476894538f, -14.592870544353783f },
            { -169.934351491784724f, -14.592870599745480f },
            { -130.371777910341081f, -27.674620066213038f },
            {   85.628222070907171f, -27.674620090476555f },
            {  -58.371777927924647f, -27.674620111321456f },
            {   13.628222137298344f, -27.674620175248929f },
            {  157.628222071891031f, -27.674620180936959f },
            {  114.325659529250402f, -60.328942473230214f },
            { -101.674340479973154f, -60.328942486402639f },
            {  -29.674340469468074f, -60.328942522375186f },
            {   42.325659562602375f, -60.328942595809103f },
            { -173.674340366378374f, -60.328942608889832f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_31_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_FWPAIR_31:
        nCH = 31;
        // clang-format off
        // Sloane t-design 3.31.6, rotated such that a pair of vertices
        // lies on the horizontal plane, bisected by the forward axis.
        // Reordered so that point ordering forms approx longest path.
        // longest path:
        //     1 27 16 31 4 5 28 11 21 3 10 18 30 2 25 13 24 9 29 19 22 6 14 20 7 15 26 8 17 12 23
        // path length:
        //     3984.0029
        float dirs_t_fwpair_31_deg[31][2] = {
            { -21.227914684628225f, -0.000000000000004f },
            { 144.166146415942535f, -39.058501810257034f },
            { -0.000000042133876f, 32.319159519795448f },
            { 160.696218451645649f, -8.646319455760947f },
            { 46.092132941458154f, 35.592827198843139f },
            { -46.092133060988431f, 35.592827131173195f },
            { -165.285328486345094f, -6.013873952385691f },
            { 35.341081142546805f, -41.005150567886659f },
            { 136.549914552511922f, 30.894355659464448f },
            { 21.227914684628200f, 0.000000000000007f },
            { -64.555276959969802f, 7.303982613202332f },
            { -139.462967778490594f, 32.677072071279753f },
            { -174.435316727748955f, -61.771453724156927f },
            { -0.000000217873171f, 70.344154027604731f },
            { -90.004136402389676f, -60.304065739604162f },
            { 156.161941041166216f, 65.716199543234879f },
            { -121.572244417656435f, -2.398536477252398f },
            { 53.378132586776388f, -7.684054402112428f },
            { -144.464385588860097f, -33.806320503878077f },
            { -5.000092606567491f, -72.215563373533598f },
            { -177.969313143670092f, 29.976473465812681f },
            { -10.970804291206807f, -31.495601569421947f },
            { 86.057785094463583f, 12.162232881350828f },
            { -94.105804021871819f, -21.464815261831014f },
            { 87.526615506354688f, 49.121590079422631f },
            { -54.075652088035646f, -29.406603527914680f },
            { 123.665584936594925f, 0.429138722951919f },
            { -100.673331810731455f, 66.536017458738542f },
            { 92.484356152494456f, -24.124555266313592f },
            { -96.656013946955284f, 28.545378419433657f },
            { 92.030804464154855f, -56.947682877906125f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_fwpair_31_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_32:
        nCH = 32;
        // clang-format off
        float dirs_t_32_deg[32][2] = { // Sloane t-design 3.32.7
            {   25.849850024681860f,   2.749789239286311f },
            {  -25.849850024681860f,  -2.749789239286311f },
            {    3.054975656697321f, -25.817891154416834f },
            {   -3.054975656697321f,  25.817891154416834f },
            {  154.150149975318129f,  -2.749789239286311f },
            { -154.150149975318129f,   2.749789239286311f },
            {  176.945024343302691f,  25.817891154416834f },
            { -176.945024343302691f, -25.817891154416834f },
            {   86.945024343302677f,  25.817891154416834f },
            {   93.054975656697337f, -25.817891154416834f },
            {  115.849850024681857f,   2.749789239286311f },
            {   64.150149975318143f,  -2.749789239286311f },
            {  -93.054975656697337f,  25.817891154416834f },
            {  -86.945024343302677f, -25.817891154416834f },
            {  -64.150149975318143f,   2.749789239286311f },
            { -115.849850024681857f,  -2.749789239286311f },
            {    6.286151868369432f,  64.014314030234402f },
            { -173.713848131630556f,  64.014314030234402f },
            {  -83.713848131630570f,  64.014314030234402f },
            {   96.286151868369430f,  64.014314030234402f },
            {   -6.286151868369432f, -64.014314030234402f },
            {  173.713848131630556f, -64.014314030234402f },
            {   83.713848131630570f, -64.014314030234402f },
            {  -96.286151868369430f, -64.014314030234402f },
            {   45.000000000000000f,  35.264389682754654f },
            {   45.000000000000000f, -35.264389682754654f },
            {  -45.000000000000000f,  35.264389682754654f },
            {  -45.000000000000000f, -35.264389682754654f },
            {  135.000000000000000f,  35.264389682754654f },
            {  135.000000000000000f, -35.264389682754654f },
            { -135.000000000000000f,  35.264389682754654f },
            { -135.000000000000000f, -35.264389682754654f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_32_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_FWPAIR_32:
        nCH = 32;
        // clang-format off
        // Sloane t-design 3.32.7, rotated such that a pair of vertices
        // lies on the horizontal plane, bisected by the forward axis.
        // Reordered so that point ordering forms approx longest path.
        // longest path:
        //     4 23 18 8 9 15 10 13 26 5 2 1 14 11 19 21 28 6 3 7 12 16 25 24 31 22 20 32 17 29 27 30
        // path length:
        //     4079.5212
        float dirs_t_fwpair_32_deg[32][2] = {
            { 22.474936964977470f, 35.860540061570866f },
            { -51.661692570888199f, -56.841443261550857f },
            { 132.122791194047920f, 25.150495560863934f },
            { -154.440394672879762f, -30.999694498343057f },
            { 78.400559330524757f, -22.952358711171289f },
            { -71.713232520574891f, 56.448906412039165f },
            { 39.273931061474336f, -69.433320853424235f },
            { -148.483682301422618f, 64.823161930483664f },
            { -6.296099525244550f, -35.391697926795729f },
            { 162.144075232645179f, -39.982123557879042f },
            { -22.474936964977424f, 35.860540061570880f },
            { 18.048167762992190f, -0.000000000000003f },
            { -95.976815857980455f, 24.912297833394142f },
            { 107.838825230384401f, -50.955692376204254f },
            { 120.087771243904299f, 60.276133011852828f },
            { -51.454156935055749f, -20.745434062829023f },
            { -57.516581228536992f, 15.822100748286299f },
            { -166.389011259887184f, 3.252378483153097f },
            { -18.048167762992190f, 0.000000000000020f },
            { 158.237765004642739f, -4.058836893320030f },
            { 38.912236318258145f, -33.337554056084080f },
            { -136.437935125723612f, 29.527173787897350f },
            { 54.684248966543350f, 5.946920541392967f },
            { -88.405491610819894f, -13.054423778212259f },
            { 173.703900474755443f, 35.391697926795729f },
            { -107.861222426008240f, -45.138998363985934f },
            { 96.898764851439552f, 11.222034560588700f },
            { -125.315751033456650f, -5.946920541392967f },
            { 71.780574384545190f, 39.645769346964109f },
            { 122.483418771463022f, -15.822100748286299f },
            { 17.626119182211323f, 72.827760005323100f },
            { -162.373880817788688f, -72.827760005323100f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_fwpair_32_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_36:
        nCH = 36;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __Tdesign_degree_8_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_48:
        nCH = 48;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __Tdesign_degree_9_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_60:
        nCH = 60;
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = __Tdesign_degree_10_dirs_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_63:
        nCH = 63;
        // clang-format off
        float dirs_t_63_deg[63][2] = { // Sloane t-design 3.63.9
            {  145.331280085063810f,  36.942890565569328f },
            {  148.169771036754327f,  -9.666104337905960f },
            {  -92.769934396819451f,  31.921829263867707f },
            {  -79.644901295642939f,  65.725913894018547f },
            {  -90.000000000000000f,   0.000000000000000f },
            {  -38.571428571441572f,   0.000000000000000f },
            {   12.857142857123044f,   0.000000000000000f },
            {   64.285714285681053f,   0.000000000000000f },
            {  115.714285714318933f,   0.000000000000000f },
            {  167.142857142876977f,   0.000000000000000f },
            { -141.428571428558428f,   0.000000000000000f },
            { -163.240148486425227f,  36.942890565569130f },
            { -160.401657534697733f,  -9.666104337900849f },
            {  -41.341362968258530f,  31.921829263883030f },
            {  -28.216329867032737f,  65.725913893997117f },
            {   34.668719914936212f, -36.942890565569328f },
            {   31.830228963245691f,   9.666104337905960f },
            {  -87.230065603180563f, -31.921829263867707f },
            { -100.355098704357076f, -65.725913894018547f },
            { -111.811577057825730f,  36.942890565581898f },
            { -108.973086106076551f,  -9.666104337901245f },
            {   10.087208460294454f,  31.921829263863639f },
            {   23.212241561504321f,  65.725913894018532f },
            {   86.097291343543787f, -36.942890565571126f },
            {   83.258800391829595f,   9.666104337906011f },
            {  -35.801494174590154f, -31.921829263888590f },
            {  -48.926527275832079f, -65.725913894042094f },
            {  -60.383005629243577f,  36.942890565557832f },
            {  -57.544514677532995f,  -9.666104337904155f },
            {   61.515779888883721f,  31.921829263865469f },
            {   74.640812990023591f,  65.725913894028992f },
            {  137.525862772137856f, -36.942890565562273f },
            {  134.687371820398852f,   9.666104337903713f },
            {   15.627077253997301f, -31.921829263884195f },
            {    2.502044152704607f, -65.725913894029617f },
            {   -8.954434200684474f,  36.942890565587042f },
            {   -6.115943248925059f,  -9.666104337898975f },
            {  112.944351317458441f,  31.921829263862254f },
            {  126.069384418725022f,  65.725913894017197f },
            { -171.045565799315540f, -36.942890565587042f },
            { -173.884056751074951f,   9.666104337898975f },
            {   67.055648682541573f, -31.921829263862254f },
            {   53.930615581274985f, -65.725913894017197f },
            {   42.474137227862144f,  36.942890565562273f },
            {   45.312628179601163f,  -9.666104337903713f },
            {  164.372922746002700f,  31.921829263884195f },
            {  177.497955847295401f,  65.725913894029617f },
            { -119.616994370756430f, -36.942890565557832f },
            { -122.455485322467027f,   9.666104337904155f },
            {  118.484220111116286f, -31.921829263865469f },
            {  105.359187009976424f, -65.725913894028992f },
            {   93.902708656456213f,  36.942890565571126f },
            {   96.741199608170405f,  -9.666104337906011f },
            { -144.198505825409853f,  31.921829263888590f },
            { -131.073472724167914f,  65.725913894042094f },
            {  -68.188422942174299f, -36.942890565581898f },
            {  -71.026913893923449f,   9.666104337901245f },
            {  169.912791539705552f, -31.921829263863639f },
            {  156.787758438495672f, -65.725913894018532f },
            {  -16.759851513574784f, -36.942890565569130f },
            {  -19.598342465302277f,   9.666104337900849f },
            { -138.658637031741478f, -31.921829263883030f },
            { -151.783670132967274f, -65.725913893997117f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_63_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_FWPAIR_63:
        nCH = 63;
        // clang-format off
        // Sloane t-design 3.63.9, rotated such that a pair of vertices
        // lies on the horizontal plane, bisected by the forward axis.
        // Reordered so that point ordering forms approx longest path.
        // longest path:
        //     62 17 46 60 8 11 57 50 5 31 29 33 49 16 39 2 21 59 4 63 6 32 61 13 55 15 51 9 58 43 47 18 44 54 26 23 36 38 56 7 37 53 48 42 20 24 30 19 12 35 10 25 28 1 27 22 40 41 45 14 34 3 52
        // path length:
        //     7285.2991
        float dirs_t_fwpair_63_deg[63][2] = {
            { 113.697644297577199f, 75.586367185940546f },
            { 166.310646192394728f, -77.684915450136458f },
            { -17.389103694898271f, 16.199603089248804f },
            { 158.822754037832453f, -12.645966321239733f },
            { 60.358089557801456f, -61.633632065821807f },
            { -55.400505826335845f, 72.236746983300861f },
            { -138.621699406575971f, 18.136412679207336f },
            { 58.826283607279628f, -1.414381990856097f },
            { -140.436766467579957f, 39.290274137497249f },
            { -49.081811554803778f, -37.081634002506441f },
            { -162.016399094256514f, 11.539415068276861f },
            { 14.462909900643668f, 0.000000000000016f },
            { -96.689514963231886f, 58.590970970599642f },
            { 121.385234198446426f, -35.154804586166392f },
            { -39.884596536300933f, -18.186308271001522f },
            { 29.176384353080813f, 18.573181877103718f },
            { -143.929354548781674f, 60.389893911892045f },
            { 90.113195006351290f, 26.173048875247730f },
            { -86.026684054801592f, -4.064365329271832f },
            { 107.359577791370299f, 40.970430011138482f },
            { -158.378140809581964f, -9.362257369704528f },
            { 59.201385791935110f, 15.036228755803268f },
            { -154.251908659831230f, -30.221887395193693f },
            { 7.331796515755554f, 67.240009845692171f },
            { -68.418666082070501f, 6.579590634863780f },
            { -90.418339489950952f, -24.173441813919823f },
            { 92.850643546644875f, 5.790286144586245f },
            { 29.499830538080069f, -15.138225248667363f },
            { 52.052426573120464f, 41.363214877831737f },
            { 109.539653840480028f, -6.242935338992117f },
            { -48.769156080158716f, 0.413993055858015f },
            { 177.107021781366853f, 41.812771277039012f },
            { -63.314596947338423f, -70.552282445636862f },
            { -58.937151467299643f, 40.255558615281394f },
            { 170.546592751151849f, -0.995005064330493f },
            { -75.184580491607932f, -40.262910557670033f },
            { -121.137777758556467f, -41.683881870833638f },
            { -3.116505077969357f, -24.715273106713525f },
            { 171.423156108720008f, 26.037622270870678f },
            { 175.861985785744793f, -56.730887113546807f },
            { 178.356183795915598f, -35.585540530141550f },
            { 46.779122330655440f, -29.137637580452569f },
            { 151.274088900546161f, 64.484910903387217f },
            { 88.207759148636541f, -34.876563400358158f },
            { -89.801268305889096f, 30.053225210144532f },
            { 77.656926399373148f, -21.278351389684364f },
            { -17.821507128036220f, -66.949086169393908f },
            { 132.771099505294984f, 35.665865658227233f },
            { -42.401567716459439f, 30.718049375001772f },
            { 129.638946492271032f, -1.716547557921829f },
            { 12.475304819954122f, 33.661183083303406f },
            { 29.965758471370243f, -48.848969323800397f },
            { -116.840219983601131f, -1.288753780714446f },
            { -14.462909900637902f, 0.000000000000003f },
            { 139.940681600195944f, 16.136446227182592f },
            { -122.541856340902655f, -58.114702705768522f },
            { 65.287791671586234f, 55.343181967946883f },
            { -10.454909013743984f, 45.637942442249106f },
            { 106.651521415697275f, -62.578150231561231f },
            { -126.085866385379063f, -14.978780360888869f },
            { 141.473502636798969f, -34.649424345748770f },
            { -107.014661653642463f, 24.017281223767185f },
            { -9.370605842054440f, -40.314579424013793f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_fwpair_63_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_64:
        nCH = 64;
        // clang-format off
        float dirs_t_64_deg[64][2] = { // Sloane t-design 3.64.10
            {  142.582977743815434f, -65.091952299642273f },
            {  -74.243705990092167f, -19.542269840809102f },
            {  -37.417022256187941f, -65.091952299642017f },
            { -159.755681973496280f,  14.827143134382457f },
            { -105.756294009902845f,  19.542269840814690f },
            {  159.755681973498895f, -14.827143134388791f },
            {  105.756294009904479f, -19.542269840810736f },
            {   20.244318026501244f,  14.827143134383412f },
            {   74.243705990096927f,  19.542269840812068f },
            {  -20.244318026501880f, -14.827143134385423f },
            { -142.582977743822170f,  65.091952299640042f },
            {   37.417022256191729f,  65.091952299638280f },
            {   40.221505467109836f, -76.268583265343608f },
            {  -81.033191139306751f,  10.442308165157815f },
            { -139.778494532884366f, -76.268583265344645f },
            {  169.431388847861200f,   8.817105959967771f },
            {  -98.966808860686598f, -10.442308165149425f },
            { -169.431388847856937f,  -8.817105959973800f },
            {   98.966808860692808f,  10.442308165152568f },
            {  -10.568611152138214f,   8.817105959971302f },
            {   81.033191139308684f, -10.442308165152049f },
            {   10.568611152141074f,  -8.817105959974150f },
            {  -40.221505467088882f,  76.268583265348781f },
            {  139.778494532902101f,  76.268583265340439f },
            {  158.451387436544934f,  52.428964324509977f },
            {   74.222037681907480f, -34.550141898548844f },
            {  -21.548612563456238f,  52.428964324517167f },
            {  -35.584838491591391f,  12.941341400175538f },
            {  105.777962318092051f,  34.550141898550926f },
            {   35.584838491593878f, -12.941341400177480f },
            { -105.777962318090829f, -34.550141898545490f },
            {  144.415161508409824f,  12.941341400175002f },
            {  -74.222037681906272f,  34.550141898555928f },
            { -144.415161508402946f, -12.941341400176658f },
            { -158.451387436542490f, -52.428964324515931f },
            {   21.548612563456263f, -52.428964324515469f },
            {  -97.097998925890138f, -57.667439498543551f },
            { -122.133691036755152f,  -3.789300312497619f },
            {   82.902001074113898f, -57.667439498545669f },
            { -175.527771396916904f, -32.055047744297980f },
            {  -57.866308963240598f,   3.789300312501178f },
            {  175.527771396919917f,  32.055047744293127f },
            {   57.866308963241487f,  -3.789300312497617f },
            {    4.472228603079609f, -32.055047744298470f },
            {  122.133691036760297f,   3.789300312499686f },
            {   -4.472228603083227f,  32.055047744297589f },
            {   97.097998925889172f,  57.667439498547544f },
            {  -82.902001074112178f,  57.667439498552788f },
            {  -26.042188966570730f, -43.044553102038073f },
            { -115.176889817343380f,  41.042861595177826f },
            {  153.957811033431000f, -43.044553102038527f },
            {  136.109822130309482f, -18.714661875619822f },
            {  -64.823110182659178f, -41.042861595174642f },
            { -136.109822130307151f,  18.714661875619750f },
            {   64.823110182656109f,  41.042861595173356f },
            {  -43.890177869689552f, -18.714661875620074f },
            {  115.176889817342641f, -41.042861595173953f },
            {   43.890177869689680f,  18.714661875619328f },
            {   26.042188966565707f,  43.044553102034335f },
            { -153.957811033432961f,  43.044553102033447f },
            { -134.999999999999801f, -35.264389682755741f },
            {   44.999999999999062f, -35.264389682756082f },
            {  -44.999999999997470f,  35.264389682755663f },
            {  135.000000000000625f,  35.264389682753951f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_64_deg[ch][i];
        break;
    case SPHDESIGN_PRESET_T_DESIGN_FWPAIR_64:
        nCH = 64;
        // clang-format off
        // Sloane t-design 3.64.10, rotated such that a pair of vertices
        // lies on the horizontal plane, bisected by the forward axis.
        // Reordered so that point ordering forms approx longest path.
        // longest path:
        //     56 11 20 18 28 32 38 30 3 24 39 42 10 54 62 53 29 1 31 16 59 26 5 8 34 51 55 40 2 12 19 49 60 23 15 46 33 57 48 22 52 63 35 27 14 7 41 43 44 6 17 47 45 21 50 58 4 9 61 25 64 36 37 13
        // path length:
        //     7377.1128
        float dirs_t_fwpair_64_deg[64][2] = {
            { -81.850726544549048f, -32.948370003536915f },
            { 157.019706789171920f, -8.249426112544091f },
            { -105.396129305646809f, 3.671620093553835f },
            { 78.095135954651425f, -24.283192307663509f },
            { -115.171720318553355f, -19.294191659643104f },
            { 92.194195748076510f, 24.584703016220168f },
            { 101.906000292977168f, -69.357284341270343f },
            { -68.206884003283463f, 42.097856309342347f },
            { -31.173325661804938f, -20.288545352948724f },
            { 160.169882384787371f, 17.075871251621489f },
            { -3.953416863825204f, 21.162593779207032f },
            { 115.837692819243316f, -0.943493217131336f },
            { -83.807898284948095f, -10.066704916491343f },
            { 124.601376927510159f, -45.561325996056439f },
            { -39.162880933693557f, 34.504638610528154f },
            { -48.179445946715717f, -45.275064516267221f },
            { 129.752910183714590f, 55.040865494686997f },
            { 15.360751652955029f, 0.000000000000001f },
            { -0.639612568851173f, -66.294555404354298f },
            { 91.928042496649766f, -0.311542218697678f },
            { -142.716419239411408f, 35.354365864824047f },
            { -10.753234220321939f, 44.331347912888106f },
            { 167.091163216749607f, -59.281915304143638f },
            { -107.645892568362768f, 34.321798620582648f },
            { 75.780063083254547f, -49.114415041094766f },
            { 38.225297385742863f, 2.064578523441790f },
            { -169.089946270824839f, 58.008020494316639f },
            { 52.585379794985691f, -20.241224160957241f },
            { -76.351470277153112f, -61.378798674100210f },
            { -169.036924769577695f, 29.355755808413154f },
            { 83.373449393996580f, 68.786355271367455f },
            { -55.229566951731805f, -17.276130468093577f },
            { 135.881593854488898f, -18.522713416571598f },
            { -174.936868884868630f, 4.458909262075689f },
            { 1.765402469312145f, -21.800868367338524f },
            { -127.809164132510389f, 12.672057891222179f },
            { -157.716297362066427f, -37.066625163878960f },
            { 24.243402862507683f, 27.352298968581707f },
            { -176.085730527442905f, -19.430789460968583f },
            { -83.214994906610514f, 20.649265335330441f },
            { 56.083401365911314f, 24.911464635040247f },
            { -140.438987030913779f, -18.591700944783291f },
            { 28.182230862772268f, -29.186466537200449f },
            { -150.004149419881344f, 3.361145571227032f },
            { -141.385717813138569f, -59.648984431331883f },
            { 35.497296189289180f, 48.583682907443119f },
            { -113.032267110877470f, -43.068798469138017f },
            { -60.836775732895390f, 65.578852021152386f },
            { -62.054663794295003f, 8.225489323190915f },
            { 66.829811079544541f, 4.413869539976249f },
            { -105.917396651309815f, -86.363534859714974f },
            { 159.054295080344076f, 40.972953615471305f },
            { 77.145545139802053f, 45.083935170616400f },
            { 2.613048926446749f, 68.232277765143081f },
            { 161.088264329345748f, -36.661729224859734f },
            { -114.534902781735283f, 56.789717068333289f },
            { 106.287643320900798f, -27.775354056078719f },
            { -167.971505590874500f, 80.958275936017117f },
            { 38.037061173911226f, -51.924513664667614f },
            { 134.479012924763111f, 14.145647610385025f },
            { 117.898710906512264f, 32.652875585453337f },
            { -38.002520218253352f, 10.639935046082352f },
            { -11.164282385973525f, -43.160984783176794f },
            { -15.360751652955038f, 0.000000000000008f } };
        // clang-format on
        for (ch = 0; ch < nCH; ch++)
            for (i = 0; i < 2; i++)
                dirsDeg[ch][i] = dirs_t_fwpair_64_deg[ch][i];
        break;
    }

    // Fill remaining slots with default coords
    for (; ch < RIRLIB_MAX_NUM_OUTPUTS; ch++)
        for (i = 0; i < 2; i++)
            dirsDeg[ch][i] = __default_LScoords64_rad[ch][i] * (180.0f / M_PI);

    // Specify new number of channels (for dynamically changing the number of TFT channels)
    (*nChanOut) = nCH;
}
