# This software module was originally developed by
#
# Ericsson AB
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Ericsson AB retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.

from typing import Any, Dict, List

from mpegi.framework import Bitstream, BitstreamComposer, Payload, ScenePayloadPacketType, tokens

from .. import _constants

syntax: List[tokens.Token] = [
    tokens.Label("AUDIOPLUS PAYLOAD"),
    tokens.writeCountOrIndex("numberOfSources"),
    tokens.Repeat(
        "numberOfSources",
        [
            tokens.writeID("audioSourceId"),
            tokens.Flag("distanceGainModel"),
            tokens.Flag("hasDiffuseness"),
            tokens.Conditional(
                "hasDiffuseness",
                false=[],
                true=[
                    tokens.QuantizedFloat(
                        "diffuseness", quant_range=_constants.audio_diffuseness_qrange
                    )
                ],
            ),
        ],
    ),
]


class AudioplusPayload(Payload):
    def __init__(self, args: Dict[str, Any], artifacts: Dict[str, Any]):
        """Serialises a Directivity packet."""
        super().__init__(args, artifacts)
        composer = BitstreamComposer(syntax)

        codedAudioplus = artifacts["audioplus"]
        composer.push(len(codedAudioplus))
        for item in codedAudioplus:
            composer.push(artifacts["bs_ids"].getId(item[0]))
            composer.push(bool(item[1]))
            composer.push(item[2])
            if item[2]:  # hasDiffuseness
                composer.push(item[3])

        self.payload_as_bitstream = composer.get_bitstream()

    @staticmethod
    def get_payload_type() -> ScenePayloadPacketType:
        return ScenePayloadPacketType.AUDIO_PLUS

    @staticmethod
    def getArtifactSubscription() -> List[str]:
        return ["audioplus", "bs_ids"]

    @staticmethod
    def getArgumentSubscription() -> List[str]:
        return []

    def get_bitstream(self) -> Bitstream:
        return self.payload_as_bitstream
