# This software module was originally developed by
#
# Fraunhofer IIS
#
# in the course of development of ISO/IEC 23090-4 for reference purposes
# and its performance may not have been optimized. This software module
# is an implementation of one or more tools as specified by the ISO/IEC
# 23090-4 standard. ISO/IEC gives you a royalty-free, worldwide,
# non-exclusive, copyright license to copy, distribute, and make
# derivative works of this software module or modifications thereof for
# use in implementations or products claiming conformance to ISO/IEC
# standards incorporating ISO/IEC 23090-4 reference software and which
# satisfy any specified conformance criteria of that standard.
#
# Those intending to use this software module in products are advised that
# its use may infringe existing patents and that the license in the
# previous paragraph grants no licenses under such patents. ISO/IEC and
# the developers of this software module have no liability for use of this
# software module or modifications thereof.
#
# Copyright is not released for products that do not conform to an ISO/IEC
# standard.
#
# Fraunhofer IIS retains full right to modify and use the code for its own
# purpose, assign or donate the code to a third party without consent of
# and without accounting to third parties, and to inhibit third parties
# from using the code for products that do not conform to MPEG-related
# ISO/IEC International Standards.
#
# This copyright notice must be included in all copies or derivative
# works.
#
# Copyright (c) ISO/IEC 2022.

from typing import List

from mpegi.framework import BitstreamComposer, tokens
from mpegi.metadata.eifplus import Anchor

from ... import _constants
from ...components import BitstreamIdentifiers

write_anchors = tokens.Function(
    "writeAnchors",
    syntax=[
        tokens.writeCountOrIndex("anchorsCount"),
        tokens.Repeat(
            "anchorsCount",
            syntax=[
                tokens.writeID("anchorId"),
                tokens.CString("anchorLsdfRef"),
                tokens.Flag("autoRescale"),
                tokens.writeTolerance("tolerance"),
            ],
        ),
    ],
)


def serialize_anchors(
    composer: BitstreamComposer,
    anchors: List[Anchor],
    bs_ids: BitstreamIdentifiers,
) -> None:
    """Push anchors into bitstream composer."""
    composer.push(len(anchors))
    for anchor in anchors:
        composer.push(bs_ids.getId(anchor))
        composer.push(anchor.lsdfRef)
        composer.push(anchor.autoRescale)
        composer.push(anchor.tolerance)
